package com.qhiot.cache.config;

import com.qhiot.cache.J2CacheSpringCacheManageAdapter;
import com.qhiot.cache.redis.DefaultCacheErrorHandler;
import com.qhiot.cache.redis.KeyExpiredMessageDelegate;
import com.qhiot.core.util.StringUtil;
import io.lettuce.core.StatefulRedisConnectionImpl;
import io.lettuce.core.api.StatefulRedisConnection;
import io.lettuce.core.codec.RedisCodec;
import io.lettuce.core.dynamic.codec.AnnotationRedisCodecResolver;
import io.lettuce.core.pubsub.StatefulRedisPubSubConnection;
import net.oschina.j2cache.J2CacheBuilder;
import net.oschina.j2cache.J2CacheConfig;
import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.CachingConfigurerSupport;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.interceptor.CacheErrorHandler;
import org.springframework.cache.interceptor.KeyGenerator;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.connection.RedisPassword;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.lettuce.DefaultLettucePool;
import org.springframework.data.redis.connection.lettuce.LettuceClientConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.connection.lettuce.LettucePoolingClientConfiguration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.listener.PatternTopic;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.data.redis.listener.adapter.MessageListenerAdapter;
import org.springframework.data.redis.serializer.GenericToStringSerializer;
import org.springframework.data.redis.serializer.JdkSerializationRedisSerializer;

import java.io.File;
import java.lang.reflect.Method;
import java.time.Duration;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

/**
 * 缓存配置
 *
 * @author 金建强(ptma @ 163.com)
 * <p>
 * Date: 2015-12-22 9:58.
 * </p>
 */
@Order(4)
@Configuration
@EnableCaching
public class CacheConfig extends CachingConfigurerSupport {

    private static final Logger LOGGER = LoggerFactory.getLogger("Init.Cache");

    @Autowired
    private ApplicationContext ctx;

    @Value("${cache.redis.hostName:localhost}")
    private String hostName;

    @Value("${cache.redis.port:6379}")
    private int port;

    @Value("${cache.redis.database:0}")
    private int database;

    @Value("${cache.redis.password:@null}")
    private String password;

    @Value("${cache.redis.expireTime:600}")
    private int expireTime;

    @Value("#{${cache.redis.expiresMap:null}}")
    private Map<String, Long> expiresMap;

    @Value("${cache.redis.maxTotal:200}")
    private int maxTotal;

    @Value("${cache.redis.maxIdle:10}")
    private int maxIdle;

    @Value("${cache.redis.minIdle:10}")
    private int minIdle;

    @Value("${cache.redis.maxWaitMillis:10000}")
    private long maxWaitMillis;

    @Value("${cache.redis.minEvictableIdleTimeMillis:60000}")
    private long minEvictableIdleTimeMillis;

    @Value("${cache.redis.numTestsPerEvictionRun:10}")
    private int numTestsPerEvictionRun;

    @Value("${cache.redis.timeBetweenEvictionRunsMillis:30000}")
    private long timeBetweenEvictionRunsMillis;

//    @Bean
//    public DefaultLettucePool lettucePool() {
//        DefaultLettucePool pool = new DefaultLettucePool();
//        pool.setHostName(hostName);
//        pool.setPort(port);
//        if (StringUtil.isNotEmpty(password) && !StringUtil.AT_NULL.equals(password)) {
//            pool.setPassword(password);
//        }
//        pool.setDatabase(database);
//        pool.setTimeout(5000);
//        GenericObjectPoolConfig poolConfig = new GenericObjectPoolConfig<>();
//        //设置最大连接总数
//        poolConfig.setMaxTotal(maxTotal);
//        //设置最大空闲数
//        poolConfig.setMaxIdle(maxIdle);
//        //设置最小空闲数
//        poolConfig.setMinIdle(minIdle);
//        poolConfig.setMaxWaitMillis(maxWaitMillis);
//        //在获取连接的时候检查有效性, 默认false
//        poolConfig.setTestOnBorrow(true);
//        poolConfig.setTestOnReturn(false);
//        //Idle时进行连接扫描
//        poolConfig.setTestWhileIdle(true);
//        //逐出扫描的时间间隔(毫秒) 如果为负数,则不运行逐出线程, 默认-1
//        //poolConfig.setTimeBetweenEvictionRunsMillis(timeBetweenEvictionRunsMillis);
//        //表示idle object evitor每次扫描的最多的对象数
//        //poolConfig.setNumTestsPerEvictionRun(numTestsPerEvictionRun);
//        //表示一个对象至少停留在idle状态的最短时间，然后才能被idle object evitor扫描并驱逐；这一项只有在timeBetweenEvictionRunsMillis大于0时才有意义
//        //poolConfig.setMinEvictableIdleTimeMillis(minEvictableIdleTimeMillis);
////        AnnotationRedisCodecResolver annotationRedisCodecResolver = new AnnotationRedisCodecResolver(new ArrayList<RedisCodec<?, ?>>());
//        pool.setPoolConfig(poolConfig);
//        return pool;
//    }
//
//    @Bean(value = "redisConnectionFactory", destroyMethod = "destroy")
//    public LettuceConnectionFactory redisConnectionFactory() {
//        LettuceConnectionFactory connectionFactory = new LettuceConnectionFactory(lettucePool());
////        LettuceConnectionFactory connectionFactory = new DefaultLettucePoolingClientConfiguration();
//        connectionFactory.setDatabase(database);
//        //connectionFactory.setUsePool(true);
//        return connectionFactory;
//    }

    @Bean(value = "redisConnectionFactory", destroyMethod = "destroy")
    public LettuceConnectionFactory redisConnectionFactory() {
        RedisStandaloneConfiguration redisStandaloneConfiguration = new RedisStandaloneConfiguration();
        redisStandaloneConfiguration.setDatabase(database);
        redisStandaloneConfiguration.setHostName(hostName);
        redisStandaloneConfiguration.setPort(port);
        if (StringUtil.isNotEmpty(password) && !StringUtil.AT_NULL.equals(password)) {
            redisStandaloneConfiguration.setPassword(RedisPassword.of(password));
        }

        GenericObjectPoolConfig poolConfig = new GenericObjectPoolConfig();
        //设置最大连接总数
        poolConfig.setMaxTotal(maxTotal);
        //设置最大空闲数
        poolConfig.setMaxIdle(maxIdle);
        //设置最小空闲数
        poolConfig.setMinIdle(minIdle);
        poolConfig.setMaxWaitMillis(maxWaitMillis);
        //在获取连接的时候检查有效性, 默认false
        poolConfig.setTestOnBorrow(true);
        poolConfig.setTestOnReturn(false);
        //Idle时进行连接扫描
        poolConfig.setTestWhileIdle(true);
        poolConfig.setTimeBetweenEvictionRunsMillis(100);

        LettuceClientConfiguration clientConfig = LettucePoolingClientConfiguration.builder()
                .commandTimeout(Duration.ofMillis(5000))
                .poolConfig(poolConfig)
                .build();

        LettuceConnectionFactory factory = new LettuceConnectionFactory(redisStandaloneConfiguration,clientConfig);
//        factory.setShareNativeConnection(false);
        return factory;
    }

    @SuppressWarnings("unchecked")
    @Bean
    public RedisTemplate redisTemplate() {
        RedisTemplate template = new RedisTemplate();
        template.setConnectionFactory(redisConnectionFactory());
        template.setKeySerializer(new GenericToStringSerializer<Object>(Object.class));
        template.setValueSerializer(new JdkSerializationRedisSerializer());
        return template;
    }

    @Override
    public CacheManager cacheManager() {
        // 引入配置
        J2CacheConfig config = null;
        try {
            config = J2CacheConfig.initFromConfig(File.separator + "j2cache.properties");
        } catch (Exception e) {
            System.out.println("文件丟失"+e.getMessage());
        }
        // 生成 J2CacheBuilder
        J2CacheBuilder j2CacheBuilder = J2CacheBuilder.init(config);
        // 构建适配器
        J2CacheSpringCacheManageAdapter j2CacheSpringCacheManageAdapter =
                new J2CacheSpringCacheManageAdapter(j2CacheBuilder, true);

        return j2CacheSpringCacheManageAdapter;
    }


//    @Bean
//    @Override public CacheManager cacheManager() {
//      RedisCacheManager cacheManager = new ExtendedRedisCacheManager(redisTemplate());
//      cacheManager.setUsePrefix(true);
//      cacheManager.setDefaultExpiration(expireTime);
//      if (null != expiresMap && !expiresMap.isEmpty()) {
//          cacheManager.setExpires(expiresMap);
//      }
//      return cacheManager;
//    }


    @Bean
    @Override
    public KeyGenerator keyGenerator() {
        return new KeyGenerator() {

            @Override
            public Object generate(Object o, Method method, Object... objects) {
                StringBuilder sb = new StringBuilder();
                sb.append(o.getClass().getName()).append(".");
                sb.append(method.getName()).append("(");
                boolean first = true;
                for (Object obj : objects) {
                    if (!first) {
                        sb.append(",");
                    }
                    sb.append(obj.hashCode());
                    first = false;
                }
                return sb.append(")").toString();
            }
        };
    }

    @Override
    @Bean("cacheErrorHandler")
    public CacheErrorHandler errorHandler() {
        return new DefaultCacheErrorHandler();
    }

    @Bean
    public KeyExpiredMessageDelegate messageDelegate() {
        return new KeyExpiredMessageDelegate();
    }

    @Bean
    public MessageListenerAdapter messageListenerAdapter() {
        MessageListenerAdapter adapter = new MessageListenerAdapter();
        adapter.setDelegate(messageDelegate());
        return adapter;
    }

    @Bean
    public RedisMessageListenerContainer messageListenerContainer() {
        RedisMessageListenerContainer container = new RedisMessageListenerContainer();
        container.setConnectionFactory(redisConnectionFactory());
        Collection<PatternTopic> topics = new ArrayList<PatternTopic>();
        PatternTopic topic = new PatternTopic("__key*__:expired");
        topics.add(topic);

        container.addMessageListener(messageListenerAdapter(), topics);
        return container;
    }
}
