//package com.qhiot.cache.config;
//
//import com.qhiot.cache.redis.CustomizedRedisCache;
//import com.qhiot.core.util.StringUtil;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;
//import org.springframework.cache.Cache;
//import org.springframework.data.redis.cache.RedisCacheManager;
//import org.springframework.data.redis.core.RedisOperations;
//
//import java.util.Map;
//import java.util.concurrent.ConcurrentHashMap;
//
///**
// * 扩展 RedisCacheManager，支持自定义有效期
// *
// * <p>
// * 使用方法：有效期设置放入缓存名内， 缓存名字和有效期的分隔符默认为#
// * </p>
// * <p>
// *
// * @Cacheable(value = "cache1#600", key = "")， 就表示cache1 的 有效期为 600 秒
// *                  <p>
// *
// * @author 金建强(ptma/@163.com)
// * @version 2017/5/11 17:07
// */
//public class ExtendedRedisCacheManager extends RedisCacheManager {
//
//    private static final Logger LOGGER            = LoggerFactory.getLogger(ExtendedRedisCacheManager.class);
//
//    private String              separator         = "#";
//
//    private long                defaultExpiration = 0;
//    private Map<String, Long>   expires           = null;
//
//    public ExtendedRedisCacheManager(RedisOperations redisOperations) {
//        super(redisOperations);
//    }
//
//    @Override
//    public Cache getCache(String name) {
//        String cacheName = getRealCacheName(name);
//
//        if (StringUtil.isBlank(cacheName)) {
//            return null;
//        }
//
//        long expirationSecondTime = computeExpiration(cacheName);
//
//        Cache cache = super.getCache(cacheName);
//        if (null == cache) {
//            return cache;
//        }
//        CustomizedRedisCache redisCache = new CustomizedRedisCache(cacheName, (isUsePrefix() ? getCachePrefix().prefix(cacheName) : null), getRedisOperations(), expirationSecondTime);
//        return redisCache;
//
//    }
//
//    @Override
//    protected long computeExpiration(String name) {
//        String cacheName = getRealCacheName(name);
//
//        // 优先级： expires -> #(expiration) -> defaultExpiration
//        Long expiration = null;
//        if (expires != null) {
//            expiration = expires.get(cacheName);
//        }
//        if (expiration != null) {
//            return expiration.longValue();
//        }
//
//        String[] cacheParams = name.split(separator);
//        if (cacheParams.length > 1) {
//            try {
//                expiration = Long.parseLong(cacheParams[1]);
//            } catch (NumberFormatException ex) {
//                LOGGER.error(String.format("Cannnot separate expiration time from cache: '%s'", name), ex);
//            }
//
//        }
//
//        return (expiration != null ? expiration.longValue() : defaultExpiration);
//    }
//
//    @Override
//    public void setExpires(Map<String, Long> expires) {
//        super.setExpires(expires);
//        this.expires = (expires != null ? new ConcurrentHashMap<String, Long>(expires) : null);
//    }
//
//    @Override
//    public void setDefaultExpiration(long defaultExpireTime) {
//        super.setDefaultExpiration(defaultExpireTime);
//        defaultExpiration = defaultExpireTime;
//    }
//
//    public void setSeparator(String separator) {
//        this.separator = separator;
//    }
//
//    private String getRealCacheName(String name) {
//        String[] cacheParams = name.split(separator);
//        return cacheParams[0];
//    }
//}
