package com.qhiot.cache.util;

import io.lettuce.core.ClientOptions;
import io.lettuce.core.StatefulRedisConnectionImpl;
import io.lettuce.core.protocol.RedisCommand;
import io.lettuce.core.pubsub.RedisPubSubListener;
import io.lettuce.core.pubsub.StatefulRedisPubSubConnection;
import io.lettuce.core.pubsub.api.async.RedisPubSubAsyncCommands;
import io.lettuce.core.pubsub.api.reactive.RedisPubSubReactiveCommands;
import io.lettuce.core.pubsub.api.sync.RedisPubSubCommands;
import io.lettuce.core.resource.ClientResources;
import net.oschina.j2cache.CacheChannel;
import net.oschina.j2cache.CacheObject;
import net.oschina.j2cache.J2Cache;

import java.time.Duration;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

/**
 * @author 金建强(ptma @ 163.com)
 * @version 2020/5/7 13:00
 */
public class CacheUtil {

    public static void del(String region, String... key) {
        J2Cache.getChannel().evict(region, key);
    }

    public static void del(String region, Collection<String> keys) {
        if (keys != null && !keys.isEmpty()) {
            J2Cache.getChannel().evict(region, keys.toArray(new String[0]));
        }
    }

    public static void deleteByPrefix(String region, String keyPrefix) {
        CacheChannel cacheChannel = J2Cache.getChannel();
        Collection<String> keys = cacheChannel.keys(region);
        Iterator<String> iterator = keys.iterator();
        String key;
        while (iterator.hasNext()) {
            key = iterator.next();
            if (key != null && key.startsWith(keyPrefix)) {
                cacheChannel.evict(region, key);
            }
        }
    }

    public static boolean exists(String region, String key) {
        return J2Cache.getChannel().exists(region, key);
    }

    public static boolean existsL2(String region, String key) {
        return J2Cache.getChannel().existsL2(region, key);
    }

    public static Object get(String region, String key) {
        if (key == null) {
            return null;
        }
        CacheObject cacheObject = J2Cache.getChannel().get(region, key);
        return cacheObject.getValue();
    }

    public static String getString(String region, String key) {
        return key == null ? null : (String) get(region, key);
    }

    public static Object getL2(String region, String key) {
        if (key == null) {
            return null;
        }
        CacheObject cacheObject = J2Cache.getChannel().getL2(region, key);
        return cacheObject.getValue();
    }

    public static String getStringL2(String region, String key) {
        return key == null ? null : (String) getL2(region, key);
    }

    public static void set(String region, String key, Object value) {
        J2Cache.getChannel().set(region, key, value);
    }

    /**
     * 调用此方法的同时， 必须在 caffeine.properties 中添加缓存时间配置， 时间与timeToLiveInSeconds必须一致
     * @param region
     * @param key
     * @param value
     * @param timeToLiveInSeconds 时间(秒) timeToLiveInSeconds 要大于0 如果time小于等于0 将设置无限期
     * @return
     */
    public static void setL2(String region, String key, Object value, long timeToLiveInSeconds) {
        J2Cache.getChannel().setL2(region, key, value, timeToLiveInSeconds);
    }

    public static void setString(String region, String key, String value) {
        set(region, key, value);
    }

    /**
     * 调用此方法的同时， 必须在 caffeine.properties 中添加缓存时间配置， 时间与timeToLiveInSeconds必须一致
     * @param region
     * @param key
     * @param value
     * @param timeToLiveInSeconds
     */
    public static void setStringL2(String region, String key, String value, long timeToLiveInSeconds) {
        J2Cache.getChannel().setL2(region, key, value, timeToLiveInSeconds);
    }

    public static Collection<String> keys(String region) {
        return J2Cache.getChannel().keys(region);
    }

    /**
     * 批量读取缓存中的对象（用户无需判断返回的对象是否为空）
     * @param region Cache region name
     * @param keys cache keys
     * @return cache object
     */
    public static Map<String, CacheObject> get(String region, Collection<String> keys)  {
        return J2Cache.getChannel().get(region, keys);
    }

    public static void clear(String region) {
        J2Cache.getChannel().clear(region);
    }
}
