package com.qhiot.core.bean;

import org.apache.ibatis.session.RowBounds;

import java.util.ArrayList;
import java.util.List;

/**
 * 分页对象
 *
 * @param <E> 分页结果集对象的类型
 * @author 金建强(ptma @ 163.com)
 * <p>
 * Date: 2013-4-2 19:58
 * </p>
 */
public class Pager<E> extends ArrayList<E> {

    public static final int DEFAULT_PAGESIZE = 20;
    private static final long serialVersionUID = 3910983921269295804L;
    /**
     * 是否进行count查询
     */
    private boolean countEnabled = true;

    private int pageNumber;
    private int pageSize;
    private long total;
    private int pages;

    public Pager(int pageNumber) {
        this(pageNumber, DEFAULT_PAGESIZE, true);
    }

    public Pager(int pageNumber, int pageSize) {
        this(pageNumber, pageSize, true);
    }

    public Pager(int pageNumber, int pageSize, boolean countEnabled) {
        this(pageNumber, pageSize, -1);
        this.countEnabled = countEnabled;
    }

    public Pager(int pageNumber, int pageSize, int total) {
        super(pageSize);
        this.pageNumber = pageNumber;
        this.pageSize = pageSize;
        this.total = total;
    }

    public Pager(RowBounds rowBounds, boolean countEnabled) {
        this(rowBounds, -1);
        this.countEnabled = countEnabled;
    }

    public Pager(RowBounds rowBounds, int total) {
        super(rowBounds.getLimit());
        this.pageSize = rowBounds.getLimit();
        this.total = total;
    }

    public boolean isCountEnabled() {
        return countEnabled;
    }

    public List<E> getResult() {
        return this;
    }

    public int getPages() {
        return pages;
    }

    public void setPages(int pages) {
        this.pages = pages;
    }

    public int getPageNumber() {
        return pageNumber;
    }

    public void setPageNumber(int pageNumber) {
        this.pageNumber = pageNumber;
    }

    public int getPageSize() {
        return pageSize;
    }

    public void setPageSize(int pageSize) {
        this.pageSize = pageSize;
    }

    public long getTotal() {
        return total;
    }

    public void setTotal(long total) {
        this.total = total;
        this.pages = (int) (total / this.pageSize + ((total % this.pageSize == 0) ? 0 : 1));
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Pager)) {
            return false;
        }
        if (!super.equals(o)) {
            return false;
        }

        Pager<?> pager = (Pager<?>) o;

        if (countEnabled != pager.countEnabled) {
            return false;
        }
        if (pageNumber != pager.pageNumber) {
            return false;
        }
        if (pageSize != pager.pageSize) {
            return false;
        }
        if (total != pager.total) {
            return false;
        }
        return pages == pager.pages;

    }

    @Override
    public int hashCode() {
        int result = super.hashCode();
        result = 31 * result + (countEnabled ? 1 : 0);
        result = 31 * result + pageNumber;
        result = 31 * result + pageSize;
        result = 31 * result + (int) (total ^ (total >>> 32));
        result = 31 * result + pages;
        return result;
    }
}
