package com.qhiot.core.jackson;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.deser.std.FromStringDeserializer;

/**
 * @author 金建强(ptma / @ 163.com)
 * @version 2017/4/14 10:28
 */
public class HexJsonDeserializer extends FromStringDeserializer<byte[]> {

    public HexJsonDeserializer() {
        super(byte[].class);
    }

    @Override
    protected byte[] _deserialize(String value, DeserializationContext ctxt) throws JsonProcessingException {
        try {
            return hexToBytes(value);
        } catch (Exception e) {
            throw ctxt.weirdStringException(value, byte[].class, "Must be valid HEX value.");
        }
    }

    private byte[] hexToBytes(String hexStr) {
        if (hexStr == null) {
            return new byte[0];
        }
        String str = hexStr.trim().replace(" ", "");
        int len = str.length();
        if (len == 0) {
            return new byte[0];
        }
        if ((len % 2) != 0) {
            str = '0' + str;
        }

        byte[] b = new byte[len / 2];
        for (int i = 0, n = str.length(); i < n; i += 2) {
            b[i / 2] = (byte) Integer.decode("0x" + str.substring(i, i + 2)).intValue();
        }
        return b;
    }
}
