package com.qhiot.core.jackson;

import com.fasterxml.jackson.core.JsonGenerationException;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.jsontype.TypeSerializer;

import java.io.IOException;

/**
 * @author 金建强(ptma / @ 163.com)
 * @version 2017/4/14 10:34
 */
public class HexJsonSerializer extends JsonSerializer<byte[]> {

    private static final String HEX_DIGITS = "0123456789ABCDEF";

    @Override
    public void serialize(byte[] value, JsonGenerator jgen, SerializerProvider provider) throws IOException, JsonGenerationException {
        jgen.writeString(toHexString(value).toUpperCase());
    }

    @Override
    public void serializeWithType(byte[] value, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer) throws IOException, JsonGenerationException {
        typeSer.writeTypePrefixForScalar(value, jgen);
        this.serialize(value, jgen, provider);
        typeSer.writeTypeSuffixForScalar(value, jgen);
    }

    private String toHexString(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0, n = bytes.length; i < n; i++) {
            byte b = bytes[i];
            int high = (b & 0xf0) >> 4;
            int low = b & 0x0f;
            sb.append(HEX_DIGITS.charAt(high));
            sb.append(HEX_DIGITS.charAt(low));
        }
        return sb.toString();
    }
}
