package com.qhiot.core.sql.mybatis.plugin;

import org.apache.ibatis.executor.Executor;
import org.apache.ibatis.logging.Log;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.plugin.*;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.session.RowBounds;

import java.util.Properties;

/**
 * SQL执行耗时检测拦截器 <br>
 * <br>
 * 在开发和测试环境中中使用, 实际运行环境中不推荐加载本插件
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2014-8-20 10:24:42
 */
@Intercepts({
        @Signature(type = Executor.class, method = "update", args = {MappedStatement.class, Object.class}),
        @Signature(type = Executor.class, method = "query", args = {MappedStatement.class, Object.class,
                RowBounds.class, ResultHandler.class})})
public class SqlTimeConsumingInterceptor implements Interceptor {

    private int timeConsumingWarnUpperBound = 2000;

    @Override
    public Object intercept(Invocation invocation) throws Exception {
        MappedStatement mappedStatement = (MappedStatement) invocation.getArgs()[0];
        Object returnValue = null;
        long start = System.currentTimeMillis();
        try {
            returnValue = invocation.proceed();
        } finally {
            long end = System.currentTimeMillis();
            long time = (end - start);
            // 打印查询时间大于50ms的sql语句
            Log logger = mappedStatement.getStatementLog();
            if (time > timeConsumingWarnUpperBound) {
                logger.warn("耗时: " + time + "ms");
            } else {
                logger.debug("耗时: " + time + "ms");
            }
        }

        return returnValue;
    }

    @Override
    public Object plugin(Object target) {
        return Plugin.wrap(target, this);
    }

    @Override
    public void setProperties(Properties properties) {
        String timeConsumingWarnUpperBound = properties.getProperty("timeConsumingWarnUpperBound");
        if (timeConsumingWarnUpperBound != null && timeConsumingWarnUpperBound.trim().length() > 0) {
            this.timeConsumingWarnUpperBound = Integer.parseInt(timeConsumingWarnUpperBound);
        }
    }
}
