package com.qhiot.core.util;

import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.Set;
import java.util.TreeSet;

/**
 * 数组工具
 * <p>
 * User: 金建强(ptma@163.com)
 * </p>
 * <p>
 * Date: 2015-11-11 17:02.
 * </p>
 */
public final class ArrayUtil {

    private ArrayUtil() {
    }

    public static boolean isEmpty(final Object[] array) {
        return null == array || 0 == array.length;
    }

    public static boolean isEmpty(final long[] array) {
        return array == null || array.length == 0;
    }

    public static boolean isEmpty(final int[] array) {
        return array == null || array.length == 0;
    }

    public static boolean isEmpty(final short[] array) {
        return array == null || array.length == 0;
    }

    public static boolean isEmpty(final char[] array) {
        return array == null || array.length == 0;
    }

    public static boolean isEmpty(final byte[] array) {
        return array == null || array.length == 0;
    }

    public static boolean isEmpty(final double[] array) {
        return array == null || array.length == 0;
    }

    public static boolean isEmpty(final float[] array) {
        return array == null || array.length == 0;
    }

    public static boolean isEmpty(final boolean[] array) {
        return array == null || array.length == 0;
    }

    public static boolean isNotEmpty(final Object[] array) {
        return array != null && array.length != 0;
    }

    public static boolean isNotEmpty(final long[] array) {
        return array != null && array.length != 0;
    }

    public static boolean isNotEmpty(final int[] array) {
        return array != null && array.length != 0;
    }

    public static boolean isNotEmpty(final short[] array) {
        return array != null && array.length != 0;
    }

    public static boolean isNotEmpty(final char[] array) {
        return array != null && array.length != 0;
    }

    public static boolean isNotEmpty(final byte[] array) {
        return array != null && array.length != 0;
    }

    public static boolean isNotEmpty(final double[] array) {
        return array != null && array.length != 0;
    }

    public static boolean isNotEmpty(final float[] array) {
        return array != null && array.length != 0;
    }

    public static boolean isNotEmpty(final boolean[] array) {
        return array != null && array.length != 0;
    }

    /**
     * 将参数包装为数组.
     */
    public static <T> T[] array(T... elements) {
        return elements;
    }

    /**
     * 将参数包装为数组.
     */
    public static byte[] array(byte... elements) {
        return elements;
    }

    /**
     * 将参数包装为数组.
     */
    public static char[] array(char... elements) {
        return elements;
    }

    /**
     * 将参数包装为数组.
     */
    public static short[] array(short... elements) {
        return elements;
    }

    /**
     * 将参数包装为数组.
     */
    public static int[] array(int... elements) {
        return elements;
    }

    /**
     * 将参数包装为数组.
     */
    public static long[] array(long... elements) {
        return elements;
    }

    /**
     * 将参数包装为数组.
     */
    public static float[] array(float... elements) {
        return elements;
    }

    /**
     * 将参数包装为数组.
     */
    public static double[] array(double... elements) {
        return elements;
    }

    /**
     * 将参数包装为数组.
     */
    public static boolean[] array(boolean... elements) {
        return elements;
    }

    /**
     * 合并多个数组
     *
     * @param first 第一个数组
     * @param rest  剩余数组
     * @param <T>   泛型
     * @return 新的合并后的数组
     */
    @SuppressWarnings("unchecked")
    public static <T> T[] concat(T[] first, T[]... rest) {
        int totalLength = 0;
        int offset = 0;
        if (first != null) {
            totalLength += first.length;
        }

        for (T[] array : rest) {
            if (array != null) {
                totalLength += array.length;
            }
        }
        T[] result = (T[]) Array.newInstance(first.getClass().getComponentType(), totalLength);

        if (first != null) {
            System.arraycopy(first, 0, result, offset, first.length);
            offset += first.length;
        }
        for (T[] array : rest) {
            if (array != null) {
                System.arraycopy(array, 0, result, offset, array.length);
                offset += array.length;
            }
        }
        return result;
    }

    /**
     * 合并多个数组
     *
     * @param first 第一个数组
     * @param rest  剩余数组
     * @return 新的合并后的数组
     */
    public static boolean[] concat(boolean[] first, boolean[]... rest) {
        int totalLength = 0;
        int offset = 0;
        if (first != null) {
            totalLength += first.length;
        }

        for (boolean[] array : rest) {
            if (array != null) {
                totalLength += array.length;
            }
        }
        boolean[] result = new boolean[totalLength];

        if (first != null) {
            System.arraycopy(first, 0, result, offset, first.length);
            offset += first.length;
        }
        for (boolean[] array : rest) {
            if (array != null) {
                System.arraycopy(array, 0, result, offset, array.length);
                offset += array.length;
            }
        }
        return result;
    }

    /**
     * 合并多个数组
     *
     * @param first 第一个数组
     * @param rest  剩余数组
     * @return 新的合并后的数组
     */
    public static char[] concat(char[] first, char[]... rest) {
        int totalLength = 0;
        int offset = 0;
        if (first != null) {
            totalLength += first.length;
        }

        for (char[] array : rest) {
            if (array != null) {
                totalLength += array.length;
            }
        }
        char[] result = new char[totalLength];

        if (first != null) {
            System.arraycopy(first, 0, result, offset, first.length);
            offset += first.length;
        }
        for (char[] array : rest) {
            if (array != null) {
                System.arraycopy(array, 0, result, offset, array.length);
                offset += array.length;
            }
        }
        return result;
    }

    /**
     * 合并多个数组
     *
     * @param first 第一个数组
     * @param rest  剩余数组
     * @return 新的合并后的数组
     */
    public static byte[] concat(byte[] first, byte[]... rest) {
        int totalLength = 0;
        int offset = 0;
        if (first != null) {
            totalLength += first.length;
        }

        for (byte[] array : rest) {
            if (array != null) {
                totalLength += array.length;
            }
        }
        byte[] result = new byte[totalLength];

        if (first != null) {
            System.arraycopy(first, 0, result, offset, first.length);
            offset += first.length;
        }
        for (byte[] array : rest) {
            if (array != null) {
                System.arraycopy(array, 0, result, offset, array.length);
                offset += array.length;
            }
        }
        return result;
    }

    /**
     * 合并多个数组
     *
     * @param first 第一个数组
     * @param rest  剩余数组
     * @return 新的合并后的数组
     */
    public static short[] concat(short[] first, short[]... rest) {
        int totalLength = 0;
        int offset = 0;
        if (first != null) {
            totalLength += first.length;
        }

        for (short[] array : rest) {
            if (array != null) {
                totalLength += array.length;
            }
        }
        short[] result = new short[totalLength];

        if (first != null) {
            System.arraycopy(first, 0, result, offset, first.length);
            offset += first.length;
        }
        for (short[] array : rest) {
            if (array != null) {
                System.arraycopy(array, 0, result, offset, array.length);
                offset += array.length;
            }
        }
        return result;
    }

    /**
     * 合并多个数组
     *
     * @param first 第一个数组
     * @param rest  剩余数组
     * @return 新的合并后的数组
     */
    public static int[] concat(int[] first, int[]... rest) {
        int totalLength = 0;
        int offset = 0;
        if (first != null) {
            totalLength += first.length;
        }

        for (int[] array : rest) {
            if (array != null) {
                totalLength += array.length;
            }
        }
        int[] result = new int[totalLength];

        if (first != null) {
            System.arraycopy(first, 0, result, offset, first.length);
            offset += first.length;
        }
        for (int[] array : rest) {
            if (array != null) {
                System.arraycopy(array, 0, result, offset, array.length);
                offset += array.length;
            }
        }
        return result;
    }

    /**
     * 合并多个数组
     *
     * @param first 第一个数组
     * @param rest  剩余数组
     * @return 新的合并后的数组
     */
    public static long[] concat(long[] first, long[]... rest) {
        int totalLength = 0;
        int offset = 0;
        if (first != null) {
            totalLength += first.length;
        }

        for (long[] array : rest) {
            if (array != null) {
                totalLength += array.length;
            }
        }
        long[] result = new long[totalLength];

        if (first != null) {
            System.arraycopy(first, 0, result, offset, first.length);
            offset += first.length;
        }
        for (long[] array : rest) {
            if (array != null) {
                System.arraycopy(array, 0, result, offset, array.length);
                offset += array.length;
            }
        }
        return result;
    }

    /**
     * 合并多个数组
     *
     * @param first 第一个数组
     * @param rest  剩余数组
     * @return 新的合并后的数组
     */
    public static float[] concat(float[] first, float[]... rest) {
        int totalLength = 0;
        int offset = 0;
        if (first != null) {
            totalLength += first.length;
        }

        for (float[] array : rest) {
            if (array != null) {
                totalLength += array.length;
            }
        }
        float[] result = new float[totalLength];

        if (first != null) {
            System.arraycopy(first, 0, result, offset, first.length);
            offset += first.length;
        }
        for (float[] array : rest) {
            if (array != null) {
                System.arraycopy(array, 0, result, offset, array.length);
                offset += array.length;
            }
        }
        return result;
    }

    /**
     * 合并多个数组
     *
     * @param first 第一个数组
     * @param rest  剩余数组
     * @return 新的合并后的数组
     */
    public static double[] concat(double[] first, double[]... rest) {
        int totalLength = 0;
        int offset = 0;
        if (first != null) {
            totalLength += first.length;
        }

        for (double[] array : rest) {
            if (array != null) {
                totalLength += array.length;
            }
        }
        double[] result = new double[totalLength];

        if (first != null) {
            System.arraycopy(first, 0, result, offset, first.length);
            offset += first.length;
        }
        for (double[] array : rest) {
            if (array != null) {
                System.arraycopy(array, 0, result, offset, array.length);
                offset += array.length;
            }
        }
        return result;
    }

    /**
     * 反转数组
     */
    public static <T> T[] reverse(T[] array) {
        if (array == null) {
            return null;
        }
        T[] duplicate = Arrays.copyOf(array, array.length);
        int i = 0;
        int j = array.length - 1;
        T tmp;
        while (j > i) {
            tmp = duplicate[j];
            duplicate[j] = duplicate[i];
            duplicate[i] = tmp;
            j--;
            i++;
        }
        return duplicate;
    }

    /**
     * 反转数组
     */
    public static short[] reverse(short[] array) {
        if (array == null) {
            return null;
        }
        short[] duplicate = Arrays.copyOf(array, array.length);
        int i = 0;
        int j = array.length - 1;
        short tmp;
        while (j > i) {
            tmp = duplicate[j];
            duplicate[j] = duplicate[i];
            duplicate[i] = tmp;
            j--;
            i++;
        }
        return duplicate;
    }

    /**
     * 反转数组
     */
    public static int[] reverse(int[] array) {
        if (array == null) {
            return null;
        }
        int[] duplicate = Arrays.copyOf(array, array.length);
        int i = 0;
        int j = array.length - 1;
        int tmp;
        while (j > i) {
            tmp = duplicate[j];
            duplicate[j] = duplicate[i];
            duplicate[i] = tmp;
            j--;
            i++;
        }
        return duplicate;
    }

    /**
     * 反转数组
     */
    public static long[] reverse(long[] array) {
        if (array == null) {
            return null;
        }
        long[] duplicate = Arrays.copyOf(array, array.length);
        int i = 0;
        int j = array.length - 1;
        long tmp;
        while (j > i) {
            tmp = duplicate[j];
            duplicate[j] = duplicate[i];
            duplicate[i] = tmp;
            j--;
            i++;
        }
        return duplicate;
    }

    /**
     * 反转数组
     */
    public static char[] reverse(char[] array) {
        if (array == null) {
            return null;
        }
        char[] duplicate = Arrays.copyOf(array, array.length);
        int i = 0;
        int j = array.length - 1;
        char tmp;
        while (j > i) {
            tmp = duplicate[j];
            duplicate[j] = duplicate[i];
            duplicate[i] = tmp;
            j--;
            i++;
        }
        return duplicate;
    }

    /**
     * 反转数组
     */
    public static byte[] reverse(byte[] array) {
        if (array == null) {
            return null;
        }
        byte[] duplicate = Arrays.copyOf(array, array.length);
        int i = 0;
        int j = array.length - 1;
        byte tmp;
        while (j > i) {
            tmp = duplicate[j];
            duplicate[j] = duplicate[i];
            duplicate[i] = tmp;
            j--;
            i++;
        }
        return duplicate;
    }

    /**
     * 反转数组
     */
    public static double[] reverse(double[] array) {
        if (array == null) {
            return null;
        }
        double[] duplicate = Arrays.copyOf(array, array.length);
        int i = 0;
        int j = array.length - 1;
        double tmp;
        while (j > i) {
            tmp = duplicate[j];
            duplicate[j] = duplicate[i];
            duplicate[i] = tmp;
            j--;
            i++;
        }
        return duplicate;
    }

    /**
     * 反转数组
     */
    public static float[] reverse(float[] array) {
        if (array == null) {
            return null;
        }
        float[] duplicate = Arrays.copyOf(array, array.length);
        int i = 0;
        int j = array.length - 1;
        float tmp;
        while (j > i) {
            tmp = duplicate[j];
            duplicate[j] = duplicate[i];
            duplicate[i] = tmp;
            j--;
            i++;
        }
        return duplicate;
    }

    /**
     * 反转数组
     */
    public static boolean[] reverse(boolean[] array) {
        if (array == null) {
            return null;
        }
        boolean[] duplicate = Arrays.copyOf(array, array.length);
        int i = 0;
        int j = array.length - 1;
        boolean tmp;
        while (j > i) {
            tmp = duplicate[j];
            duplicate[j] = duplicate[i];
            duplicate[i] = tmp;
            j--;
            i++;
        }
        return duplicate;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] resize(T[] buffer, int newSize) {
        Class<T> componentType = (Class<T>) buffer.getClass().getComponentType();
        T[] temp = (T[]) Array.newInstance(componentType, newSize);
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    public static String[] resize(String[] buffer, int newSize) {
        String[] temp = new String[newSize];
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    public static byte[] resize(byte[] buffer, int newSize) {
        byte[] temp = new byte[newSize];
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    public static char[] resize(char[] buffer, int newSize) {
        char[] temp = new char[newSize];
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    public static short[] resize(short[] buffer, int newSize) {
        short[] temp = new short[newSize];
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    public static int[] resize(int[] buffer, int newSize) {
        int[] temp = new int[newSize];
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    public static long[] resize(long[] buffer, int newSize) {
        long[] temp = new long[newSize];
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    public static float[] resize(float[] buffer, int newSize) {
        float[] temp = new float[newSize];
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    public static double[] resize(double[] buffer, int newSize) {
        double[] temp = new double[newSize];
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 调整数组大小, 并复制元素到新数组
     */
    public static boolean[] resize(boolean[] buffer, int newSize) {
        boolean[] temp = new boolean[newSize];
        System.arraycopy(buffer, 0, temp, 0, buffer.length >= newSize ? newSize : buffer.length);
        return temp;
    }

    /**
     * 追加元素到数组.
     */
    public static <T> T[] append(T[] buffer, T newElement) {
        T[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 追加元素到数组.
     */
    public static String[] append(String[] buffer, String newElement) {
        String[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 追加元素到数组.
     */
    public static byte[] append(byte[] buffer, byte newElement) {
        byte[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 追加元素到数组.
     */
    public static char[] append(char[] buffer, char newElement) {
        char[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 追加元素到数组.
     */
    public static short[] append(short[] buffer, short newElement) {
        short[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 追加元素到数组.
     */
    public static int[] append(int[] buffer, int newElement) {
        int[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 追加元素到数组.
     */
    public static long[] append(long[] buffer, long newElement) {
        long[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 追加元素到数组.
     */
    public static float[] append(float[] buffer, float newElement) {
        float[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 追加元素到数组.
     */
    public static double[] append(double[] buffer, double newElement) {
        double[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 追加元素到数组.
     */
    public static boolean[] append(boolean[] buffer, boolean newElement) {
        boolean[] t = resize(buffer, buffer.length + 1);
        t[buffer.length] = newElement;
        return t;
    }

    /**
     * 移除子数组
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] remove(T[] buffer, int offset, int length) {
        Class<T> componentType = (Class<T>) buffer.getClass().getComponentType();
        return remove(buffer, offset, length, componentType);
    }

    /**
     * 移除子数组
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] remove(T[] buffer, int offset, int length, Class<T> componentType) {
        int len2 = buffer.length - length;
        T[] temp = (T[]) Array.newInstance(componentType, len2);
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 移除子数组
     */
    public static String[] remove(String[] buffer, int offset, int length) {
        int len2 = buffer.length - length;
        String[] temp = new String[len2];
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 移除子数组
     */
    public static byte[] remove(byte[] buffer, int offset, int length) {
        int len2 = buffer.length - length;
        byte[] temp = new byte[len2];
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 移除子数组
     */
    public static char[] remove(char[] buffer, int offset, int length) {
        int len2 = buffer.length - length;
        char[] temp = new char[len2];
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 移除子数组
     */
    public static short[] remove(short[] buffer, int offset, int length) {
        int len2 = buffer.length - length;
        short[] temp = new short[len2];
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 移除子数组
     */
    public static int[] remove(int[] buffer, int offset, int length) {
        int len2 = buffer.length - length;
        int[] temp = new int[len2];
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 移除子数组
     */
    public static long[] remove(long[] buffer, int offset, int length) {
        int len2 = buffer.length - length;
        long[] temp = new long[len2];
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 移除子数组
     */
    public static float[] remove(float[] buffer, int offset, int length) {
        int len2 = buffer.length - length;
        float[] temp = new float[len2];
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 移除子数组
     */
    public static double[] remove(double[] buffer, int offset, int length) {
        int len2 = buffer.length - length;
        double[] temp = new double[len2];
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 移除子数组
     */
    public static boolean[] remove(boolean[] buffer, int offset, int length) {
        int len2 = buffer.length - length;
        boolean[] temp = new boolean[len2];
        System.arraycopy(buffer, 0, temp, 0, offset);
        System.arraycopy(buffer, offset + length, temp, offset, len2 - offset);
        return temp;
    }

    /**
     * 取子数组
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] subarray(T[] buffer, int offset, int length) {
        Class<T> componentType = (Class<T>) buffer.getClass().getComponentType();
        return subarray(buffer, offset, length, componentType);
    }

    /**
     * 取子数组
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] subarray(T[] buffer, int offset, int length, Class<T> componentType) {
        T[] temp = (T[]) Array.newInstance(componentType, length);
        System.arraycopy(buffer, offset, temp, 0, length);
        return temp;
    }

    /**
     * 取子数组
     */
    public static byte[] subarray(byte[] buffer, int offset, int length) {
        byte[] temp = new byte[length];
        System.arraycopy(buffer, offset, temp, 0, length);
        return temp;
    }

    /**
     * 取子数组
     */
    public static char[] subarray(char[] buffer, int offset, int length) {
        char[] temp = new char[length];
        System.arraycopy(buffer, offset, temp, 0, length);
        return temp;
    }

    /**
     * 取子数组
     */
    public static short[] subarray(short[] buffer, int offset, int length) {
        short[] temp = new short[length];
        System.arraycopy(buffer, offset, temp, 0, length);
        return temp;
    }

    /**
     * 取子数组
     */
    public static int[] subarray(int[] buffer, int offset, int length) {
        int[] temp = new int[length];
        System.arraycopy(buffer, offset, temp, 0, length);
        return temp;
    }

    /**
     * 取子数组
     */
    public static long[] subarray(long[] buffer, int offset, int length) {
        long[] temp = new long[length];
        System.arraycopy(buffer, offset, temp, 0, length);
        return temp;
    }

    /**
     * 取子数组
     */
    public static float[] subarray(float[] buffer, int offset, int length) {
        float[] temp = new float[length];
        System.arraycopy(buffer, offset, temp, 0, length);
        return temp;
    }

    /**
     * 取子数组
     */
    public static double[] subarray(double[] buffer, int offset, int length) {
        double[] temp = new double[length];
        System.arraycopy(buffer, offset, temp, 0, length);
        return temp;
    }

    /**
     * 取子数组
     */
    public static boolean[] subarray(boolean[] buffer, int offset, int length) {
        boolean[] temp = new boolean[length];
        System.arraycopy(buffer, offset, temp, 0, length);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] insert(T[] dest, T[] src, int offset) {
        Class<T> componentType = (Class<T>) dest.getClass().getComponentType();
        return insert(dest, src, offset, componentType);
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] insert(T[] dest, T src, int offset) {
        Class<T> componentType = (Class<T>) dest.getClass().getComponentType();
        return insert(dest, src, offset, componentType);
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] insert(T[] dest, T[] src, int offset, Class<T> componentType) {
        T[] temp = (T[]) Array.newInstance(componentType, dest.length + src.length);
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] insert(T[] dest, T src, int offset, Class<T> componentType) {
        T[] temp = (T[]) Array.newInstance(componentType, dest.length + 1);
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static String[] insert(String[] dest, String[] src, int offset) {
        String[] temp = new String[dest.length + src.length];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static String[] insert(String[] dest, String src, int offset) {
        String[] temp = new String[dest.length + 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static byte[] insert(byte[] dest, byte[] src, int offset) {
        byte[] temp = new byte[dest.length + src.length];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static byte[] insert(byte[] dest, byte src, int offset) {
        byte[] temp = new byte[dest.length + 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static char[] insert(char[] dest, char[] src, int offset) {
        char[] temp = new char[dest.length + src.length];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static char[] insert(char[] dest, char src, int offset) {
        char[] temp = new char[dest.length + 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static short[] insert(short[] dest, short[] src, int offset) {
        short[] temp = new short[dest.length + src.length];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static short[] insert(short[] dest, short src, int offset) {
        short[] temp = new short[dest.length + 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static int[] insert(int[] dest, int[] src, int offset) {
        int[] temp = new int[dest.length + src.length];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static int[] insert(int[] dest, int src, int offset) {
        int[] temp = new int[dest.length + 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static long[] insert(long[] dest, long[] src, int offset) {
        long[] temp = new long[dest.length + src.length];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static long[] insert(long[] dest, long src, int offset) {
        long[] temp = new long[dest.length + 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static float[] insert(float[] dest, float[] src, int offset) {
        float[] temp = new float[dest.length + src.length];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static float[] insert(float[] dest, float src, int offset) {
        float[] temp = new float[dest.length + 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static double[] insert(double[] dest, double[] src, int offset) {
        double[] temp = new double[dest.length + src.length];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static double[] insert(double[] dest, double src, int offset) {
        double[] temp = new double[dest.length + 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static boolean[] insert(boolean[] dest, boolean[] src, int offset) {
        boolean[] temp = new boolean[dest.length + src.length];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset, temp, src.length + offset, dest.length - offset);
        return temp;
    }

    /**
     * 插入一个数组到目标数组的指定位置
     */
    public static boolean[] insert(boolean[] dest, boolean src, int offset) {
        boolean[] temp = new boolean[dest.length + 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        temp[offset] = src;
        System.arraycopy(dest, offset, temp, offset + 1, dest.length - offset);
        return temp;
    }

    // ---------------------------------------------------------------- insertAt

    /**
     * Inserts one array into another at given offset.
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] insertAt(T[] dest, T[] src, int offset) {
        Class<T> componentType = (Class<T>) dest.getClass().getComponentType();
        return insertAt(dest, src, offset, componentType);
    }

    /**
     * Inserts one array into another at given offset.
     */
    @SuppressWarnings({"unchecked"})
    public static <T> T[] insertAt(T[] dest, T[] src, int offset, Class<T> componentType) {
        T[] temp = (T[]) Array.newInstance(componentType, dest.length + src.length - 1);
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    /**
     * Inserts one array into another by replacing specified offset.
     */
    public static String[] insertAt(String[] dest, String[] src, int offset) {
        String[] temp = new String[dest.length + src.length - 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    /**
     * Inserts one array into another by replacing specified offset.
     */
    public static byte[] insertAt(byte[] dest, byte[] src, int offset) {
        byte[] temp = new byte[dest.length + src.length - 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    /**
     * Inserts one array into another by replacing specified offset.
     */
    public static char[] insertAt(char[] dest, char[] src, int offset) {
        char[] temp = new char[dest.length + src.length - 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    /**
     * Inserts one array into another by replacing specified offset.
     */
    public static short[] insertAt(short[] dest, short[] src, int offset) {
        short[] temp = new short[dest.length + src.length - 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    /**
     * Inserts one array into another by replacing specified offset.
     */
    public static int[] insertAt(int[] dest, int[] src, int offset) {
        int[] temp = new int[dest.length + src.length - 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    /**
     * Inserts one array into another by replacing specified offset.
     */
    public static long[] insertAt(long[] dest, long[] src, int offset) {
        long[] temp = new long[dest.length + src.length - 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    /**
     * Inserts one array into another by replacing specified offset.
     */
    public static float[] insertAt(float[] dest, float[] src, int offset) {
        float[] temp = new float[dest.length + src.length - 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    /**
     * Inserts one array into another by replacing specified offset.
     */
    public static double[] insertAt(double[] dest, double[] src, int offset) {
        double[] temp = new double[dest.length + src.length - 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    /**
     * Inserts one array into another by replacing specified offset.
     */
    public static boolean[] insertAt(boolean[] dest, boolean[] src, int offset) {
        boolean[] temp = new boolean[dest.length + src.length - 1];
        System.arraycopy(dest, 0, temp, 0, offset);
        System.arraycopy(src, 0, temp, offset, src.length);
        System.arraycopy(dest, offset + 1, temp, src.length + offset, dest.length - offset - 1);
        return temp;
    }

    // ---------------------------------------------------------------- convert

    /**
     * Converts to primitive array.
     */
    public static byte[] values(Byte[] array) {
        byte[] dest = new byte[array.length];
        for (int i = 0; i < array.length; i++) {
            Byte v = array[i];
            if (v != null) {
                dest[i] = v;
            }
        }
        return dest;
    }

    /**
     * Converts to object array.
     */
    public static Byte[] valuesOf(byte[] array) {
        Byte[] dest = new Byte[array.length];
        for (int i = 0; i < array.length; i++) {
            dest[i] = array[i];
        }
        return dest;
    }

    /**
     * Converts to primitive array.
     */
    public static char[] values(Character[] array) {
        char[] dest = new char[array.length];
        for (int i = 0; i < array.length; i++) {
            Character v = array[i];
            if (v != null) {
                dest[i] = v;
            }
        }
        return dest;
    }

    /**
     * Converts to object array.
     */
    public static Character[] valuesOf(char[] array) {
        Character[] dest = new Character[array.length];
        for (int i = 0; i < array.length; i++) {
            dest[i] = array[i];
        }
        return dest;
    }

    /**
     * Converts to primitive array.
     */
    public static short[] values(Short[] array) {
        short[] dest = new short[array.length];
        for (int i = 0; i < array.length; i++) {
            Short v = array[i];
            if (v != null) {
                dest[i] = v;
            }
        }
        return dest;
    }

    /**
     * Converts to object array.
     */
    public static Short[] valuesOf(short[] array) {
        Short[] dest = new Short[array.length];
        for (int i = 0; i < array.length; i++) {
            dest[i] = array[i];
        }
        return dest;
    }

    /**
     * Converts to primitive array.
     */
    public static int[] values(Integer[] array) {
        int[] dest = new int[array.length];
        for (int i = 0; i < array.length; i++) {
            Integer v = array[i];
            if (v != null) {
                dest[i] = v;
            }
        }
        return dest;
    }

    /**
     * Converts to object array.
     */
    public static Integer[] valuesOf(int[] array) {
        Integer[] dest = new Integer[array.length];
        for (int i = 0; i < array.length; i++) {
            dest[i] = array[i];
        }
        return dest;
    }

    /**
     * Converts to primitive array.
     */
    public static long[] values(Long[] array) {
        long[] dest = new long[array.length];
        for (int i = 0; i < array.length; i++) {
            Long v = array[i];
            if (v != null) {
                dest[i] = v;
            }
        }
        return dest;
    }

    /**
     * Converts to object array.
     */
    public static Long[] valuesOf(long[] array) {
        Long[] dest = new Long[array.length];
        for (int i = 0; i < array.length; i++) {
            dest[i] = array[i];
        }
        return dest;
    }

    /**
     * Converts to primitive array.
     */
    public static float[] values(Float[] array) {
        float[] dest = new float[array.length];
        for (int i = 0; i < array.length; i++) {
            Float v = array[i];
            if (v != null) {
                dest[i] = v;
            }
        }
        return dest;
    }

    /**
     * Converts to object array.
     */
    public static Float[] valuesOf(float[] array) {
        Float[] dest = new Float[array.length];
        for (int i = 0; i < array.length; i++) {
            dest[i] = array[i];
        }
        return dest;
    }

    /**
     * Converts to primitive array.
     */
    public static double[] values(Double[] array) {
        double[] dest = new double[array.length];
        for (int i = 0; i < array.length; i++) {
            Double v = array[i];
            if (v != null) {
                dest[i] = v;
            }
        }
        return dest;
    }

    /**
     * Converts to object array.
     */
    public static Double[] valuesOf(double[] array) {
        Double[] dest = new Double[array.length];
        for (int i = 0; i < array.length; i++) {
            dest[i] = array[i];
        }
        return dest;
    }

    /**
     * Converts to primitive array.
     */
    public static boolean[] values(Boolean[] array) {
        boolean[] dest = new boolean[array.length];
        for (int i = 0; i < array.length; i++) {
            Boolean v = array[i];
            if (v != null) {
                dest[i] = v;
            }
        }
        return dest;
    }

    /**
     * Converts to object array.
     */
    public static Boolean[] valuesOf(boolean[] array) {
        Boolean[] dest = new Boolean[array.length];
        for (int i = 0; i < array.length; i++) {
            dest[i] = array[i];
        }
        return dest;
    }

    // ---------------------------------------------------------------- indexof

    /**
     * Finds the first occurrence of an element in an array.
     */
    public static int indexOf(byte[] array, byte value) {
        for (int i = 0; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Returns <code>true</code> if an array contains given value.
     */
    public static boolean contains(byte[] array, byte value) {
        return indexOf(array, value) != -1;
    }

    /**
     * Finds the first occurrence of given value in an array from specified given position.
     */
    public static int indexOf(byte[] array, byte value, int startIndex) {
        for (int i = startIndex; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 在指定的区间内查找指定值在源数组中第一次出现的位置.
     */
    public static int indexOf(byte[] array, byte value, int startIndex, int endIndex) {
        for (int i = startIndex; i < endIndex; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Finds the first occurrence of an element in an array.
     */
    public static int indexOf(char[] array, char value) {
        for (int i = 0; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Returns <code>true</code> if an array contains given value.
     */
    public static boolean contains(char[] array, char value) {
        return indexOf(array, value) != -1;
    }

    /**
     * Finds the first occurrence of given value in an array from specified given position.
     */
    public static int indexOf(char[] array, char value, int startIndex) {
        for (int i = startIndex; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 在指定的区间内查找指定值在源数组中第一次出现的位置.
     */
    public static int indexOf(char[] array, char value, int startIndex, int endIndex) {
        for (int i = startIndex; i < endIndex; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Finds the first occurrence of an element in an array.
     */
    public static int indexOf(short[] array, short value) {
        for (int i = 0; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Returns <code>true</code> if an array contains given value.
     */
    public static boolean contains(short[] array, short value) {
        return indexOf(array, value) != -1;
    }

    /**
     * Finds the first occurrence of given value in an array from specified given position.
     */
    public static int indexOf(short[] array, short value, int startIndex) {
        for (int i = startIndex; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 在指定的区间内查找指定值在源数组中第一次出现的位置.
     */
    public static int indexOf(short[] array, short value, int startIndex, int endIndex) {
        for (int i = startIndex; i < endIndex; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Finds the first occurrence of an element in an array.
     */
    public static int indexOf(int[] array, int value) {
        for (int i = 0; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Returns <code>true</code> if an array contains given value.
     */
    public static boolean contains(int[] array, int value) {
        return indexOf(array, value) != -1;
    }

    /**
     * Finds the first occurrence of given value in an array from specified given position.
     */
    public static int indexOf(int[] array, int value, int startIndex) {
        for (int i = startIndex; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 在指定的区间内查找指定值在源数组中第一次出现的位置.
     */
    public static int indexOf(int[] array, int value, int startIndex, int endIndex) {
        for (int i = startIndex; i < endIndex; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Finds the first occurrence of an element in an array.
     */
    public static int indexOf(long[] array, long value) {
        for (int i = 0; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Returns <code>true</code> if an array contains given value.
     */
    public static boolean contains(long[] array, long value) {
        return indexOf(array, value) != -1;
    }

    /**
     * Finds the first occurrence of given value in an array from specified given position.
     */
    public static int indexOf(long[] array, long value, int startIndex) {
        for (int i = startIndex; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 在指定的区间内查找指定值在源数组中第一次出现的位置.
     */
    public static int indexOf(long[] array, long value, int startIndex, int endIndex) {
        for (int i = startIndex; i < endIndex; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Finds the first occurrence of an element in an array.
     */
    public static int indexOf(boolean[] array, boolean value) {
        for (int i = 0; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Returns <code>true</code> if an array contains given value.
     */
    public static boolean contains(boolean[] array, boolean value) {
        return indexOf(array, value) != -1;
    }

    /**
     * Finds the first occurrence of given value in an array from specified given position.
     */
    public static int indexOf(boolean[] array, boolean value, int startIndex) {
        for (int i = startIndex; i < array.length; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 在指定的区间内查找指定值在源数组中第一次出现的位置.
     */
    public static int indexOf(boolean[] array, boolean value, int startIndex, int endIndex) {
        for (int i = startIndex; i < endIndex; i++) {
            if (array[i] == value) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Finds the first occurrence of value in <code>float</code> array.
     */
    public static int indexOf(float[] array, float value) {
        for (int i = 0; i < array.length; i++) {
            if (Float.compare(array[i], value) == 0) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Returns <code>true</code> if <code>float</code> array contains given value.
     */
    public static boolean contains(float[] array, float value) {
        return indexOf(array, value) != -1;
    }

    /**
     * Finds the first occurrence of given value in <code>float</code> array from specified given position.
     */
    public static int indexOf(float[] array, float value, int startIndex) {
        for (int i = startIndex; i < array.length; i++) {
            if (Float.compare(array[i], value) == 0) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Finds the first occurrence in <code>float</code> array from specified given position and upto given length.
     */
    public static int indexOf(float[] array, float value, int startIndex, int endIndex) {
        for (int i = startIndex; i < endIndex; i++) {
            if (Float.compare(array[i], value) == 0) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Finds the first occurrence of value in <code>double</code> array.
     */
    public static int indexOf(double[] array, double value) {
        for (int i = 0; i < array.length; i++) {
            if (Double.compare(array[i], value) == 0) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Returns <code>true</code> if <code>double</code> array contains given value.
     */
    public static boolean contains(double[] array, double value) {
        return indexOf(array, value) != -1;
    }

    /**
     * Finds the first occurrence of given value in <code>double</code> array from specified given position.
     */
    public static int indexOf(double[] array, double value, int startIndex) {
        for (int i = startIndex; i < array.length; i++) {
            if (Double.compare(array[i], value) == 0) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Finds the first occurrence in <code>double</code> array from specified given position and upto given length.
     */
    public static int indexOf(double[] array, double value, int startIndex, int endIndex) {
        for (int i = startIndex; i < endIndex; i++) {
            if (Double.compare(array[i], value) == 0) {
                return i;
            }
        }
        return -1;
    }

    /**
     * 查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(Object[] array, Object value) {
        for (int i = 0; i < array.length; i++) {
            if (array[i].equals(value)) {
                return i;
            }
        }
        return -1;
    }

    public static boolean contains(Object[] array, Object value) {
        return indexOf(array, value) != -1;
    }

    /**
     * 在指定的起始位置后查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(Object[] array, Object value, int startIndex) {
        for (int i = startIndex; i < array.length; i++) {
            if (array[i].equals(value)) {
                return i;
            }
        }
        return -1;
    }

    public static boolean contains(Object[] array, Object value, int startIndex) {
        return indexOf(array, value, startIndex) != -1;
    }

    // ---------------------------------------------------------------- indexof 2

    /**
     * 查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(byte[] array, byte[] sub) {
        return indexOf(array, sub, 0, array.length);
    }

    public static boolean contains(byte[] array, byte[] sub) {
        return indexOf(array, sub) != -1;
    }

    /**
     * 在指定的起始位置后查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(byte[] array, byte[] sub, int startIndex) {
        return indexOf(array, sub, startIndex, array.length);
    }

    /**
     * 在指定的区间内查找数组在源数组中第一次出现的位置.
     */
    public static int indexOf(byte[] array, byte[] sub, int startIndex, int endIndex) {
        int sublen = sub.length;
        if (sublen == 0) {
            return startIndex;
        }
        int total = endIndex - sublen + 1;
        byte c = sub[0];
        for (int i = startIndex; i < total; i++) {
            if (array[i] != c) {
                continue;
            }
            int j = 1;
            int k = i + 1;
            while (j < sublen) {
                if (sub[j] != array[k]) {
                    break;
                }
                j++;
                k++;
            }
            return i;
        }
        return -1;
    }

    /**
     * 查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(char[] array, char[] sub) {
        return indexOf(array, sub, 0, array.length);
    }

    public static boolean contains(char[] array, char[] sub) {
        return indexOf(array, sub) != -1;
    }

    /**
     * 在指定的起始位置后查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(char[] array, char[] sub, int startIndex) {
        return indexOf(array, sub, startIndex, array.length);
    }

    /**
     * 在指定的区间内查找数组在源数组中第一次出现的位置.
     */
    public static int indexOf(char[] array, char[] sub, int startIndex, int endIndex) {
        int sublen = sub.length;
        if (sublen == 0) {
            return startIndex;
        }
        int total = endIndex - sublen + 1;
        char c = sub[0];
        for (int i = startIndex; i < total; i++) {
            if (array[i] != c) {
                continue;
            }
            int j = 1;
            int k = i + 1;
            while (j < sublen) {
                if (sub[j] != array[k]) {
                    break;
                }
                j++;
                k++;
            }
            return i;
        }
        return -1;
    }

    /**
     * 查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(short[] array, short[] sub) {
        return indexOf(array, sub, 0, array.length);
    }

    public static boolean contains(short[] array, short[] sub) {
        return indexOf(array, sub) != -1;
    }

    /**
     * 在指定的起始位置后查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(short[] array, short[] sub, int startIndex) {
        return indexOf(array, sub, startIndex, array.length);
    }

    /**
     * 在指定的区间内查找数组在源数组中第一次出现的位置.
     */
    public static int indexOf(short[] array, short[] sub, int startIndex, int endIndex) {
        int sublen = sub.length;
        if (sublen == 0) {
            return startIndex;
        }
        int total = endIndex - sublen + 1;
        short c = sub[0];
        for (int i = startIndex; i < total; i++) {
            if (array[i] != c) {
                continue;
            }
            int j = 1;
            int k = i + 1;
            while (j < sublen) {
                if (sub[j] != array[k]) {
                    break;
                }
                j++;
                k++;
            }
            return i;
        }
        return -1;
    }

    /**
     * 查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(int[] array, int[] sub) {
        return indexOf(array, sub, 0, array.length);
    }

    public static boolean contains(int[] array, int[] sub) {
        return indexOf(array, sub) != -1;
    }

    /**
     * 在指定的起始位置后查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(int[] array, int[] sub, int startIndex) {
        return indexOf(array, sub, startIndex, array.length);
    }

    /**
     * 在指定的区间内查找数组在源数组中第一次出现的位置.
     */
    public static int indexOf(int[] array, int[] sub, int startIndex, int endIndex) {
        int sublen = sub.length;
        if (sublen == 0) {
            return startIndex;
        }
        int total = endIndex - sublen + 1;
        int c = sub[0];
        for (int i = startIndex; i < total; i++) {
            if (array[i] != c) {
                continue;
            }
            int j = 1;
            int k = i + 1;
            while (j < sublen) {
                if (sub[j] != array[k]) {
                    break;
                }
                j++;
                k++;
            }
            return i;
        }
        return -1;
    }

    /**
     * 查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(long[] array, long[] sub) {
        return indexOf(array, sub, 0, array.length);
    }

    public static boolean contains(long[] array, long[] sub) {
        return indexOf(array, sub) != -1;
    }

    /**
     * 在指定的起始位置后查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(long[] array, long[] sub, int startIndex) {
        return indexOf(array, sub, startIndex, array.length);
    }

    /**
     * 在指定的区间内查找数组在源数组中第一次出现的位置.
     */
    public static int indexOf(long[] array, long[] sub, int startIndex, int endIndex) {
        int sublen = sub.length;
        if (sublen == 0) {
            return startIndex;
        }
        int total = endIndex - sublen + 1;
        long c = sub[0];
        for (int i = startIndex; i < total; i++) {
            if (array[i] != c) {
                continue;
            }
            int j = 1;
            int k = i + 1;
            while (j < sublen) {
                if (sub[j] != array[k]) {
                    break;
                }
                j++;
                k++;
            }
            return i;
        }
        return -1;
    }

    /**
     * 查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(boolean[] array, boolean[] sub) {
        return indexOf(array, sub, 0, array.length);
    }

    public static boolean contains(boolean[] array, boolean[] sub) {
        return indexOf(array, sub) != -1;
    }

    /**
     * 在指定的起始位置后查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(boolean[] array, boolean[] sub, int startIndex) {
        return indexOf(array, sub, startIndex, array.length);
    }

    /**
     * 在指定的区间内查找数组在源数组中第一次出现的位置.
     */
    public static int indexOf(boolean[] array, boolean[] sub, int startIndex, int endIndex) {
        int sublen = sub.length;
        if (sublen == 0) {
            return startIndex;
        }
        int total = endIndex - sublen + 1;
        boolean c = sub[0];
        for (int i = startIndex; i < total; i++) {
            if (array[i] != c) {
                continue;
            }
            int j = 1;
            int k = i + 1;
            while (j < sublen) {
                if (sub[j] != array[k]) {
                    break;
                }
                j++;
                k++;
            }
            return i;
        }
        return -1;
    }

    /**
     * 查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(float[] array, float[] sub) {
        return indexOf(array, sub, 0, array.length);
    }

    public static boolean contains(float[] array, float[] sub) {
        return indexOf(array, sub) != -1;
    }

    /**
     * 在指定的起始位置后查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(float[] array, float[] sub, int startIndex) {
        return indexOf(array, sub, startIndex, array.length);
    }

    /**
     * 在指定的区间内查找数组在源数组中第一次出现的位置.
     */
    public static int indexOf(float[] array, float[] sub, int startIndex, int endIndex) {
        int sublen = sub.length;
        if (sublen == 0) {
            return startIndex;
        }
        int total = endIndex - sublen + 1;
        float c = sub[0];
        for (int i = startIndex; i < total; i++) {
            if (Float.compare(array[i], c) != 0) {
                continue;
            }
            int j = 1;
            int k = i + 1;
            while (j < sublen) {
                if (Float.compare(sub[j], array[k]) != 0) {
                    break;
                }
                j++;
                k++;
            }
            return i;
        }
        return -1;
    }

    /**
     * 查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(double[] array, double[] sub) {
        return indexOf(array, sub, 0, array.length);
    }

    public static boolean contains(double[] array, double[] sub) {
        return indexOf(array, sub) != -1;
    }

    /**
     * 在指定的起始位置后查找数组在源数组中第一次出现的位置
     */
    public static int indexOf(double[] array, double[] sub, int startIndex) {
        return indexOf(array, sub, startIndex, array.length);
    }

    /**
     * 在指定的区间内查找数组在源数组中第一次出现的位置.
     */
    public static int indexOf(double[] array, double[] sub, int startIndex, int endIndex) {
        int sublen = sub.length;
        if (sublen == 0) {
            return startIndex;
        }
        int total = endIndex - sublen + 1;
        double c = sub[0];
        for (int i = startIndex; i < total; i++) {
            if (Double.compare(array[i], c) != 0) {
                continue;
            }
            int j = 1;
            int k = i + 1;
            while (j < sublen) {
                if (Double.compare(sub[j], array[k]) != 0) {
                    break;
                }
                j++;
                k++;
            }
            return i;
        }
        return -1;
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(Object[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(String[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(byte[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(char[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(short[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(int[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(long[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(float[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(double[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串, 每个元素间用逗号分隔, 返回字符串不包含方括号
     */
    public static String toString(boolean[] array) {
        if (array == null) {
            return StringUtil.NULL;
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < array.length; i++) {
            if (i != 0) {
                sb.append(',');
            }
            sb.append(array[i]);
        }
        return sb.toString();
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(Object[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = StringUtil.toString(array[i]);
        }
        return result;
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(String[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = String.valueOf(array[i]);
        }
        return result;
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(byte[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = String.valueOf(array[i]);
        }
        return result;
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(char[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = String.valueOf(array[i]);
        }
        return result;
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(short[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = String.valueOf(array[i]);
        }
        return result;
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(int[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = String.valueOf(array[i]);
        }
        return result;
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(long[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = String.valueOf(array[i]);
        }
        return result;
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(float[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = String.valueOf(array[i]);
        }
        return result;
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(double[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = String.valueOf(array[i]);
        }
        return result;
    }

    /**
     * 转换数组为字符串数组
     */
    public static String[] toStringArray(boolean[] array) {
        if (array == null) {
            return new String[0];
        }
        String[] result = new String[array.length];
        for (int i = 0; i < array.length; i++) {
            result[i] = String.valueOf(array[i]);
        }
        return result;
    }

    /**
     * 克隆数组
     *
     * @param array 被克隆的数组
     * @return 新数组
     */
    public static <T> T[] clone(T[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * 克隆数组
     */
    public static char[] clone(char[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * 克隆数组
     */
    public static byte[] clone(byte[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * 克隆数组
     */
    public static int[] clone(int[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * 克隆数组
     */
    public static short[] clone(short[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * 克隆数组
     */
    public static long[] clone(long[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * 克隆数组
     */
    public static float[] clone(float[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * 克隆数组
     */
    public static double[] clone(double[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * 克隆数组
     */
    public static boolean[] clone(boolean[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * 字符串数组去重复值
     *
     * @param arrayStr
     * @return
     */
    public static String[] removeSame(String[] arrayStr) {
        Set<String> set = new TreeSet<String>();
        for (int i = 0; i < arrayStr.length; i++) {
            set.add(arrayStr[i]);
        }
        arrayStr = (String[]) set.toArray(new String[0]);
        return arrayStr;
    }

}
