package com.qhiot.core.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

@Slf4j
public class CommonsCsvUtil {
    public static void exportByList(List<String> headersList, List<List<String>> dataList,
                                    OutputStream os) {
        OutputStreamWriter osw = null;
        CSVFormat csvFormat = null;
        CSVPrinter csvPrinter = null;
        try {
            osw = new OutputStreamWriter(os, "GBK");// 如果是UTF-8时，WPS打开是正常显示，而微软的excel打开是乱码,
            csvFormat = CSVFormat.DEFAULT
                    .withHeader(headersList.toArray(new String[headersList.size()]));
            csvPrinter = new CSVPrinter(osw, csvFormat);
            for (int i = 0; i < dataList.size(); i++) {
                List<String> values = dataList.get(i);
                csvPrinter.printRecord(values);
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            close(os, csvPrinter);
        }
    }

    private static void close(OutputStream os, CSVPrinter csvPrinter) {
        try {
            if (csvPrinter != null) {
                csvPrinter.flush();
                csvPrinter.close();
            }
            if (os != null) {
                os.flush();
                os.close();
            }
        } catch (IOException e) {
            log.error(e.getMessage());
        }
    }

    public static List<String> importCsv(InputStream in) {
        List<String> dataList = new ArrayList<String>();
        BufferedReader br = null;
        try {
            br = new BufferedReader(new InputStreamReader(in));
            String line = "";
            while ((line = br.readLine()) != null) {
                dataList.add(line);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (br != null) {
                try {
                    br.close();
                    br = null;
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return dataList;
    }

}
