package com.qhiot.core.util;

import org.apache.commons.mail.EmailAttachment;
import org.apache.commons.mail.EmailException;
import org.apache.commons.mail.HtmlEmail;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

/**
 * 发送邮件Util
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017/11/8 23:33
 */
public class MailUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(MailUtil.class);

    // 邮箱
    private static final String MAIL_SERVER_HOST = "smtp.mxhichina.com";
    private static final String MAIL_SENDER_ADDRESS = "seem@qh-iot.com";
    private static final String MAIL_SENDER_NICK = "SEEM";
    private static final String MAIL_AUTH_USERNAME = "seem@qh-iot.com";
    private static final String MAIL_AUTH_PASSWORD = "7(8n@Z1t50dWO5cx";

    /**
     * 发送邮件 (Html)
     *
     * @param subject     主题
     * @param htmlContent 正文内容
     * @param sendTo      收件人
     */
    public static void sendEmail(String subject, String htmlContent, String... sendTo) throws EmailException {
        sendEmail(subject, htmlContent, null, null, sendTo);
    }

    /**
     * 发送内嵌图片的邮件 (Html)
     *
     * @param subject     主题
     * @param htmlContent 正文内容
     * @param pictures    嵌入HTML 的图片, Map&lt;placeHolder, pictrueFile&gt;, 在邮件正文内的图片地址使用占位符
     *                    <pre>&lt;img src="cid:placeHolder"/&gt;</pre>
     * @param sendTo      收件人
     */
    public static void sendEmail(String subject, String htmlContent, Map<String, File> pictures, String... sendTo)
            throws EmailException {
        sendEmail(subject, htmlContent, pictures, null, sendTo);
    }

    /**
     * 发送内嵌图片并且附加附件的邮件 (Html)
     *
     * @param subject     主题
     * @param htmlContent 正文内容
     * @param pictures    嵌入HTML 的图片, Map&lt;placeHolder, pictrueFile&gt;, 在邮件正文内的图片地址使用占位符
     *                    <pre>&lt;img src="cid:placeHolder"/&gt;</pre>
     * @param attachments M附件
     * @param sendTo      收件人
     */
    public static void sendEmail(String subject, String htmlContent, Map<String, File> pictures,
                                 Collection<File> attachments, String... sendTo) throws EmailException {
        HtmlEmail email = new HtmlEmail();
        // 配置信息
        email.setHostName(MAIL_SERVER_HOST);
        email.setSSLOnConnect(true);
        email.setFrom(MAIL_SENDER_ADDRESS, MAIL_SENDER_NICK);
        email.setAuthentication(MAIL_AUTH_USERNAME, MAIL_AUTH_PASSWORD);
        email.setCharset("UTF-8");
        email.setSubject(subject);
        if (pictures != null && !pictures.isEmpty()) {
            Iterator<Map.Entry<String, File>> itrPlaceHolder = pictures.entrySet().iterator();
            while (itrPlaceHolder.hasNext()) {
                Map.Entry<String, File> entry = itrPlaceHolder.next();
                String cid = email.embed(entry.getValue()); // 将图片嵌入邮件中，返回cid
                htmlContent = htmlContent.replaceAll(entry.getKey(), cid); // 替换html邮件正文中的占位符
            }
        }
        email.setHtmlMsg(htmlContent);

        if (attachments != null && !attachments.isEmpty()) {
            Iterator<File> itrAttachment = attachments.iterator();
            EmailAttachment emailAttachment = null;
            while (itrAttachment.hasNext()) {
                emailAttachment = new EmailAttachment();
                File file = itrAttachment.next();
                emailAttachment.setPath(file.getPath());
                emailAttachment.setName(file.getName());
                emailAttachment.setDisposition(EmailAttachment.ATTACHMENT);
                email.attach(emailAttachment);
            }
        }

        if (null != sendTo && sendTo.length > 0) {
            for (int i = 0; i < sendTo.length; i++) {
                email.addTo(sendTo[i]);
            }
        }
        email.send();
    }
}
