package com.qhiot.core.util;

import com.artofsolving.jodconverter.DocumentConverter;
import com.artofsolving.jodconverter.openoffice.connection.OpenOfficeConnection;
import com.artofsolving.jodconverter.openoffice.connection.SocketOpenOfficeConnection;
import com.artofsolving.jodconverter.openoffice.converter.StreamOpenOfficeDocumentConverter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.ConnectException;

public class Office2PdfUtil {

    private static final Logger LOGGER = LoggerFactory.getLogger(Office2PdfUtil.class);
    private String openOfficeHost;
    private int openOfficePort;
    private String storePath;

    public Office2PdfUtil(String openOfficeHost, int openOfficePort, String storePath) {
        this.openOfficeHost = openOfficeHost;
        this.openOfficePort = openOfficePort;
        this.storePath = storePath;
    }

    public File file2pdf(HttpServletRequest request, InputStream fromFileInputStream, String suffixName)
            throws IOException {
        File file = new File(storePath + File.separatorChar + "pdftemp");
        if (!file.exists()) {
            file.mkdirs();
        }
        String filePath = file.getPath();
        String docInputFileName = "srctemp";
        String docOutPutFileName = "desctemp.pdf";
        File outputFile = new File(filePath + File.separatorChar + docOutPutFileName);
        File docInputFile = new File(filePath + File.separatorChar + docInputFileName + suffixName);
        if (outputFile.exists()) {
            outputFile.delete();
        }
        outputFile.createNewFile();
        if (docInputFile.exists()) {
            docInputFile.delete();
        }
        docInputFile.createNewFile();
        try {
            OutputStream os = new FileOutputStream(docInputFile);
            int bytesRead = 0;
            byte[] buffer = new byte[1024 * 8];
            while ((bytesRead = fromFileInputStream.read(buffer)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            fromFileInputStream.close();
        } catch (IOException e) {
        }
//        OpenOfficeConnection connection = new SocketOpenOfficeConnection(8100);
        OpenOfficeConnection connection = new SocketOpenOfficeConnection(openOfficeHost, openOfficePort);
        try {
            connection.connect();
        } catch (ConnectException e) {
            LOGGER.error("文件转换出错，请检查OpenOffice服务是否启动!");
        }

//        DocumentConverter converter = new OpenOfficeDocumentConverter(connection);
        DocumentConverter converter = new StreamOpenOfficeDocumentConverter(connection);
        converter.convert(docInputFile, outputFile);
        connection.disconnect();
        docInputFile.delete();
        return outputFile;
    }

}
