package tk.mybatis.mapper.entity;


import java.util.HashMap;
import java.util.Map;

/**
 * 继承Example的全部属性和方法，在此基础上实现自定义参数Map<name,value>
 * 使用方法：
 * #{myCondition.name}
 * 1 * @Author: luoj
 * 2 * @Date: 2020/6/28 16:16
 * 3
 */

public class ExpandExample extends Example {
    //添加数据 私有变量
    private Map<String, Object> expandCondition = new HashMap<>();

    public ExpandExample(Class<?> entityClass) {
        super(entityClass, true);
    }

    /**
     * 带exists参数的构造方法，默认notNull为false，允许为空
     *
     * @param entityClass
     * @param exists      - true时，如果字段不存在就抛出异常，false时，如果不存在就不使用该字段的条件
     */
    public ExpandExample(Class<?> entityClass, boolean exists) {
        super(entityClass, exists, false);
    }

    /**
     * 带exists参数的构造方法
     *
     * @param entityClass
     * @param exists      - true时，如果字段不存在就抛出异常，false时，如果不存在就不使用该字段的条件
     * @param notNull     - true时，如果值为空，就会抛出异常，false时，如果为空就不使用该字段的条件
     */
    public ExpandExample(Class<?> entityClass, boolean exists, boolean notNull) {
        super(entityClass, exists, notNull);
    }

    public ExpandExample(Example example) {
        super(example.getEntityClass());
        this.orderByClause = example.orderByClause;

        this.distinct = example.distinct;

        this.distinct = example.exists;

        this.distinct = example.notNull;

        this.distinct = example.forUpdate;

        //查询字段
        this.selectColumns = example.selectColumns;

        //排除的查询字段
        this.excludeColumns = example.excludeColumns;

        this.countColumn = example.countColumn;

        this.oredCriteria = example.oredCriteria;

        this.entityClass = example.entityClass;

        this.table = example.table;
        //属性和列对应
        this.propertyMap = example.propertyMap;
        //动态表名
        this.tableName = example.tableName;

        this.ORDERBY = example.ORDERBY;
    }

    public void addMyCondition(String name, Object value) {
        expandCondition.put(name, value);
    }

    public Map<String, Object> getMyCondition() {
        return expandCondition;
    }
}

