package com.qhiot.influxdb.config;

import com.qhiot.influxdb.connect.InfluxDBConnection;
import com.qhiot.influxdb.mapper.InfluxDBBaseMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;

/**
 * 缓存配置
 *
 * @author 罗军
 * <p>
 * Date: 2019-6-3 9:58.
 * </p>
 */
@Order(4)
@Configuration
public class InfluxDBConfig {
    private static final Logger LOGGER = LoggerFactory.getLogger("Init.Influxdb");

    @Autowired
    private ApplicationContext ctx;

    @Value("${influxDB.host:http://localhost}")
    private String host;

    @Value("${influxDB.port:8086}")
    private int port;

    @Value("${influxDB.database:seem}")
    private String database;

    @Value("${influxDB.username:admin}")
    private String username;

    @Value("${influxDB.password:admin}")
    private String password;

    @Value("${influxDB.retentionPolicy:autogen}")
    private String retentionPolicy;

    @Value("${influxDB.enableBatch:true}")
    private boolean enableBatch;

    @Value("${influxDB.maxPoints:1000}")
    private int maxPoints;

    @Value("${influxDB.maxTime:5000}")
    private int maxTime;

    @Value("${influxDB.connectTimeOut:20}")
    private int connectTimeOut;

    @Value("${influxDB.readTimeOut:300}")
    private int readTimeOut;

    @Value("${influxDB.writeTimeOut:30}")
    private int writeTimeOut;

    @Value("${influxDB.retention.decode.error.logtime:20d}")
    private String logTime;

    @Value("${decode.error.log.retention.policy:decode_log_policy}")
    private String decodeLogPolicy;

    @Value("${decode.error.log.time:20d}")
    private String decodeErrorLogTime;


    @Bean
    public InfluxDBConnection influxDBConnection() {
        LOGGER.info("连接influxdb");
        String openUrl = host + ":" + port;
        InfluxDBConnection influxDBConnectionUtil = new InfluxDBConnection(username, password, openUrl, database, retentionPolicy,
                enableBatch, maxPoints, maxTime,
                connectTimeOut, readTimeOut, writeTimeOut);
        //创建默认存储策略，用于decodelog
        createOrAlertLogRetentionPolicy(influxDBConnectionUtil);
        return influxDBConnectionUtil;
    }

    @Bean
    public InfluxDBBaseMapper influxDBBaseMapper() {
        return influxDBConnection().getInfluxDBMapper();
    }

    /**
     * 创建默认日志记录策略
     */
    private void createOrAlertLogRetentionPolicy(InfluxDBConnection influxDBConnection){
        try {
            influxDBConnection.createRetentionPolicy(decodeLogPolicy,database, decodeErrorLogTime,  null,1, false);
        } catch (Exception e) {
            LOGGER.error("创建解析日志存储策略失败");
        }
    }
}
