package org.influxdb.querybuilder;

import org.influxdb.querybuilder.Where;
import org.influxdb.querybuilder.WhereCoreImpl;
import org.influxdb.querybuilder.WhereNested;
import org.influxdb.querybuilder.*;
import org.influxdb.querybuilder.clauses.Clause;
import org.influxdb.querybuilder.clauses.ConjunctionClause;

import java.util.List;

public class WhereQueryImpl<T extends SelectQueryImpl> extends BuiltQuery implements Where, Select {

    private final T query;
    private final WhereCoreImpl whereCore;

    WhereQueryImpl(final T query, final WhereCoreImpl whereCore) {
        super(null);
        this.query = query;
        this.whereCore = whereCore;
    }

    @Override
    public org.influxdb.querybuilder.WhereQueryImpl where() {
        return query.where();
    }

    @Override
    public org.influxdb.querybuilder.WhereQueryImpl where(final Clause clause) {
        return query.where(clause);
    }

    @Override
    public org.influxdb.querybuilder.WhereQueryImpl where(final String text) {
        return query.where(text);
    }

    @Override
    public org.influxdb.querybuilder.WhereQueryImpl and(final Clause clause) {
        whereCore.and(clause);
        return this;
    }

    @Override
    public org.influxdb.querybuilder.WhereQueryImpl or(final Clause clause) {
        whereCore.or(clause);
        return this;
    }

    @Override
    public org.influxdb.querybuilder.WhereQueryImpl<T> append(final Clause clause) {
        whereCore.append(clause);
        return this;
    }

    @Override
    public List<ConjunctionClause> getClauses() {
        return whereCore.getClauses();
    }

    @Override
    public WhereNested<org.influxdb.querybuilder.WhereQueryImpl<T>> andNested() {
        return new WhereNested<>(this, false);
    }

    @Override
    public WhereNested<org.influxdb.querybuilder.WhereQueryImpl<T>> orNested() {
        return new WhereNested<>(this, true);
    }

    @Override
    public SelectQueryImpl orderBy(final Ordering orderings) {
        return query.orderBy(orderings);
    }

    @Override
    public SelectQueryImpl groupBy(final Object... columns) {
        return query.groupBy(columns);
    }

    @Override
    public SelectQueryImpl fill(final Number value) {
        return query.fill(value);
    }

    @Override
    public SelectQueryImpl fill(final String value) {
        return query.fill(value);
    }

    @Override
    public SelectQueryImpl limit(final int limit) {
        return query.limit(limit);
    }

    @Override
    public SelectQueryImpl limit(final int limit, final long offSet) {
        return query.limit(limit, offSet);
    }

    @Override
    public SelectQueryImpl sLimit(final int sLimit) {
        return query.sLimit(sLimit);
    }

    @Override
    public SelectQueryImpl sLimit(final int sLimit, final long sOffSet) {
        return query.sLimit(sLimit, sOffSet);
    }

    @Override
    public SelectQueryImpl tz(final String timezone) {
        return query.tz(timezone);
    }

    @Override
    public String getDatabase() {
        return query.getDatabase();
    }

    @Override
    public StringBuilder buildQueryString(final StringBuilder stringBuilder) {
        return query.buildQueryString(stringBuilder);
    }

    @Override
    public StringBuilder buildQueryString() {
        return query.buildQueryString(new StringBuilder());
    }
}
