package com.qhiot.kafka;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.util.CollectionUtil;
import com.qhiot.transport.db.service.KafkaSwitchConfigService;
import org.apache.kafka.clients.admin.AdminClient;
import org.apache.kafka.clients.admin.TopicDescription;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.util.concurrent.ListenableFuture;

import java.io.Serializable;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 1 * @Author: luoj
 * 2 * @Date: 2021/5/27 10:34
 * 3 实现按照开关进行判断是否发送消息
 */
public abstract class BaseJmsKafkaMessageSender {

    private static final Logger LOGGER = LoggerFactory.getLogger(BaseJmsKafkaMessageSender.class);

    @Autowired
    private KafkaSwitchConfigService kafkaSwitchConfigService;

    private KafkaTemplate kafkaTemplate;

    private String topicDestination;

    public BaseJmsKafkaMessageSender(KafkaTemplate kafkaTemplate, String topicDestination) {
        this.kafkaTemplate = kafkaTemplate;
        this.topicDestination = topicDestination;
    }

    //TODO 注入一个service，service从数据库初始化到内存，那些topic可以推送，kafka可以i独立定义一个通道，用于从外围link，推送消息到此，设置某个topic是否可以推送

    private boolean checkCanSend(String destination) throws ServiceException {
        return kafkaSwitchConfigService.checkCanPass(destination);
    }


    protected ListenableFuture<SendResult> sendMessage(Serializable serializable){
        try {
            if(!checkCanSend(topicDestination)){
                return null;
            }
            return kafkaTemplate.send(topicDestination, serializable);
        }catch (Exception e){
            LOGGER.error("Send message to topic://{} fail. {}", topicDestination, e.getMessage(), e);
        }
        return null;
    }

    protected ListenableFuture<SendResult> sendMessage(Integer partition, Serializable serializable){
        try {
            if(!checkCanSend(topicDestination)){
                return null;
            }
            return kafkaTemplate.send(topicDestination, partition, null, serializable);
        }catch (Exception e){
            LOGGER.error("Send message to topic://{} fail. {}", topicDestination, e.getMessage(), e);
        }
        return null;
    }

}
