package com.qhiot.kafka;

import java.util.HashMap;
import java.util.Map;

/**
 * 消息主题常量定义
 * */
public class KafkaTopicDestinations {

    /**
     * 日志消息推送kafka --暂时不启用
     * 发送这为：CodecLogKafkaSender
     */
    public static final String CODEC_LOG  = "codec.log";

    /**
     * 推送log
     *
     */
    public static final String  TOPIC_APP_LOG = "application.logs";

    /**
     * 对象推送ConnectionData
     */
    public static final String  CONNECTION_DATA = "connection.data";

    /**
     * 对象推送CoulometerData
     */
    public static final String  COULOMETER_DATA = "coulometer.data";

    /**
     * 对象推送ElectricCabinetData
     */
    public static final String  ELECTRIC_CABINET_DATA = "electric.cabinet.data";

    /**
     * 对象推送IccidData
     */
    public static final String  ICCID_DATA = "iccid.data";

    /**
     * 对象推送InputStateData
     */
    public static final String  INPUT_STATE_DATA = "input.state.data";

    /**
     * 推送历史数据对象
     * 对应替代发送对象为 MonitoringHistoryData 的原所有消息
     * 发送者为 MonitoringHistoryDataKafkaSender
     * 需要实现对应的消费者:
     *  1.
     */
    public static final String  MONITORING_HISTORY_DATA = "monitoring.history.data";


    /**
     * 对象推送OnboardTagData
     */
    public static final String  ONBOARD_TAG_DATA = "onboard.tag.data";


    /**
     * 对象推送OnlineData
     */
    public static final String  ONLINE_DATA = "online.data";

    /**
     * 对象推送OutputStateData
     */
    public static final String  OUTPUT_STATE_DATA = "output.state.data";

    /**
     * 对象推送PowerData
     */
    public static final String  POWER_DATA = "power.data";

    /**
     * 对象推送Rs485Data
     */
    public static final String RS485_DATA = "rs485.data";


    /**
     * 推送snapData 废弃 参见 SNAP_DATA_BURST
     */
//    public static final String  SNAP_DATA = "snap.data";

    /**
     * 推送SNAP_DATA_BURST
     */
    public static final String  SNAP_DATA_BURST = "snap.data.burst";

    /**
     * 推送Tag24gData
     */
    public static final String  TAG24G_DATA = "tag24G.data";

    /**
     * 推送Tag780mData
     */
    public static final String  TAG780M_DATA = "tag780m.data";

    /**
     * 对象推送Traffic
     */
    public static final String  TRAFFIC_DATA = "traffic.data";


    /**
     * 对象推送TransformerState
     */
    public static final String  TRANSFORMER_STATE = "transformer.state";


    /**
     * 告警对象推送warningData
     */
    public static final String  WARNING_DATA = "warning.data";


    /**
     * 告警对象推送warningData
     */
    public static final String  TRANSPORT_WARNING_DATA = "transport.warning.data";


    /**
     * ****推送所有父类为Request的对象****
     */
    public static final String  SEND_EXTENDS_REQUEST_MESSAGE = "send.extends.request.message";


    /**
     * ****推送所有父类为Response的对象****
     */
    public static final String  SEND_EXTENDS_RESPONSE_MESSAGE = "send.extends.response.message";


    /**
     * 为每一个路径创建说明信息
     */
    public static final Map<String,String> DESCRIPTIONS = new HashMap<>();

    static {
        DESCRIPTIONS.put(CODEC_LOG ,"推送解码日志数据的通道，禁用后传输日志页面将不可用，系统日志中将不会记录解码日志。");
        DESCRIPTIONS.put(TOPIC_APP_LOG,"系统常规日志的推送通道，消息量较大，建议禁用。");
        DESCRIPTIONS.put(CONNECTION_DATA,"推送终端连接记录数据的通道，禁用后终端连接数据表将不再记录终端连接数据。");
        DESCRIPTIONS.put(COULOMETER_DATA,"推送电量计数据的通道，暂时没有应用，建议禁用。");
        DESCRIPTIONS.put(ELECTRIC_CABINET_DATA,"推送电控箱状态数据的通道，暂时没有应用，建议禁用。");
        DESCRIPTIONS.put(ICCID_DATA,"推送终端上SIM卡的ICCID信息的通道，禁用后，终端列表将无法准确获取SIM卡信息。");
        DESCRIPTIONS.put(INPUT_STATE_DATA,"推送终端输入到平台的开关量信息的通道，暂时没有应用，建议禁用。");
        DESCRIPTIONS.put(MONITORING_HISTORY_DATA,"推送终端历史数据的通道，禁用后终端实时数据页面将不会及时更新最新数据，禁用不会导致link本身数据丢失，但中台和业务系统可能无法获取数据。");
        /**
         * 对象推送OnboardTagData
         */
//        DESCRIPTIONS.put( ONBOARD_TAG_DATA,"onboard.tag.data");

        DESCRIPTIONS.put(ONLINE_DATA,"推送终端离线在线状态的消息通道，禁用后可能无法及时得知当前的终端是否在线。");
        DESCRIPTIONS.put(OUTPUT_STATE_DATA,"推送终端输出到终端的开关量回应信息的通道，暂时没有应用，建议禁用。");
        DESCRIPTIONS.put(POWER_DATA,"推送终端产生的上电、断电信息的通道，禁用后可能无法及时获取终端的供电状态。");
        DESCRIPTIONS.put(SNAP_DATA_BURST,"推送终端抓拍到的图片数据，禁用后抓拍图片功能将无法得到图片。");
        DESCRIPTIONS.put(TAG24G_DATA,"推送Tag24标签数据的通道，暂时没有应用，建议禁用。");
        DESCRIPTIONS.put(TAG780M_DATA,"推送Tag78标签数据的通道，暂时没有应用，建议禁用。");
        DESCRIPTIONS.put(TRAFFIC_DATA,"推送终端与平台通讯消耗的流量数据的通道，暂时没有应用，建议禁用。");
        DESCRIPTIONS.put(TRANSFORMER_STATE,"推送互感器信息的通道，禁用后将不会收到终端上发的互感器信息。");
        DESCRIPTIONS.put(WARNING_DATA,"推送告警信息的通道，禁用后将不会收到终端上发的告警信息。");
        DESCRIPTIONS.put(TRANSPORT_WARNING_DATA,"推送告警信息的通道，禁用后将不会收到系统产生的告警信息。");
        DESCRIPTIONS.put(SEND_EXTENDS_REQUEST_MESSAGE, "系统发送信息到终端的通道,禁用后系统将无法和终端进行主动通讯（无法对终端进行控制、升级、下发指令等操作）。");
        DESCRIPTIONS.put(SEND_EXTENDS_RESPONSE_MESSAGE, "终端发送信息到系统的通道,禁用后系统将无法收到终端发送给系统的指令回应信息（抓拍图片会丢失）。");
    }

}
