package com.qhiot.log2jms;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.LoggerContext;
import org.apache.logging.log4j.core.appender.AbstractAppender;
import org.apache.logging.log4j.core.config.AbstractConfiguration;
import org.apache.logging.log4j.core.config.LoggerConfig;
import org.apache.logging.log4j.core.layout.PatternLayout;

import java.sql.Timestamp;

/**
 * JMS 日志消息发送器
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-01-21 20:37
 */
public class JmsLogAppender extends AbstractAppender {

//    private JmsLoggerSender jmsLoggerSender;
    private JmsLoggerKafkaSender jmsLoggerSender;
    private PatternLayout layout;
    private Filter        filter;
    private Level         level = Level.DEBUG;
    private String        logNode;

    public JmsLogAppender(String name, Filter filter, PatternLayout layout,
                          JmsLoggerKafkaSender jmsLoggerSender, String logNode) {
        super(name, filter, layout);
        this.layout = layout;
        this.filter = filter;
        this.jmsLoggerSender = jmsLoggerSender;
        this.logNode = logNode;
    }

    public void init() {
        this.start();
        addAppenderToRoot();
    }

    public void destroy() {
        LoggerContext ctx = (LoggerContext) LogManager.getContext(false);
        AbstractConfiguration config = (AbstractConfiguration) ctx.getConfiguration();

        config.removeAppender(getName());
        for (final LoggerConfig loggerConfig : config.getLoggers().values()) {
            loggerConfig.removeAppender(getName());
        }
        config.getRootLogger().removeAppender(getName());
        super.stop();
    }

    public void addAppenderToRoot() {
        LoggerContext loggerContext = (LoggerContext) LogManager.getContext(false);
        AbstractConfiguration configuration =
                (AbstractConfiguration) loggerContext.getConfiguration();

        configuration.removeAppender(getName());
        configuration.addAppender(this);

        configuration.getRootLogger().removeAppender(getName());
        configuration.getRootLogger().addAppender(this, getLevel(), getFilter());
        loggerContext.updateLoggers();
    }

    /**
     * 产生新的日志时， 推送格式化后的日志文本到目标订阅地址
     *
     * @param event
     */
    @Override
    public void append(LogEvent event) {
        LogMessage logMessage = new LogMessage();
        logMessage.setLevel(event.getLevel().name());
        logMessage.setLogger(event.getLoggerName());
        logMessage.setMessage(event.getMessage().getFormattedMessage());
        logMessage.setTime(new Timestamp(event.getTimeMillis()));
        logMessage.setLogNode(logNode);
        if (jmsLoggerSender != null) {
            jmsLoggerSender.sendMessage(logMessage);
        }
    }

    public void setJmsLoggerSender(JmsLoggerKafkaSender jmsLoggerSender) {
        this.jmsLoggerSender = jmsLoggerSender;
    }

    @Override
    public PatternLayout getLayout() {
        return layout;
    }

    public void setLayout(PatternLayout layout) {
        this.layout = layout;
    }

    @Override
    public Filter getFilter() {
        return filter;
    }

    public void setFilter(Filter filter) {
        this.filter = filter;
    }

    public Level getLevel() {
        return level;
    }

    public void setLevel(Level level) {
        this.level = level;
    }
}
