package com.qhiot.log2jms;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.LoggerContext;
import org.apache.logging.log4j.core.config.AbstractConfiguration;
import org.apache.logging.log4j.core.config.ConfigurationListener;
import org.apache.logging.log4j.core.config.Reconfigurable;
import org.apache.logging.log4j.core.filter.RegexFilter;
import org.apache.logging.log4j.core.layout.PatternLayout;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author 金建强(ptma@163.com)
 * @version 2016/7/1 14:24
 */
public class JmsLogAppenderInitializer {

    private static final String APPENDER_NAME = "JmsLogs";

    private static final Logger LOGGER        = LoggerFactory.getLogger("Init.log2jms.Initializer");

    private Level               level         = Level.DEBUG;

    private String              layoutPattern = "%d{yyyy-MM-dd HH:mm:ss.SSS} [%-5level] %logger - %msg%n";
    private String              regexPattern  = ".*";

//    private JmsLoggerSender     jmsLoggerSender;
    private JmsLoggerKafkaSender jmsLoggerSender;

    private int              node;

    private JmsLogAppender      appender;

    /**
     * 构造方法
     *
     * @param jmsLoggerSender JMS发送器
     */
    public JmsLogAppenderInitializer(JmsLoggerKafkaSender jmsLoggerSender, int node) {
        this.jmsLoggerSender = jmsLoggerSender;
        this.node = node;
        resetAppender();
    }

    /**
     * 重置 Appender
     */
    public void resetAppender() {
        LoggerContext context = (LoggerContext) LogManager.getContext(false);
        AbstractConfiguration config = (AbstractConfiguration) context.getConfiguration();

        config.removeAppender(APPENDER_NAME);

        PatternLayout layout = PatternLayout.newBuilder().withPattern(layoutPattern).withConfiguration(config).build();

        RegexFilter filter = null;
        try {
            filter = RegexFilter.createFilter(regexPattern, null, false, Filter.Result.ACCEPT, Filter.Result.DENY);
            filter.start();
        } catch (Exception e) {

        }

        if (appender == null) {
            appender = new JmsLogAppender(APPENDER_NAME, filter, layout, jmsLoggerSender, "node_"+node);
            appender.start();
        }

        config.addAppender(appender);
        /*
         * 下列语句会导致日志重复输出 for (final LoggerConfig loggerConfig : config.getLoggers().values()) {
         * loggerConfig.addAppender(appender, level, filter); }
         */
        config.getRootLogger().addAppender(appender, level, filter);

        // 监听配置改动， 一旦发现改动重新添加 Appender
        config.addListener(new ConfigurationListener() {
            @Override
            public void onChange(Reconfigurable reconfigurable) {
                resetAppender();
                LOGGER.info(
                        "Logging configuration has been modified and a new JmsLogAppender has been configured on the root logger");
            }
        });
        context.updateLoggers(config);
    }

    public void stop() {
        if (appender != null) {
            appender.destroy();
        }
    }

    public String getLevel() {
        return level.name();
    }

    public void setLevel(String levelName) {
        level = Level.toLevel(levelName, Level.ALL);
    }

    public String getLayoutPattern() {
        return layoutPattern;
    }

    public void setLayoutPattern(String layoutPattern) {
        this.layoutPattern = layoutPattern;
    }

    public String getRegexPattern() {
        return regexPattern;
    }

    public void setRegexPattern(String regexPattern) {
        this.regexPattern = regexPattern;
    }
}
