package com.qhiot.qhlinkbasic.logListener;

import com.qhiot.core.util.StringUtil;
import org.slf4j.MDC;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.context.event.ApplicationEnvironmentPreparedEvent;
import org.springframework.boot.context.event.ApplicationFailedEvent;
import org.springframework.boot.context.event.ApplicationPreparedEvent;
import org.springframework.boot.context.event.ApplicationStartingEvent;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.event.ContextClosedEvent;
import org.springframework.context.event.GenericApplicationListener;
import org.springframework.core.Ordered;
import org.springframework.core.ResolvableType;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.core.env.MutablePropertySources;
import org.springframework.core.env.PropertySource;

import java.util.Iterator;
/**
 * 初始化MDC，用于log4j2.xml文件的模板
 * @author 罗军(luoj@qh-iot.com)
 * @version 2021/6/22
 */
public class ApplicationStartedEventListener implements GenericApplicationListener {

    public static final int DEFAULT_ORDER = Ordered.HIGHEST_PRECEDENCE + 10;

    private static Class<?>[] EVENT_TYPES = { ApplicationStartingEvent.class,
            ApplicationEnvironmentPreparedEvent.class, ApplicationPreparedEvent.class,
            ContextClosedEvent.class, ApplicationFailedEvent.class };

    private static Class<?>[] SOURCE_TYPES = { SpringApplication.class,
            ApplicationContext.class };

    @Override
    public void onApplicationEvent(ApplicationEvent event) {
        if (event instanceof ApplicationEnvironmentPreparedEvent) {

            ConfigurableEnvironment envi = ((ApplicationEnvironmentPreparedEvent) event).getEnvironment();
            MutablePropertySources mps = envi.getPropertySources();
            Iterator<PropertySource<?>> iterator = mps.iterator();
            PropertySource<?> ps = null;
            while (iterator.hasNext()) {
                PropertySource<?> psIn = iterator.next();
                if(StringUtil.containsIgnoreCase(psIn.getName(),"application.yml")){
                    ps = psIn;
                    break;
                }
            }


            if(ps != null){
                if(ps.containsProperty("log2es.clusterNodes")){
                    String clusterNodes = (String) ps.getProperty("log2es.clusterNodes");
                    //System.out.println(kafkaUrl);
                    MDC.put("log2es.clusterNodes", clusterNodes);
                }else {
                    MDC.put("log2es.clusterNodes", "127.0.0.1:9300");
                }
                if(ps.containsProperty("log2es.clusterName")){
                    String clusterName = (String) ps.getProperty("log2es.clusterName");
                    //System.out.println(kafkaUrl);
                    MDC.put("log2es.clusterName", clusterName);
                }else {
                    MDC.put("log2es.clusterName", "elasticsearch");
                }
                if(ps.containsProperty("log2es.index")){
                    String index = (String) ps.getProperty("log2es.index");
                    //System.out.println(kafkaUrl);
                    MDC.put("log2es.index", index);
                }else {
                    MDC.put("log2es.index", "qhseem");
                }

                if(ps.containsProperty("app.node")){
                    Integer appNode = (Integer) ps.getProperty("app.node");
                    if(appNode == null){
                        appNode = 0;
                    }
                    //System.out.println(kafkaUrl);
                    MDC.put("app.node", String.valueOf(appNode));
                }else {
                    MDC.put("app.node", "0");
                }
            }

        }

    }

    /*
     * (non-Javadoc)
     *
     * @see org.springframework.core.Ordered#getOrder()
     */
    @Override
    public int getOrder() {
        // TODO Auto-generated method stub
        return DEFAULT_ORDER;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.springframework.context.event.GenericApplicationListener#
     * supportsEventType(org.springframework.core.ResolvableType)
     */
    @Override
    public boolean supportsEventType(ResolvableType resolvableType) {
        return isAssignableFrom(resolvableType.getRawClass(), EVENT_TYPES);
    }

    @Override
    public boolean supportsSourceType(Class<?> sourceType) {
        return isAssignableFrom(sourceType, SOURCE_TYPES);
    }

    private boolean isAssignableFrom(Class<?> type, Class<?>... supportedTypes) {
        if (type != null) {
            for (Class<?> supportedType : supportedTypes) {
                if (supportedType.isAssignableFrom(type)) {
                    return true;
                }
            }
        }
        return false;
    }
}