package com.qhiot.transport.db.config;

import com.qhiot.core.util.CollectionUtil;
import com.qhiot.influxdb.connect.InfluxDBConnection;
import com.qhiot.influxdb.statement.Statement;
import com.qhiot.transport.db.model.InfluxSaveDurationConfig;
import com.qhiot.transport.db.service.InfluxSaveDurationConfigService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;

import java.util.List;

@Order(9999999)
@Configuration
public class InitInfluxdbPolicyConfig {

    private static  final Logger LOGGER = LoggerFactory.getLogger("Init.influx");

    @Value("${influxDB.database:seem}")
    private String database;

    @Autowired
    private InfluxSaveDurationConfigService influxSaveDurationConfigService;

    @Autowired
    private InfluxDBConnection influxDBConnection;
    @Bean
    public String initPolicy() {
        LOGGER.info("初始化存储策略默认值");
        try {
            List<InfluxSaveDurationConfig> configList = influxSaveDurationConfigService.selectAll();
            if (CollectionUtil.isNotEmpty(configList)) {
                for (InfluxSaveDurationConfig config : configList) {
                    int duration = Statement.DEFAULT_SAVE_HISTORY_TIME;
                    if(config.getDuration() != null){
                        duration = config.getDuration().intValue();
                    }
                    String policyName = Statement.TENANT_POLICY_PREFIX + config.getTenantId();
                    influxDBConnection.createOrAlertRetentionPolicy(policyName, database, duration + "m", "1s", 1, false);
                }
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage(),e);
        }
        return new String();
    }
}
