package com.qhiot.transport.db.dto;

import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.persistence.Column;
import javax.persistence.Transient;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.qhiot.core.util.JsonUtil;
import com.qhiot.core.util.StringUtil;
import lombok.Getter;
import lombok.Setter;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.db.model.TransportResourceMonitoring;

/**
 * @author 金建强(ptma/@163.com)
 * @version 2017/4/20 16:28
 */
@Getter
@Setter
public class TransportMonitoringDataDTO extends TransportResourceMonitoring {

    private static final long serialVersionUID = 2107343271725347596L;

    @Column(name = "A.monitoring_id")
    private Long              monitoringId;
    @Column(name = "A.device_id")
    private String            deviceId;

    @Column(name = "A.valid")
    private Short   valid = 0;
    @Column(name = "Y.valid")
    private Short   yalid = 0;
    private Double            dataValue;
    @Column(name = "A.config_index")
    private Short   configIndex;
    @Column(name = "Y.config_index")
    private Short   yonfigIndex;
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date              dataTime;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date              receivedTime;

    /**
     * 指标单位
     */
    @Column(name = "A.unit")
    private String unit;

    /**
     * Column: value_precision - 精度
     */
    @Column(name = "A.value_precision")
    private Short valuePrecision;

    /**
     * 数据类型，数值型，开工量输入，开工量输出
     */
    private Short             dataType;

    @Column(name = "A.data_map")
    private String            dataMap;

    /**
     * 图标
     */
    private String            icon;


    public long getDataTimestamp() {
        return dataTime == null ? 0 : dataTime.getTime();
    }

    public long getReceivedTimestamp() {
        return receivedTime == null ? 0 : receivedTime.getTime();
    }

    @Transient
    public boolean isActive() {
        return dataTime == null ? false : (DateUtil.diffSeconds(dataTime, new Date()) < 60 * 30);
    }

    @Override
    public Short getValuePrecision() {
        return valuePrecision == null ? 2 : valuePrecision;
    }


    public String getIcon() {
        return icon == null ? "icon-unknow" : icon;
    }

    public String getUnit() {
        return unit == null ? "" : unit;
    }

    @Override
    public Map<String, String> getDataMapObject() {
        if (StringUtil.isEmpty(dataMap)) {
            return null;
        } else {
            try {
                ObjectMapper mapper = JsonUtil.newMapper();
                mapper.configure(JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, false);
                mapper.configure(JsonParser.Feature.ALLOW_SINGLE_QUOTES, false);
                return mapper.readValue(dataMap, new TypeReference<LinkedHashMap<String, String>>() {});
            } catch (Exception e) {
                return null;
            }
        }
    }
}
