/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-07-26 09:54:20
 */
package com.qhiot.transport.db.mapper;


import java.util.Date;
import java.util.List;

import com.qhiot.transport.db.dto.MonitoringStatsIndexDataDTO;
import org.apache.ibatis.annotations.Param;

import tk.mybatis.mapper.common.ConditionMapper;
import tk.mybatis.mapper.common.Mapper;
import tk.mybatis.mapper.entity.Example;

import com.qhiot.transport.db.dto.MonitoringDataDayDTO;
import com.qhiot.transport.db.dto.MonitoringStatsDataDayDTO;
import com.qhiot.transport.db.model.MonitoringDataDay;

/**
 * MyBatis Mapper 接口 <br/>
 * Table: monitoring_data_day - 测点日数据 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-07-26 09:54:20
 */
public interface MonitoringDataDayMapper extends Mapper<MonitoringDataDay>, ConditionMapper<MonitoringDataDay> {

    MonitoringDataDayDTO selectDTOByExample(Example example);

    List<MonitoringDataDayDTO> selectMonitoringStatsDataHourAvg(@Param("startTime") Date startTime, @Param("endTime") Date endTime);

    List<MonitoringDataDayDTO> selectMonitoringStatsDataHourOnTime(@Param("startTime") Date startTime, @Param("endTime") Date endTime);

    List<MonitoringDataDayDTO> selectMonitorngDataAccumulated();

    /**
     * 根据时间范围查询测点数据
     *
     * @param deviceId 终端编号
     * @param indexId 监测指标编号
     * @param monitoringId 测点编号
     * @param smId 站点测点编号
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return
     */
    MonitoringDataDay selectMonitorngDataValue(@Param("deviceId") String deviceId,
                                               @Param("indexId") Long indexId,
                                               @Param("monitoringId") Long monitoringId,
                                               @Param("smId") Long smId,
                                               @Param("startTime") Date startTime,
                                               @Param("endTime") Date endTime);
    /**
     * 根据时间范围查询测点数据
     *
     * @param deviceId 终端编号
     * @param indexId 监测指标编号
     * @param monitoringId 测点编号
     * @param smId 站点测点编号
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @return
     */
    MonitoringDataDay selectMonitorngDataValueFirst(@Param("deviceId") String deviceId,
                                                    @Param("indexId") Long indexId,
                                                    @Param("monitoringId") Long monitoringId,
                                                    @Param("smId") Long smId,
                                                    @Param("startTime") Date startTime,
                                                    @Param("endTime") Date endTime);

    MonitoringStatsDataDayDTO selectDTOAVGDay(@Param("monitoringId") Long monitoringId,
                                              @Param("smId") Long smId,
                                              @Param("startTime") Date startTime,
                                              @Param("endTime") Date endTime);

    /**
     * 根据smId查询前一日统计数据
     * @param smId
     * @return
     * */
    MonitoringDataDay selectYesterdayDataBySmId(@Param("smId") Long smId);

    /**
     * 根据smId、ruleDelayTime查询延时前数据
     * @param smId
     * @return
     * */
    List<MonitoringDataDay> selectDataBySmIdAndDelayTime(@Param("smId") Long smId);

    /**
     * 根据smId查询backTime秒之前的数据
     * @param smId
     * @param backTime
     * @return List<MonitoringDataDay>
     * */
    List<MonitoringDataDay> selectBySmIdAndBackTime(@Param("smId") Long smId, @Param("backTime") Long backTime);

    /**
     * 根据smId查询backTime小时之前的数据
     * @param smId
     * @param backTime
     * @return List<MonitoringDataDay>
     * */
    List<MonitoringDataDay> selectBySmIdAndHourBackTime(@Param("smId") Long smId, @Param("backTime") Long backTime);

    /**
     * 根据smId查询backTime日之前的数据
     * @param smId
     * @param backTime
     * @return List<MonitoringDataDay>
     * */
    List<MonitoringDataDay> selectBySmIdAndDayBackTime(@Param("smId") Long smId, @Param("backTime") Long backTime);

    /**
     * 根据smId查询backTime月之前的数据
     * @param smId
     * @param backTime
     * @return List<MonitoringDataDay>
     * */
    List<MonitoringDataDay> selectBySmIdAndMonthBackTime(@Param("smId") Long smId, @Param("backTime") Long backTime);

    /**
     *按时间查询站点指标日均值数据
     * @param stationId
     * @param indexId
     * @param startTime
     * @param endTime
     * @return MonitoringStatsIndexDataDTO
     * */
    MonitoringStatsIndexDataDTO selectDTOAVGDayByIndexId(@Param("stationId") Long stationId,
                                                         @Param("indexId") Long indexId,
                                                         @Param("startTime") Date startTime,
                                                         @Param("endTime") Date endTime);

    /**
     * 查询小时站点指标数据
     * @param stationId
     * @param indexId
     * @param backTime
     * @return List<MonitoringStatsIndexDataDTO>
     * */
    List<MonitoringStatsIndexDataDTO> selectByIndexIdAndHourBackTime(@Param("stationId") Long stationId,
                                                                     @Param("indexId") Long indexId,
                                                                     @Param("backTime") Long backTime);

    /**
     * 查询日站点指标数据
     * @param stationId
     * @param indexId
     * @param backTime
     * @return List<MonitoringStatsIndexDataDTO>
     * */
    List<MonitoringStatsIndexDataDTO> selectByIndexIdAndDayBackTime(@Param("stationId") Long stationId,
                                                                    @Param("indexId") Long indexId,
                                                                    @Param("backTime") Long backTime);

    /**
     * 查询月站点指标数据
     * @param stationId
     * @param indexId
     * @param backTime
     * @return List<MonitoringStatsIndexDataDTO>
     * */
    List<MonitoringStatsIndexDataDTO> selectByIndexIdAndMonthBackTime(@Param("stationId") Long stationId,
                                                                      @Param("indexId") Long indexId,
                                                                      @Param("backTime") Long backTime);
}
