/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-04-20 10:18:47
 */
package com.qhiot.transport.db.mapper;

import com.qhiot.transport.db.dto.MonitoringChartData;
import com.qhiot.transport.db.dto.MonitoringDataDayDTO;
import com.qhiot.transport.db.dto.MonitoringDataHistoryDTO;
import com.qhiot.transport.db.dto.MonitoringGisDataHistoryDTO;
import com.qhiot.transport.db.model.MonitoringDataHistory;
import org.apache.ibatis.annotations.Param;
import tk.mybatis.mapper.common.ConditionMapper;
import tk.mybatis.mapper.common.Mapper;
import tk.mybatis.mapper.entity.Example;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * MyBatis Mapper 接口 <br/>
 * Table: monitoring_data_history - 测点历史数据 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-04-20 10:18:47
 */
public interface MonitoringDataHistoryMapper extends Mapper<MonitoringDataHistory>, ConditionMapper<MonitoringDataHistory> {

    /**
     * 插入历史数据到多个表（日表、月表、历史表）
     *
     * @param entity        历史数据实体
     * @param today         是否是当日数据
     * @param recentlyMonth 是否是近一个月内的数据
     * @return
     */
    int insertToMultipleTableSelective(@Param("entity") MonitoringDataHistory entity,
                                       @Param("today") boolean today,
                                       @Param("recentlyMonth") boolean recentlyMonth);

    List<MonitoringChartData> selectChartDataByExampleFromDay(Example example);

    List<MonitoringChartData> selectChartDataByExampleFromMonth(Example example);

    List<MonitoringDataHistoryDTO> selectDTOByExampleFromMonth(Example example);

    List<MonitoringGisDataHistoryDTO> selectMonitoringChartDataFromMonth(Example example);

    int deleteMonitoringDataDayLessThanId(@Param("dataId") long dataId);

    int deleteMonitoringDataMonthLessThanId(@Param("dataId") long dataId);

    /**
     * 根据时间范围查询测点数据
     *
     * @param deviceId     终端编号
     * @param indexId      监测指标编号
     * @param monitoringId 测点编号
     * @param smId         站点测点编号
     * @param startTime    开始时间
     * @param endTime      结束时间
     * @return
     */
    MonitoringDataHistory selectMonitoringDataHistoryByTime(@Param("deviceId") String deviceId,
                                                            @Param("indexId") Long indexId,
                                                            @Param("monitoringId") Long monitoringId,
                                                            @Param("smId") Long smId,
                                                            @Param("startTime") Date startTime,
                                                            @Param("endTime") Date endTime);

    MonitoringDataHistory selectMonitoringDataHistoryAllByTime(@Param("deviceId") String deviceId,
                                                               @Param("indexId") Long indexId,
                                                               @Param("monitoringId") Long monitoringId,
                                                               @Param("smId") Long smId,
                                                               @Param("startTime") Date startTime,
                                                               @Param("endTime") Date endTime);

    MonitoringDataHistory selectBySmId(@Param("smId") Long smId);

    List<MonitoringDataHistory> selectMonitoringDataHistoryAllByTimeSM(@Param("smId") Long smId, @Param("startTime") Date startTime, @Param("endTime") Date endTime);

    List<MonitoringDataHistory> selectMonitoringDataHistoryByTimeSM(@Param("smId") Long smId, @Param("startTime") Date startTime, @Param("endTime") Date endTime);

    List<MonitoringDataDayDTO> selectAVGHistoryByTimeSM(@Param("smId") Long smId, @Param("startTime") Date startTime, @Param("endTime") Date endTime);

    List<MonitoringDataDayDTO> selectAVGHistoryAllByTimeSM(@Param("smId") Long smId, @Param("startTime") Date startTime, @Param("endTime") Date endTime);

}
