/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 * 
 * Date: 2020-11-27 11:06:35
 */
package com.qhiot.transport.db.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;

import com.qhiot.core.util.DateUtil;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;

import lombok.Data;

/**
 * Table: codec_log_statstics -  <br/>
 *
 * @author 罗军(luoj@qh-iot.com)
 * @version 2020-11-27 11:06:35
 */
@Data
@Entity
@Table(name = "codec_log_statstics")
public class CodecLogStatstics implements Serializable {

    /** Column: context_id -  */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "context_id")
    private String contextId;

    private Short node;

    /** Column: device_id -  */
    @Length(max=20, message="db.codecLogStatstics.deviceId.Length")
    @NotBlank(message="db.codecLogStatstics.deviceId.NotBlank")
    private String deviceId;

    /** Column: number -  */
    private Integer countNumber;

    /** Column: first_time -  */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date firstTime;

    /** Column: last_time -  */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastTime;

    /** Column: used_max_millis -  */
    private Long usedMaxMillis;

    /** Column: used_min_millis -  */
    private Long usedMinMillis;

    /** Column: used_sum_millis -  */
    private Long usedSumMillis;

    /** Column: package_number 组装测点数量 -  */
    private Integer packageNumber;

    /** Column: package_first_time -  */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date packageFirstTime;

    /** Column: package_last_time -  */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date packageLastTime;

    /** Column: package_used_max_millis -  */
    private Long packageUsedMaxMillis;

    /** Column: package_used_min_millis -  */
    private Long packageUsedMinMillis;

    /** Column: package_used_sum_millis -  */
    private Long packageUsedSumMillis;

    /** Column: go_save_max_millis -  */
    private Long goSaveMaxMillis;

    /** Column: go_save_min_millis -  */
    private Long goSaveMinMillis;

    /** Column: go_save_sum_millis -  */
    private Long goSaveSumMillis;

    /** Column: save_number -  */
    private Integer saveNumber;

    /** Column: save_mysql_first_time -  */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date saveMysqlFirstTime;

    /** Column: save_mysql_last_time -  */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date saveMysqlLastTime;

    /** Column: save_mysql_used_max_millis -  */
    private Long saveMysqlUsedMaxMillis;

    /** Column: save_mysql_used_min_millis -  */
    private Long saveMysqlUsedMinMillis;

    /** Column: save_mysql_used_sum_millis -  */
    private Long saveMysqlUsedSumMillis;

    /** Column: save_influx_first_time -  */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date saveInfluxFirstTime;

    /** Column: save_influx_last_time -  */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date saveInfluxLastTime;

    /** Column: save_influx_used_max_millis -  */
    private Long saveInfluxUsedMaxMillis;

    /** Column: save_influx_used_min_millis -  */
    private Long saveInfluxUsedMinMillis;

    /** Column: save_influx_used_sum_millis -  */
    private Long saveInfluxUsedSumMillis;

    public void addNumber(int number){
        this.countNumber =  this.countNumber == null? number : (this.countNumber + number);
    }

    public void judgeFirstTime(Date firstTime){
        if(this.firstTime == null){
            this.firstTime = firstTime;
        }else {
            this.firstTime = DateUtil.min(this.firstTime,firstTime);
        }
    }

    public void judgeLastTime(Date lastTime){
        if(this.lastTime == null){
            this.lastTime = lastTime;
        }else {
            this.lastTime = DateUtil.max(this.lastTime,lastTime);
        }
    }

    public void judgeUsedMaxAndMinMillis(long usedMillis){
        if(this.usedMaxMillis == null){
            this.usedMaxMillis = usedMillis;
        }else {
            this.usedMaxMillis = this.usedMaxMillis > usedMaxMillis ? this.usedMaxMillis : usedMaxMillis;
        }
        if(this.usedMinMillis == null){
            this.usedMinMillis = usedMillis;
        }else {
            this.usedMinMillis = this.usedMinMillis < usedMillis ? this.usedMinMillis : usedMillis;
        }
        this.usedSumMillis =  this.usedSumMillis == null? usedMillis : (this.usedSumMillis + usedMillis);
    }

    /**
     * 最早进入数据组装的时间-出解码时间
     * @param goPackageUsedMillis
     */
    public long getGoPackageMillis(long goPackageUsedMillis){
        if(lastTime != null && packageFirstTime != null){
            return packageFirstTime.getTime() - lastTime.getTime();
        }
        return -1L;
    }

    /** 组装 */
    public void addPackageNumber(int packageNumber){
        this.packageNumber =  this.packageNumber == null? packageNumber : (this.packageNumber + packageNumber);
    }

    public void judgePackageFirstTime(Date packageFirstTime){
        if(this.packageFirstTime == null){
            this.packageFirstTime = packageFirstTime;
        }else {
            this.packageFirstTime = DateUtil.min(this.packageFirstTime,packageFirstTime);
        }
    }

    public void judgePackageLastTime(Date packageLastTime){
        if(this.packageLastTime == null){
            this.packageLastTime = packageLastTime;
        }else {
            this.packageLastTime = DateUtil.max(this.packageLastTime,packageLastTime);
        }
    }

    public void judgePackageUsedMaxAndMinMillis(long packageUsedMillis){
        if(this.packageUsedMaxMillis == null){
            this.packageUsedMaxMillis = packageUsedMillis;
        }else {
            this.packageUsedMaxMillis = this.packageUsedMaxMillis > packageUsedMillis ? this.packageUsedMaxMillis : packageUsedMillis;
        }
        if(this.packageUsedMinMillis == null){
            this.packageUsedMinMillis = packageUsedMillis;
        }else {
            this.packageUsedMinMillis = this.packageUsedMinMillis < packageUsedMillis ? this.packageUsedMinMillis : packageUsedMillis;
        }
        this.packageUsedSumMillis =  this.packageUsedSumMillis == null? packageUsedMillis : (this.packageUsedSumMillis + packageUsedMillis);
    }

    /** 进入入库前 */

    public void judgeGoSaveUsedMaxAndMinMillis(long goSaveUsedMillis){
        if(this.goSaveMaxMillis == null){
            this.goSaveMaxMillis = goSaveUsedMillis;
        }else {
            this.goSaveMaxMillis = this.goSaveMaxMillis > goSaveUsedMillis ? this.goSaveMaxMillis : goSaveUsedMillis;
        }
        if(this.goSaveMinMillis == null){
            this.goSaveMinMillis = goSaveUsedMillis;
        }else {
            this.goSaveMinMillis = this.goSaveMinMillis < goSaveUsedMillis ? this.goSaveMinMillis : goSaveUsedMillis;
        }
        this.goSaveSumMillis =  this.goSaveSumMillis == null? goSaveUsedMillis : (this.goSaveSumMillis + goSaveUsedMillis);
    }

    public void addSaveNumber(int saveNumber){
        this.saveNumber =  this.saveNumber == null? saveNumber : (this.saveNumber + saveNumber);
    }

    public void judgeSaveMysqlFirstTime(Date saveMysqlFirstTime){
        if(this.saveMysqlFirstTime == null){
            this.saveMysqlFirstTime = saveMysqlFirstTime;
        }else {
            this.saveMysqlFirstTime = DateUtil.min(this.saveMysqlFirstTime,saveMysqlFirstTime);
        }
    }

    public void judgeSaveMysqlLastTime(Date saveMysqlLastTime){
        if(this.saveMysqlLastTime == null){
            this.saveMysqlLastTime = saveMysqlLastTime;
        }else {
            this.saveMysqlLastTime = DateUtil.max(this.saveMysqlLastTime,saveMysqlLastTime);
        }
    }

    public void judgeSaveMysqlUsedMaxAndMinMillis(long saveMysqlUsedMillis){
        if(this.saveMysqlUsedMaxMillis == null){
            this.saveMysqlUsedMaxMillis = saveMysqlUsedMillis;
        }else {
            this.saveMysqlUsedMaxMillis = this.saveMysqlUsedMaxMillis > saveMysqlUsedMillis ? this.saveMysqlUsedMaxMillis : saveMysqlUsedMillis;
        }
        if(this.saveMysqlUsedMinMillis == null){
            this.saveMysqlUsedMinMillis = saveMysqlUsedMillis;
        }else {
            this.saveMysqlUsedMinMillis = this.saveMysqlUsedMinMillis < saveMysqlUsedMillis ? this.saveMysqlUsedMinMillis : saveMysqlUsedMillis;
        }
        this.saveMysqlUsedSumMillis =  this.saveMysqlUsedSumMillis == null? saveMysqlUsedMillis : (this.saveMysqlUsedSumMillis + saveMysqlUsedMillis);
    }

    public void judgeSaveInfluxFirstTime(Date saveInfluxFirstTime){
        if(this.saveInfluxFirstTime == null){
            this.saveInfluxFirstTime = firstTime;
        }else {
            this.saveInfluxFirstTime = DateUtil.min(this.saveInfluxFirstTime,saveInfluxFirstTime);
        }
    }

    public void judgeSaveInfluxLastTime(Date saveInfluxLastTime){
        if(this.saveInfluxLastTime == null){
            this.saveInfluxLastTime = saveInfluxLastTime;
        }else {
            this.saveInfluxLastTime = DateUtil.max(this.saveInfluxLastTime,saveInfluxLastTime);
        }
    }

    public void judgeSaveInfluxUsedMaxAndMinMillis(long saveInfluxUsedMillis){
        if(this.saveInfluxUsedMaxMillis == null){
            this.saveInfluxUsedMaxMillis = saveInfluxUsedMillis;
        }else {
            this.saveInfluxUsedMaxMillis = this.saveInfluxUsedMaxMillis > saveInfluxUsedMillis ? this.saveInfluxUsedMaxMillis : saveInfluxUsedMillis;
        }
        if(this.saveInfluxUsedMinMillis == null){
            this.saveInfluxUsedMinMillis = saveInfluxUsedMillis;
        }else {
            this.saveInfluxUsedMinMillis = this.saveInfluxUsedMinMillis < saveInfluxUsedMillis ? this.saveInfluxUsedMinMillis : saveInfluxUsedMillis;
        }
        this.saveInfluxUsedSumMillis =  this.saveInfluxUsedSumMillis == null? saveInfluxUsedMillis : (this.saveInfluxUsedSumMillis + saveInfluxUsedMillis);
    }

    /** 从进入解码到完全解码耗时 */
    public long getProcessFinalTime(){
        /** 计算总计耗时，最后写入实际存在时，以最后写入时间算，否则一组装时间算 */
        if(firstTime != null){
            if(saveInfluxLastTime != null || saveMysqlLastTime != null){
                if(saveInfluxLastTime != null && saveMysqlLastTime != null){
                    return DateUtil.max(saveInfluxLastTime,saveMysqlLastTime).getTime() - firstTime.getTime();
                }else if(saveInfluxLastTime != null){
                    return saveInfluxLastTime.getTime() - firstTime.getTime();
                }else {
                    return saveMysqlLastTime.getTime() - firstTime.getTime();
                }
            }else {
                if(packageLastTime != null){
                    return packageLastTime.getTime() - firstTime.getTime();
                }
            }
        }
        return -1L;
    }
}
