/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-04-18 12:03:21
 */
package com.qhiot.transport.db.model;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.qhiot.core.util.DateUtil;
import lombok.Getter;
import lombok.Setter;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;

/**
 * Table: device - 终端 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-04-18 12:03:21
 */
@Getter
@Setter
@Entity
@Table(name = "device")
@JsonIgnoreProperties(value={"lastActiveTimestamp"}, allowGetters=true)
public class Device implements Serializable {

//    private static final long serialVersionUID = 3564890470275847229L;

    /** Column: device_id - 终端编号 */
    @Id
    @Column(name = "device_id")
    @Length(max=20, message="db.device.deviceId.Length")
    private String deviceId;

    /** Column: tenant_id - 项目ID */
    @NotNull(message="db.device.tenantId.NotNull")
    private Long tenantId;

    /** Column: device_name - 终端名称 */
    @Length(max=20, message="db.device.deviceName.Length")
    @NotBlank(message="db.device.deviceName.NotBlank")
    private String deviceName;

    /** Column: model_id - 终端型号ID */
    @NotNull(message="db.device.deviceModelId.NotNull")
    private Long deviceModelId;

    private String iccid;

    /** Column: create_time - 更新时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createTime;

    /** Column: last_active_time - 最后活动时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date lastActiveTime;

    /**
     * 1上电、0掉电
     */
    private Integer powerState;

    private Integer deviceState;

    /**
     * 联网率
     */
    private Double networkingRate;

    private Long advStationEquipmentId;
    /**
     * 停用时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date downTime;

    /**
     * 终端上电、掉电告警时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date powerWarnTime;

    @Transient
    public int getLastActiveTimestamp(){
        return lastActiveTime==null?0:(int) (lastActiveTime.getTime()/1000);
    }

    @Transient
    public boolean isActive(){
        return lastActiveTime==null?false:(DateUtil.diffSeconds(lastActiveTime, new Date())< 60 * 30);
    }

}
