/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-05-19 11:33:32
 */
package com.qhiot.transport.db.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import lombok.Getter;
import lombok.Setter;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import com.qhiot.core.util.DateUtil;

/**
 * Table: device_connection - 终端连接 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-05-19 11:33:32
 */
@Getter
@Setter
@Entity
@Table(name = "device_connection")
public class DeviceConnection implements Serializable {

    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "connection_id")
    private Long connectionId;

    /** Column: device_id - 终端编号 */
    @Column(name = "device_id")
    private String deviceId;

    /** Column: connected_time - 连接时间 */
    @NotNull(message="db.deviceConnection.connectedTime.NotNull")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date connectedTime;

    /** Column: disconnected_time - 断开时间 */
    @NotNull(message="db.deviceConnection.disconnectedTime.NotNull")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date disconnectedTime;

    /** Column: connection_duration - 连接时长(秒) */
    @NotNull(message="db.deviceConnection.connectionDuration.NotNull")
    private Integer connectionDuration;

    public String getConnectionDurationPretty() {
        if(connectionDuration!=null){
            return DateUtil.prettySeconds(connectionDuration);
        }
        return "";
    }
}
