/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2018-08-30 05:25:05
 */
package com.qhiot.transport.db.model;

import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import lombok.Data;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;
import org.hibernate.validator.constraints.Range;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import java.io.Serializable;

/**
 * Table: device_peripheral - 终端外设表 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2018-08-30 05:25:05
 */
@Data
@Entity
@Table(name = "device_peripheral")
public class DevicePeripheral implements Serializable {

    /** Column: device_peripheral_id - 外设ID */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "device_peripheral_id")
    private Long devicePeripheralId;

    /** Column: device_id - 终端ID */
    @Length(max=20, message="db.devicePeripheral.deviceId.Length")
    @NotBlank(message="db.devicePeripheral.deviceId.NotBlank")
    private String deviceId;

    /** Column: device_peripheral_name - 外设名称 */
    @Length(max=200, message="db.devicePeripheral.devicePeripheralName.Length")
    @NotBlank(message="db.devicePeripheral.devicePeripheralName.NotBlank")
    private String devicePeripheralName;

    /** Column: protocol - 外设协议类型 */
    @Range(min=0, max=5, message = "db.devicePeripheral.protocol.Range")
    @NotNull(message="db.devicePeripheral.protocol.NotNull")
    private Byte protocol;

    /** Column: comm_port - 串口号 */
    @NotNull(message="db.devicePeripheral.commPort.NotNull")
    private Byte commPort;

    /** Column: baud_rate2 - 波特率 */
    @NotNull(message="db.devicePeripheral.baudRate2.NotNull")
    private Byte baudRate2;

    /** Column: data_bit - 数据位 */
    @NotNull(message="db.devicePeripheral.dataBit.NotNull")
    private Byte dataBit;

    /** Column: stop_bit - 停止位 */
    @NotNull(message="db.devicePeripheral.stopBit.NotNull")
    private Byte stopBit;

    /** Column: parity_bit - 校验 */
    @NotNull(message="db.devicePeripheral.parityBit.NotNull")
    private Byte parityBit;

    /** Column: address - 设备地址 */
    @NotNull(message="db.devicePeripheral.address.NotNull")
    @Range(min=0, max=255, message = "db.devicePeripheral.address.Range")
    private Short address;
    /** Column: ammeterNum - 电表号 */
    private Long ammeterNum;
}
