/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2018-04-20 10:40:52
 */
package com.qhiot.transport.db.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;

import lombok.Data;

import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import com.qhiot.core.util.StringUtil;

/**
 * Table: equipment_config_templet - 设备配置模板 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2018-04-20 10:40:52
 */
@Data
@Entity
@Table(name = "equipment_config_templet")
public class EquipmentConfigTemplet implements Serializable {

    /** Column: templet_id - 主键 */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "templet_id")
    private Long templetId;

    /** Column: templet_name - 模板名称 */
    @Length(max=100, message="db.equipmentConfigTemplet.templetName.Length")
    @NotBlank(message="db.equipmentConfigTemplet.templetName.NotBlank")
    private String templetName;

    /** Column: equipment_type - 设备类型 */
    @NotNull(message="db.equipmentConfigTemplet.equipmentType.NotNull")
    private Byte equipmentType;

    /** Column: manufacturers - 制造厂家 */
    @NotNull(message="db.equipmentConfigTemplet.manufacturers.NotNull")
    private Byte manufacturers;

    /** Column: equipment_model - 型号 */
    @NotNull(message="db.equipmentConfigTemplet.equipmentModel.NotNull")
    private Byte equipmentModel;

    /** Column: communication_interface - 通讯接口 */
    @NotNull(message="db.equipmentConfigTemplet.communicationInterface.NotNull")
    private Byte communicationInterface;

    /** Column: ip1_or_baud_rate - 波特率 */
    private Byte ip1OrBaudRate;

    /** Column: ip2_or_data_bit - 数据位 */
    private Byte ip2OrDataBit;

    /** Column: ip3_or_parity_bit - 奇偶校验位 */
    private Byte ip3OrParityBit;

    /** Column: ip4_or_stop_bit - 停止位 */
    private Byte ip4OrStopBit;

    @Transient
    private String ip;

    /** Column: port_or_reserved1 - 端口 */
    private Short portOrReserved1;

    /** Column: byte_order16 - 16位数据字节序 */
    private Byte byteOrder16;

    /** Column: byte_order32 - 32位数据字节序 */
    private Byte byteOrder32;

    public void setIp1OrBaudRate(Byte ip1OrBaudRate) {
        if (ip1OrBaudRate != null) {
            this.ip1OrBaudRate = ip1OrBaudRate;
        }
    }

    public void setIp2OrDataBit(Byte ip2OrDataBit) {
        if (ip2OrDataBit != null) {
            this.ip2OrDataBit = ip2OrDataBit;
        }
    }

    public void setIp3OrParityBit(Byte ip3OrParityBit) {
        if (ip3OrParityBit != null) {
            this.ip3OrParityBit = ip3OrParityBit;
        }
    }

    public void setIp4OrStopBit(Byte ip4OrStopBit) {
        if (ip4OrStopBit != null) {
            this.ip4OrStopBit = ip4OrStopBit;
        }
    }

    public String getIp(){
        if (ip1OrBaudRate!=null && ip2OrDataBit!=null && ip3OrParityBit!=null && ip4OrStopBit!=null) {
            return StringUtil.format("{}.{}.{}.{}", ip1OrBaudRate & 0xff, ip2OrDataBit & 0xff, ip3OrParityBit & 0xff, ip4OrStopBit & 0xff);
        } else {
            return "";
        }
    }



    public void setIp(String ip) {
        this.ip = ip;
        if (StringUtil.isNotBlank(ip)) {
            String[] tokens = ip.split("[.]");
            if (tokens.length != 4) {
                throw new RuntimeException(StringUtil.format("'{}' 不合法的 IPv4 地址", ip));
            }
            try {
                ip1OrBaudRate = Integer.valueOf(tokens[0]).byteValue();
                ip2OrDataBit = Integer.valueOf(tokens[1]).byteValue();
                ip3OrParityBit = Integer.valueOf(tokens[2]).byteValue();
                ip4OrStopBit = Integer.valueOf(tokens[3]).byteValue();
            } catch (NumberFormatException e) {
                throw new RuntimeException(StringUtil.format("'{}' 不合法的 IPv4 地址", ip));
            }
        }
    }
}
