/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2018-12-19 02:18:32
 */
package com.qhiot.transport.db.model;

import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import lombok.Data;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;
import org.hibernate.validator.constraints.Range;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.GroupSequence;
import javax.validation.constraints.NotNull;
import javax.validation.groups.Default;
import java.io.Serializable;

/**
 * Table: lib_equipment - 设备库 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2018-12-19 02:18:32
 */
@Data
@Entity
@Table(name = "lib_equipment")
public class LibEquipment implements Serializable {

    /** Column: equipment_id - 设备id */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "equipment_id")
    private Long equipmentId;

    /** Column: equipment_type_id - 设备类型id */
    @NotNull(message="db.libEquipment.equipmentTypeId.NotNull")
    private Long equipmentTypeId;

    /** Column: equipment_name - 设备名称 */
    @Length(max=200, message="db.libEquipment.equipmentName.Length")
    @NotBlank(message="db.libEquipment.equipmentName.NotBlank")
    private String equipmentName;

    /** Column: manufacturers - 厂家 */
    @Length(max=200, message="db.libEquipment.manufacturers.Length")
    @NotBlank(message="db.libEquipment.manufacturers.NotBlank")
    private String manufacturers;

    /** Column: model - 型号 */
    @Length(max=200, message="db.libEquipment.model.Length")
    @NotBlank(message="db.libEquipment.model.NotBlank")
    private String model;

    /** Column: protocol - 协议类型 */
    @NotNull(message="db.libEquipment.protocol.NotNull")
    private Short protocol;

    /** Column: baud_rate - 波特率 */
    private Short baudRate = 0;

    /** Column: data_bit - 数据位 */
    private Short dataBit = 0;

    /** Column: stop_bit - 停止位 */
    private Short stopBit = 0;

    /** Column: parity_bit - 校验方式 */
    private Short parityBit = 0;

    /** Column: device_modal_id - 终端型号id */
    @NotNull(message="db.libEquipment.deviceModalId.NotNull",groups = GroupA.class)
    private Long deviceModalId;

    /** Column: need_number - 是否需要编号 */
    @NotNull(message="db.libEquipment.needNumber.NotNull")
    private Short needNumber = 1;
    /**
     * Column: period - 采集周期(2 byte) (0-65535)秒
     */
    @Range(min=1, max=65535, message = "db.transportResourceConfig.period.Range")
    @NotNull(message="db.transportResourceConfig.period.NotNull")
    private Integer period = 300;

    public  interface GroupA {

    }

    @GroupSequence( { Default.class, GroupA.class })
    public interface Group {

    }
}
