/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-04-20 10:18:47
 */
package com.qhiot.transport.db.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;

import lombok.Getter;
import lombok.Setter;

import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import tk.mybatis.mapper.entity.IDynamicTableName;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;

/**
 * Table: monitoring_data_history - 测点历史数据 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-04-20 10:18:47
 */
@Getter
@Setter
@Entity
@Table(name = "monitoring_data_history")
public class MonitoringDataHistory implements IDynamicTableName, Serializable {

    private static final long serialVersionUID = -851372646559326792L;

    /** Column: data_id - 主键 */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "data_id")
    private Long dataId;

    /** Column: device_id - 终端编号 */
    @Length(max=50, message="db.monitoringDataHistory.deviceId.Length")
    @NotBlank(message="db.monitoringDataHistory.deviceId.NotBlank")
    private String deviceId;

    /** Column: monitoring_id - 测点编号 */
    @NotNull(message="db.monitoringDataHistory.monitoringId.NotNull")
    private Long monitoringId;

    private Long smId;

    private Long indexId;

    /** Column: data_value - 采样值 */
    @NotNull(message="db.monitoringDataHistory.dataValue.NotNull")
    private Double dataValue;

    /** Column: data_time - 采样时间 */
    @NotNull(message="db.monitoringDataHistory.dataTime.NotNull")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date dataTime;

    /** Column: received_time - 接收时间 */
    @NotNull(message="db.monitoringDataHistory.receivedTime.NotNull")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date receivedTime;

    @Transient//非表字段，字段名称无所谓
    private String dynamicTableName;

    public Integer getDataTimestamp() {
        return dataTime==null ? 0 : (int) (dataTime.getTime()/1000);
    }

    @Override//当该方法返回值不为空时，就会使用返回值作为表名
    public String getDynamicTableName() {
        return dynamicTableName;
    }

    public void setDynamicTableName(MonitoringDataTable table) {
        dynamicTableName = table.getName();
    }


    public enum MonitoringDataTable {
        DAY("monitoring_data_day"),MONTH("monitoring_data_month"), HISTORY("monitoring_data_history");

        private String name;

        MonitoringDataTable(String name){
            this.name = name;
        }

        public String getName() {
            return name;
        }
    }
}
