/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 * 
 * Date: 2018-09-26 03:59:32
 */
package com.qhiot.transport.db.model;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import lombok.Data;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;

/**
 * Table: sim_info -  <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2018-09-26 03:59:32
 */
@Data
@Entity
@Table(name = "sim_info")
public class SimInfo implements Serializable {

    /** Column: sim_id - ID */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "sim_id")
    private Long simId;

    /** Column: tenant_manager_id - 租户ID */
    @NotNull(message="web.simInfo.tenantManagerId.NotNull")
    private Long tenantManagerId;

    /** Column: tenant_id - 项目ID */
    @NotNull(message="web.simInfo.tenantId.NotNull")
    private Long tenantId;

    /** Column: service_provider - 运营商 */
    @NotNull(message="web.simInfo.serviceProvider.NotNull")
    private Integer serviceProvider;

    /** Column: total_ flow - 套餐总量(M) */
    @NotNull(message="web.simInfo.totalFlow.NotNull")
    private Double totalFlow;

    /** Column: sim_iccid - 平台卡号 */
    @Length(max=20, message="web.simInfo.simIccid.Length")
    @NotBlank(message="web.simInfo.simIccid.NotBlank")
    private String simIccid;

    /** Column: card_no - 实际卡号 */
    @Length(max=20, message="web.simInfo.cardNo.Length")
    @NotBlank(message="web.simInfo.cardNo.NotBlank")
    private String cardNo;

    /** Column: sendgoods_date - 发货日期 */
    @NotNull(message="web.simInfo.sendgoodsDate.NotNull")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date sendgoodsDate;

    /** Column: due_date - 续费年限 */
    @NotNull(message="web.simInfo.dueDate.NotNull")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date dueDate;

    /** Column: next_renewal_date - 次年续费日期 */
    @NotNull(message="web.simInfo.nextRenewalDate.NotNull")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date nextRenewalDate;

    /** Column: customer_name - 客户名称 */
    @Length(max=255, message="web.simInfo.customerName.Length")
    @NotBlank(message="web.simInfo.customerName.NotBlank")
    private String customerName;

    /** Column: project_name - 项目名称 */
    @Length(max=255, message="web.simInfo.projectName.Length")
    @NotBlank(message="web.simInfo.projectName.NotBlank")
    private String projectName;

    /** Column: contract_no - 合同编号 */
    @Length(max=255, message="web.simInfo.contractNo.Length")
    @NotBlank(message="web.simInfo.contractNo.NotBlank")
    private String contractNo;

    /** Column: sim_device_id - 网关编号 */
    @Length(max=255, message="web.simInfo.simDeviceId.Length")
    @NotBlank(message="web.simInfo.simDeviceId.NotBlank")
    private String simDeviceId;

    /** Column: Is_earlywarning - 是否提前提醒 */
    @NotNull(message="web.simInfo.isEarlywarning.NotNull")
    private Integer isEarlywarning;

    /** Column: sim_state - 状态 */
    @NotNull(message="web.simInfo.simState.NotNull")
    private Integer simState;

    /** Column: yesterday_flow - 前一日使用量 */
    private Double yesterdayFlow;

    /** Column: day_avg_flow - 日平均使用量 */
    private Double dayAvgFlow;

    /** Column: month_flow - 月使用量 */
    private Double monthFlow;


    private Double alreadyusedFlow;

    /** Column: overflow - 超出流量 */
    private Double overflow;

    /** Column: sim_remarks - 备注 */
    @Length(max=255, message="web.simInfo.simRemarks.Length")
    private String simRemarks;

}
