/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-03-29 04:13:27
 */
package com.qhiot.transport.db.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;

import lombok.Getter;
import lombok.Setter;

import com.qhiot.core.util.StringUtil;

/**
 * Table: transport_resource_config - 终端资源配置 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-03-29 04:13:27
 */
@Getter
@Setter
@Entity
@Table(name = "transport_equipment_config")
public class TransportEquipmentConfig implements Serializable {

    private static final long serialVersionUID = -2208300079187080050L;

    /** Column: device_id - 终端编号 */
    @Id
    @Column(name = "device_id")
    private String deviceId;

    /** Column: config_index - 配置序号 */
    @Id
    @Column(name = "config_index")
    private Short configIndex;

    private Byte valid;

    /** Column: equipment_type - 设备类型 */
    @NotNull(message="db.equipmentConfigTemplet.equipmentType.NotNull")
    private Byte equipmentType;

    /** Column: manufacturers - 制造厂家 */
    @NotNull(message="db.equipmentConfigTemplet.manufacturers.NotNull")
    private Byte manufacturers;

    /** Column: equipment_model - 型号 */
    @NotNull(message="db.equipmentConfigTemplet.equipmentModel.NotNull")
    private Byte equipmentModel;

    /** Column: communication_interface - 通讯接口 */
    @NotNull(message="db.equipmentConfigTemplet.communicationInterface.NotNull")
    private Byte communicationInterface;

    /** Column: ip1_or_baud_rate - 波特率 */
    private Byte ip1OrBaudRate;

    /** Column: ip2_or_data_bit - 数据位 */
    private Byte ip2OrDataBit;

    /** Column: ip3_or_parity_bit - 奇偶校验位 */
    private Byte ip3OrParityBit;

    /** Column: ip4_or_stop_bit - 停止位 */
    private Byte ip4OrStopBit;

    @Transient
    private String ip;

    /** Column: port_or_reserved1 - 端口 */
    private Integer portOrReserved1;

    /** Column: byte_order16 - 16位数据字节序 */
    private Byte byteOrder16;

    /** Column: byte_order32 - 32位数据字节序 */
    private Byte byteOrder32;

    /** Column: union_index - 连接序号 */
    private Short unionIndex;

    public void setIp1OrBaudRate(Byte ip1OrBaudRate) {
        if (ip1OrBaudRate != null) {
            this.ip1OrBaudRate = ip1OrBaudRate;
        }
    }

    public void setIp2OrDataBit(Byte ip2OrDataBit) {
        if (ip2OrDataBit != null) {
            this.ip2OrDataBit = ip2OrDataBit;
        }
    }

    public void setIp3OrParityBit(Byte ip3OrParityBit) {
        if (ip3OrParityBit != null) {
            this.ip3OrParityBit = ip3OrParityBit;
        }
    }

    public void setIp4OrStopBit(Byte ip4OrStopBit) {
        if (ip4OrStopBit != null) {
            this.ip4OrStopBit = ip4OrStopBit;
        }
    }

    public String getIp(){
        if (ip1OrBaudRate!=null && ip2OrDataBit!=null && ip3OrParityBit!=null && ip4OrStopBit!=null) {
            return StringUtil.format("{}.{}.{}.{}", ip1OrBaudRate & 0xff, ip2OrDataBit & 0xff, ip3OrParityBit & 0xff, ip4OrStopBit & 0xff);
        } else {
            return "";
        }
    }



    public void setIp(String ip) {
        this.ip = ip;
        if (StringUtil.isNotBlank(ip)) {
            String[] tokens = ip.split("[.]");
            if (tokens.length != 4) {
                throw new RuntimeException(StringUtil.format("'{}' 不合法的 IPv4 地址", ip));
            }
            try {
                ip1OrBaudRate = Integer.valueOf(tokens[0]).byteValue();
                ip2OrDataBit = Integer.valueOf(tokens[1]).byteValue();
                ip3OrParityBit = Integer.valueOf(tokens[2]).byteValue();
                ip4OrStopBit = Integer.valueOf(tokens[3]).byteValue();
            } catch (NumberFormatException e) {
                throw new RuntimeException(StringUtil.format("'{}' 不合法的 IPv4 地址", ip));
            }
        }
    }

    public Byte getIp1OrBaudRate() {
        return ip1OrBaudRate == null ? 0 : ip1OrBaudRate;
    }

    public Byte getIp2OrDataBit() {
        return ip2OrDataBit == null ? 0 : ip2OrDataBit;
    }

    public Byte getIp3OrParityBit() {
        return ip3OrParityBit == null ? 0 : ip3OrParityBit;
    }

    public Byte getIp4OrStopBit() {
        return ip4OrStopBit == null ? 0 : ip4OrStopBit;
    }

    public Integer getPortOrReserved1() {
        return portOrReserved1 == null ? 0 : portOrReserved1;
    }

    public Short getUnionIndex() {
        return unionIndex == null ? 0 : unionIndex;
    }
}
