/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-03-29 04:13:27
 */
package com.qhiot.transport.db.model;

import com.qhiot.core.util.StringUtil;
import lombok.Getter;
import lombok.Setter;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;
import org.hibernate.validator.constraints.Range;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.io.Serializable;

/**
 * Table: transport_resource_config - 终端资源配置 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-03-29 04:13:27
 */
@Getter
@Setter
@Entity
@Table(name = "transport_resource_config")
public class TransportResourceConfig implements Serializable {

    private static final long serialVersionUID = -8715176254444705863L;

    /** Column: device_id - 终端编号 */
    @Id
    @Column(name = "device_id")
    private String deviceId;

    /** Column: config_index - 配置序号 */
    @Id
    @Column(name = "config_index")
    private Short configIndex;

    /** Column: config_name- 配置名称 */
    @Column(name = "config_name")
    @NotBlank(message = "db.transportResourceConfig.configName.NotBlank")
    private String configName;

    /** Column: valid - 配置有效位(1 byte), 0 或 1 */
    @NotNull(message="db.transportResourceConfig.valid.NotNull")
    @Range(min=0, max=1, message = "db.transportResourceConfig.valid.Range")
    private Byte valid;

    /** Column: protocol - 协议类型 (1 byte)
     *
     * 0: 终端内部资源, <br/>
     * 1: Modbus外设, <br/>
     * 2: 电表协议1997, <br/>
     * 3: 电表协议2007, <br/>
     * 4: Hostlink协议, <br/>
     * 5: 其它协议
     */
    @Range(min=0, max=5, message = "db.transportResourceConfig.protocol.Range")
    @NotNull(message="db.transportResourceConfig.protocol.NotNull")
    private Byte protocol;

    /** Column: resource - 资源类型(1 byte)
     *
     * 0-15: 4-20ma, <br/>
     * 16-23: 0-5V, <br/>
     * 24: 温度, <br/>
     * 25: 湿度, <br/>
     * 26: 掉电检测电压(V), <br/>
     * 30-45: 电量数据(安培), <br/>
     * 50-69: 开关量输入, <br/>
     * 70-89: 开关量输出, <br/>
     * 100: 外设
     */
    @Range(min=0, max=100, message = "db.transportResourceConfig.resource.Range")
    @NotNull(message="db.transportResourceConfig.resource.NotNull")
    private Byte resource;

    /** Column: comm_port - 串口号(1 byte)
     *
     * 1: 串口1, <br/>
     * 2: 串口2, <br/>
     * 3: 串口3, <br/>
     * 4: 串口4, <br/>
     * 5: 串口5, <br/>
     * 6: 其它
     */
    @Range(min=0, max=6, message = "db.transportResourceConfig.commPort.Range")
    @NotNull(message="db.transportResourceConfig.commPort.NotNull")
    private Byte commPort = 0;

    /** Column: baud_rate - 波特率(1 byte)
     *
     * 0: 1200, <br/>
     * 1: 2400, <br/>
     * 2: 4800, <br/>
     * 3: 9600, <br/>
     * 4: 19200, <br/>
     * 5: 38400, <br/>
     * 6: 57600, <br/>
     * 7: 115200
     */
    @Range(min=0, max=7, message = "db.transportResourceConfig.baudRate.Range")
    @NotNull(message="db.transportResourceConfig.baudRate.NotNull")
    private Byte baudRate = 0;

    /** Column: data_bit - 数据位(1byte)
     *
     * 0: 5位数据, <br/>
     * 1: 6位数据, <br/>
     * 2: 7位数据, <br/>
     * 3: 8位数据
     */
    @Range(min=0, max=3, message = "db.transportResourceConfig.dataBit.Range")
    @NotNull(message="db.transportResourceConfig.dataBit.NotNull")
    private Byte dataBit = 0;

    /** Column: stop_bit - 停止位(1 byte)
     *
     * 0: 1位, <br/>
     * 1: 1.5位, <br/>
     * 2: 2位
     */
    @Range(min=0, max=2, message = "db.transportResourceConfig.stopBit.Range")
    @NotNull(message="db.transportResourceConfig.stopBit.NotNull")
    private Byte stopBit = 0;

    /** Column: parity_bit - 奇偶校验位(1 byte)
     *
     * 0: 无校验, <br/>
     * 1: 奇校验, <br/>
     * 2: 偶校验
     */
    @Range(min=0, max=2, message = "db.transportResourceConfig.parityBit.Range")
    @NotNull(message="db.transportResourceConfig.parityBit.NotNull")
    private Byte parityBit = 0;

    /**
     * Column: period - 采集周期(2 byte) (0-65535)秒
     */
    @Range(min=1, max=65535, message = "db.transportResourceConfig.period.Range")
    @NotNull(message="db.transportResourceConfig.period.NotNull")
    private Integer period = 300;

    /** Column: address - 设备地址(1 byte) */
    @NotNull(message="db.transportResourceConfig.address.NotNull")
    @Range(min=0, max=255, message = "db.transportResourceConfig.address.Range")
    private Short address = 0;

    /** Column: function - 功能码(1 byte)
     * 01 读线圈
     * 02 读离散输入
     * 03 读保持寄存器
     * 04 读输入寄存器
     */
    @NotNull(message="db.transportResourceConfig.functionCode.NotNull")
    @Range(min=0, max=4, message = "db.transportResourceConfig.period.Range")
    private Byte functionCode = 0;

    /** Column: starting - 起始地址(2 byte) */
    @NotNull(message="db.transportResourceConfig.startingAddr.NotNull")
    private Integer startingAddr = 0;

    /** Column: method - 数据解码(1 byte)
     *
     * 0:Bit <br/>
     * 1:INT8S 8位有符号 <br/>
     * 2:INT8U 8位无符号 <br/>
     * 3.INT16S_AB <br/>
     * 4.INT16S_BA <br/>
     * 5.INT16U_AB <br/>
     * 6.INT16U_BA <br/>
     * 7:Long ABCD <br/>
     * 8:Long CDAB <br/>
     * 9:Long BADC <br/>
     * 10:Long DCBA <br/>
     * 11:float ABCD <br/>
     * 12:float CDAB <br/>
     * 13:float BADC <br/>
     * 14:float DCBA <br/>
     */
    @Range(min=0, max=14, message = "db.transportResourceConfig.method.Range")
    @NotNull(message="db.transportResourceConfig.method.NotNull")
    private Byte method;

    /** Column: quantity - 测点数量(2 byte) */
    @NotNull(message="db.transportResourceConfig.dataQuantity.NotNull")
    private Integer dataQuantity = 1;

    /**
     * 指令长度(1 byte) 0-32 <br/>
     */
    private Byte commandLength = 0;

    /** Column: command - 采集指令 */
    @Length(max=100, message="db.transportResourceConfig.command.Length")
    private String command;

    /** Column: register_type - 寄存器类型(1 byte)
     *
     * 0: bit/只读, <br/>
     * 1: bit/可读可写, <br/>
     * 2: 2字节/只读, <br/>
     * 3: 2字节/可读可写
     */
    @Range(min=0, max=4, message = "db.transportResourceConfig.registerType.Range")
    @NotNull(message="db.transportResourceConfig.registerType.NotNull")
    private Byte registerType;

    /** Column: store_address - 存放地址(2 byte) */
    @Range(min=0, max=300, message = "db.transportResourceConfig.storeAddress.Range")
    @NotNull(message="db.transportResourceConfig.storeAddress.NotNull")
    private Integer storeAddress;


    /** 上限(4 byte) Float DCBA */
    private Float upperBound;

    /** 下限(4 byte) Float DCBA */
    private Float lowerBound;

    @Range(min=0, max=1, message = "db.transportResourceConfig.upperBoundEnabled.Range")
    @NotNull(message="db.transportResourceConfig.upperBoundEnabled.NotNull")
    private Byte upperBoundEnabled;

    @Range(min=0, max=1, message = "db.transportResourceConfig.lowerBoundEnabled.Range")
    @NotNull(message="db.transportResourceConfig.lowerBoundEnabled.NotNull")
    private Byte lowerBoundEnabled;

    private Short needsUpdate;
    /** 电表号电表协议的时候使用 */
    private Long ammeterNum;

    private Long resourceTempId;

    private Long devicePeripheralId;


    @Transient
    private String shortCommand;

    @Transient
    public String getShortCommand(){
        shortCommand = command;
        while (shortCommand !=null && StringUtil.endsWithIgnoreCase(shortCommand, "00")) {
            shortCommand = StringUtil.removeSuffix(shortCommand, "00").trim();
        }
        return shortCommand;
    }

}
