/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2019-06-11 05:48:13
 */
package com.qhiot.transport.db.model;

import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import lombok.Data;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import java.io.Serializable;

/**
 * Table: transport_resource_core - 配置信息核心表，包含配置信息的各种重要信息 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2019-06-11 05:48:13
 */
@Data
@Entity
@Table(name = "transport_resource_core")
public class TransportResourceCore implements Serializable {

    /** Column: core_id - 主键ID */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "core_id")
    private Long coreId;

    /** Column: device_id - 设备编号 */
    @Length(max=20, message="db.transportResourceCore.deviceId.Length")
    @NotBlank(message="db.transportResourceCore.deviceId.NotBlank")
    private String deviceId;

    /** Column: device_id - 项目id */
    @Length(max=20, message="db.transportResourceCore.tenantId.Length")
    @NotBlank(message="db.transportResourceCore.tenantId.NotBlank")
    private Long tenantId;

    /** Column: valid - 有效 */
    @NotBlank(message="db.transportResourceCore.valid.NotBlank")
    private Short valid;

    /** Column: config_index - 配置序号 */
    @NotNull(message="db.transportResourceCore.configIndex.NotNull")
    private Short configIndex;

    /** Column: data_quantity - 数据数量 */
    @NotNull(message="db.transportResourceCore.dataQuantity.NotNull")
    private Integer dataQuantity;

    /** Column: method - 解码方式 */
    @NotNull(message="db.transportResourceCore.method.NotNull")
    private Short method;

    /** Column: store_address - 存放地址 */
    @NotNull(message="db.transportResourceCore.storeAddress.NotNull")
    private Short storeAddress;

    /** Column: monitoring_id - 主键ID */
    @NotNull(message="db.transportResourceCore.monitoringId.NotNull")
    private Long monitoringId;

    /** Column: data_index - 序号 */
    @NotNull(message="db.transportResourceCore.dataIndex.NotNull")
    private Short dataIndex;

    /** Column: formula - 换算公式 */
    @Length(max=200, message="db.transportResourceCore.formula.Length")
    private String formula;

    /** Column: write_address - 写入地址(-1表示不可写入) */
    @NotNull(message="db.transportResourceCore.writeAddress.NotNull")
    private Integer writeAddress;

    /** Column: index_id - 监测指标ID */
    /** Column: write_address - 写入地址(-1表示不可写入) */
    @NotNull(message="db.transportResourceCore.indexId.NotNull")
    private Long indexId;

    @NotNull(message = "db.transportResourceCore.dataType.NotNull")
    private Integer dataType;
    /**
     * Column: value_type - 统计方法
     */
    @NotNull(message = "db.transportResourceCore.valueType.NotNull")
    private Integer valueType;

    /** Column: station_id - 站点ID */
//    @NotNull(message="db.transportResourceCore.stationId.NotNull")
    private Long stationId;

    /** Column: area_id - 区域主键 */
//    @NotNull(message="db.transportResourceCore.areaId.NotNull")
    private Long areaId;

    /** Column: region_code - 标准区划代码 */
//    @Length(max=12, message="db.transportResourceCore.regionCode.Length")
    private String regionCode;

    /** Column: sm_id - ID */
//    @NotNull(message="db.transportResourceCore.smId.NotNull")
    private Long smId;

    /** Column: monitoringCode - 网关内测点编码 */
//    @NotNull(message="db.transportResourceCore.monitoringcode.NotNull")
    private Integer monitoringCode;

    /** Column: smCode - 站点内测点编码 */
//    @NotNull(message="db.transportResourceCore.smcode.NotNull")
    private Integer smCode;

    /** Column: adv_station_equipment_id - 站点设备类型主表 */
//    @NotNull(message="db.transportResourceCore.advStationEquipmentId.NotNull")
    private Long advStationEquipmentId;

    /** Column: adv_station_monitor_id - 设备测点id */
//    @NotNull(message="db.transportResourceCore.advStationMonitorId.NotNull")
    private Long advStationMonitorId;

    private String monitorCode;

    private Integer equipmentSequence;

    private Integer monitoringSequence;

    private Long tempId;

    private Long typeId;
}
