/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-03-24 09:52:45
 */
package com.qhiot.transport.db.model;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import lombok.Getter;
import lombok.Setter;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;

/**
 * Table: transport_upgrade - 固件升级记录 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-03-24 09:52:45
 */
@Getter
@Setter
@Entity
@Table(name = "transport_upgrade")
public class TransportUpgrade implements Serializable {

    /** Column: id - 主键 */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "id")
    private Long id;

    /** Column: device_id - 终端编号 */
    @Length(max=50, message="db.transportUpgrade.deviceId.Length")
    @NotBlank(message="db.transportUpgrade.deviceId.NotBlank")
    private String deviceId;

    /** Column: device_type - 终端类型 */
    @NotNull(message="db.transportUpgrade.deviceType.NotNull")
    private Byte deviceType;

    /** Column: firmware - 固件 */
    @NotNull(message="db.transportUpgrade.firmware.NotNull")
    private byte[] firmware;

    /** Column: create_time - 创建时间 */
    @NotNull(message="db.transportUpgrade.createTime.NotNull")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createTime;

    /** Column: segment_index - 已发送分片号 */
    private Integer segmentIndex;

    /** Column: ignore_errors - 忽略终端类型错误 */
    @NotNull(message="db.transportUpgrade.ignoreErrors.NotNull")
    private Integer ignoreErrors;

    /** Column: timeout - 超时时间 */
    @NotNull(message="db.transportUpgrade.timeout.NotNull")
    private Integer timeout;

    /** Column: retry_count - 超时时间 */
    private Integer retryCount = 30;

    /** Column: segment_length - 包分片长度 */
    private Integer segmentLength = 512;

    /** Column: startup_time - 已发送分片号 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date startupTime;

    private Long firmwareId;

}
