/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-03-24 09:53:57
 */
package com.qhiot.transport.db.model;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import lombok.Getter;
import lombok.Setter;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;

/**
 * Table: transport_upgrade_log - 固件升级日志 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-03-24 09:53:57
 */
@Getter
@Setter
@Entity
@Table(name = "transport_upgrade_log")
public class TransportUpgradeLog implements Serializable {

    /** Column: id - 主键 */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "id")
    private Long id;

    /** Column: device_id - 终端编号 */
    @Length(max=50, message="db.transportUpgradeLog.deviceId.Length")
    @NotBlank(message="db.transportUpgradeLog.deviceId.NotBlank")
    private String deviceId;

    /** Column: device_type - 终端类型 */
    @NotNull(message="db.transportUpgradeLog.deviceType.NotNull")
    private Byte deviceType;

    /** Column: start_time - 开始时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date startTime;

    /** Column: complete_time - 结束时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date completeTime;

    /** Column: upgrade_result - 升级结果 */
    @NotNull(message="db.transportUpgradeLog.upgradeResult.NotNull")
    private Integer upgradeResult;

    /** Column: result_message - 升级结果详情 */
    @Length(max=255, message="db.transportUpgradeLog.resultMessage.Length")
    private String resultMessage;

    /** Column: firmware - 固件 */
    @NotNull(message="db.transportUpgradeLog.firmware.NotNull")
    private byte[] firmware;

    private Long firmwareId;

    /** Column: segment_length - 包分片长度 */
    private Integer segmentLength = 512;

    private Integer ignoreErrors;
}
