/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-04-28 09:57:32
 */
package com.qhiot.transport.db.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;

import lombok.Getter;
import lombok.Setter;
import tk.mybatis.mapper.annotation.ColumnType;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.qhiot.core.sql.annotation.Key;
import com.qhiot.core.sql.annotation.KeyType;
import com.qhiot.transport.db.mapper.type.WarningCodeTypeHandler;

/**
 * Table: transport_warning - 终端告警 <br/>
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-04-28 09:57:32
 */
@Getter
@Setter
@Entity
@Table(name = "transport_warning")
public class TransportWarning implements Serializable {

    /** Column: id - 主键ID */
    @Id
    @Key(type = KeyType.SNOWFLAKE)
    @Column(name = "id")
    private Long id;

    /** Column: device_id - 终端编号 */
    private String deviceId;

    /** Column: warning_code - 告警代码 */
    @ColumnType(typeHandler = WarningCodeTypeHandler.class)
    private WarningCode warningCode;

    /** Column: warning - 告警 */
    private String warning;

    /** Column: warning_count - 告警次数 */
    private Integer warningCount;

    /** Column: first_time - 首次告警时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date firstTime;

    /** Column: latest_time - 最后告警时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date latestTime;


    /**
     * 告警代码
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum WarningCode {

        POWER("W001", "终端掉电"),
        OFFLINE("W002", "终端离线"),
        ELECTRIC_CABINET("W003", "电控箱非法开启"),
        TRANSFINITE("W004", "数据超限"),
        RULE_MATCH("W005", "规则告警"),
        UNKNOW("W000", "未知代码");

        private String code;

        private String desc;

        private WarningCode(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @JsonCreator
        public static WarningCode codeOf(String code) {
            WarningCode[] enumConstants = WarningCode.class.getEnumConstants();
            for (WarningCode e : enumConstants) {
                if (e.getCode().equals(code)) {
                    return e;
                }
            }
            return UNKNOW;
        }

        public String getCode() {
            return code;
        }

        public String getDesc() {
            return desc;
        }


        @Override
        public String toString() {
            return code;
        }
    }
}
