/*
 * ! CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-05-19 11:33:32
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.service.BaseService;
import com.qhiot.transport.db.dto.DeviceConnectionNetworkDTO;
import com.qhiot.transport.db.mapper.DeviceConnectionMapper;
import com.qhiot.transport.db.model.DeviceConnection;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 业务层 <br/>
 * Table: device_connection - 终端连接 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-05-19 11:33:32
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class DeviceConnectionService extends BaseService<DeviceConnection> {

    private static final Logger LOGGER = LoggerFactory.getLogger(DeviceConnectionService.class);

    private static final String INSERT_SQL = "INSERT INTO device_connection (connection_id, device_id, connected_time, disconnected_time, connection_duration) VALUES (?, ?, ?, ?, ?)";
    private static final String UPDATE_SQL = "UPDATE device_connection SET disconnected_time = ?, connection_duration = ? WHERE connection_id = ?";

    @Autowired
    private DeviceConnectionMapper deviceConnectionMapper;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public int insertBatch(List<DeviceConnection> entries) throws ServiceException {
        if (entries == null || entries.isEmpty()) {
            return 0;
        }
        int totalRowCount = 0;
        try {
            List<Object[]> args = new ArrayList<Object[]>(entries.size());
            for (DeviceConnection item : entries) {
                Object[] argItem = new Object[5];
                argItem[0] = item.getConnectionId();
                argItem[1] = item.getDeviceId();
                argItem[2] = item.getConnectedTime();
                argItem[3] = item.getDisconnectedTime();
                argItem[4] = item.getConnectionDuration();
                args.add(argItem);
            }
            int[] rowCounts = jdbcTemplate.batchUpdate(INSERT_SQL, args);
            for (int j = 0; j < rowCounts.length; j++) {
                if (rowCounts[j] == Statement.SUCCESS_NO_INFO) {
                    // do nothing
                } else if (rowCounts[j] == Statement.EXECUTE_FAILED) {
                    LOGGER.warn("The batched statement at index " + j + " failed to execute.");
                } else {
                    totalRowCount += rowCounts[j];
                }
            }
            return totalRowCount;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }

    public int updateBatch(List<DeviceConnection> entries) throws ServiceException {
        if (entries == null || entries.isEmpty()) {
            return 0;
        }
        int totalRowCount = 0;
        try {
            List<Object[]> args = new ArrayList<Object[]>(entries.size());
            for (DeviceConnection item : entries) {
                Object[] argItem = new Object[3];
                argItem[0] = item.getDisconnectedTime();
                argItem[1] = item.getConnectionDuration();
                argItem[2] = item.getConnectionId();
                args.add(argItem);
            }
            int[] rowCounts = jdbcTemplate.batchUpdate(UPDATE_SQL, args);

            for (int j = 0; j < rowCounts.length; j++) {
                if (rowCounts[j] == Statement.SUCCESS_NO_INFO) {
                    // do nothing
                } else if (rowCounts[j] == Statement.EXECUTE_FAILED) {
                    LOGGER.warn("The batched statement at index " + j + " failed to execute.");
                } else {
                    totalRowCount += rowCounts[j];
                }
            }
            return totalRowCount;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }

    public DeviceConnectionNetworkDTO selectNetworkingRateBystationId(String deviceId, String startTime, String endTime) throws ServiceException {
        try {
            return deviceConnectionMapper.selectNetworkingRateBystationId(deviceId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }

    public DeviceConnection selectNormalTimeByDeviceId(String deviceId, String startTime, String endTime) throws ServiceException {
        try {
            return deviceConnectionMapper.selectNormalTimeByDeviceId(deviceId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }

    public Integer deleteByDisconnectedTime(Date disconnectedTime, Integer limit) throws ServiceException {
        try {
            return deviceConnectionMapper.deleteByDisconnectedTime(disconnectedTime, limit);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }
}
