/*
 * ! CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-04-18 12:03:21
 */
package com.qhiot.transport.db.service;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.dto.DeviceDTO;
import com.qhiot.transport.db.dto.DeviceSimDTO;
import com.qhiot.transport.db.mapper.DeviceMapper;
import com.qhiot.transport.db.model.Device;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 业务层 <br/>
 * Table: device - 终端 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-04-18 12:03:21
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class DeviceService {

    private static final Logger LOGGER = LoggerFactory.getLogger(DeviceService.class);

    private static final String UPDATE_SQL = "UPDATE device SET last_active_time = ? WHERE device_id = ?";

    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    public List<Device> selectDeviceNetWorkingRateNormal() throws ServiceException {
        try {
            return deviceMapper.selectDeviceNetWorkingRateNormal();
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public void update2Null(String deviceId) throws ServiceException {
        try {
            deviceMapper.update2Null(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<Device> selectDeviceNetWorkingRateLowPower() throws ServiceException {
        try {
            return deviceMapper.selectDeviceNetWorkingRateLowPower();
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CachePut(value = "seem:transport:device", key = "#entity.deviceId")
    public Device insertSelective(Device entity) throws ServiceException {
        try {
            deviceMapper.insertSelective(entity);
            return entity;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int insertSelectiveList(List<Device> entitys) throws ServiceException {
        try {
            int count = 0;
            for (Device entity : entitys) {
                count += deviceMapper.insertSelective(entity);
            }
            return count;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }


    @CacheEvict(value = "seem:transport:device", key = "#entity.deviceId")
    public int updateSelective(Device entity) throws ServiceException {
        try {
            // 删除该终端相关推送 api 缓存
            CacheUtil.del("seem:transport:pushapi", entity.getDeviceId());

            return deviceMapper.updateByPrimaryKeySelective(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:device", key = "#deviceId")
    public int updateIccid(String deviceId, String iccid) throws ServiceException {
        try {
            Device device = new Device();
            device.setDeviceId(deviceId);
            device.setIccid(iccid);
            return deviceMapper.updateByPrimaryKeySelective(device);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:device", key = "#deviceId")
    public int updateLastActiveTime(String deviceId, Date lastActiveTime) throws ServiceException {
        try {
            Device device = new Device();
            device.setDeviceId(deviceId);
            device.setLastActiveTime(lastActiveTime);
            return deviceMapper.updateByPrimaryKeySelective(device);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:device", key = "#deviceId")
    public int updateNetworkingRate(String deviceId, Double networkingRate) throws ServiceException {
        try {
            Device device = new Device();
            device.setDeviceId(deviceId);
            device.setNetworkingRate(networkingRate);
            return deviceMapper.updateByPrimaryKeySelective(device);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int batchUpdateLastActiveTime(List<Device> entries) throws ServiceException {
        if (entries == null || entries.isEmpty()) {
            return 0;
        }
        int totalRowCount = 0;
        try {
            List<Object[]> args = new ArrayList<Object[]>(entries.size());
            for (Device item : entries) {
                Object[] argItem = new Object[2];
                argItem[0] = item.getLastActiveTime();
                argItem[1] = item.getDeviceId();
                args.add(argItem);
            }
            int[] rowCounts = jdbcTemplate.batchUpdate(UPDATE_SQL, args);
            for (int j = 0; j < rowCounts.length; j++) {
                if (rowCounts[j] == Statement.SUCCESS_NO_INFO) {
                    // do nothing
                } else if (rowCounts[j] == Statement.EXECUTE_FAILED) {
                    LOGGER.warn("The batched statement at index " + j + " failed to execute.");
                } else {
                    totalRowCount += rowCounts[j];
                }
            }
            return totalRowCount;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }

    @CacheEvict(value = "seem:transport:device", key = "#deviceId")
    public int deleteByPrimaryKey(String deviceId) throws ServiceException {
        try {
            // 删除该终端相关推送 api 缓存
            CacheUtil.del("seem:transport:pushapi", deviceId);

            return deviceMapper.deleteByPrimaryKey(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Cacheable(value = "seem:transport:device", key = "#deviceId")
    public Device selectByPrimaryKey(String deviceId) throws ServiceException {
        try {
            return deviceMapper.selectByPrimaryKey(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public boolean deviceRegistered(String deviceId) throws ServiceException {
        try {
            return deviceMapper.existsWithPrimaryKey(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public boolean hasDeviceForTenant(long tenantId, String deviceId) throws ServiceException {
        try {
            Example example = new Example(Device.class);
            example.and().andEqualTo("tenantId", tenantId).andEqualTo("deviceId", deviceId);

            return deviceMapper.selectCountByExample(example) > 0;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<Device> selectByExample(Example example) throws ServiceException {
        try {
            return deviceMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<Device> selectByExample1(Example example) throws ServiceException {
        try {
            return deviceMapper.selectByExample1(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<DeviceDTO> selectDTOByExample(Example example) throws ServiceException {
        try {
            return deviceMapper.selectDTOByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }


    /**
     * 导出--低功耗终端查询
     *
     * @param example
     * @return
     * @throws ServiceException
     */
    public List<DeviceDTO> selectRateListDTOLowPower(Example example, Long tenantId) throws ServiceException {
        try {
            return deviceMapper.selectRatePageDTOLowPower(example, tenantId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 导出--常供电终端查询
     *
     * @param example
     * @return
     * @throws ServiceException
     */
    public List<DeviceDTO> selectReteListDTONormal(Example example, Long tenantId) throws ServiceException {
        try {
            return deviceMapper.selectRatePageDTONormal(example, tenantId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<DeviceSimDTO> getDeviceSimDTOList() throws ServiceException {
        try {
            return deviceMapper.getDeviceSimDTOList();
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public DeviceDTO selectDTOByDeviceId(String deviceId) throws ServiceException {
        try {
            return deviceMapper.selectDTOByDeviceId(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public DeviceDTO selectDeviceDTOByDeviceId(String deviceId) throws ServiceException {
        try {
            return deviceMapper.selectDeviceDTOByDeviceId(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public DeviceDTO selectDTOByDeviceId4Send(String deviceId) throws ServiceException {
        try {
            return deviceMapper.selectDTOByDeviceId4Send(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }


    @CacheEvict(value = "seem:transport:device", key = "#entity.deviceId")
    public int updateByPrimaryKey(Device entity) throws ServiceException {
        try {
            // 删除该终端相关推送 api 缓存
            CacheUtil.del("seem:transport:pushapi", entity.getDeviceId());

            return deviceMapper.updateByPrimaryKey(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
