/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2021-07-06 03:46:47
 */
package com.qhiot.transport.db.service;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.mapper.InfluxSaveDurationConfigMapper;
import com.qhiot.transport.db.model.InfluxSaveDurationConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 业务层 <br/>
 * Table: influx_save_duration_config - 历史数据存储周期 <br/>
 *
 * @author 罗军(luoj@qh-iot.com)
 * @version 2021-07-06 03:46:47
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class InfluxSaveDurationConfigService {

    private static final String region = "seem:influx:save:duration";

    @Autowired
    private InfluxSaveDurationConfigMapper influxSaveDurationConfigMapper;

    public int deleteByPrimaryKey(Long tenantId) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix(region, String.valueOf(tenantId));
            return influxSaveDurationConfigMapper.deleteByPrimaryKey(tenantId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int saveOrUpdateSelective(InfluxSaveDurationConfig entity) throws ServiceException {
        CacheUtil.deleteByPrefix(region, String.valueOf(entity.getTenantId()));
        try {
            if (null == influxSaveDurationConfigMapper.selectByPrimaryKey(entity)) {
                return influxSaveDurationConfigMapper.insertSelective(entity);
            } else {
                return influxSaveDurationConfigMapper.updateByPrimaryKeySelective(entity);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<InfluxSaveDurationConfig> selectAll() throws ServiceException {
        try {
            return influxSaveDurationConfigMapper.selectAllTenantConfig();
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Cacheable(value = region, key = "#tenantId")
    public InfluxSaveDurationConfig selectByTenantId(Long tenantId) throws ServiceException {
        try {
            return influxSaveDurationConfigMapper.selectByPrimaryKey(tenantId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

}
