/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2019-08-07 02:09:06
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.service.BaseService;
import com.qhiot.transport.db.mapper.LibMonitoringNextNoMapper;
import com.qhiot.transport.db.model.LibMonitoringNextNo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 业务层 <br/>
 * Table: lib_monitoring_next_no - 设备库设备下一个序号 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2019-08-07 02:09:06
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class LibMonitoringNextNoService extends BaseService<LibMonitoringNextNo> {

    @Autowired
    private LibMonitoringNextNoMapper libMonitoringNextNoMapper;

    public int showNextNo(Long libEquipmentTypeId, boolean isType) throws ServiceException {
        LibMonitoringNextNo keyParam = new LibMonitoringNextNo();
        keyParam.setLibEquipmentMonitorId(libEquipmentTypeId);
        if (isType) {
            keyParam.setIsType(1);
        } else {
            keyParam.setIsType(0);
        }

        LibMonitoringNextNo libMonitoringNextNo = libMonitoringNextNoMapper.selectByPrimaryKey(keyParam);
        if (libMonitoringNextNo == null || libMonitoringNextNo.getNextNo() == null) {
            if (isType) {
                return 0;
            } else {
                return 1000;
            }

        } else {
            return libMonitoringNextNo.getNextNo();
        }
    }

    /**
     * @param libEquipmentTypeId
     * @param isType             true表示类型级别的测点序号分布
     * @return
     * @throws ServiceException
     */
    public int getNextNo(Long libEquipmentTypeId, boolean isType) throws ServiceException {
        LibMonitoringNextNo keyParam = new LibMonitoringNextNo();
        keyParam.setLibEquipmentMonitorId(libEquipmentTypeId);
        if (isType) {
            keyParam.setIsType(1);
        } else {
            keyParam.setIsType(0);
        }
        LibMonitoringNextNo libMonitoringNextNo = libMonitoringNextNoMapper.selectByPrimaryKey(keyParam);
        if (libMonitoringNextNo == null || libMonitoringNextNo.getNextNo() == null) {
            libMonitoringNextNo = new LibMonitoringNextNo();
            libMonitoringNextNo.setLibEquipmentMonitorId(libEquipmentTypeId);
            if (isType) {
                libMonitoringNextNo.setIsType(1);
                libMonitoringNextNo.setNextNo(0);
            } else {
                libMonitoringNextNo.setIsType(0);
                libMonitoringNextNo.setNextNo(1000);
            }
            saveOrUpdate(libMonitoringNextNo);
            if (isType) {
                return 0;
            } else {
                return 1000;
            }
        } else {
            int now = libMonitoringNextNo.getNextNo();
            libMonitoringNextNo.setNextNo(now + 1);
            saveOrUpdate(libMonitoringNextNo);
            return now + 1;
        }
    }
}
