/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-07-26 09:54:20
 */
package com.qhiot.transport.db.service;


import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.service.BaseService;
import com.qhiot.transport.db.dto.MonitoringDataDayDTO;
import com.qhiot.transport.db.dto.MonitoringStatsDataDayDTO;
import com.qhiot.transport.db.dto.MonitoringStatsIndexDataDTO;
import com.qhiot.transport.db.mapper.MonitoringDataDayMapper;
import com.qhiot.transport.db.model.MonitoringDataDay;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.Date;
import java.util.List;

/**
 * 业务层 <br/>
 * Table: monitoring_data_day - 测点日数据 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-07-26 09:54:20
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class MonitoringDataDayService extends BaseService<MonitoringDataDay> {

    @Autowired
    private MonitoringDataDayMapper monitoringDataDayMapper;

    public MonitoringDataDayDTO selectDTOExample(Example example) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectDTOByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDayDTO> selectMonitoringStatsDataHourAvg(Date startTime, Date endTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectMonitoringStatsDataHourAvg(startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }

    }

    public List<MonitoringDataDayDTO> selectMonitoringStatsDataHourOnTime(Date startTime, Date endTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectMonitoringStatsDataHourOnTime(startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDayDTO> selectMonitorngDataAccumulated() throws ServiceException {
        try {
            return monitoringDataDayMapper.selectMonitorngDataAccumulated();
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据时间范围查询最后一条测点数据
     *
     * @param deviceId     终端编号
     * @param indexId      监测指标编号
     * @param monitoringId 测点编号
     * @param smId         站点测点编号
     * @param startTime    开始时间
     * @param endTime      结束时间
     * @return
     * @throws ServiceException
     */
    public MonitoringDataDay selectMonitorngDataValue(String deviceId,
                                                      Long indexId,
                                                      Long monitoringId,
                                                      Long smId,
                                                      Date startTime,
                                                      Date endTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectMonitorngDataValue(deviceId, indexId, monitoringId, smId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据时间范围查询最先一条测点数据
     *
     * @param deviceId     终端编号
     * @param indexId      监测指标编号
     * @param monitoringId 测点编号
     * @param smId         站点测点编号
     * @param startTime    开始时间
     * @param endTime      结束时间
     * @return
     * @throws ServiceException
     */
    public MonitoringDataDay selectMonitorngDataValueFirst(String deviceId,
                                                           Long indexId,
                                                           Long monitoringId,
                                                           Long smId,
                                                           Date startTime,
                                                           Date endTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectMonitorngDataValueFirst(deviceId, indexId, monitoringId, smId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public MonitoringStatsDataDayDTO selectDTOAVGDay(Long smId, Long monitoringId, Date dayTimeStart, Date dayTimeEnd) throws ServiceException {
        try {
            MonitoringStatsDataDayDTO msddd = monitoringDataDayMapper.selectDTOAVGDay(monitoringId, smId, dayTimeStart, dayTimeEnd);
            return msddd;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public MonitoringDataDay selectYesterdayDataBySmId(Long smId) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectYesterdayDataBySmId(smId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDay> selectDataBySmIdAndDelayTime(Long smId) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectDataBySmIdAndDelayTime(smId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDay> selectBySmIdAndBackTime(Long smId, Long backTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectBySmIdAndBackTime(smId, backTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDay> selectBySmIdAndHourBackTime(Long smId, Long backTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectBySmIdAndHourBackTime(smId, backTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDay> selectBySmIdAndDayBackTime(Long smId, Long backTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectBySmIdAndDayBackTime(smId, backTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDay> selectBySmIdAndMonthBackTime(Long smId, Long backTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectBySmIdAndMonthBackTime(smId, backTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public MonitoringStatsIndexDataDTO selectDTOAVGDayByIndexId(Long stationId, Long indexId, Date startTime, Date endTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectDTOAVGDayByIndexId(stationId, indexId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringStatsIndexDataDTO> selectByIndexIdAndHourBackTime(Long stationId, Long indexId, Long backTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectByIndexIdAndHourBackTime(stationId, indexId, backTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringStatsIndexDataDTO> selectByIndexIdAndDayBackTime(Long stationId, Long indexId, Long backTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectByIndexIdAndDayBackTime(stationId, indexId, backTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringStatsIndexDataDTO> selectByIndexIdAndMonthBackTime(Long stationId, Long indexId, Long backTime) throws ServiceException {
        try {
            return monitoringDataDayMapper.selectByIndexIdAndMonthBackTime(stationId, indexId, backTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
