/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-04-20 10:15:54
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.service.BaseService;
import com.qhiot.transport.db.dto.MonitoringDataDTO;
import com.qhiot.transport.db.dto.MonitoringStatsDataHourDTO;
import com.qhiot.transport.db.dto.MonitoringStatsIndexDataDTO;
import com.qhiot.transport.db.mapper.MonitoringDataMapper;
import com.qhiot.transport.db.model.MonitoringData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * 业务层 <br/>
 * Table: monitoring_data - 测点数据 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-04-20 10:15:54
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class MonitoringDataService extends BaseService<MonitoringData> {

    @Autowired
    private MonitoringDataMapper monitoringDataMapper;

    /**
     * 插入或更新， 仅插入或更新非 NULL 字段
     *
     * @param entity 实体类
     * @return 影响的行数
     * @throws ServiceException 业务出错
     */
    @Override
    public int saveOrUpdateSelective(MonitoringData entity) throws ServiceException {
        try {
            return monitoringDataMapper.saveOrUpdateByPrimaryKeyAndDataTime(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public MonitoringStatsDataHourDTO selectDTOAVGHour(Long smId, Long monitoringId, Date dayTimeStart, Date dayTimeEnd) throws ServiceException {
        try {
            return monitoringDataMapper.selectDTOAVGHour(monitoringId, smId, dayTimeStart, dayTimeEnd);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public MonitoringDataDTO selectOneDTOByParam(String deviceId, Long indexId) throws ServiceException {
        try {
            return monitoringDataMapper.selectOneDTOByParam(deviceId, indexId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<MonitoringDataDTO> selectDataDTOList(List<String> deviceIdList, List<String> indexIdsList) throws ServiceException {
        try {
            return monitoringDataMapper.selectDataDTOList(deviceIdList, indexIdsList);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public MonitoringStatsIndexDataDTO selectDTOAVGHourByIndexId(Long stationId, Long indexId, Date startTime, Date endTime) throws ServiceException {
        try {
            return monitoringDataMapper.selectDTOAVGHourByIndexId(stationId, indexId, startTime, endTime);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
