/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-02-24 10:16:42
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.service.BaseService;
import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.db.dto.TransportAsciiParamDTO;
import com.qhiot.transport.db.mapper.TransportAsciiParamMapper;
import com.qhiot.transport.db.model.TransportAsciiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;

/**
 * 业务层 <br/>
 * Table: transport_ascii_param - 动态参数 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-02-24 10:16:42
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportAsciiParamService extends BaseService<TransportAsciiParam> {

    @Autowired
    private TransportAsciiParamMapper transportAsciiParamMapper;

    public int updateSynchronousByDeviceId(String deviceId) throws ServiceException {
        try {
            return transportAsciiParamMapper.updateSynchronousByDeviceId(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int updateByDeviceIdAndParamKey(TransportAsciiParam entity) throws ServiceException {
        try {
            return transportAsciiParamMapper.updateByDeviceIdAndParamKey(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据终端编号查找未经同步的参数(所有读的参数，和尚未同步的写参数)
     *
     * @param deviceId
     * @return
     * @throws ServiceException
     */
    public List<TransportAsciiParam> selectNotSynchronousAndReadableByDeviceId(String deviceId) throws ServiceException {
        try {
            return transportAsciiParamMapper.selectNotSynchronousAndReadableByDeviceId(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportAsciiParamDTO> selectDTOByDeviceId(String deviceId) throws ServiceException {
        try {
            return transportAsciiParamMapper.selectDTOByDeviceId(deviceId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int insertIfNotExists(String deviceId, String paramKey) throws ServiceException {
        try {
            Example example = new Example(TransportAsciiParam.class);
            example.createCriteria().andEqualTo("paramKey", paramKey).andEqualTo("deviceId", deviceId);
            if (transportAsciiParamMapper.selectCountByExample(example) == 0) {
                TransportAsciiParam transportAsciiParam = new TransportAsciiParam();
                transportAsciiParam.setDeviceId(deviceId);
                transportAsciiParam.setParamKey(paramKey);
                transportAsciiParam.setSetupTime(DateUtil.now());
                transportAsciiParam.setReadable((short) 0);
                return insert(transportAsciiParam);
            }
            return 0;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
