/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2018-05-18 11:45:50
 */
package com.qhiot.transport.db.service;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.mapper.TransportEquipmentConfigMapper;
import com.qhiot.transport.db.model.TransportEquipmentConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;

/**
 * 业务层 <br/>
 * Table: transport_equipment_config - 设备配置 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2018-05-18 11:45:50
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportEquipmentConfigService {

    @Autowired
    private TransportEquipmentConfigMapper transportEquipmentConfigMapper;

    public int saveOrUpdateSelective(TransportEquipmentConfig entity) throws ServiceException {
        CacheUtil.deleteByPrefix("seem:transport:datacollect:equipment", entity.getDeviceId());
        try {
            if (null == transportEquipmentConfigMapper.selectByPrimaryKey(entity)) {
                return transportEquipmentConfigMapper.insertSelective(entity);
            } else {
                return transportEquipmentConfigMapper.updateByPrimaryKeySelective(entity);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByDeviceId(String deviceId) throws ServiceException {
        CacheUtil.deleteByPrefix("seem:transport:datacollect:equipment", deviceId);
        try {
            Example example = new Example(TransportEquipmentConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            return transportEquipmentConfigMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportEquipmentConfig> selectByDeviceId(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportEquipmentConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            example.orderBy("configIndex");
            return transportEquipmentConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportEquipmentConfig> selectValidByDeviceId(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportEquipmentConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId).andEqualTo("valid", 1);
            example.orderBy("configIndex");
            return transportEquipmentConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Cacheable(value = "seem:transport:datacollect:equipment", key = "#deviceId+'_'+#configIndex")
    public TransportEquipmentConfig selectByPrimaryKey(Short configIndex, String deviceId) throws ServiceException {
        try {
            TransportEquipmentConfig key = new TransportEquipmentConfig();
            key.setDeviceId(deviceId);
            key.setConfigIndex(configIndex);
            return transportEquipmentConfigMapper.selectByPrimaryKey(key);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:datacollect:equipment", key = "#deviceId+'_'+#configIndex")
    public int deleteByPrimaryKey(Short configIndex, String deviceId) throws ServiceException {
        try {
            TransportEquipmentConfig key = new TransportEquipmentConfig();
            key.setDeviceId(deviceId);
            key.setConfigIndex(configIndex);
            return transportEquipmentConfigMapper.deleteByPrimaryKey(key);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
