/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2018-05-18 01:28:56
 */
package com.qhiot.transport.db.service;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.mapper.TransportEquipmentMonitoringMapper;
import com.qhiot.transport.db.model.TransportEquipmentMonitoring;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;

/**
 * 业务层 <br/>
 * Table: transport_equipment_monitoring - 设备测点配置 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2018-05-18 01:28:56
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportEquipmentMonitoringService {

    @Autowired
    private TransportEquipmentMonitoringMapper transportEquipmentMonitoringMapper;

    public List<TransportEquipmentMonitoring> selectByDeviceId(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportEquipmentMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            example.orderBy("configIndex");
            return transportEquipmentMonitoringMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByDeviceId(String deviceId) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:datacollect:config", deviceId);
            Example example = new Example(TransportEquipmentMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            example.orderBy("configIndex");
            return transportEquipmentMonitoringMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public TransportEquipmentMonitoring selectByPrimaryKey(String deviceId, Short configIndex) throws ServiceException {
        try {
            TransportEquipmentMonitoring key = new TransportEquipmentMonitoring();
            key.setDeviceId(deviceId);
            key.setConfigIndex(configIndex);
            return transportEquipmentMonitoringMapper.selectByPrimaryKey(key);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByPrimaryKey(String deviceId, Short configIndex) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:datacollect:config", deviceId);
            TransportEquipmentMonitoring key = new TransportEquipmentMonitoring();
            key.setDeviceId(deviceId);
            key.setConfigIndex(configIndex);
            return transportEquipmentMonitoringMapper.deleteByPrimaryKey(key);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int saveOrUpdateSelective(TransportEquipmentMonitoring entity) throws ServiceException {
        CacheUtil.deleteByPrefix("seem:transport:datacollect:config", entity.getDeviceId());
        try {
            if (null == transportEquipmentMonitoringMapper.selectByPrimaryKey(entity)) {
                return transportEquipmentMonitoringMapper.insertSelective(entity);
            } else {
                return transportEquipmentMonitoringMapper.updateByPrimaryKeySelective(entity);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Cacheable(value = "seem:transport:datacollect:config", key = "#deviceId+'_'+#registerType+'_'+#storeAddress")
    public TransportEquipmentMonitoring selectByRegister(String deviceId, byte registerType, int storeAddress) throws ServiceException {
        try {
            Example example = new Example(TransportEquipmentMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("storeRegisterType", registerType)
                    .andEqualTo("storeRegisterAddress", storeAddress);
            List<TransportEquipmentMonitoring> list = transportEquipmentMonitoringMapper.selectByExample(example);
            return (list != null && list.size() > 0) ? list.get(0) : null;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
