/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2019-08-07 02:09:06
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.service.BaseService;
import com.qhiot.transport.db.mapper.TransportNextMonitoringCodeMapper;
import com.qhiot.transport.db.mapper.TransportResourceCoreMapper;
import com.qhiot.transport.db.model.TransportNextMonitoringCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 业务层 <br/>
 * Table: lib_monitoring_next_no - 设备库设备下一个序号 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2019-08-07 02:09:06
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportNextMonitoringCodeService extends BaseService<TransportNextMonitoringCode> {

    @Autowired
    private TransportNextMonitoringCodeMapper transportNextMonitoringCodeMapper;
    @Autowired
    private TransportResourceCoreMapper transportResourceCoreMapper;

    public int showNextNo(String deviceId) throws ServiceException {
        TransportNextMonitoringCode param = new TransportNextMonitoringCode();
        param.setDeviceId(deviceId);
        TransportNextMonitoringCode transportNextMonitoringCode = transportNextMonitoringCodeMapper.selectByPrimaryKey(param);
        //补充逻辑，查询是否存在最大值
        Integer coreMaxMonitoringCode = transportResourceCoreMapper.selectMaxMonitorCodeByDeviceId(deviceId);
        if (transportNextMonitoringCode == null || transportNextMonitoringCode.getNextMonitoringCode() == null) {
            if (coreMaxMonitoringCode != null && coreMaxMonitoringCode != 0) {
                transportNextMonitoringCode = new TransportNextMonitoringCode();
                transportNextMonitoringCode.setDeviceId(deviceId);
                transportNextMonitoringCode.setNextMonitoringCode(coreMaxMonitoringCode + 1);
                saveOrUpdate(transportNextMonitoringCode);
                return coreMaxMonitoringCode + 1;
            }
            return 0;
        } else {
            if (coreMaxMonitoringCode != null && coreMaxMonitoringCode >= transportNextMonitoringCode.getNextMonitoringCode()) {
                transportNextMonitoringCode = new TransportNextMonitoringCode();
                transportNextMonitoringCode.setDeviceId(deviceId);
                transportNextMonitoringCode.setNextMonitoringCode(coreMaxMonitoringCode + 1);
                saveOrUpdate(transportNextMonitoringCode);
                return coreMaxMonitoringCode + 1;
            }
            return transportNextMonitoringCode.getNextMonitoringCode();
        }
    }

    /**
     * @return
     * @throws ServiceException
     */
    public int getNextNo(String deviceId) throws ServiceException {
        TransportNextMonitoringCode param = new TransportNextMonitoringCode();
        param.setDeviceId(deviceId);
        //补充逻辑，查询是否存在最大值
        Integer coreMaxMonitoringCode = transportResourceCoreMapper.selectMaxMonitorCodeByDeviceId(deviceId);
        TransportNextMonitoringCode transportNextMonitoringCode = transportNextMonitoringCodeMapper.selectByPrimaryKey(param);
        if (transportNextMonitoringCode == null || transportNextMonitoringCode.getNextMonitoringCode() == null) {
            transportNextMonitoringCode = new TransportNextMonitoringCode();
            transportNextMonitoringCode.setDeviceId(deviceId);
            if (coreMaxMonitoringCode != null && coreMaxMonitoringCode != 0) {
                transportNextMonitoringCode.setNextMonitoringCode(coreMaxMonitoringCode + 2);
                saveOrUpdate(transportNextMonitoringCode);
                return coreMaxMonitoringCode + 1;
            }
            transportNextMonitoringCode.setNextMonitoringCode(1);
            saveOrUpdate(transportNextMonitoringCode);
            return 0;
        } else {
            int now = transportNextMonitoringCode.getNextMonitoringCode();
            if (coreMaxMonitoringCode != null && coreMaxMonitoringCode >= now) {
                transportNextMonitoringCode.setNextMonitoringCode(coreMaxMonitoringCode + 2);
                saveOrUpdate(transportNextMonitoringCode);
                return coreMaxMonitoringCode + 1;
            }
            transportNextMonitoringCode.setNextMonitoringCode(now + 1);
            saveOrUpdate(transportNextMonitoringCode);
            return now;
        }
    }
}
