/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2019-08-07 02:09:06
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.service.BaseService;
import com.qhiot.transport.db.mapper.TransportNextSmCodeMapper;
import com.qhiot.transport.db.mapper.TransportResourceCoreMapper;
import com.qhiot.transport.db.model.TransportNextSmCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 业务层 <br/>
 * Table: lib_monitoring_next_no - 设备库设备下一个序号 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2019-08-07 02:09:06
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportNextSmCodeService extends BaseService<TransportNextSmCode> {

    @Autowired
    private TransportNextSmCodeMapper transportNextSmCodeMapper;

    @Autowired
    private TransportResourceCoreMapper transportResourceCoreMapper;

    public int showNextNo(Long stationId) throws ServiceException {
        TransportNextSmCode param = new TransportNextSmCode();
        param.setStationId(stationId);
        TransportNextSmCode transportNextSmCode = transportNextSmCodeMapper.selectByPrimaryKey(param);

        //补充逻辑，查询是否存在最大值
        Integer coreMaxSmCode = transportResourceCoreMapper.selectMaxSmCodeByStationId(stationId);
        if (transportNextSmCode == null || transportNextSmCode.getNextSmCode() == null) {
            if (coreMaxSmCode != null && coreMaxSmCode != 0) {
                transportNextSmCode = new TransportNextSmCode();
                transportNextSmCode.setStationId(stationId);
                transportNextSmCode.setNextSmCode(coreMaxSmCode + 1);
                saveOrUpdate(transportNextSmCode);
                return coreMaxSmCode + 1;
            }
            return 0;
        } else {
            if (coreMaxSmCode != null && coreMaxSmCode >= transportNextSmCode.getNextSmCode()) {
                transportNextSmCode = new TransportNextSmCode();
                transportNextSmCode.setStationId(stationId);
                transportNextSmCode.setNextSmCode(coreMaxSmCode + 1);
                saveOrUpdate(transportNextSmCode);
                return coreMaxSmCode + 1;
            }
            return transportNextSmCode.getNextSmCode();
        }
    }

    /**
     * @return
     * @throws ServiceException
     */
    public int getNextNo(Long stationId) throws ServiceException {
        TransportNextSmCode param = new TransportNextSmCode();
        param.setStationId(stationId);
        //补充逻辑，查询是否存在最大值
        Integer coreMaxSmCode = transportResourceCoreMapper.selectMaxSmCodeByStationId(stationId);
        TransportNextSmCode transportNextSmCode = transportNextSmCodeMapper.selectByPrimaryKey(param);
        if (transportNextSmCode == null || transportNextSmCode.getNextSmCode() == null) {
            transportNextSmCode = new TransportNextSmCode();
            transportNextSmCode.setStationId(stationId);

            if (coreMaxSmCode != null && coreMaxSmCode != 0) {
                transportNextSmCode.setNextSmCode(coreMaxSmCode + 2);
                saveOrUpdate(transportNextSmCode);
                return coreMaxSmCode + 1;
            }
            transportNextSmCode.setNextSmCode(1);
            saveOrUpdate(transportNextSmCode);
            return 0;
        } else {
            int now = transportNextSmCode.getNextSmCode();
            if (coreMaxSmCode != null && coreMaxSmCode >= now) {
                transportNextSmCode.setNextSmCode(coreMaxSmCode + 2);
                saveOrUpdate(transportNextSmCode);
                return coreMaxSmCode + 1;
            }
            transportNextSmCode.setNextSmCode(now + 1);
            saveOrUpdate(transportNextSmCode);
            return now;
        }
    }
}
