/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-03-29 04:13:28
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.model.TransportResourceConfig;
import com.qhiot.transport.db.model.TransportResourceMonitoring;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 业务层 <br/>
 * Table: transport_resource_config - 终端资源配置 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-03-29 04:13:28
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportResourceConfigCopyService {

    @Autowired
    private TransportResourceConfigService transportResourceConfigService;
    @Autowired
    private TransportResourceMonitoringService transportResourceMonitoringService;

    public void copyResourceConfigAndMonitoring(String deviceIdFrom, String deviceIdTo) throws ServiceException {
        try {
            List<TransportResourceConfig> configList = transportResourceConfigService.selectAllByDeviceId(deviceIdFrom);
            if (configList.isEmpty()) {
                throw new ServiceException("源终端配置信息不存在");
            }
            transportResourceConfigService.deleteByDeviceId(deviceIdTo);
            for (TransportResourceConfig config : configList) {
                config.setDeviceId(deviceIdTo);
                // 配置有效的才需要下发更新
                config.setNeedsUpdate(Short.valueOf(config.getValid()));
                transportResourceConfigService.saveOrUpdateSelective(config);
            }

            List<TransportResourceMonitoring> monitoringList = transportResourceMonitoringService.selectByDevice(deviceIdFrom);
            transportResourceMonitoringService.deleteByDevice(deviceIdTo);
            for (TransportResourceMonitoring monitoring : monitoringList) {
                monitoring.setMonitoringId(null);
                monitoring.setDeviceId(deviceIdTo);
                monitoring.setAdvStationMonitorId(null);
                transportResourceMonitoringService.saveOrUpdateSelective(monitoring);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
