/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-03-29 04:13:28
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.mapper.TransportResourceConfigOneStripMapper;
import com.qhiot.transport.db.model.TransportResourceConfig;
import com.qhiot.transport.db.model.TransportResourceConfigOneStrip;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 业务层 <br/>
 * Table: transport_resource_config - 终端资源配置 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-03-29 04:13:28
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportResourceConfigOneStripService {

    @Autowired
    private TransportResourceConfigOneStripMapper transportResourceConfigMapper;

    /**
     * 按主键删除 （终端编号 + 配置序号）
     */
    public int deleteByPrimaryKey(String deviceId, short configIndex) throws ServiceException {
        try {
//            redisUtil.deleteByPrefix("seem:transport:resource:config:" + deviceId);

            Example example = new Example(TransportResourceConfigOneStrip.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("configIndex", configIndex);
            return transportResourceConfigMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByDeviceId(String deviceId) throws ServiceException {
        try {
//            redisUtil.deleteByPrefix("seem:transport:resource:config:" + deviceId);

            Example example = new Example(TransportResourceConfigOneStrip.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            return transportResourceConfigMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 按页删除
     *
     * @param page 0-2
     */
    public int deleteByPage(String deviceId, short page) throws ServiceException {
        try {
//            redisUtil.deleteByPrefix("seem:transport:resource:config:" + deviceId);

            Example example = new Example(TransportResourceConfigOneStrip.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andBetween("configIndex", page * 10, page * 10 + 9);
            return transportResourceConfigMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int saveOrUpdateSelective(TransportResourceConfigOneStrip entity) throws ServiceException {
//        redisUtil.deleteByPrefix("seem:transport:resource:config:" + entity.getDeviceId());
        try {
            if (null == transportResourceConfigMapper.selectByPrimaryKey(entity)) {
                return transportResourceConfigMapper.insertSelective(entity);
            } else {
                return transportResourceConfigMapper.updateByPrimaryKeySelective(entity);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据TransportResourceConfig插入TransportResourceConfigOneStrip
     *
     * @param transportResourceConfig
     * @return
     * @throws ServiceException
     */
    public int saveOrUpdateSelective(TransportResourceConfig transportResourceConfig) throws ServiceException {
        TransportResourceConfigOneStrip entity = new TransportResourceConfigOneStrip();
        entity.setConfigIndex(transportResourceConfig.getConfigIndex());
        entity.setDeviceId(transportResourceConfig.getDeviceId());
        entity.setConfigName(transportResourceConfig.getConfigName());
        entity.setValid(transportResourceConfig.getValid());
        entity.setProtocol(transportResourceConfig.getProtocol());
        entity.setResource(transportResourceConfig.getResource());
        entity.setCommPort(transportResourceConfig.getCommPort());
        entity.setBaudRate(transportResourceConfig.getBaudRate());
        entity.setDataBit(transportResourceConfig.getDataBit());
        entity.setStopBit(transportResourceConfig.getStopBit());
        entity.setPeriod(transportResourceConfig.getPeriod());
        entity.setAddress(transportResourceConfig.getAddress());
        entity.setFunctionCode(transportResourceConfig.getFunctionCode());
        entity.setStartingAddr(transportResourceConfig.getStartingAddr());
        entity.setMethod(transportResourceConfig.getMethod());
        entity.setDataQuantity(transportResourceConfig.getDataQuantity());
        entity.setCommandLength(transportResourceConfig.getCommandLength());
        entity.setCommand(transportResourceConfig.getCommand());
        entity.setRegisterType(transportResourceConfig.getRegisterType());
        entity.setStoreAddress(transportResourceConfig.getStoreAddress());
        entity.setUpperBound(transportResourceConfig.getUpperBound());
        entity.setUpperBoundEnabled(transportResourceConfig.getUpperBoundEnabled());
        entity.setLowerBound(transportResourceConfig.getLowerBound());
        entity.setLowerBoundEnabled(transportResourceConfig.getLowerBoundEnabled());
        entity.setNeedsUpdate(transportResourceConfig.getNeedsUpdate());
        entity.setAmmeterNum(transportResourceConfig.getAmmeterNum());
        entity.setResourceTempId(transportResourceConfig.getResourceTempId());
        entity.setDevicePeripheralId(transportResourceConfig.getDevicePeripheralId());
        try {
            if (null == transportResourceConfigMapper.selectByPrimaryKey(entity)) {
                return transportResourceConfigMapper.insertSelective(entity);
            } else {
                return transportResourceConfigMapper.updateByPrimaryKeySelective(entity);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int updateSelective(TransportResourceConfigOneStrip entity) throws ServiceException {
//        redisUtil.deleteByPrefix("seem:transport:resource:config:" + entity.getDeviceId());
        try {
            return transportResourceConfigMapper.updateByPrimaryKeySelective(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfigOneStrip> selectByPage(String deviceId, short page) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfigOneStrip.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andBetween("configIndex", page * 10, page * 10 + 9);
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfigOneStrip> selectAllByDeviceId(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfigOneStrip.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfigOneStrip> selectNeedsUpdate(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfigOneStrip.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("needsUpdate", 1);
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfigOneStrip> selectValid(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfigOneStrip.class);
            example.createCriteria().andEqualTo("deviceId", deviceId).andEqualTo("valid", 1);
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    //    @Cacheable(value = "seem:transport:resource:config", key = "#deviceId+'_'+#registerType+'_'+#storeAddress")
    public TransportResourceConfigOneStrip selectByRegister(String deviceId, byte registerType, int storeAddress) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfigOneStrip.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("registerType", registerType)
                    .andEqualTo("storeAddress", storeAddress);
            List<TransportResourceConfigOneStrip> list = transportResourceConfigMapper.selectByExample(example);
            return (list != null && list.size() > 0) ? list.get(0) : null;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public TransportResourceConfigOneStrip selectByPrimaryKey(TransportResourceConfigOneStrip entity) throws ServiceException {
        try {
            return transportResourceConfigMapper.selectByPrimaryKey(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public TransportResourceConfigOneStrip selectByPrimaryKey(String deviceId, short configIndex) throws ServiceException {
        try {
            TransportResourceConfigOneStrip config = new TransportResourceConfigOneStrip();
            config.setDeviceId(deviceId);
            config.setConfigIndex(configIndex);
            return selectByPrimaryKey(config);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据Example的条件进行更新实体中的非null字段
     *
     * @param transportResourceConfig 实体
     * @param example                 条件
     * @return 一个分页的实体集合
     * @throws ServiceException 业务出错
     */
    public int updateByExampleSelective(TransportResourceConfigOneStrip transportResourceConfig, Example example) throws ServiceException {
        try {
            return transportResourceConfigMapper.updateByExampleSelective(transportResourceConfig, example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据条件查询已经使用了的存放地址
     *
     * @param registerType 寄存器类型
     * @param deviceId     网关编号
     *                     Map<"地址","占用几位">
     * @return 一个分页的实体集合
     * @throws ServiceException 业务出错
     */
    public Map<Integer, Integer> selectUsedStoreAddress(String deviceId, Byte registerType) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfigOneStrip.class);
            example.createCriteria().andEqualTo("deviceId", deviceId).andEqualTo("registerType", registerType);
            List<TransportResourceConfigOneStrip> list = transportResourceConfigMapper.selectByExample(example);
            Map<Integer, Integer> map = new HashMap<Integer, Integer>();
            for (TransportResourceConfigOneStrip transportResourceConfig : list) {
                Integer gap = transportResourceConfig.getMethod() > 6 ? 2 : 1;
                Integer storAddress = transportResourceConfig.getStoreAddress();
                int dataQuantity = transportResourceConfig.getDataQuantity();
                //占位换算
                for (int i = 0; i < dataQuantity; i++) {
                    map.put(storAddress + gap * i, gap);
                }
            }
            return map;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfigOneStrip> selectByExample(Example example) throws ServiceException {
        try {
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
