/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-03-29 04:13:28
 */
package com.qhiot.transport.db.service;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.mapper.TransportResourceConfigMapper;
import com.qhiot.transport.db.model.TransportResourceConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 业务层 <br/>
 * Table: transport_resource_config - 终端资源配置 <br/>
 * 注意：如果需要新增insert,save,uodate,delete相关的方法，请检查ConfigServiceAspect的AOP拦截
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-03-29 04:13:28
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportResourceConfigService {

    @Autowired
    private TransportResourceConfigMapper transportResourceConfigMapper;

    /**
     * 按主键删除 （终端编号 + 配置序号）
     */
    public int deleteByPrimaryKey(String deviceId, short configIndex) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:config", deviceId);

            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("configIndex", configIndex);
            return transportResourceConfigMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByDeviceId(String deviceId) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:config", deviceId);

            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            return transportResourceConfigMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 按页删除
     *
     * @param page 0-2
     */
    public int deleteByPage(String deviceId, int page) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:config", deviceId);

            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andBetween("configIndex", page * 10, page * 10 + 9);
            return transportResourceConfigMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int saveOrUpdateSelective(TransportResourceConfig entity) throws ServiceException {
        CacheUtil.deleteByPrefix("seem:transport:resource:config", entity.getDeviceId());
        try {
            if (null == transportResourceConfigMapper.selectByPrimaryKey(entity)) {
                return transportResourceConfigMapper.insertSelective(entity);
            } else {
                return transportResourceConfigMapper.updateByPrimaryKeySelective(entity);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int updateSelective(TransportResourceConfig entity) throws ServiceException {
        CacheUtil.deleteByPrefix("seem:transport:resource:config", entity.getDeviceId());
        try {
            return transportResourceConfigMapper.updateByPrimaryKeySelective(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfig> selectByPage(String deviceId, short page) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andBetween("configIndex", page * 10, page * 10 + 9);
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfig> selectAllByDeviceId(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId).andNotEqualTo("configIndex", -1L);
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfig> selectNeedsUpdate(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("needsUpdate", 1).andNotEqualTo("configIndex", -1L);
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfig> selectValid(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId).andEqualTo("valid", 1).andNotEqualTo("configIndex", -1L);
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public TransportResourceConfig selectValid(String deviceId, short configIndex) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId).andEqualTo("valid", 1).andEqualTo("configIndex", configIndex);
            List<TransportResourceConfig> list = transportResourceConfigMapper.selectByExample(example);
            return (list != null && list.size() > 0) ? list.get(0) : null;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Cacheable(value = "seem:transport:resource:config", key = "#deviceId+'_'+#registerType+'_'+#storeAddress")
    public TransportResourceConfig selectByRegister(String deviceId, byte registerType, int storeAddress) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("registerType", registerType)
                    .andEqualTo("storeAddress", storeAddress)
                    .andEqualTo("valid", 1)
                    .andNotEqualTo("configIndex", -1L);
            List<TransportResourceConfig> list = transportResourceConfigMapper.selectByExample(example);
            return (list != null && list.size() > 0) ? list.get(0) : null;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public TransportResourceConfig selectByPrimaryKey(TransportResourceConfig entity) throws ServiceException {
        try {
            return transportResourceConfigMapper.selectByPrimaryKey(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public TransportResourceConfig selectByPrimaryKey(String deviceId, short configIndex) throws ServiceException {
        try {
            TransportResourceConfig config = new TransportResourceConfig();
            config.setDeviceId(deviceId);
            config.setConfigIndex(configIndex);
            return selectByPrimaryKey(config);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据Example的条件进行更新实体中的非null字段
     *
     * @param transportResourceConfig 实体
     * @param example                 条件
     * @return 一个分页的实体集合
     * @throws ServiceException 业务出错
     */
    public int updateByExampleSelective(TransportResourceConfig transportResourceConfig, Example example) throws ServiceException {
        try {
            return transportResourceConfigMapper.updateByExampleSelective(transportResourceConfig, example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据条件查询已经使用了的存放地址
     *
     * @param registerType 寄存器类型
     * @param deviceId     网关编号
     *                     Map<"地址","占用几位">
     * @return 一个分页的实体集合
     * @throws ServiceException 业务出错
     */
    public Map<Integer, Integer> selectUsedStoreAddress(String deviceId, Byte registerType) throws ServiceException {
        try {
            Example example = new Example(TransportResourceConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId).andEqualTo("registerType", registerType).andNotEqualTo("configIndex", -1L);
            List<TransportResourceConfig> list = transportResourceConfigMapper.selectByExample(example);
            Map<Integer, Integer> map = new HashMap<Integer, Integer>(16);
            for (TransportResourceConfig transportResourceConfig : list) {
                byte method = transportResourceConfig.getMethod();
                int dataQuantity = transportResourceConfig.getDataQuantity();
                Integer storAddress = transportResourceConfig.getStoreAddress();
                //位存

                if (registerType == 1 || registerType == 2) {
                    int gap = 1;
                    for (int i = 0; i < dataQuantity; i++) {
                        map.put(storAddress + gap * i, gap);
                    }
                } else {
                    //字存
                    //位解
                    if (method == 0) {
                        int y = dataQuantity % 16;
                        int x = dataQuantity / 16;
                        int length = x + (y != 0 ? 1 : 0);
                        int gap = 1;
                        for (int i = 0; i < length; i++) {
                            map.put(storAddress + gap * i, gap);
                        }
                    } else if (method <= 6) {
                        //占一位
                        int gap = 1;
                        for (int i = 0; i < dataQuantity; i++) {
                            map.put(storAddress + gap * i, gap);
                        }
                    } else {
                        //站2位
                        int gap = 2;
                        for (int i = 0; i < dataQuantity; i++) {
                            map.put(storAddress + gap * i, gap);
                        }
                    }
                }
            }
            return map;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceConfig> selectByExample(Example example) throws ServiceException {
        try {
            return transportResourceConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
