/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2019-06-11 05:48:13
 */
package com.qhiot.transport.db.service;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.bean.Pager;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.sql.mybatis.plugin.PageHelper;
import com.qhiot.transport.db.dto.TransportResourceCoreDTO;
import com.qhiot.transport.db.mapper.DbStationMonitoringMapper;
import com.qhiot.transport.db.mapper.TransportResourceCoreMapper;
import com.qhiot.transport.db.model.DbStationMonitoring;
import com.qhiot.transport.db.model.TransportResourceCore;
import com.qhiot.transport.db.model.TransportResourceMonitoring;
import org.apache.ibatis.session.RowBounds;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;

/**
 * 业务层 <br/>
 * Table: transport_resource_core - 配置信息核心表，包含配置信息的各种重要信息 <br/>
 * 这个
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2019-06-11 05:48:13
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportResourceCore4dbService {
    @Autowired
    private TransportResourceCoreMapper transportResourceCoreMapper;
    @Autowired
    private DbStationMonitoringMapper dbStationMonitoringMapper;
    @Autowired
    private TransportResourceMonitoringService transportResourceMonitoringService;

    public TransportResourceCore selectBySmId(Long smId) {
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("smId", smId);
        List<TransportResourceCore> coreList = transportResourceCoreMapper.selectByExample(example);
        if (coreList != null && !coreList.isEmpty()) {
            return coreList.get(0);
        }
        return null;
    }

    public TransportResourceCore selectByMonitoringId(Long monitoringId) {
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("monitoringId", monitoringId);
        List<TransportResourceCore> coreList = transportResourceCoreMapper.selectByExample(example);
        if (coreList != null && !coreList.isEmpty()) {
            return coreList.get(0);
        }
        return null;
    }

    public List<TransportResourceCore> selectByStationId(Long stationId) {
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("stationId", stationId);
        return transportResourceCoreMapper.selectByExample(example);
    }

    public TransportResourceCore selectByMonitoringIdIfNotExsistCreate(Long monitoringId) throws ServiceException {
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("monitoringId", monitoringId);
        List<TransportResourceCore> coreList = transportResourceCoreMapper.selectByExample(example);
        if (coreList != null && !coreList.isEmpty()) {
            return coreList.get(0);
        } else {
            try {
                TransportResourceMonitoring transportResourceMonitoring = transportResourceMonitoringService.selectByPrimaryKey(monitoringId);
                transportResourceMonitoringService.saveOrUpdateSelective(transportResourceMonitoring);
                coreList = transportResourceCoreMapper.selectByExample(example);
                if (coreList != null && !coreList.isEmpty()) {
                    return coreList.get(0);
                }
            } catch (Exception e) {
                return null;
            }

        }
        return null;
    }

    public TransportResourceCore selectBySMIdIfNotExsistCreate(Long smId) throws ServiceException {
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("smId", smId);

        List<TransportResourceCore> coreList = transportResourceCoreMapper.selectByExample(example);
        if (coreList != null && !coreList.isEmpty()) {
            return coreList.get(0);
        } else {
            try {
                DbStationMonitoring dbStationMonitoring = dbStationMonitoringMapper.selectByPrimaryKey(smId);
                if (dbStationMonitoring == null) {
                    return null;
                }
                TransportResourceMonitoring transportResourceMonitoring = transportResourceMonitoringService.selectByPrimaryKey(dbStationMonitoring.getMonitoringId());
                if (transportResourceMonitoring != null) {
                    transportResourceMonitoringService.saveOrUpdateSelective(transportResourceMonitoring);
                    coreList = transportResourceCoreMapper.selectByExample(example);
                    if (coreList != null && !coreList.isEmpty()) {
                        return coreList.get(0);
                    }
                } else {
                    return null;
                }
            } catch (Exception e) {
                return null;
            }

        }
        return null;
    }

    public TransportResourceCore selectByDeviceIdAndConfigIndexAndDataIndex(String deviceId, Short configIndex, Short dataIndex) throws ServiceException {
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("deviceId", deviceId)
                .andEqualTo("configIndex", configIndex).andEqualTo("dataIndex", dataIndex);
        List<TransportResourceCore> transportResourceCores = transportResourceCoreMapper.selectByExample(example);
        if (null != transportResourceCores && !transportResourceCores.isEmpty()) {
            return transportResourceCores.get(0);
        }
        return null;
    }

    public int deleteByDeviceId(String deviceId) {
        CacheUtil.deleteByPrefix("seem:transport:resource:core", deviceId);
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("deviceId", deviceId);
        return transportResourceCoreMapper.deleteByExample(example);
    }

    public int deleteByMonitoringId(Long monitoringId) {
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("monitoringId", monitoringId);
        return transportResourceCoreMapper.deleteByExample(example);
    }

    public int deleteByDeviceIdAndConfigIndex(String deviceId, Short configIndex) {
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("deviceId", deviceId)
                .andEqualTo("configIndex", configIndex);
        CacheUtil.deleteByPrefix("seem:transport:resource:core", deviceId);
        return transportResourceCoreMapper.deleteByExample(example);
    }

    public int deleteByPage(String deviceId, int page) {
        CacheUtil.deleteByPrefix("seem:transport:resource:core", deviceId);
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("deviceId", deviceId)
                .andBetween("configIndex", page * 10, page * 10 + 9);
        return transportResourceCoreMapper.deleteByExample(example);
    }

    public int updateByDeviceAndConfigIndexOrDataIndex(TransportResourceCore entity) throws ServiceException {
        Example example = new Example(TransportResourceCore.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("deviceId", entity.getDeviceId())
                .andEqualTo("configIndex", entity.getConfigIndex());
        if (entity.getDataIndex() != null) {
            criteria.andEqualTo("dataIndex", entity.getDataIndex());
        }
        CacheUtil.deleteByPrefix("seem:transport:resource:core", entity.getDeviceId());
        return transportResourceCoreMapper.updateByExampleSelective(entity, example);
    }

    public List<TransportResourceCoreDTO> selectDTOByExample(Example example) throws ServiceException {
        try {
            return transportResourceCoreMapper.selectDTOByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }

    }

    public TransportResourceCoreDTO selectOneDTOByExample(Example example) throws ServiceException {
        try {
            List<TransportResourceCoreDTO> transportResourceCoreDTOS = transportResourceCoreMapper.selectDTOByExample(example);
            if (transportResourceCoreDTOS != null && !transportResourceCoreDTOS.isEmpty()) {
                return transportResourceCoreDTOS.get(0);
            }
            return null;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Cacheable(value = "seem:transport:resource:core", key = "#deviceId+'_'+#monitoringId")
    public TransportResourceCore selectByDeviceIdAndMonitoringId(String deviceId, Long monitoringId) throws ServiceException {
        Example example = new Example(TransportResourceCore.class);
        example.createCriteria().andEqualTo("deviceId", deviceId)
                .andEqualTo("monitoringId", monitoringId);
        return selectOneByExample(example);
    }

//    /**
//     * 插入，仅保存非 NULL 字段
//     *
//     * @param entity 实体类
//     * @return 影响的行数
//     * @throws ServiceException 业务出错
//     */
//    public int insertSelective(TransportResourceCore entity) throws ServiceException {
//        try {
//            return transportResourceCoreMapper.insertSelective(entity);
//        } catch (Exception e) {
//            throw new ServiceException(e.getMessage(), e);
//        }
//    }

    /**
     * 根据主键更新
     *
     * @param entity 实体类
     * @return 影响的行数
     * @throws ServiceException 业务出错
     */
    public int update(TransportResourceCore entity) throws ServiceException {
        try {
            return transportResourceCoreMapper.updateByPrimaryKey(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 根据主键更新， 仅更新非 NULL 字段
     *
     * @param entity 实体类
     * @return 影响的行数
     * @throws ServiceException 业务出错
     */
    public int updateSelective(TransportResourceCore entity) throws ServiceException {
        try {
            return transportResourceCoreMapper.updateByPrimaryKeySelective(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int updateByExampleSelective(TransportResourceCore entity, Object example) throws ServiceException {
        try {
            return transportResourceCoreMapper.updateByExampleSelective(entity, example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 插入或更新
     *
     * @param entity 实体类
     * @return 影响的行数
     * @throws ServiceException 业务出错
     */
    @CacheEvict(value = "seem:transport:resource:core", key = "#entity.deviceId+'_'+#entity.monitoringId")
    public int saveOrUpdate(TransportResourceCore entity) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:core", entity.getDeviceId());
            if (!transportResourceCoreMapper.existsWithPrimaryKey(entity)) {
                return transportResourceCoreMapper.insert(entity);
            } else {
                return transportResourceCoreMapper.updateByPrimaryKey(entity);
            }
        } catch (Exception e) {
            if (e instanceof DuplicateKeyException) {
                Example example = new Example(TransportResourceCore.class);
                example.createCriteria().andEqualTo("deviceId", entity.getDeviceId())
                        .andEqualTo("configIndex", entity.getConfigIndex())
                        .andEqualTo("dataIndex", entity.getDataIndex());
                transportResourceCoreMapper.deleteByExample(example);
                if (!transportResourceCoreMapper.existsWithPrimaryKey(entity)) {
                    return transportResourceCoreMapper.insert(entity);
                } else {
                    return transportResourceCoreMapper.updateByPrimaryKey(entity);
                }
            } else {
                throw new ServiceException(e.getMessage(), e);
            }
        }
    }

    /**
     * 插入或更新， 仅插入或更新非 NULL 字段
     *
     * @param entity 实体类
     * @return 影响的行数
     * @throws ServiceException 业务出错
     */
    @CacheEvict(value = "seem:transport:resource:core", key = "#entity.deviceId+'_'+#entity.monitoringId")
    public int saveOrUpdateSelective(TransportResourceCore entity) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:core", entity.getDeviceId());
            if (null == transportResourceCoreMapper.selectByPrimaryKey(entity)) {
                return transportResourceCoreMapper.insertSelective(entity);
            } else {
                return transportResourceCoreMapper.updateByPrimaryKeySelective(entity);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 删除
     *
     * @param entity 实体类
     * @return 影响的行数
     * @throws ServiceException 业务出错
     */
    public int delete(TransportResourceCore entity) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:resource:core", entity.getDeviceId());
            return transportResourceCoreMapper.deleteByPrimaryKey(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 删除
     *
     * @param key 主键
     * @return 影响的行数
     * @throws ServiceException 业务出错
     */
    public int deleteByPrimaryKey(Object key) throws ServiceException {
        try {
            return transportResourceCoreMapper.deleteByPrimaryKey(key);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public List<TransportResourceCore> selectAll() throws ServiceException {
        try {
            return transportResourceCoreMapper.selectAll();
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 按条件查询
     *
     * @param example 查询条件
     * @return 符合条件的实体集合
     * @throws ServiceException 业务出错
     */
    public List<TransportResourceCore> selectByExample(Example example) throws ServiceException {
        try {
            return transportResourceCoreMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 按条件查询单条
     *
     * @param example 查询条件
     * @return 第一个符合条件的实体
     * @throws ServiceException 业务出错
     */
    public TransportResourceCore selectOneByExample(Example example) throws ServiceException {
        try {
            List<TransportResourceCore> list = transportResourceCoreMapper.selectByExampleAndRowBounds(example, new RowBounds(0, 1));
            if (null != list && !list.isEmpty()) {
                return list.get(0);
            } else {
                return null;
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 分页查询
     *
     * @param pageNum  页码
     * @param pageSize 每页的大小
     * @return 一个分页的实体集合
     * @throws ServiceException 业务出错
     */
    public Pager<TransportResourceCore> selectPage(int pageNum, int pageSize) throws ServiceException {
        try {
            PageHelper.startPage(pageNum, pageSize);
            return (Pager<TransportResourceCore>) transportResourceCoreMapper.select(null);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 分页查询
     *
     * @param pageNum  页码
     * @param pageSize 每页的大小
     * @return 一个分页的实体集合
     * @throws ServiceException 业务出错
     */
    public Pager<TransportResourceCore> selectPage(int pageNum, int pageSize, TransportResourceCore entity) throws ServiceException {
        try {
            PageHelper.startPage(pageNum, pageSize);
            return (Pager<TransportResourceCore>) transportResourceCoreMapper.select(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    /**
     * 按条件分页查询
     *
     * @param pageNum  页码
     * @param pageSize 每页的大小
     * @param example  查询条件
     * @return 一个分页的实体集合
     * @throws ServiceException 业务出错
     */
    public Pager<TransportResourceCore> selectPageByExample(int pageNum, int pageSize, Example example) throws ServiceException {
        try {
            PageHelper.startPage(pageNum, pageSize);
            return (Pager<TransportResourceCore>) transportResourceCoreMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

}
