/*
 * ! CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-06-01 10:50:22
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.mapper.TransportTransferMapper;
import com.qhiot.transport.db.model.TransportTransfer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;

/**
 * 业务层 <br/>
 * Table: transport_transfer - 数据转发配置 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-06-01 10:50:22
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportTransferService {

    @Autowired
    private TransportTransferMapper transportTransferMapper;

    public TransportTransfer selectByPrimaryKey(Long transferId) throws ServiceException {
        try {
            return transportTransferMapper.selectByPrimaryKey(transferId);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:transfer", key = "#entity.deviceId")
    public int insert(TransportTransfer entity) throws ServiceException {
        try {
            return transportTransferMapper.insert(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:transfer", key = "#entity.deviceId")
    public int updateSelective(TransportTransfer entity) throws ServiceException {
        try {
            return transportTransferMapper.updateByPrimaryKeySelective(entity);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:transfer", key = "#result.deviceId", condition = "#result != null")
    public TransportTransfer deleteByPrimaryKey(Long transferId) throws ServiceException {
        TransportTransfer transfer;
        try {
            transfer = transportTransferMapper.selectByPrimaryKey(transferId);
            transportTransferMapper.deleteByPrimaryKey(transferId);
            return transfer;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Cacheable(value = "seem:transport:transfer", key = "#deviceId")
    public List<TransportTransfer> selectByDeviceId(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportTransfer.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            return transportTransferMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

}
