/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2017-03-24 09:52:45
 */
package com.qhiot.transport.db.service;

import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.db.mapper.TransportUpgradeMapper;
import com.qhiot.transport.db.model.TransportUpgrade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

/**
 * 业务层 <br/>
 * Table: transport_upgrade - 固件升级记录 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2017-03-24 09:52:45
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportUpgradeService {

    @Autowired
    private TransportUpgradeMapper transportUpgradeMapper;

    @CachePut(value = "seem:transport:upgrade", key = "#entity.deviceType+'_'+#entity.deviceId")
    public TransportUpgrade insert(TransportUpgrade entity) throws ServiceException {
        try {
            transportUpgradeMapper.insert(entity);
            return entity;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CachePut(value = "seem:transport:upgrade", key = "#entity.deviceType+'_'+#entity.deviceId")
    public TransportUpgrade insertSelective(TransportUpgrade entity) throws ServiceException {
        try {
            transportUpgradeMapper.insertSelective(entity);
            return entity;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:upgrade", key = "#deviceType+'_'+#deviceId")
    public void deleteByDevice(String deviceId, Byte deviceType) throws ServiceException {
        try {
            Example example = new Example(TransportUpgrade.class);
            example.createCriteria().andEqualTo("deviceId", deviceId).andEqualTo("deviceType", deviceType);
            transportUpgradeMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:upgrade", key = "#deviceType+'_'+#deviceId")
    public void updateStartupTime(String deviceId, Byte deviceType) throws ServiceException {
        try {
            TransportUpgrade upgrade = new TransportUpgrade();
            upgrade.setDeviceId(deviceId);
            upgrade.setDeviceType(deviceType);
            upgrade.setStartupTime(DateUtil.now());
            transportUpgradeMapper.updateStartupTime(upgrade);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @CacheEvict(value = "seem:transport:upgrade", key = "#deviceType+'_'+#deviceId")
    public void updateSegmentIndex(String deviceId, Byte deviceType, Integer segmentIndex) throws ServiceException {
        try {
            TransportUpgrade upgrade = new TransportUpgrade();
            upgrade.setDeviceId(deviceId);
            upgrade.setDeviceType(deviceType);
            upgrade.setSegmentIndex(segmentIndex);
            transportUpgradeMapper.updateSegmentIndex(upgrade);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public TransportUpgrade selectByPrimaryKey(Long id) throws ServiceException {
        try {
            return transportUpgradeMapper.selectByPrimaryKey(id);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Cacheable(value = "seem:transport:upgrade", key = "#deviceType+'_'+#deviceId")
    public TransportUpgrade selectByDevice(String deviceId, Byte deviceType) throws ServiceException {
        try {
            TransportUpgrade upgrade = new TransportUpgrade();
            upgrade.setDeviceId(deviceId);
            upgrade.setDeviceType(deviceType);
            return transportUpgradeMapper.selectOne(upgrade);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int selectCountByExample(Example example) throws ServiceException {
        try {
            return transportUpgradeMapper.selectCountByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

}
