/*!
 * CodeGenerator(https://git.oschina.net/ptma/CodeGenerator)
 *
 * Date: 2018-06-06 11:56:27
 */
package com.qhiot.transport.db.service;

import com.qhiot.cache.util.CacheUtil;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.transport.db.mapper.TransportWarningConfigMapper;
import com.qhiot.transport.db.model.TransportEquipmentMonitoring;
import com.qhiot.transport.db.model.TransportWarningConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;

/**
 * 业务层 <br/>
 * Table: transport_warning_config - 报警配置 <br/>
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2018-06-06 11:56:27
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TransportWarningConfigService {

    @Autowired
    private TransportWarningConfigMapper transportWarningConfigMapper;

    public List<TransportWarningConfig> selectByDeviceId(String deviceId) throws ServiceException {
        try {
            Example example = new Example(TransportEquipmentMonitoring.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            example.orderBy("configIndex");
            return transportWarningConfigMapper.selectByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByDeviceId(String deviceId) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:datacollect:warning", deviceId);
            Example example = new Example(TransportWarningConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId);
            example.orderBy("configIndex");
            return transportWarningConfigMapper.deleteByExample(example);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public TransportWarningConfig selectByPrimaryKey(Short configIndex, String deviceId) throws ServiceException {
        try {
            TransportWarningConfig key = new TransportWarningConfig();
            key.setDeviceId(deviceId);
            key.setConfigIndex(configIndex);
            return transportWarningConfigMapper.selectByPrimaryKey(key);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int deleteByPrimaryKey(Short configIndex, String deviceId) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:datacollect:warning", deviceId);
            TransportWarningConfig key = new TransportWarningConfig();
            key.setDeviceId(deviceId);
            key.setConfigIndex(configIndex);
            return transportWarningConfigMapper.deleteByPrimaryKey(key);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public int saveOrUpdateSelective(TransportWarningConfig entity) throws ServiceException {
        try {
            CacheUtil.deleteByPrefix("seem:transport:datacollect:warning", entity.getDeviceId());
            if (null == transportWarningConfigMapper.selectByPrimaryKey(entity)) {
                return transportWarningConfigMapper.insertSelective(entity);
            } else {
                return transportWarningConfigMapper.updateByPrimaryKeySelective(entity);
            }
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Cacheable(value = "seem:transport:datacollect:warning", key = "#deviceId+'_'+#storeAddress")
    public TransportWarningConfig selectByAddress(String deviceId, int storeAddress) throws ServiceException {
        try {
            Example example = new Example(TransportWarningConfig.class);
            example.createCriteria().andEqualTo("deviceId", deviceId)
                    .andEqualTo("warningRegisterAddress", storeAddress);
            List<TransportWarningConfig> list = transportWarningConfigMapper.selectByExample(example);
            return (list != null && list.size() > 0) ? list.get(0) : null;
        } catch (Exception e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }
}
