package com.qhiot.transport.db.util;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.transport.db.model.TransportEquipmentConfig;
import com.qhiot.transport.db.model.TransportEquipmentMonitoring;
import com.qhiot.transport.db.model.TransportResourceConfig;
import com.qhiot.transport.db.model.TransportWarningConfig;
import com.qhiot.transport.message.bean.EquipmentConfig;
import com.qhiot.transport.message.bean.EquipmentMonitoring;
import com.qhiot.transport.message.bean.ResourceConfig;
import com.qhiot.transport.message.bean.WarningConfig;

/**
 * @author 金建强(ptma/@163.com)
 * @version 2017/7/20 19:34
 */
public class BeanTransformUtil {

    public static TransportResourceConfig transform(ResourceConfig config) {
        TransportResourceConfig resource = new TransportResourceConfig();
        resource.setValid(config.getValid());
        resource.setProtocol(config.getProtocol());
        resource.setResource(config.getResource());
        resource.setCommPort(config.getCommPort());
        resource.setBaudRate(config.getBaudRate());
        resource.setDataBit(config.getDataBit());
        resource.setStopBit(config.getStopBit());
        resource.setParityBit(config.getParityBit());
        resource.setPeriod(config.getPeriod());
        resource.setAddress(config.getAddress());
        resource.setFunctionCode(config.getFunctionCode());
        resource.setStartingAddr(config.getStartingAddr());
        resource.setMethod(config.getMethod());
        resource.setDataQuantity(config.getDataQuantity());
        resource.setCommandLength(config.getCommandLength());
        resource.setCommand(ByteUtil.toHexString(config.getCommand()));
        resource.setRegisterType(config.getRegisterType());
        resource.setStoreAddress(config.getStoreAddress());
        resource.setUpperBound(config.getUpperBound());
        resource.setLowerBound(config.getLowerBound());
        resource.setUpperBoundEnabled(config.getUpperBoundEnabled());
        resource.setLowerBoundEnabled(config.getLowerBoundEnabled());
        return resource;
    }

    public static ResourceConfig transform(TransportResourceConfig resource) {
        ResourceConfig config = new ResourceConfig();
        config.setValid(resource.getValid());
        config.setProtocol(resource.getProtocol());
        config.setResource(resource.getResource());
        config.setCommPort(resource.getCommPort());
        config.setBaudRate(resource.getBaudRate());
        config.setDataBit(resource.getDataBit());
        config.setStopBit(resource.getStopBit());
        config.setParityBit(resource.getParityBit());
        config.setPeriod(resource.getPeriod());
        config.setAddress(resource.getAddress());
        config.setFunctionCode(resource.getFunctionCode());
        config.setStartingAddr(resource.getStartingAddr());
        config.setMethod(resource.getMethod());
        config.setDataQuantity(resource.getDataQuantity());
        config.setCommandLength(resource.getCommandLength());
        config.setCommand(ByteUtil.hexToBytes(resource.getCommand()));
        config.setRegisterType(resource.getRegisterType());
        config.setStoreAddress(resource.getStoreAddress());
        config.setUpperBound(resource.getUpperBound()==null?0:resource.getUpperBound());
        config.setLowerBound(resource.getLowerBound()==null?0:resource.getLowerBound());
        config.setUpperBoundEnabled(resource.getUpperBoundEnabled());
        config.setLowerBoundEnabled(resource.getLowerBoundEnabled());
        return config;
    }

    public static EquipmentConfig transform(TransportEquipmentConfig source) {
        EquipmentConfig config = new EquipmentConfig();
        config.setValid(source.getValid());
        config.setCommandIndex(source.getConfigIndex());
        config.setEquipmentType(source.getEquipmentType());
        config.setManufacturers(source.getManufacturers());
        config.setEquipmentModel(source.getEquipmentModel());
        config.setCommunicationInterface(source.getCommunicationInterface());
        config.setIp1OrBaudRate(source.getIp1OrBaudRate());
        config.setIp2OrDataBit(source.getIp2OrDataBit());
        config.setIp3OrParityBit(source.getIp3OrParityBit());
        config.setIp4OrStopBit(source.getIp4OrStopBit());
        config.setPortOrReserved1(source.getPortOrReserved1());

        config.setByteOrder16(source.getByteOrder16());
        config.setByteOrder32(source.getByteOrder32());
        config.setUnionIndex(source.getUnionIndex());
        return config;
    }

    public static TransportEquipmentConfig transform(EquipmentConfig source) {
        TransportEquipmentConfig config = new TransportEquipmentConfig();
        config.setValid(source.getValid());
        config.setConfigIndex(source.getCommandIndex());
        config.setEquipmentType(source.getEquipmentType());
        config.setManufacturers(source.getManufacturers());
        config.setEquipmentModel(source.getEquipmentModel());
        config.setCommunicationInterface(source.getCommunicationInterface());
        config.setIp1OrBaudRate(source.getIp1OrBaudRate());
        config.setIp2OrDataBit(source.getIp2OrDataBit());
        config.setIp3OrParityBit(source.getIp3OrParityBit());
        config.setIp4OrStopBit(source.getIp4OrStopBit());
        config.setPortOrReserved1(source.getPortOrReserved1());

        config.setByteOrder16(source.getByteOrder16());
        config.setByteOrder32(source.getByteOrder32());
        config.setUnionIndex(source.getUnionIndex());
        return config;
    }

    public static EquipmentMonitoring transform(TransportEquipmentMonitoring source) {
        EquipmentMonitoring config = new EquipmentMonitoring();
        config.setValid(source.getValid());
        config.setConfigIndex(source.getConfigIndex());
        config.setEquipmentIndex(source.getEquipmentIndex());
        config.setProtocol(source.getProtocol());
        config.setAddress(source.getAddress());
        config.setRegisterType(source.getRegisterType());
        config.setRegisterAddress(source.getRegisterAddress());
        config.setDataLength(source.getDataLength());
        config.setMethod(source.getMethod());
        config.setDataQuantity(source.getDataQuantity());
        config.setDeadZone(source.getDeadZone());

        config.setPeriod(source.getPeriod());
        config.setStoreRegisterType(source.getStoreRegisterType());
        config.setStoreRegisterAddress(source.getStoreRegisterAddress());
        config.setRw(source.getRw());
        config.setUnionIndex(source.getUnionIndex());
        return config;
    }

    public static TransportEquipmentMonitoring transform(EquipmentMonitoring source) {
        TransportEquipmentMonitoring config = new TransportEquipmentMonitoring();
        config.setValid(source.getValid());
        config.setConfigIndex(source.getConfigIndex());
        config.setEquipmentIndex(source.getEquipmentIndex());
        config.setProtocol(source.getProtocol());
        config.setAddress(source.getAddress());
        config.setRegisterType(source.getRegisterType());
        config.setRegisterAddress(source.getRegisterAddress());
        config.setDataLength(source.getDataLength());
        config.setMethod(source.getMethod());
        config.setDataQuantity(source.getDataQuantity());
        config.setDeadZone(source.getDeadZone());

        config.setPeriod(source.getPeriod());
        config.setStoreRegisterType(source.getStoreRegisterType());
        config.setStoreRegisterAddress(source.getStoreRegisterAddress());
        config.setRw(source.getRw());
        config.setUnionIndex(source.getUnionIndex());
        return config;
    }


    public static WarningConfig transform(TransportWarningConfig source) {
        WarningConfig config = new WarningConfig();
        config.setConfigIndex(source.getConfigIndex());
        config.setValid(source.getValid());
        config.setRegisterType1(source.getRegisterType1());
        config.setRegisterAddress1(source.getRegisterAddress1());
        config.setMethod1(source.getMethod1());
        config.setRegisterType2(source.getRegisterType2());
        config.setRegisterAddress2(source.getRegisterAddress2());
        config.setMethod2(source.getMethod2());
        config.setParamRelation(source.getParamRelation());
        config.setCondition1(source.getCondition1());
        config.setCondition2(source.getCondition2());
        config.setConditionRelation(source.getConditionRelation());
        config.setValue1(source.getValue1());
        config.setValue2(source.getValue2());
        config.setWarningRegisterType(source.getWarningRegisterType());
        config.setWarningRegisterAddress(source.getWarningRegisterAddress());
        config.setWarningMethod(source.getWarningMethod());
        config.setPeriod(source.getPeriod());

        return config;
    }

    public static TransportWarningConfig transform(WarningConfig source) {
        TransportWarningConfig config = new TransportWarningConfig();
        config.setConfigIndex(source.getConfigIndex());
        config.setValid(source.getValid());
        config.setRegisterType1(source.getRegisterType1());
        config.setRegisterAddress1(source.getRegisterAddress1());
        config.setMethod1(source.getMethod1());
        config.setRegisterType2(source.getRegisterType2());
        config.setRegisterAddress2(source.getRegisterAddress2());
        config.setMethod2(source.getMethod2());
        config.setParamRelation(source.getParamRelation());
        config.setCondition1(source.getCondition1());
        config.setCondition2(source.getCondition2());
        config.setConditionRelation(source.getConditionRelation());
        config.setValue1(source.getValue1());
        config.setValue2(source.getValue2());
        config.setWarningRegisterType(source.getWarningRegisterType());
        config.setWarningRegisterAddress(source.getWarningRegisterAddress());
        config.setWarningMethod(source.getWarningMethod());
        config.setPeriod(source.getPeriod());
        return config;
    }
}
