package com.qhiot.transport.db.util.simapi.chinaunicom;

import com.alibaba.fastjson.JSON;
import com.qhiot.transport.db.util.simapi.chinaunicom.response.DevicesAudiTrailsResponse;
import com.qhiot.transport.db.util.simapi.chinaunicom.response.DevicesCtdUsagesResponse;
import com.qhiot.transport.db.util.simapi.chinaunicom.response.DevicesDetilResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

public class UnicomAPIUtil {
    private static final Logger LOGGER = LoggerFactory.getLogger(UnicomAPIUtil.class);
    private static final String authorization = "Basic Z2FveWFuMjo0MTNmMjNkNy0yMjIwLTQzYzctOWY4NC01YzE0ZDc4OWFiNTA=";
    private static final String url = "https://api.10646.cn/rws/api/v1/devices/";

    /**
     * API接口
     *
     * @param urlstr
     * @return
     * @throws MalformedURLException
     */
    public static String ltAPI(String urlstr) throws MalformedURLException {
        StringBuilder data = new StringBuilder();
        URL url = new URL(urlstr);
        int counts = 0;
        HttpURLConnection connection = null;
        while (true) {
            try {
                connection = (HttpURLConnection) url.openConnection();
                connection.setRequestProperty("Authorization", "Basic Z2FveWFuMjo0MTNmMjNkNy0yMjIwLTQzYzctOWY4NC01YzE0ZDc4OWFiNTA=");
                connection.setConnectTimeout(5 * 1000);
                connection.connect();
                BufferedReader br = new BufferedReader(new InputStreamReader(connection.getInputStream(), "UTF-8"));
                String line = null;
                while ((line = br.readLine()) != null) {
                    data.append(line);
                    data.append("\r\n");
                }
                connection.disconnect();
                break;
            } catch (Exception e) {
                if (counts >= 5) {
                    LOGGER.info("联通SIM卡API接口{}链接失败！链接结束", urlstr);
                    return null;
                }
                counts++;
                LOGGER.info("联通SIM卡API接口链接失败后，再次尝试");
                continue;
            } finally {
                if (connection != null) {
                    connection.disconnect();
                }
            }
        }
        return data.toString();
    }


    /**
     * 查询SIM卡详情
     *
     * @param iccid
     * @return
     * @throws MalformedURLException
     */
    public static DevicesDetilResponse devicesInfo(String iccid) throws MalformedURLException {
        StringBuffer sbf = new StringBuffer(UnicomAPIUtil.url).append(iccid);
        String rs = ltAPI(sbf.toString());
        System.out.println(rs);
        LOGGER.info("SIM卡详情：" + rs);
        return JSON.parseObject(rs, DevicesDetilResponse.class);
    }


    /**
     * 查询SIM卡设备变更记录
     *
     * @param iccid
     * @return
     * @throws MalformedURLException
     */
    public static DevicesAudiTrailsResponse auditTrails(String iccid) throws MalformedURLException {
        StringBuffer sbf = new StringBuffer(UnicomAPIUtil.url).append(iccid).append("/auditTrails");
        String rs = ltAPI(sbf.toString());
        LOGGER.info("SIM卡变更记录：" + rs);
        return JSON.parseObject(rs, DevicesAudiTrailsResponse.class);
    }

    /**
     * 查询SIM卡设备变更记录
     *
     * @param iccid
     * @return
     * @throws MalformedURLException
     */
    public static DevicesCtdUsagesResponse ctdUsages(String iccid) throws MalformedURLException {
        StringBuffer sbf = new StringBuffer(UnicomAPIUtil.url).append(iccid).append("/ctdUsages");
        String rs = ltAPI(sbf.toString());
        LOGGER.info("SIM卡计费周期使用量：" + rs);
        return JSON.parseObject(rs, DevicesCtdUsagesResponse.class);
    }

    public static void main(String[] args) {
        try {
            DevicesDetilResponse devicesDetilResponse = UnicomAPIUtil.devicesInfo("89860617040020496677");
            DevicesAudiTrailsResponse devicesAudiTrailsResponse = UnicomAPIUtil.auditTrails("89860617040020496677");
            DevicesCtdUsagesResponse devicesCtdUsagesResponse = UnicomAPIUtil.ctdUsages("89860617040020496677");
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
    }
}
