package com.qhiot.transport.influxdb.model;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.qhiot.influxdb.model.BaseInfluxModel;
import com.qhiot.transport.db.model.TransportWarning;
import lombok.Data;
import org.influxdb.annotation.Column;
import org.influxdb.annotation.Measurement;

import java.time.Instant;
import java.util.Date;

/**
 * Measurement: transportWarningInflux - 终端告警 <br/>
 *
 * 原mysql中transport_warning表
 * @see TransportWarning
 */
@Data
@JsonFormat(shape = JsonFormat.Shape.OBJECT)
@Measurement(name = "transportWarningInflux",database = "seem")
public class TransportWarningInflux extends BaseInfluxModel {

    public TransportWarningInflux() {}

    public TransportWarningInflux(TransportWarning transportWarning) {
        setDate(transportWarning.getFirstTime());
        setId(transportWarning.getId());
        this.deviceId = transportWarning.getDeviceId();
        this.warningCode = transportWarning.getWarningCode().getCode();
        this.warning = transportWarning.getWarning();
        this.warningCount = transportWarning.getWarningCount();
        this.firstTime = transportWarning.getFirstTime().getTime();
        this.latestTime = transportWarning.getLatestTime().getTime();
    }

    @Column(name = "id", tag = true)
    private String id;

    /** Column: device_id - 终端编号 */
    @Column(name = "deviceId", tag = true)
    private String deviceId;

    /** Column: warning_code - 告警代码 */
    @Column(name = "warningCode", tag = true)
    private String warningCode;

    /** Column: warning - 告警 */
    @Column(name = "warning")
    private String warning;

    /** Column: warning_count - 告警次数 */
    @Column(name = "warningCount")
    private Integer warningCount;

    /** Column: first_time - 首次告警时间（以时间戳存储） */
    @Column(name = "firstTime")
    private Long firstTime;

    /** Column: latest_time - 最后告警时间（以时间戳存储） */
    @Column(name = "latestTime")
    private Long latestTime;

    @Column(name = "time")
    private Instant time;

    public Long getId() {
        if (null != this.id) {
            return Long.valueOf(this.id);
        }
        return null;
    }

    public void setId(Long id) {
        if (null != id) {
            this.id = id.toString();
        }
    }

    public WarningCode getWarningCode() {
        return WarningCode.codeOf(this.warningCode);
    }

    public void setWarningCode(WarningCode warningCode) {
        this.warningCode = warningCode.getCode();
    }

    public Date getFirstTimeDate() {
        if (null != this.firstTime) {
            return new Date(this.firstTime);
        }
        return null;
    }

    public void setFirstTimeDate(Date firstTime) {
        if (null != firstTime) {
            this.firstTime = firstTime.getTime();
        }
    }

    public Date getLatestTimeDate() {
        if (null != this.latestTime) {
            return new Date(this.latestTime);
        }
        return null;
    }

    public void setLatestTimeDate(Date latestTime) {
        if (null != latestTime) {
            this.latestTime = latestTime.getTime();
        }
    }

    /**
     * 获取当前时区的时间
     * @return
     */
    public Date getDate() {
        if (null != time) {
            Date date = new Date(time.toEpochMilli());
            return date;
        }
        return null;
    }
    /**
     * 以当前的时间设置Instant时间
     * @return
     */
    public void setDate(Date date) {
        if (null != date) {
            Instant instant = Instant.ofEpochMilli(date.getTime());
            this.time = instant;
        }
    }

    /**
     * 告警代码
     */
    @JsonFormat(shape = JsonFormat.Shape.OBJECT)
    public enum WarningCode {

        POWER("W001", "终端掉电"),
        OFFLINE("W002", "终端离线"),
        ELECTRIC_CABINET("W003", "电控箱非法开启"),
        TRANSFINITE("W004", "数据超限"),
        RULE_MATCH("W005", "规则告警"),
        UNKNOW("W000", "未知代码");

        private String code;

        private String desc;

        private WarningCode(String code, String desc) {
            this.code = code;
            this.desc = desc;
        }

        @JsonCreator
        public static WarningCode codeOf(String code) {
            WarningCode[] enumConstants = WarningCode.class.getEnumConstants();
            for (WarningCode e : enumConstants) {
                if (e.getCode().equals(code)) {
                    return e;
                }
            }
            return UNKNOW;
        }

        public String getCode() {
            return code;
        }

        public String getDesc() {
            return desc;
        }


        @Override
        public String toString() {
            return code;
        }
    }
}
