package com.qhiot.transport.influxdb.service;

import com.qhiot.core.bean.Pager;
import com.qhiot.core.exception.ServiceException;
import com.qhiot.core.sql.generator.SnowflakeID;
import com.qhiot.core.util.CollectionUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.influxdb.service.InfluxDBBaseService;
import com.qhiot.influxdb.util.InfluxDBHandleUtil;
import com.qhiot.transport.db.model.TransportWarning;
import com.qhiot.transport.db.service.TransportWarningService;
import com.qhiot.transport.influxdb.model.TransportWarningInflux;
import lombok.extern.slf4j.Slf4j;
import org.influxdb.dto.Query;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import tk.mybatis.mapper.entity.Example;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class TransportWarningInfluxService extends InfluxDBBaseService<TransportWarningInflux> {

    @Autowired
    private SnowflakeID snowflakeID;

    @Autowired
    private TransportWarningService transportWarningService;

    public void insert(TransportWarningInflux transportWarningInflux) {
        if (null == transportWarningInflux.getDate()) {
            transportWarningInflux.setDate(transportWarningInflux.getFirstTimeDate());
        }
        if (null == transportWarningInflux.getId()) {
            transportWarningInflux.setId(snowflakeID.next());
        }
        super.insert(transportWarningInflux);
    }

    /**
     * 根据id更新记录
     * @param transportWarningInflux
     * @throws ServiceException
     */
    public void updateById(TransportWarningInflux transportWarningInflux) throws ServiceException {
        if (transportWarningInflux.getId() == null) {
            throw new ServiceException("非法的TransportWarning记录");
        }
        StringBuffer sql = new StringBuffer("select * from ").append(getMeasurementName()).append(" where id = '")
            .append(transportWarningInflux.getId()).append("' tz('Local')");
        Query query = new Query(sql.toString(),getDatabaseName());
        List<TransportWarningInflux> resultList = myQuery(query);
        if (resultList.size() > 1) {
            throw new ServiceException("重复的TransportWarning记录");
        }
//        if (resultList.size() == 0) {
//            throw new ServiceException("未找到匹配的TransportWarning记录");
//        }
        if (resultList.size() == 1) {
            TransportWarningInflux old = resultList.get(0);
            if (!old.getId().equals(transportWarningInflux.getId()) || !old.getDeviceId().equals(transportWarningInflux.getDeviceId())
                || !old.getWarningCode().equals(transportWarningInflux.getWarningCode())) {
                throw new ServiceException("原id、deviceId、warningCode字段不可修改");
            }
            old.setDeviceId(transportWarningInflux.getDeviceId());
            old.setWarningCode(transportWarningInflux.getWarningCode());
            old.setWarning(transportWarningInflux.getWarning());
            old.setWarningCount(transportWarningInflux.getWarningCount());
            old.setFirstTime(transportWarningInflux.getFirstTime());
            old.setLatestTime(transportWarningInflux.getLatestTime());
            super.insert(old);
        }
    }

    /**
     * 根据网关号与告警代码查询
     * @param deviceId
     * @param warningCode
     * @return
     * @throws ServiceException
     */
    public List<TransportWarningInflux> selectByDeviceIdAndWarningCode(String deviceId, String warningCode) throws ServiceException {
        StringBuffer sql = new StringBuffer("select * from ").append(getMeasurementName()).append(" where deviceId = '").append(deviceId)
            .append("' and warningCode = '").append(warningCode).append("' tz('Local')");
        Query query = new Query(sql.toString(),getDatabaseName());
        List<TransportWarningInflux> resultList = myQuery(query);
        return resultList;
    }

    /**
     * 获取一网关最近告警时间
     * @param deviceId
     * @return
     * @throws ServiceException
     */
    public TransportWarningInflux selectLatestWarnTime(String deviceId) throws ServiceException {
        StringBuffer sql = new StringBuffer("select max(latestTime),* from ").append(getMeasurementName()).append(" where deviceId = '").append(deviceId)
            .append("' tz('Local')");
        Query query = new Query(sql.toString(),getDatabaseName());
        List<TransportWarningInflux> resultList = myQuery(query);
        if (CollectionUtil.isNotEmpty(resultList)) {
            return resultList.get(0);
        }
        return null;
    }

    /**
     * 获取一网关今日的告警数据
     * @param deviceId
     * @return
     * @throws ServiceException
     */
    public List<TransportWarningInflux> selectTodayWarning(String deviceId) throws ServiceException {
        StringBuffer sql = new StringBuffer("select * from ").append(getMeasurementName()).append(" where deviceId = '").append(deviceId)
            .append("' and time > '").append(InfluxDBHandleUtil.dataFormat(DateUtil.beginTimeOfDay(DateUtil.now())))
            .append("' and time < '").append(InfluxDBHandleUtil.dataFormat(DateUtil.endTimeOfDay(DateUtil.now())))
            .append("' tz('Local')");
        Query query = new Query(sql.toString(),getDatabaseName());
        List<TransportWarningInflux> resultList = myQuery(query);
        resultList = resultList.parallelStream().sorted(Comparator.comparing(TransportWarningInflux::getLatestTime)).collect(Collectors.toList());
        return resultList;
    }

    /**
     * 根据系统参数中需屏蔽的告警进行查询
     * @return
     */
    public Pager<TransportWarningInflux> selectByDeviceIdExceptPager(String deviceId, List<String> exceptList, int pageNum, int pageSize, String s) {
        StringBuffer sql = new StringBuffer("select * from ").append(getMeasurementName()).append(" where deviceId = '").append(deviceId)
            .append("'");
        if (CollectionUtil.isNotEmpty(exceptList)) {
            for (String warningCode : exceptList) {
                sql.append(" and warningCode != '").append(warningCode).append("' ");
            }
        }
        sql.append(s);
        sql.append("limit ").append(pageSize).append(" offset ").append(pageNum * pageSize);
        sql.append(" tz('Local')");
        log.info(sql.toString());
        Query query = new Query(sql.toString(),getDatabaseName());
        List<TransportWarningInflux> resultList = myQuery(query);
        resultList = resultList.parallelStream().sorted(Comparator.comparing(TransportWarningInflux::getLatestTime)).collect(Collectors.toList());

        //计算个数
        sql = new StringBuffer("select * from ").append(getMeasurementName()).append(" where deviceId = '").append(deviceId)
            .append("'");
        if (CollectionUtil.isNotEmpty(exceptList)) {
            for (String warningCode : exceptList) {
                sql.append(" and warningCode != '").append(warningCode).append("' ");
            }
        }
        sql.append(s);
        sql.append(" tz('Local')");
        log.info(sql.toString());
        Query query1 = new Query(sql.toString(),getDatabaseName());
        List<TransportWarningInflux> resultList1 = myQuery(query1);
        Pager<TransportWarningInflux> pager = new Pager<>(pageNum, pageSize);
        pager.addAll(resultList);
        pager.setTotal(resultList1.size());

        return pager;
    }


    public Pager<TransportWarningInflux> selectByDeviceIdListPager(List<String> deviceIdList, int pageNum, int pageSize, String extra) {
        if (CollectionUtil.isEmpty(deviceIdList)) {
            return new Pager<>(pageNum, pageSize);
        }
        StringBuffer sql = new StringBuffer("select * from ").append(getMeasurementName()).append(" where (");
        for (int i = 0; i < deviceIdList.size(); i++) {
            sql.append(" deviceId = '").append(deviceIdList.get(i)).append("' ");
            if (i != deviceIdList.size() - 1) {
                sql.append(" or");
            }
        }
        sql.append(")");
        sql.append(extra);
        sql.append(" limit ").append(pageSize).append(" offset ").append(pageNum * pageSize);
        sql.append(" tz('Local')");
        log.info(sql.toString());
        Query query = new Query(sql.toString(),getDatabaseName());
        List<TransportWarningInflux> resultList = myQuery(query);

        sql = new StringBuffer("select * from ").append(getMeasurementName()).append(" where (");
        for (int i = 0; i < deviceIdList.size(); i++) {
            sql.append(" deviceId = '").append(deviceIdList.get(i)).append("' ");
            if (i != deviceIdList.size() - 1) {
                sql.append(" or");
            }
        }
        sql.append(")");
        sql.append(extra);
        sql.append(" tz('Local')");
        log.info(sql.toString());
        Query query1 = new Query(sql.toString(),getDatabaseName());
        List<TransportWarningInflux> resultList1 = myQuery(query1);

        Pager<TransportWarningInflux> pager = new Pager<>(pageNum, pageSize);
        pager.addAll(resultList);
        pager.setTotal(resultList1.size());

        return pager;
    }

    /**
     * 将mysql中transport_warning表中近3个月的数据迁移至influxdb
     */
    public void initInfluxMasurement() throws ServiceException {
        Example example = new Example(TransportWarning.class);
        example.createCriteria().andGreaterThan("firstTime", DateUtil.offsetMonth(DateUtil.now(),-3));
        List<TransportWarning> transportWarningList = transportWarningService.selectByExample(example);

        List<TransportWarningInflux> transportWarningInfluxList = transportWarningList.parallelStream().map(TransportWarningInflux::new).collect(Collectors.toList());

        super.batchInsert(transportWarningInfluxList);

    }


//    public String build(Example example) {
//        StringBuffer s = new StringBuffer();
//        List<Example.Criteria> oredList = example.getOredCriteria();
//        if (CollectionUtil.isNotEmpty(oredList)) {
//            s.append(" and ");
//            List<Example.Criterion> criteriaList = oredList.get(0).getCriteria();
//            for (Example.Criterion criterion : criteriaList) {
//                s.append(criterion.getCondition());
//                s.append(" = '").append(criterion.getValue()).append("' and ");
//            }
//            s.append("1=1 ");
//        }
//        return s.toString();
//    }

}
