package com.qhiot.transport.message;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 编码解码日志消息
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-06-30 23:55:33
 */
public class CodecLogMessage implements Serializable {

    private static final long serialVersionUID = 3362041533376689640L;

    private String            channelId;
    /** 终端编号码 */
    private String            deviceId;

    /** 上下文编号 */
    private String            contextId;
    /** 方向 */
    private String            direction;
    private String            message;
    private final Date        time;

    private CodecLogMessage() {
        time = new Date();
    }

    /**
     * 下行消息
     */
    public static CodecLogMessage downstreamMessage(String channelId, String deviceId, String contextId, String message) {
        CodecLogMessage msg = new CodecLogMessage();
        msg.channelId = channelId;
        msg.deviceId = deviceId;
        msg.contextId = contextId;
        msg.direction = ">>";
        msg.message = message;
        return msg;
    }

    /**
     * 上行消息
     */
    public static CodecLogMessage upstreamMessage(String channelId, String deviceId, String contextId, String message) {
        CodecLogMessage msg = new CodecLogMessage();
        msg.channelId = channelId;
        msg.deviceId = deviceId;
        msg.contextId = contextId;
        msg.direction = "<<";
        msg.message = message;
        return msg;
    }

    /**
     * 其它消息
     */
    public static CodecLogMessage analyzeMessage(String channelId, String deviceId, String contextId, String message) {
        CodecLogMessage msg = new CodecLogMessage();
        msg.channelId = channelId;
        msg.deviceId = deviceId;
        msg.contextId = contextId;
        msg.direction = "==";
        msg.message = "命令解析, " + message;
        return msg;
    }

    public static CodecLogMessage normalMessage(String channelId, String deviceId, String contextId, String message) {
        CodecLogMessage msg = new CodecLogMessage();
        msg.channelId = channelId;
        msg.deviceId = deviceId;
        msg.contextId = contextId;
        msg.direction = "##";
        msg.message = message;
        return msg;
    }

    public String getChannelId() {
        return channelId;
    }

    public String getDeviceId() {
        return deviceId == null ? "unknow" : deviceId;
    }

    public String getDirection() {
        return direction;
    }

    public String getMessage() {
        if (contextId != null && contextId.trim().length()>0) {
            return String.format("[%s, %s] %s %s, 上下文=%s", channelId, deviceId, direction, message, contextId);
        } else {
            return String.format("[%s, %s] %s %s", channelId, deviceId, direction, message);
        }
    }

    public Date getTime() {
        return time == null ? null : (Date) time.clone();
    }

    public String getTimeString() {
        if (time == null) {
            return "";
        } else {
            SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
            return formatter.format(time);
        }
    }

}
