package com.qhiot.transport.message.bean;

import java.io.Serializable;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * ASCII 参数
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-06-30 23:43:48
 */
public class AsciiParam implements Serializable {

    private static final long    serialVersionUID = -1998945812200843075L;

    private static final Pattern KEY_VALUE_PATTERN = Pattern.compile("\\s*([^=:]*)[=|:](.*)");

    protected byte[]             data;

    public byte[] getData() {
        return data == null ? new byte[0] : data;
    }

    public void setData(byte[] data) {
        this.data = data;
    }

    public String getDataString() {
        return data != null ? new String(data) : "";
    }

    public Map<String, String> getParamMap() {
        Map<String, String> paramMap = new HashMap<String, String>();
        if (data != null) {
            String asciiStr = new String(data);
            String[] items = asciiStr.split("\\u0000");

            Matcher m;
            for (String item : items) {
                m = KEY_VALUE_PATTERN.matcher(item);
                if (m.matches()) {
                    String key = m.group(1).trim();
                    String value = m.group(2).trim();
                    paramMap.put(key, value);
                }
            }
        }
        return paramMap;
    }

    public void setParamMap(Map<String, String> paramMap) {
        StringBuilder asciiStr = new StringBuilder();

        char spliter = 0x00;

        for (Map.Entry<String, String> entry : paramMap.entrySet()) {
            if (entry.getValue() == null) {
                continue;
            }
            asciiStr.append(entry.getKey());
            if ("?".equals(entry.getValue())) {
                asciiStr.append("?");
            } else {
                asciiStr.append(":").append(entry.getValue());
            }
            asciiStr.append(spliter);
        }

        data = asciiStr.toString().getBytes(Charset.forName("GBK"));
    }

}
