package com.qhiot.transport.message.bean;

import com.qhiot.transport.message.util.MessageBeanUtil;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 终端参数
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-06-21 15:59
 */
public class DeviceParam implements Serializable {

    private static final long serialVersionUID = 7755298526333218461L;

    public static final int DATA_LENGTH     = 104;

    /** 初始化标志 1B */
    protected short           initMark;
    /** 终端名称 30B ASCII */
    protected byte[]          deviceName      = new byte[30];
    /** 序列号 12B ASCII */
    protected byte[]          serialNumber    = new byte[12];
    /** 软件版本号 4B ASCII */
    protected byte[]          softwareVersion = new byte[4];
    /** 硬件版本号 4B ASCII */
    protected byte[]          hardwareVersion = new byte[4];
    /** 终端编号 4B */
    protected int             deviceAddr;
    /** 天线地址1 2B */
    protected int             antenna1Addr;
    /** 天线地址2 2B */
    protected int             antenna2Addr;
    /** 心跳周期 2B */
    protected int             heartbeatPeriod;
    /** 激励包发送周期 2B */
    protected int             excitePeriod;
    /** 超声波探测周期 2B */
    protected int             ultrasonicDetectPeriod;
    /** 超声波探测距离 2B */
    protected int             ultrasonicDetectDistance;
    /** 125K频率 2B */
    protected int             frequency125K;
    /** 路由有效标志 1B */
    protected short           routeValidMark;
    /** 路由终端编号 4B */
    protected int             routeGatewayAddr;
    /** 时间戳 4B */
    protected int             timestamp;
    /** 累计工作时间 2B */
    protected int             accumulatedRunningTime;
    /** 预留参数1 2B */
    protected short           reserved1;
    /** 预留参数2 2B */
    protected int             reserved2;
    /** 预留参数3 2B */
    protected int             reserved3;
    /** 预留参数4 2B */
    protected int             reserved4;
    /** 预留参数5 2B */
    protected int             reserved5;
    /** 预留参数6 2B */
    protected int             reserved6;
    /** 芯片唯一ID 12B */
    protected byte[]          chipId          = new byte[12];

    public short getInitMark() {
        return initMark;
    }

    public void setInitMark(short initMark) {
        this.initMark = initMark;
    }

    public byte[] getDeviceName() {
        return deviceName.clone();
    }

    public String getDeviceNameString() {
        return MessageBeanUtil.toGBKString(deviceName);
    }

    public void setDeviceName(byte[] deviceName) {
        this.deviceName = deviceName.clone();
    }

    public byte[] getSerialNumber() {
        return serialNumber.clone();
    }

    public String getSerialNumberString() {
        return MessageBeanUtil.toGBKString(serialNumber);
    }

    public void setSerialNumber(byte[] serialNumber) {
        this.serialNumber = serialNumber.clone();
    }

    public byte[] getSoftwareVersion() {
        return softwareVersion.clone();
    }

    public String getSoftwareVersionString() {
        return MessageBeanUtil.toGBKString(softwareVersion);
    }

    public void setSoftwareVersion(byte[] softwareVersion) {
        this.softwareVersion = softwareVersion.clone();
    }

    public byte[] getHardwareVersion() {
        return hardwareVersion.clone();
    }

    public String getHardwareVersionString() {
        return MessageBeanUtil.toGBKString(hardwareVersion);
    }

    public void setHardwareVersion(byte[] hardwareVersion) {
        this.hardwareVersion = hardwareVersion.clone();
    }

    public short getReserved1() {
        return reserved1;
    }

    public void setReserved1(short reserved1) {
        this.reserved1 = reserved1;
    }

    public int getReserved2() {
        return reserved2;
    }

    public void setReserved2(int reserved2) {
        this.reserved2 = reserved2;
    }

    public int getReserved3() {
        return reserved3;
    }

    public void setReserved3(int reserved3) {
        this.reserved3 = reserved3;
    }

    public int getReserved4() {
        return reserved4;
    }

    public void setReserved4(int reserved4) {
        this.reserved4 = reserved4;
    }

    public int getReserved5() {
        return reserved5;
    }

    public void setReserved5(int reserved5) {
        this.reserved5 = reserved5;
    }

    public int getTimestamp() {
        return timestamp;
    }

    public String getFormattedTimestamp() {
        Date d = new Date(timestamp * 1000L);
        return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(d);
    }

    public void setTimestamp(int timestamp) {
        this.timestamp = timestamp;
    }

    public int getAccumulatedRunningTime() {
        return accumulatedRunningTime;
    }

    public void setAccumulatedRunningTime(int accumulatedRunningTime) {
        this.accumulatedRunningTime = accumulatedRunningTime;
    }

    public byte[] getChipId() {
        return chipId.clone();
    }

    public void setChipId(byte[] chipId) {
        for (int i = 0; i < 12; i++) {
            if (i < chipId.length) {
                this.chipId[i] = chipId[i];
            } else {
                this.chipId[i] = 0x00;
            }
        }
    }

    public String getChipIdHex() {
        return MessageBeanUtil.toHexString(chipId, true);
    }

    public int getDeviceAddr() {
        return deviceAddr;
    }

    public void setDeviceAddr(int deviceAddr) {
        this.deviceAddr = deviceAddr;
    }

    public int getAntenna1Addr() {
        return antenna1Addr;
    }

    public void setAntenna1Addr(int antenna1Addr) {
        this.antenna1Addr = antenna1Addr;
    }

    public int getAntenna2Addr() {
        return antenna2Addr;
    }

    public void setAntenna2Addr(int antenna2Addr) {
        this.antenna2Addr = antenna2Addr;
    }

    public int getHeartbeatPeriod() {
        return heartbeatPeriod;
    }

    public void setHeartbeatPeriod(int heartbeatPeriod) {
        this.heartbeatPeriod = heartbeatPeriod;
    }

    public int getExcitePeriod() {
        return excitePeriod;
    }

    public void setExcitePeriod(int excitePeriod) {
        this.excitePeriod = excitePeriod;
    }

    public int getUltrasonicDetectPeriod() {
        return ultrasonicDetectPeriod;
    }

    public void setUltrasonicDetectPeriod(int ultrasonicDetectPeriod) {
        this.ultrasonicDetectPeriod = ultrasonicDetectPeriod;
    }

    public int getUltrasonicDetectDistance() {
        return ultrasonicDetectDistance;
    }

    public void setUltrasonicDetectDistance(int ultrasonicDetectDistance) {
        this.ultrasonicDetectDistance = ultrasonicDetectDistance;
    }

    public int getFrequency125K() {
        return frequency125K;
    }

    public void setFrequency125K(int frequency125k) {
        frequency125K = frequency125k;
    }

    public short getRouteValidMark() {
        return routeValidMark;
    }

    public void setRouteValidMark(short routeValidMark) {
        this.routeValidMark = routeValidMark;
    }

    public int getRouteGatewayAddr() {
        return routeGatewayAddr;
    }

    public void setRouteGatewayAddr(int routeGatewayAddr) {
        this.routeGatewayAddr = routeGatewayAddr;
    }

    public int getReserved6() {
        return reserved6;
    }

    public void setReserved6(int reserved6) {
        this.reserved6 = reserved6;
    }

}
