package com.qhiot.transport.message.bean;

import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;

/**
 * 设备配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2018-05-10 23:42:35
 */
@Getter
@Setter
public class EquipmentConfig implements Serializable {

    private static final long serialVersionUID = 8081617001382508968L;

    public static final int   DATA_LENGTH      = 22;
    public static final int   DATA_COUNT       = 100;
    /**
     * 指令有效位(1b)
     */
    protected byte            valid;
    /**
     * 指令序号(2b)
     */
    protected short            commandIndex;
    /**
     * 设备类型(1b) 0：流量计, 1：PLC, 2：电表, 3：网关
     */
    protected byte            equipmentType;
    /**
     * 制造厂家(1b) 0：衢州柯联, 1:西门子, 2：跃进, 3：青泓
     */
    protected byte            manufacturers;
    /**
     * 设备型号(1b) 0：S7_200_SMART
     */
    protected byte            equipmentModel;
    /**
     * 通讯接口(1b) 0：485-1, 1：485-2, 2：232-1, 3：LAN-1, 4：LAN-2, 5：Local
     */
    protected byte            communicationInterface;

    /**
     * 波特率(1 byte)
     */
    protected byte            ip1OrBaudRate;

    /**
     * 数据位(1byte)<br/>
     * 0: 5位数据, <br/>
     * 1: 6位数据, <br/>
     * 2: 7位数据, <br/>
     * 3: 8位数据
     */
    protected byte            ip2OrDataBit;

    /**
     * 奇偶校验位(1 byte)<br/>
     * 0: 无校验, <br/>
     * 1: 奇校验, <br/>
     * 2: 偶校验
     */
    protected byte            ip3OrParityBit;

    /**
     * 停止位(1 byte)<br/>
     * 0: 1位, <br/>
     * 1: 1.5位, <br/>
     * 2: 2位
     */
    protected byte            ip4OrStopBit;

    /**
     * 保留 / 端口 (2b)
     */
    protected int           portOrReserved1;

    /**
     * 保留(2b)
     */
    protected byte[]          reserved2 = new byte[2];
    /**
     * 16位数据字节序 (1b)
     */
    protected byte            byteOrder16;

    /** 32位数据字节序 (1b) */
    protected byte            byteOrder32;

    /** 连接序号 (2b) */
    protected short            unionIndex;

    /**
     * 预留（3b）
     */
    protected byte[]          reserved3 = new byte[3];

    public boolean isAllFieldZero() {
        boolean allFieldZero = true;

        allFieldZero &= (valid == 0);
        allFieldZero &= (commandIndex == 0);
        allFieldZero &= (equipmentType == 0);
        allFieldZero &= (manufacturers == 0);
        allFieldZero &= (equipmentModel == 0);
        allFieldZero &= (communicationInterface == 0);
        allFieldZero &= (ip1OrBaudRate == 0);
        allFieldZero &= (ip2OrDataBit == 0);
        allFieldZero &= (ip3OrParityBit == 0);
        allFieldZero &= (ip4OrStopBit == 0);
        allFieldZero &= (portOrReserved1 == 0);
        allFieldZero &= (byteOrder16 == 0);
        allFieldZero &= (byteOrder32 == 0);
        allFieldZero &= (unionIndex == 0);
        
        return allFieldZero;
    }

}
