package com.qhiot.transport.message.bean;

import com.qhiot.transport.message.util.MessageBeanUtil;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * 终端参数
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-06-30 23:43:48
 */
public class GatewayParam implements Serializable {

    private static final long serialVersionUID = -765232907747808982L;

    public static final int DATA_LENGTH     = 124;

    /** 初始化标志 1B */
    protected short         initMark;
    /** 终端名称 30B ASCII */
    protected byte[]        deviceName      = new byte[30]; // String.getBytes("US-ASCII");
    /** 序列号 12B ASCII */
    protected byte[]        serialNumber    = new byte[12];
    /** 软件版本号 4B ASCII */
    protected byte[]        softwareVersion = new byte[4];
    /** 硬件版本号 4B ASCII */
    protected byte[]        hardwareVersion = new byte[4];
    /** 终端编号 4B */
    protected int           gatewayAddr;
    /** 服务器IP 4B */
    protected long          serverIP;
    /** 服务器端口 2B */
    protected int           serverPort;
    /** 出厂服务器IP 4B */
    protected long          server2IP;
    /** 出厂服务器端口 2B */
    protected int           server2Port;
    /** 网关IP 4B */
    protected long          gatewayIP;
    /** 子网掩码 4B */
    protected long          subnetMask;
    /** 本机IP 4B */
    protected long          deviceIP;
    /** 本机端口 2B */
    protected int           devicePort;
    /** 心跳周期 2B */
    protected int           heartbeatPeriod;
    /** GPRS信号值 1B */
    protected byte          gprsRssi;
    /** 433无线发送功率 1B */
    protected byte          wirelessPower;
    /** 喇叭关闭标志 1B */
    protected byte          speakerClosed;
    /** 输入输出手动运行标志 1B */
    protected byte          manualIO;

    /** 预留参数1-5 2B */
    protected int           reserved1;
    protected int           reserved2;
    protected int           reserved3;
    protected int           reserved4;
    protected int           reserved5;
    /** 预留参数6-10 1B */
    protected byte          reserved6;
    protected byte          reserved7;
    protected byte          reserved8;
    protected byte          reserved9;
    protected byte          reserved10;

    /** 时间戳 4B */
    protected int           timestamp;
    /** 累计工作时间 2B */
    protected int           cumulativeOperatingTime;
    /** 出厂时间 4B */
    protected int           manufactureDate;
    /** 芯片唯一ID 12B */
    protected byte[]        chipId          = new byte[12];

    public short getInitMark() {
        return initMark;
    }

    public void setInitMark(short initMark) {
        this.initMark = initMark;
    }

    public byte[] getDeviceName() {
        return MessageBeanUtil.clone(deviceName);
    }

    public String getDeviceNameString() {
        return MessageBeanUtil.toGBKString(deviceName);
    }

    public void setDeviceName(byte[] deviceName) {
        this.deviceName = MessageBeanUtil.clone(deviceName);
    }

    public byte[] getSerialNumber() {
        return MessageBeanUtil.clone(serialNumber);
    }

    public String getSerialNumberString() {
        return MessageBeanUtil.toGBKString(serialNumber);
    }

    public void setSerialNumber(byte[] serialNumber) {
        this.serialNumber = MessageBeanUtil.clone(serialNumber);
    }

    public byte[] getSoftwareVersion() {
        return MessageBeanUtil.clone(softwareVersion);
    }

    public String getSoftwareVersionString() {
        return MessageBeanUtil.toGBKString(softwareVersion);
    }

    public void setSoftwareVersion(byte[] softwareVersion) {
        this.softwareVersion = MessageBeanUtil.clone(softwareVersion);
    }

    public byte[] getHardwareVersion() {
        return MessageBeanUtil.clone(hardwareVersion);
    }

    public String getHardwareVersionString() {
        return MessageBeanUtil.toGBKString(hardwareVersion);
    }

    public void setHardwareVersion(byte[] hardwareVersion) {
        this.hardwareVersion = MessageBeanUtil.clone(hardwareVersion);
    }

    public int getGatewayAddr() {
        return gatewayAddr;
    }

    public void setGatewayAddr(int gatewayAddr) {
        this.gatewayAddr = gatewayAddr;
    }

    public long getServerIP() {
        return serverIP;
    }

    public String getServerIPString() {
        return MessageBeanUtil.long2Ip(serverIP);
    }

    public void setServerIP(long serverIP) {
        this.serverIP = serverIP;
    }

    public void setServerIP(String serverIP) {
        this.serverIP = MessageBeanUtil.ip2Long(serverIP);
    }

    public int getServerPort() {
        return serverPort;
    }

    public void setServerPort(int serverPort) {
        this.serverPort = serverPort;
    }

    public long getServer2IP() {
        return server2IP;
    }

    public String getServer2IPString() {
        return MessageBeanUtil.long2Ip(server2IP);
    }

    public void setServer2IP(long server2ip) {
        server2IP = server2ip;
    }

    public void setServer2IP(String server2ip) {
        server2IP = MessageBeanUtil.ip2Long(server2ip);
    }

    public int getServer2Port() {
        return server2Port;
    }

    public void setServer2Port(int server2Port) {
        this.server2Port = server2Port;
    }

    public long getGatewayIP() {
        return gatewayIP;
    }

    public String getGatewayIPString() {
        return MessageBeanUtil.long2Ip(gatewayIP);
    }

    public void setGatewayIP(long gatewayIP) {
        this.gatewayIP = gatewayIP;
    }

    public void setGatewayIP(String gatewayIP) {
        this.gatewayIP = MessageBeanUtil.ip2Long(gatewayIP);
    }

    public long getSubnetMask() {
        return subnetMask;
    }

    public String getSubnetMaskString() {
        return MessageBeanUtil.long2Ip(subnetMask);
    }

    public void setSubnetMask(long subnetMask) {
        this.subnetMask = subnetMask;
    }

    public void setSubnetMask(String subnetMask) {
        this.subnetMask = MessageBeanUtil.ip2Long(subnetMask);
    }

    public long getDeviceIP() {
        return deviceIP;
    }

    public String getDeviceIPString() {
        return MessageBeanUtil.long2Ip(deviceIP);
    }

    public void setDeviceIP(long deviceIP) {
        this.deviceIP = deviceIP;
    }

    public void setDeviceIP(String deviceIP) {
        this.deviceIP = MessageBeanUtil.ip2Long(deviceIP);
    }

    public int getDevicePort() {
        return devicePort;
    }

    public void setDevicePort(int devicePort) {
        this.devicePort = devicePort;
    }

    public int getHeartbeatPeriod() {
        return heartbeatPeriod;
    }

    public void setHeartbeatPeriod(int heartbeatPeriod) {
        this.heartbeatPeriod = heartbeatPeriod;
    }

    public byte getGprsRssi() {
        return gprsRssi;
    }

    public void setGprsRssi(byte gprsRssi) {
        this.gprsRssi = gprsRssi;
    }

    public int getTimestamp() {
        return timestamp;
    }

    public String getFormattedTimestamp() {
        Date d = new Date(timestamp * 1000L);
        return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(d);
    }

    public void setTimestamp(int timestamp) {
        this.timestamp = timestamp;
    }

    public int getManufactureDate() {
        return manufactureDate;
    }

    public String getFormattedManufactureDate() {
        Date d = new Date(manufactureDate * 1000L);
        return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(d);
    }

    public void setManufactureDate(int manufactureDate) {
        this.manufactureDate = manufactureDate;
    }

    public byte[] getChipId() {
        return MessageBeanUtil.clone(chipId);
    }

    public void setChipId(byte[] chipId) {
        for (int i = 0; i < 12; i++) {
            if (i < chipId.length) {
                this.chipId[i] = chipId[i];
            } else {
                this.chipId[i] = 0x00;
            }
        }
    }

    public String getChipIdHex() {
        return MessageBeanUtil.toHexString(chipId, true);
    }

    public byte getWirelessPower() {
        return wirelessPower;
    }

    public void setWirelessPower(byte wirelessPower) {
        this.wirelessPower = wirelessPower;
    }

    public byte getSpeakerClosed() {
        return speakerClosed;
    }

    public void setSpeakerClosed(byte speakerClosed) {
        this.speakerClosed = speakerClosed;
    }

    public byte getManualIO() {
        return manualIO;
    }

    public void setManualIO(byte manualIO) {
        this.manualIO = manualIO;
    }

    public int getReserved1() {
        return reserved1;
    }

    public void setReserved1(int reserved1) {
        this.reserved1 = reserved1;
    }

    public int getReserved2() {
        return reserved2;
    }

    public void setReserved2(int reserved2) {
        this.reserved2 = reserved2;
    }

    public int getReserved3() {
        return reserved3;
    }

    public void setReserved3(int reserved3) {
        this.reserved3 = reserved3;
    }

    public int getReserved4() {
        return reserved4;
    }

    public void setReserved4(int reserved4) {
        this.reserved4 = reserved4;
    }

    public int getReserved5() {
        return reserved5;
    }

    public void setReserved5(int reserved5) {
        this.reserved5 = reserved5;
    }

    public byte getReserved6() {
        return reserved6;
    }

    public void setReserved6(byte reserved6) {
        this.reserved6 = reserved6;
    }

    public byte getReserved7() {
        return reserved7;
    }

    public void setReserved7(byte reserved7) {
        this.reserved7 = reserved7;
    }

    public byte getReserved8() {
        return reserved8;
    }

    public void setReserved8(byte reserved8) {
        this.reserved8 = reserved8;
    }

    public byte getReserved9() {
        return reserved9;
    }

    public void setReserved9(byte reserved9) {
        this.reserved9 = reserved9;
    }

    public byte getReserved10() {
        return reserved10;
    }

    public void setReserved10(byte reserved10) {
        this.reserved10 = reserved10;
    }

    public int getCumulativeOperatingTime() {
        return cumulativeOperatingTime;
    }

    public void setCumulativeOperatingTime(int cumulativeOperatingTime) {
        this.cumulativeOperatingTime = cumulativeOperatingTime;
    }

}
