package com.qhiot.transport.message.bean;

import com.qhiot.transport.message.util.MessageBeanUtil;

import java.io.Serializable;
import java.util.Arrays;

/**
 * 历史数据 32 * N
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-06-30 23:43:48
 */
public class History implements Serializable {

    private static final long serialVersionUID = 4385462219663342520L;

    /** 单条数据的最小长度 */
    public static final int MIN_DATA_LENGTH = 20;

    /**
     * 删除标志(1 byte)
     */
    protected byte deleteMark;
    /**
     * 数据有效标志(1 byte)
     */
    protected byte valid;
    /**
     * 数据类型(1 byte)
     */
    protected byte dataType;
    /**
     * 数据序号(4 byte)
     */
    protected int dataIndex;
    /**
     * 寄存器类型(1 byte)
     */
    protected byte registerType;
    /**
     * 解码类型(1 byte)
     */
    protected byte method;
    /**
     * 存储地址(2 byte)
     */
    protected short storeAddress;
    /**
     * 数据长度(1 byte)
     */
    protected short dataLength;
    /**
     * 采集时间(4 byte)
     */
    protected int timestamp;
    /**
     * 校验(1 byte)
     */
    protected byte crc;

    /**
     * 报警状态 (1 byte) ’0‘:低于下限值,1’:正常,‘2’:高于上限值
     */
    protected byte alert;
    /**
     * 预留字节(2 byte)
     */
    protected byte[] reserved = new byte[2];
    /**
     * 数据(dataLength个 byte)
     */
    protected byte[] data;

    public byte getDeleteMark() {
        return deleteMark;
    }

    public void setDeleteMark(byte deleteMark) {
        this.deleteMark = deleteMark;
    }

    public byte getValid() {
        return valid;
    }

    public void setValid(byte valid) {
        this.valid = valid;
    }

    public byte getDataType() {
        return dataType;
    }

    public void setDataType(byte dataType) {
        this.dataType = dataType;
    }

    public int getDataIndex() {
        return dataIndex;
    }

    public void setDataIndex(int dataIndex) {
        this.dataIndex = dataIndex;
    }

    public byte getRegisterType() {
        return registerType;
    }

    public void setRegisterType(byte registerType) {
        this.registerType = registerType;
    }

    public byte getMethod() {
        return method;
    }

    public void setMethod(byte method) {
        this.method = method;
    }

    public short getStoreAddress() {
        return storeAddress;
    }

    public void setStoreAddress(short storeAddress) {
        this.storeAddress = storeAddress;
    }

    public short getDataLength() {
        return dataLength;
    }

    public void setDataLength(short dataLength) {
        this.dataLength = dataLength;
    }

    public int getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(int timestamp) {
        this.timestamp = timestamp;
    }

    public byte getCrc() {
        return crc;
    }

    public void setCrc(byte crc) {
        this.crc = crc;
    }

    public byte getAlert() {
        return alert;
    }

    public void setAlert(byte alert) {
        this.alert = alert;
    }

    public byte[] getReserved() {
        return reserved;
    }

    public void setReserved(byte[] reserved) {
        this.reserved = reserved;
    }

    public byte[] getData() {
        return data;
    }

    public void setData(byte[] data) {
        this.data = data;
    }

    public String getDataHex() {
        return MessageBeanUtil.toHexString(data, true);
    }

    @Override
    public String toString() {
        return "History{" +
            "deleteMark=" + deleteMark +
            ", valid=" + valid +
            ", dataType=" + dataType +
            ", dataIndex=" + dataIndex +
            ", registerType=" + registerType +
            ", method=" + method +
            ", storeAddress=" + storeAddress +
            ", dataLength=" + dataLength +
            ", timestamp=" + timestamp +
            ", crc=" + crc +
            ", alert=" + alert +
            ", reserved=" + Arrays.toString(reserved) +
            ", data=" + Arrays.toString(data) +
            '}';
    }
}
