package com.qhiot.transport.message.bean;

import com.qhiot.transport.message.util.MessageBeanUtil;

import java.io.Serializable;

/**
 * 485 外设指令配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-04-06 22:51
 */
public class Rs485Config implements Serializable {

    private static final long serialVersionUID = 3512244935893377826L;

    public static final int   DATA_LENGTH      = 25;
    public static final int   DATA_COUNT       = 30;

    /**
     * 指令有效位 0 或 1
     */
    protected byte            commandValid;
    /**
     * 串口号[1-10]
     */
    protected byte            commPort;
    /**
     * 波特率: [0:2400 ,1:4800 ,2:9600, 3:19200, 4:38400, 5:57600, 6:115200, 7:1200]
     */
    protected byte            baudRate;
    /**
     * 奇偶校验位: [0:无校验, 1:奇校验, 2:偶校验]
     */
    protected byte            parityBit;
    /**
     * 采集周期 2字节 (0-65535)秒
     */
    protected int             period;
    /**
     * 指令长度
     */
    protected short           commandLength;
    /**
     * 485通道号
     */
    protected byte            channel;

    /**
     * 数据类型(流量计,水质...)
     */
    protected short           dataType;

    /** 具体的指令,最长16个字节 */
    protected byte[]          command          = new byte[16];


    public String getCommandHex() {
        return MessageBeanUtil.toHexString(command, true);
    }

    public byte getCommandValid() {
        return commandValid;
    }

    public void setCommandValid(byte commandValid) {
        this.commandValid = commandValid;
    }

    public byte getCommPort() {
        return commPort;
    }

    public void setCommPort(byte commPort) {
        this.commPort = commPort;
    }

    public byte getBaudRate() {
        return baudRate;
    }

    public void setBaudRate(byte baudRate) {
        this.baudRate = baudRate;
    }

    public byte getParityBit() {
        return parityBit;
    }

    public void setParityBit(byte parityBit) {
        this.parityBit = parityBit;
    }

    public int getPeriod() {
        return period;
    }

    public void setPeriod(int period) {
        this.period = period;
    }

    public short getCommandLength() {
        return commandLength;
    }

    public void setCommandLength(short commandLength) {
        this.commandLength = commandLength;
    }

    public byte getChannel() {
        return channel;
    }

    public void setChannel(byte channel) {
        this.channel = channel;
    }

    public byte[] getCommand() {
        return command.clone();
    }

    public void setCommand(byte[] command) {
        this.command = command.clone();
    }

    public short getDataType() {
        return dataType;
    }

    public void setDataType(short dataType) {
        this.dataType = dataType;
    }

}
