package com.qhiot.transport.message.bean;

import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;

/**
 * 报警配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2018-05-22 10:56:35
 */
@Getter
@Setter
public class WarningConfig implements Serializable {

    private static final long serialVersionUID = -934458120013818998L;

    public static final int   DATA_LENGTH      = 33;
    public static final int   DATA_COUNT       = 200;
    /**
     * 指令有效位(1b)
     */
    protected byte            valid;
    /**
     * 配置序号(2b)
     */
    protected short           configIndex;

    /**
     * 寄存器1类型(1b)
     */
    protected byte            registerType1;
    /**
     * 寄存器1地址(2 byte)
     */
    protected short           registerAddress1;
    /**
     * 寄存器1解码方式(1 byte)
     */
    protected byte            method1;

    /**
     * 寄存器2类型(1b)
     */
    protected byte            registerType2;
    /**
     * 寄存器2地址(2 byte)
     */
    protected short           registerAddress2;
    /**
     * 寄存器2解码方式(1 byte)
     */
    protected byte            method2;

    /**
     * 参数连接关系(1 byte)
     */
    protected byte            paramRelation;

    /**
     * 报警条件1(1 byte)
     */
    protected byte            condition1;
    /**
     * 报警条件2(1 byte)
     */
    protected byte            condition2;

    /**
     * 参数连接关系(1 byte)
     */
    protected byte            conditionRelation;

    /**
     * 报警值1(4 byte)
     */
    protected float           value1;
    /**
     * 报警值1(4 byte)
     */
    protected float           value2;

    /**
     * 报警输出寄存器类型(1b)
     */
    protected byte            warningRegisterType;
    /**
     * 报警输出寄存器地址(2 byte)
     */
    protected short           warningRegisterAddress;
    /**
     * 报警输出寄存器解码方式(1 byte)
     */
    protected byte            warningMethod;

    /**
     * 采集周期 (2b) 0-65535
     */
    protected int             period;

    /**
     * 预留（4b）
     */
    protected byte[]          reserved4 = new byte[4];

    public boolean isAllFieldZero() {
        boolean allFieldZero = true;

        allFieldZero &= (valid == 0);
        allFieldZero &= (configIndex == 0);
        allFieldZero &= (registerType1 == 0);
        allFieldZero &= (registerAddress1 == 0);
        allFieldZero &= (method1 == 0);
        allFieldZero &= (registerType2 == 0);
        allFieldZero &= (registerAddress2 == 0);
        allFieldZero &= (method2 == 0);
        allFieldZero &= (paramRelation == 0);
        allFieldZero &= (condition1 == 0);
        allFieldZero &= (condition2 == 0);
        allFieldZero &= (conditionRelation == 0);
        allFieldZero &= (value1 == 0);
        allFieldZero &= (value2 == 0);
        allFieldZero &= (warningRegisterType == 0);
        allFieldZero &= (warningRegisterAddress == 0);
        allFieldZero &= (warningMethod == 0);
        allFieldZero &= (period == 0);

        return allFieldZero;
    }
}
