package com.qhiot.transport.message.define;

/**
 * Modbus 错误代码
 *
 * @author 金建强(ptma@163.com)
 * @version 2017-06-12 14:24
 */
public enum ModbusReturnCode {

    ILLEGAL_FUNCTION(0x01, "非法功能"),
    ILLEGAL_DATA_ADDRESS(0x02, "非法数据地址"),
    ILLEGAL_DATA_VALUE(0x03, "非法数据值"),
    SLAVE_DEVICE_FAILURE(0x04, "从站设备故障"),
    ACKNOWLEDGE(0x05, "确认"),
    SLAVE_DEVICE_BUSY(0x07, "从属设备忙"),
    MEMORY_PARITY_ERROR(0x08, "存储奇偶性差错"),
    GATEWAY_PATH_UNAVAILABLE(0x0A, "不可用终端路径"),
    GATEWAY_TARGET_DEVICE_FAILED_TO_RESPOND(0x0B, "目标设备响应失败"),

    UNKNOWN_EXCEPTION(0x100, "未知异常代码"),
    NO_EXCEPTION(0x00, "成功");

    private final int   value;
    private final String name;

    private ModbusReturnCode(int value, String name) {
        this.value = value;
        this.name = name;
    }

    @Override
    public String toString() {
        return name;
    }

    public static ModbusReturnCode valueOf(int value) {
        for (ModbusReturnCode type : ModbusReturnCode.values()) {
            if (type.value == value) {
                return type;
            }
        }
        return UNKNOWN_EXCEPTION;
    }

    public int getValue() {
        return value;
    }

    public String getName() {
        return name;
    }

}
