package com.qhiot.transport.message.define;

import java.io.Serializable;

/**
 * 固件更新回应代码
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-23 14:24
 */
public class UpgradeCode implements Serializable {

    private static final long serialVersionUID = -8053202494606857612L;

    /** 固件更新回应代码 - 允许更新 */
    public static final UpgradeCode ENABLED            = new UpgradeCode((byte) 0x01, "允许更新");
    /** 固件更新回应代码 - 对象错误 */
    public static final UpgradeCode TARGET_ERROR       = new UpgradeCode((byte) 0x02, "对象错误");
    /** 固件更新回应代码 - 代码太大 */
    public static final UpgradeCode CODE_TOO_LARGE     = new UpgradeCode((byte) 0x03, "代码太大");
    /** 固件更新回应代码 - 单片CRC错误 */
    public static final UpgradeCode SEGMENT_CRC_ERROR  = new UpgradeCode((byte) 0x04, "单片CRC错误");
    /** 固件更新回应代码 - 文件CRC错误 */
    public static final UpgradeCode FILE_CRC_ERROR     = new UpgradeCode((byte) 0x05, "文件CRC错误");
    /** 固件更新回应代码 - 终端类型错误 */
    public static final UpgradeCode GATEWAY_TYPE_ERROR = new UpgradeCode((byte) 0x06, "终端类型错误");

    public static final UpgradeCode ERROR              = new UpgradeCode((byte) 0x00, "错误");

    private static final UpgradeCode[] RESPONSES          = {ENABLED, TARGET_ERROR, CODE_TOO_LARGE,
            SEGMENT_CRC_ERROR, FILE_CRC_ERROR, GATEWAY_TYPE_ERROR};
    private final String               name;
    private final byte                 value;

    private UpgradeCode(byte value, String name) {
        this.value = value;
        this.name = name;
    }

    @Override
    public String toString() {
        return this.name;
    }

    public static UpgradeCode valueOf(byte value) throws IllegalArgumentException {
        if ((value < ENABLED.value) || (value > GATEWAY_TYPE_ERROR.value)) {
            return error("未知响应代码");
        }
        // 因为数值是从 1开始， 而数组索引是从0开始的， 所以索引比实际数值少1
        return RESPONSES[value - 1];
    }

    public static UpgradeCode error(String error) throws IllegalArgumentException {
        return new UpgradeCode((byte) 0x00, error);
    }

    public byte value() {
        return this.value;
    }

    @Override
    public boolean equals(Object object) {
        return object instanceof UpgradeCode && (((UpgradeCode) object).value == this.value);
    }

    public boolean equals(UpgradeCode status) {
        return status != null && (this.value == status.value);
    }

    @Override
    public int hashCode() {
        return this.value;
    }

}
