package com.qhiot.transport.message.util;

/**
 * 消息辅助工具
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/6/30 15:52
 */
public class MessageBeanUtil {

    public static final String  HEX_DIGITS  = "0123456789ABCDEF";

    /**
     * 字节数组转 GBK 字符串
     *
     * @param bytes 源字节数组
     * @return GBK字符串
     */
    public static String toGBKString(byte[] bytes) {
        String result;
        byte[] copyBytes = new byte[bytes.length];
        for (int i = 0, n = bytes.length; i < n; i++) {
            if (bytes[i] == 0x00) {
                copyBytes[i] = 0x20;
            } else {
                copyBytes[i] = bytes[i];
            }
        }
        try {
            result = new String(copyBytes, "GBK");
        } catch (Exception e) {
            result = new String(copyBytes);
        }
        return result;
    }



    /**
     * 字节数组转十六进制字符串
     *
     * @param bytes 源字节数组
     * @param space 是否用空格分隔
     * @return 返回十六进制字符串
     */
    public static String toHexString(byte[] bytes, boolean space) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0, n = bytes.length; i < n; i++) {
            byte b = bytes[i];
            if (space && (i > 0)) {
                sb.append(" ");
            }
            int high = (b & 0xf0) >> 4;
            int low = b & 0x0f;
            sb.append(HEX_DIGITS.charAt(high));
            sb.append(HEX_DIGITS.charAt(low));
        }
        return sb.toString();
    }

    /**
     * 克隆数组
     */
    public static byte[] clone(byte[] array) {
        if (array == null) {
            return null;
        }
        return array.clone();
    }

    /**
     * IP地址字符串转换为整数
     */
    public static long ip2Long(String ip) {
        String[] tokens = ip.split("[.]");
        if (tokens.length != 4) {
            throw new RuntimeException("'"+ip+"' 不合法的 IPv4 地址");
        }
        long result = 0;
        for (String token : tokens) {
            try {
                result = (result << 8) | Integer.parseInt(token);
            } catch (NumberFormatException e) {
                throw new RuntimeException("Incorrect number", e);
            }
        }
        return result;
    }

    /**
     * 整数转换为IP地址字符串
     */
    public static String long2Ip(long ip) {
        String result = String.valueOf((ip >> 24 & 0xff)) + "." +
            (ip >> 16 & 0xff) + "." +
            (ip >> 8 & 0xff) + "." +
            (ip & 0xff);

        return result;
    }

    /**
     * 字节转二进制字符串
     *
     * @param b 源字节
     * @return 二进制字符串
     */
    public static String toBinaryString(byte b) {
        return toBinaryString(new byte[] { b });
    }

    /**
     * 字节数组转二进制字符串
     *
     * @param bytes 源字节数组
     * @return 二进制字符串
     */
    public static String toBinaryString(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        int n = Byte.SIZE * bytes.length;
        for (int i = 0; i < n; i++) {
            sb.append((bytes[i / Byte.SIZE] << i % Byte.SIZE & 0x80) == 0 ? '0' : '1');
        }
        return sb.toString();
    }
}
