package com.qhiot.transport.codec.bean.v1;

import com.qhiot.core.util.ArrayUtil;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;

/**
 * 杭州亿脑基本协议(v20141113) 心跳包
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19.
 */
public class HeartbeatV1 {

    private int    deviceAddr;
    private int    timestamp;
    private int    version1;
    private int    version2;
    private int    version3;
    private byte[] reserve;

    public static HeartbeatV1 read(ByteBuf dataBuf) throws DecodeException {
        dataBuf.markReaderIndex();
        try {
            HeartbeatV1 heartbeat = new HeartbeatV1();
            heartbeat.deviceAddr = dataBuf.readIntLE();
            heartbeat.timestamp = dataBuf.readIntLE();
            heartbeat.version1 = dataBuf.readByte();
            heartbeat.version2 = dataBuf.readByte();
            heartbeat.version3 = dataBuf.readShortLE();
            heartbeat.reserve = ByteBufUtil.getBytesRemaining(dataBuf);
            //dataBuf.readBytes(heartbeat.reserve);
            return heartbeat;
        } catch (RuntimeException e) {
            dataBuf.resetReaderIndex();
            throw new DecodeException(e);
        }
    }

    public int getDeviceAddr() {
        return deviceAddr;
    }

    public void setDeviceAddr(int deviceAddr) {
        this.deviceAddr = deviceAddr;
    }

    public int getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(int timestamp) {
        this.timestamp = timestamp;
    }

    public int getVersion1() {
        return version1;
    }

    public void setVersion1(int version1) {
        this.version1 = version1;
    }

    public int getVersion2() {
        return version2;
    }

    public void setVersion2(int version2) {
        this.version2 = version2;
    }

    public int getVersion3() {
        return version3;
    }

    public void setVersion3(int version3) {
        this.version3 = version3;
    }

    public byte[] getReserve() {
        return ArrayUtil.clone(reserve);
    }

    public void setReserve(byte[] reserve) {
        this.reserve = ArrayUtil.clone(reserve);
    }
}
