package com.qhiot.transport.codec.bean.v2;

import com.qhiot.core.util.ArrayUtil;
import com.qhiot.core.util.CRCUtil;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;

import java.io.Serializable;

/**
 * 固件更新数据包（分片包）
 *
 * @author jjq
 */
public class FirmwareSegment implements Serializable {

    private static final long serialVersionUID = -766169114735375995L;

    /** 目标终端类型 */
    private DeviceType        deviceType;
    /** 目标终端编号 */
    private String            deviceId;
    /** 当前片序号 */
    private int               segmentIndex;
    /** 当前片CRC */
    private int               segmentCrc;
    /** 当前片数据 */
    private byte[]            segmentData;

    public FirmwareSegment(DeviceType deviceType, String deviceId, int segmentIndex, byte[] segmentData) {
        this.segmentIndex = segmentIndex;
        this.deviceType = deviceType;
        this.deviceId = deviceId;
        this.segmentData = segmentData==null ? null : segmentData.clone();
        segmentCrc = CRCUtil.crc16(segmentData);
    }

    public byte[] toBytes() {
        ByteBuf buf = ByteBufUtil.directBuffer();
        try {
            buf.writeByte(deviceType.getValue());
            buf.writeIntLE(Integer.parseInt(deviceId));
            buf.writeShortLE(segmentIndex);
            buf.writeShortLE(segmentCrc);
            buf.writeBytes(segmentData);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }

    public DeviceType getDeviceType() {
        return deviceType;
    }

    public FirmwareSegment setDeviceType(DeviceType deviceType) {
        this.deviceType = deviceType;
        return this;
    }

    public String getDeviceId() {
        return deviceId;
    }

    public FirmwareSegment setDeviceId(String deviceId) {
        this.deviceId = deviceId;
        return this;
    }

    public int getSegmentIndex() {
        return segmentIndex;
    }

    public void setSegmentIndex(int segmentIndex) {
        this.segmentIndex = segmentIndex;
    }

    public int getSegmentCrc() {
        return segmentCrc;
    }

    public void setSegmentCrc(int segmentCrc) {
        this.segmentCrc = segmentCrc;
    }

    public byte[] getSegmentData() {
        return ArrayUtil.clone(segmentData);
    }

    public void setSegmentData(byte[] segmentData) {
        this.segmentData = ArrayUtil.clone(segmentData);
    }
}
