package com.qhiot.transport.codec.cache;

import com.fasterxml.jackson.annotation.JsonIgnore;

import java.io.Serializable;
import java.util.Date;

/**
 * 图片抓拍容器
 *
 * @author 金建强(ptma@163.com)
 * @version 2017/4/13 20:07
 */
public class SnapCache implements Serializable {

    private static final long serialVersionUID = 7162003991689745213L;

    private String            deviceId;

    private int               cameraAddr;

    /** 块数 */
    private int               blockCount;

    /** 时间 */
    private Date              snapTime;
    /** 数据 */
    private byte[][]          data;

    public SnapCache(String deviceId, int cameraAddr, int blockCount, int timestamp) {
        this.deviceId = deviceId;
        this.cameraAddr = cameraAddr;
        this.blockCount = blockCount;
        snapTime = new Date(timestamp * 1000L);
        data = new byte[blockCount][];
    }

    /**
     * 添加块数据
     *
     * @param blockNumber
     * @param blockData
     */
    public void setBlockData(int blockNumber, byte[] blockData) {
        data[blockNumber - 1] = blockData;
    }

    @JsonIgnore
    public boolean isCompleted() {
        for (int i = data.length - 1; i >= 0; i--) {
            if (data[i] == null) {
                return false;
            }
        }
        return true;
    }

    /**
     * 已接收
     */
    public int completedBlock() {
        int blocks = 0;
        for (int i = 0, j = data.length; i < j; i++) {
            if (data[i] != null) {
                blocks++;
            }
        }
        return blocks;
    }

    @JsonIgnore
    public byte[] getBytes() {
        byte[] allBytes = new byte[0];
        for (int i = 0, j = data.length; i < j; i++) {
            if (data[i] != null) {
                if (allBytes.length == 0) {
                    allBytes = data[i];
                } else {
                    allBytes = combine(allBytes, data[i]);
                }
            }
        }
        return allBytes;
    }

    public static byte[] combine(byte[] a, byte[] b) {
        int alen = a.length;
        int blen = b.length;
        int clen = alen + blen;

        // 数组合并
        byte[] c = new byte[clen];
        System.arraycopy(a, 0, c, 0, alen);
        System.arraycopy(b, 0, c, alen, blen);
        return c;
    }

    public String getDeviceId() {
        return deviceId;
    }

    public int getBlockCount() {
        return blockCount;
    }

    public byte[][] getData() {
        return data;
    }

    public int getCameraAddr() {
        return cameraAddr;
    }

    public Date getSnapTime() {
        return snapTime;
    }

}
