package com.qhiot.transport.codec.decoder;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.CRCUtil;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFutureListener;

/**
 * 加码器基类， Spring Bean，提供一些常用的工具或服务类
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/10 22:04
 */
public abstract class BaseV2Decoder extends BaseDecoder {

    @Override
    public String version() {
        return "v2";
    }

    /**
     * 构造完整的回应报文
     *
     * @param pkg
     * @param commandFieldData
     * @return
     */
    protected byte[] encoderResponsePacklet(PacketV2 pkg, byte[] commandFieldData) {
        if (commandFieldData == null) {
            return new byte[0];
        }
        ByteBuf responseBuf = ByteBufUtil.directBuffer(PacketV2.MIN_LENGTH + commandFieldData.length);
        try {
            responseBuf.writeBytes(pkg.getPrefix());
            responseBuf.writeByte(pkg.getDeviceType());
            responseBuf.writeIntLE(Integer.parseInt(pkg.getDeviceId()));
            responseBuf.writeByte(pkg.getDstDevType());
            responseBuf.writeIntLE(pkg.getDstDevAddr());
            responseBuf.writeByte(pkg.getCommand());
            responseBuf.writeShortLE(commandFieldData.length);
            if (commandFieldData.length > 0) {
                responseBuf.writeBytes(commandFieldData);
            }

            // CRC校验数据域
            int crc;
            if (ByteUtil.equals(pkg.getPrefix(), PacketV2.PREFIX_STA)) {
                crc = CRCUtil.crc16CcittExt(commandFieldData);
            } else {
                crc = CRCUtil.crc16(commandFieldData);
            }
            responseBuf.writeShortLE(crc);
            responseBuf.writeBytes(PacketV2.SUFFIX_END);

            return ByteBufUtil.getBytesRemaining(responseBuf);
        } finally {
            ByteBufUtil.safeRelease(responseBuf);
        }
    }

    /**
     * 写回应
     * @param channel
     * @param packetV2 PacketV2
     * @param commandFieldData 命令域数据
     */
    protected void writeResponseV2(Channel channel, PacketV2 packetV2, byte[] commandFieldData) {
        byte[] responsePacket = encoderResponsePacklet(packetV2, commandFieldData);
        if (responsePacket != null && responsePacket.length > 0) {
            ByteBuf frames = ByteBufUtil.directBuffer(responsePacket.length);
            frames.writeBytes(responsePacket);
            channel.writeAndFlush(frames).addListener(ChannelFutureListener.CLOSE_ON_FAILURE);

            super.printDownstreamMessageToLogTopic(packetV2, "平台回应[{}] = {}", responsePacket.length, ByteUtil.toHexString(responsePacket, true));
        }
    }
}
