package com.qhiot.transport.codec.decoder;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 协议解析器接口<br/>
 * 如果协议有多层的结构时， 建议在解码器中增加一个 DecoderChain 来处理
 * @author 金建强(ptma@163.com)
 */
public interface Decoder extends Cloneable {

    /**
     * <P>
     * 具体解码执行的方法, 子类根据各自的解码规则实现, 不能抛出异常, 否则会导致后续解码器失效
     * </P>
     * <p>
     * 回应: 各解码只需负责各自对应部分的回应即可, 整个 DecoderChain 树中各层负责拼接或整合,
     * <b>return null 表示无需回应</b>
     * </p>
     *
     * @param channel TCP会话通道
     * @param dataBuf 缓冲区
     * @param context 上下文
     * @return 回应数据
     */
    byte[] decode(final Channel channel, ByteBuf dataBuf, final ChainContext context, final DecoderChain chain);

}
