package com.qhiot.transport.codec.decoder.v1;

import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.bean.v1.HeartbeatV1;
import com.qhiot.transport.codec.decoder.BaseV1Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 杭州亿脑基本协议(v20141113) 心跳包
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class HeartbeatV1Decoder extends BaseV1Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf dataBuf, ChainContext context, DecoderChain chain) {
        dataBuf.markReaderIndex();
        try {
            final PacketV1 packetV1 = (PacketV1) context.get(ChainContext.PACKET_V1);
            if (packetV1 == null || packetV1.getDataType() != 0x00) {
                return chain.doDecode(channel, dataBuf, context);
            }

            final HeartbeatV1 heartbeat = HeartbeatV1.read(dataBuf);

            // 判断是否是无线传感器小板
            PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null) {
                printAnalyzeMessageToLogTopic(packetV1, "心跳: {}", packetV1.getDeviceId());
            } else {
                printAnalyzeMessageToLogTopic(packetV1, "心跳: {} - {}", DeviceType.nameOf(packetV2.getDeviceType()), packetV1.getDeviceId());
            }

            return encoderResponsePacklet(packetV1, heartbeat);
        } catch (DecodeException e) {
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }
    }

    /**
     * 只需要回复数据域部分
     */
    private byte[] encoderResponsePacklet(PacketV1 packetV1, HeartbeatV1 heartbeat) {
        if (heartbeat == null) {
            return null;
        }
        ByteBuf repDataBuf = ByteBufUtil.directBuffer(19);
        try {
            repDataBuf.writeByte((byte) 0x80);
            repDataBuf.writeShortLE(16);
            repDataBuf.writeIntLE(heartbeat.getDeviceAddr());
            repDataBuf.writeIntLE(DateUtil.getUTCSeconds());
            repDataBuf.writeByte(heartbeat.getVersion1());
            repDataBuf.writeByte(heartbeat.getVersion2());
            repDataBuf.writeShortLE(heartbeat.getVersion3());
            repDataBuf.writeBytes(heartbeat.getReserve());

            return ByteBufUtil.getBytesRemaining(repDataBuf);
        } finally {
            ByteBufUtil.safeRelease(repDataBuf);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }

}
