package com.qhiot.transport.codec.decoder.v1.sensor;

import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.codec.decoder.BaseV1Decoder;

/**
 * 传感器解码器基类，包含了共用的一些方法和算法
 *
 * @author 金建强(ptma @ 163.com)
 * @version 2020/1/10 11:54
 */
public abstract class BaseSensorDecoder extends BaseV1Decoder {

    /**
     * 时间戳计算, 节点传上来的时间戳是1/4秒为单位的，所以要进行转换
     *
     * @param timestamp 节点传上来的时间戳
     * @return
     */
    public int calcTimestamp(int timestamp) {
        int localTime = DateUtil.getUTCSeconds();
        return ((timestamp >> 2) & 0x3fffffff) | (localTime & 0xc0000000);
    }

    /**
     * 传感器电池电量的计算
     *
     * @param p 为数据包中1字节的电量数据
     * @return
     */
    public double calcBattery(short p) {
        return p / 256d * 2.56d * 51d / 18d;
    }
}
