package com.qhiot.transport.codec.decoder.v1.sensor;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.NumberUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.decoder.v1.SensorDataDecoder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 3. CO2传感器(探头：COZIR)
 *
 * <p>
 * 上一级解码器 {@link SensorDataDecoder}
 * </p>
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-07-28 12:14
 */
public class Co2CozirDecoder extends BaseSensorDecoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf dataBuf, ChainContext context,
                         DecoderChain chain) {
        final PacketV1 packetV1 = (PacketV1) context.get(ChainContext.PACKET_V1);
        if (packetV1 == null) {
            return chain.doDecode(channel, dataBuf, context);
        }

        dataBuf.markReaderIndex();

        byte d1 = dataBuf.readByte();
        byte type = (byte) ByteUtil.getHigh4(d1);
        byte pkgLenOrJump = (byte) ByteUtil.getLow4(d1);
        byte dataType = dataBuf.readByte();

        if (type != 0x01 || dataType != 0x00) {
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }
        int nodeId = dataBuf.readUnsignedShortLE();
        int timestamp = calcTimestamp(dataBuf.readIntLE());

        byte d9 = dataBuf.readByte();
        byte d10 = dataBuf.readByte();
        byte d11 = dataBuf.readByte();
        short d12 = dataBuf.readUnsignedByte();
        short d13 = dataBuf.readUnsignedByte();
        byte d14 = dataBuf.readByte();
        byte d15 = dataBuf.readByte();

        int co2 = d12 + d13 * 256; // ppm

        double battery = calcBattery(dataBuf.readUnsignedByte());
        String sB = NumberUtil.toString(battery, 2);
        String dateStr = DateUtil.formatUTC(timestamp);
        logger.debug("CO₂传感器:{} [{}], 电量={}, CO₂={}ppm", nodeId, dateStr, sB, co2);
        return null;
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
