package com.qhiot.transport.codec.decoder.v1.sensor;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.NumberUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.decoder.v1.SensorDataDecoder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 11.	光照传感器(探头：新叶光照)
 *
 * <p>
 * 上一级解码器 {@link SensorDataDecoder}
 * </p>
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-07-28 12:14
 */
public class IlluminationDecoder extends BaseSensorDecoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf dataBuf, ChainContext context, DecoderChain chain) {
        final PacketV1 packetV1 = (PacketV1) context.get(ChainContext.PACKET_V1);
        if (packetV1 == null) {
            return chain.doDecode(channel, dataBuf, context);
        }
        dataBuf.markReaderIndex();

        byte d1 = dataBuf.readByte();
        byte type = (byte) ByteUtil.getHigh4(d1);
        byte pkgLenOrJump = (byte) ByteUtil.getLow4(d1);
        byte dataType = dataBuf.readByte();

        if (type != 0x03 || dataType != 0x24) {
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }
        int nodeId = dataBuf.readUnsignedShortLE();
        int timestamp = calcTimestamp(dataBuf.readIntLE());

        float d9 = dataBuf.readFloat();
        byte[] d13_15 = new byte[3];
        dataBuf.readBytes(d13_15);

        double battery = calcBattery(dataBuf.readUnsignedByte());
        String sB = NumberUtil.toString(battery, 2);
        String dateStr = DateUtil.formatUTC(timestamp);

        logger.debug("光照传感器:{} [{}], 电量={}, 照度={}lx", nodeId, dateStr, sB, d9);
        return null;
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
