package com.qhiot.transport.codec.decoder.v1.sensor;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.NumberUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.decoder.v1.SensorDataDecoder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 2. SHT21/SHT25温湿度传感器
 *
 * <p>
 * 上一级解码器 {@link SensorDataDecoder}
 * </p>
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-07-28 12:14
 */
public class SHT2xDecoder extends BaseSensorDecoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf dataBuf, ChainContext context, DecoderChain chain) {
        final PacketV1 packetV1 = (PacketV1) context.get(ChainContext.PACKET_V1);
        if (packetV1 == null) {
            return chain.doDecode(channel, dataBuf, context);
        }

        dataBuf.markReaderIndex();

        byte d1 = dataBuf.readByte();
        byte type = (byte) ByteUtil.getHigh4(d1);
        byte pkgLenOrJump = (byte) ByteUtil.getLow4(d1);
        byte dataType = dataBuf.readByte();

        if (type != 0x00 || dataType != 0x01) {
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }
        int nodeId = dataBuf.readUnsignedShortLE();
        int timestamp = calcTimestamp(dataBuf.readIntLE());

        // 湿度：(D9+D10*256)/65536*125-6;
        // 温度：(D11+D12*256)/65536*175.72-46.85;

        int d9_d10 = dataBuf.readUnsignedShortLE();
        int d11_d12 = dataBuf.readUnsignedShortLE();
        short d13 = dataBuf.readUnsignedByte();
        short d14 = dataBuf.readUnsignedByte();
        short d15 = dataBuf.readUnsignedByte();

        double h = (double) d9_d10 / 65536d * 125d - 6;
        double t = (double) d11_d12 / 65536d * 175.72d - 46.85d;

        String sH = NumberUtil.toString(h, 2);
        String sT = NumberUtil.toString(t, 2);

        double battery = calcBattery(dataBuf.readUnsignedByte());
        String sB = NumberUtil.toString(battery, 2);
        String dateStr = DateUtil.formatUTC(timestamp);

        logger.debug("SHT21/SHT25传感器:{} [{}], 电量={}, 温度={}℃, 湿度={}%", nodeId, dateStr, sB, sT, sH);

        return null;
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
