package com.qhiot.transport.codec.decoder.v1.sensor;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.NumberUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.decoder.v1.SensorDataDecoder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 1. SHT75/SHT1xx温湿度传感器
 *
 * <p>
 * 上一级解码器 {@link SensorDataDecoder}
 * </p>
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-07-28 12:14
 */
public class SHT75Decoder extends BaseSensorDecoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf dataBuf, ChainContext context, DecoderChain chain) {
        final PacketV1 packetV1 = (PacketV1) context.get(ChainContext.PACKET_V1);
        if (packetV1 == null) {
            return chain.doDecode(channel, dataBuf, context);
        }

        dataBuf.markReaderIndex();

        byte d1 = dataBuf.readByte();
        byte type = (byte) ByteUtil.getHigh4(d1);
        byte pkgLenOrJump = (byte) ByteUtil.getLow4(d1);
        byte dataType = dataBuf.readByte();

        if (type != 0x00 || dataType != 0x00) {
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }
        int nodeId = dataBuf.readUnsignedShortLE();
        int timestamp = calcTimestamp(dataBuf.readIntLE());

        short d9 = dataBuf.readUnsignedByte();
        short d10 = dataBuf.readUnsignedByte();
        short d11 = dataBuf.readUnsignedByte();
        short d12 = dataBuf.readUnsignedByte();
        short d13 = dataBuf.readUnsignedByte();
        short d14 = dataBuf.readUnsignedByte();
        short d15 = dataBuf.readUnsignedByte();

        int h = (d11 & 0x0F - 0x10) * 16 + d9;
        int t = (d11 & 0x0F) * 256 + d10;

        double tt = -39.66d + 0.04d * t;

        double h_liner = -4d + 0.0405d * h + Math.pow(2.8d, -6) * h * h;
        double hh = (tt - 25d) * (0.04d + Math.pow(8, -5) * h) + h_liner;

        String sH = NumberUtil.toString(hh, 2);
        String sT = NumberUtil.toString(tt, 2);

        double battery = calcBattery(dataBuf.readUnsignedByte());
        String sB = NumberUtil.toString(battery, 2);
        String dateStr = DateUtil.formatUTC(timestamp);

        logger.debug("SHT75/SHT1xx传感器:{} [{}], 电量={}, 温度={}℃, 湿度={}%",
                nodeId, dateStr, sB, sT, sH);

        return null;
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
