package com.qhiot.transport.codec.decoder.v1.sensor;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.NumberUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.decoder.v1.SensorDataDecoder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 5 土壤温度传感器（探头：DS18B20）
 *
 * <p>
 * 上一级解码器 {@link SensorDataDecoder}
 * </p>
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-07-28 12:14
 */
public class SoilTemperatureDecoder extends BaseSensorDecoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf dataBuf, ChainContext context, DecoderChain chain) {
        final PacketV1 packetV1 = (PacketV1) context.get(ChainContext.PACKET_V1);
        if (packetV1 == null) {
            return chain.doDecode(channel, dataBuf, context);
        }

        dataBuf.markReaderIndex();

        byte d1 = dataBuf.readByte();
        byte type = (byte) ByteUtil.getHigh4(d1);
        byte pkgLenOrJump = (byte) ByteUtil.getLow4(d1);
        byte dataType = dataBuf.readByte();

        if (type != 0x03 || dataType != 0x03) {
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }
        int nodeId = dataBuf.readUnsignedShortLE();
        int timestamp = calcTimestamp(dataBuf.readIntLE());

        short d9 = dataBuf.readUnsignedByte();
        short d10 = dataBuf.readUnsignedByte();
        short d11 = dataBuf.readUnsignedByte();
        byte[] d12_15 = new byte[4];
        dataBuf.readBytes(d12_15);

        double t = (double) ((d11 & 0x07) * 256d + d10) * 0.0625d * (d9 * 2 - 1)* (-1);
        String sT = NumberUtil.toString(t, 2);

        double battery = calcBattery(dataBuf.readUnsignedByte());
        String sB = NumberUtil.toString(battery, 2);
        String dateStr = DateUtil.formatUTC(timestamp);

        logger.debug("土壤温度传感器:{} [{}], 电量={}, 温度={}℃", nodeId, dateStr, sB, sT);
        return null;
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
