package com.qhiot.transport.codec.decoder.v1.sensor;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.NumberUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.codec.decoder.v1.SensorDataDecoder;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.text.DecimalFormat;

/**
 * 42. TMP102 温度传感器
 *
 * <p>
 * 上一级解码器 {@link SensorDataDecoder}
 * </p>
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-07-28 12:14
 */
public class TMP102Decoder extends BaseSensorDecoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf dataBuf, ChainContext context, DecoderChain chain) {
        final PacketV1 packetV1 = (PacketV1) context.get(ChainContext.PACKET_V1);
        if (packetV1 == null) {
            return chain.doDecode(channel, dataBuf, context);
        }

        dataBuf.markReaderIndex();

        byte d1 = dataBuf.readByte();
        byte type = (byte) ByteUtil.getHigh4(d1);
        byte pkgLenOrJump = (byte) ByteUtil.getLow4(d1);
        byte dataType = dataBuf.readByte();

        if (type != 0x03 || dataType != 0x40) {
            dataBuf.resetReaderIndex();
            return chain.doDecode(channel, dataBuf, context);
        }
        int nodeId = dataBuf.readUnsignedShortLE();
        int timestamp = calcTimestamp(dataBuf.readIntLE());

        short d9 = dataBuf.readUnsignedByte();
        int d10_d11 = dataBuf.readUnsignedShortLE();
        short d12 = dataBuf.readUnsignedByte();
        short d13 = dataBuf.readUnsignedByte();
        short d14 = dataBuf.readUnsignedByte();
        short d15 = dataBuf.readUnsignedByte();

        double t = (d9 * 2 - 1) * (-1) * d10_d11 / 100d;

        DecimalFormat df = new DecimalFormat("#0.00");
        String sT = NumberUtil.toString(t, 2);

        double battery = calcBattery(dataBuf.readUnsignedByte());
        String sB = NumberUtil.toString(battery, 2);
        String dateStr = DateUtil.formatUTC(timestamp);

        logger.debug("TMP102传感器:{} [{}], 电量={}, 温度={}℃", nodeId, dateStr, sB, sT);
        return null;
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
