package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.response.AtCommandResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 *  AT 指令回应
 *
 * @author 金建强(ptma@163.com)
 * @version 2017/4/19 15:58
 */
public class AtCommandDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || packetV2.getCommand() != Command.AT_COMMAND_REPORT ) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            String report = new String(packetV2.getCommandData());

            printAnalyzeMessageToLogTopic(packetV2, "AT指令回应: {}", report);
            AtCommandResponse response = new AtCommandResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
            sendTransportResponseToTopic(response);

            return new byte[0];
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
