package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.response.camrea.SnapResponse;
import com.qhiot.transport.message.response.camrea.SnapStopUploadResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 摄像头拍照指令解析
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19.
 */
public class CameraSnapDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.CAM_SNAP
                    && packetV2.getCommand() != Command.CAM_SNAP_FAIL && packetV2.getCommand() != Command.CAM_SNAP_STOP_UPLOADING)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.CAM_SNAP:
                    SnapResponse response = new SnapResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    response.setSucceed(Boolean.TRUE);
                    super.sendTransportResponseToTopic(response);

                    printAnalyzeMessageToLogTopic(packetV2, "摄像头拍照成功");
                    break;
                case Command.CAM_SNAP_FAIL:
                    SnapResponse failResponse = new SnapResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    failResponse.setSucceed(Boolean.FALSE);
                    super.sendTransportResponseToTopic(failResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "摄像头拍照失败");
                    break;
                case Command.CAM_SNAP_STOP_UPLOADING:
                    SnapStopUploadResponse stopResponse = new SnapStopUploadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(stopResponse);

                    printAnalyzeMessageToLogTopic(packetV2, "停止上传照片");
                    break;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
