package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.ArrayUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.data.CoulometerData;
import com.qhiot.transport.message.response.io.CoulometerDataResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 电量计 (0x15 0x16)
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/21 19:08
 */
public class CoulometerDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || (packetV2.getCommand() != Command.COULOMETER_DATA
                && packetV2.getCommand() != Command.COULOMETER_DATA_READ)) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            int dataCount = packetV2.getCommandDataLength() / 4;
            int[] values = new int[dataCount];
            final CoulometerData coulometerData = new CoulometerData(packetV2.getDeviceType(), packetV2.getDeviceId());
            CoulometerDataResponse response = new CoulometerDataResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
            for(int i=0;i<dataCount;i++){
                int value = commandDataBuf.readIntLE();
                values[i] = value;
                coulometerData.setValue(i, value);
                response.setValue(i, value);
            }
//            super.sendTransportDataToQueueAndTopic(coulometerData);
            super.sendCoulometerData(coulometerData);
            super.sendTransportResponseToTopic(response);

            super.printAnalyzeMessageToLogTopic(packetV2, "电量计: [{}]", ArrayUtil.toString(values));

            return new byte[0];
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
