package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.DeviceParam;
import com.qhiot.transport.message.bean.DeviceParamBuilder;
import com.qhiot.transport.message.bean.GatewayParam;
import com.qhiot.transport.message.bean.GatewayParamBuilder;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.response.DeviceParamResponse;
import com.qhiot.transport.message.response.GatewayParamResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 终端信息读取(0x02)
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19.
 */
public class DeviceParamReadDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.READ_DEV_PARAM) {
                return chain.doDecode(channel, commandDataBuf, context);
            }
            if (!DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                && !DeviceType.EXCITER.equals(packetV2.getDeviceType())
                && !DeviceType.REPEATER.equals(packetV2.getDeviceType()) ) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            if (DeviceType.GATEWAY.equals(packetV2.getDeviceType())) {
                final GatewayParam gatewayParam = GatewayParamBuilder.read(commandDataBuf);
                printAnalyzeMessageToLogTopic(packetV2, "读终端信息回应: {}={}", packetV2.getDeviceName(), packetV2.getDeviceId());
                GatewayParamResponse response = new GatewayParamResponse(packetV2.getDeviceId());
                response.setGatewayParam(gatewayParam);
                sendTransportResponseToTopic(response);
            } else if (DeviceType.EXCITER.equals(packetV2.getDeviceType())
                       || DeviceType.REPEATER.equals(packetV2.getDeviceType())) {

                final DeviceParam deviceParam = DeviceParamBuilder.read(commandDataBuf);
                printAnalyzeMessageToLogTopic(packetV2, "读终端信息回应: {}={}", packetV2.getDeviceName(), packetV2.getDeviceId());
                DeviceParamResponse response = new DeviceParamResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                response.setDeviceParam(deviceParam);
                sendTransportResponseToTopic(response);
            } else {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
