package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.response.DeviceWriteResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 终端信息写入(0x03)、终端信息低级写入(0xFD)
 * <p>
 * User: 金建强(ptma@163.com)
 * </p>
 * <p>
 * Date: 2015-12-17 16:19.
 * </p>
 */
public class DeviceParamWriteDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.WRITE_DEV_PARAM && packetV2.getCommand() != Command.LOW_LEVEL_WRITE)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            if (!DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                && !DeviceType.EXCITER.equals(packetV2.getDeviceType())
                && !DeviceType.REPEATER.equals(packetV2.getDeviceType())) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            if (DeviceType.GATEWAY.equals(packetV2.getDeviceType())) {
                printAnalyzeMessageToLogTopic(packetV2, "写终端信息回应: {}={}", packetV2.getDeviceName(), packetV2.getDeviceId());
                sendTransportResponseToTopic(new DeviceWriteResponse(packetV2.getDeviceType(),packetV2.getDeviceId()));

            } else if (DeviceType.EXCITER.equals(packetV2.getDeviceType())
                       || DeviceType.REPEATER.equals(packetV2.getDeviceType())) {

                printAnalyzeMessageToLogTopic(packetV2, "写终端信息回应: {}={}", packetV2.getDeviceName(), packetV2.getDeviceId());
                sendTransportResponseToTopic(new DeviceWriteResponse(packetV2.getDeviceType(),packetV2.getDeviceId()));

            } else {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
