package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.response.DeviceFormatResponse;
import com.qhiot.transport.message.response.DeviceRebootResponse;
import com.qhiot.transport.message.response.DeviceRestoreResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 终端重启回应(0x0C)、恢复出厂设置回应(0x0D)、低级格式化回应(0xFE)
 * <p>
 * User: 金建强(ptma@163.com)
 * </p>
 * <p>
 * Date: 2015-12-17 16:19.
 * </p>
 */
public class DeviceRebootDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.REBOOT && packetV2.getCommand() != Command.RESTORE && packetV2.getCommand() != Command.LOW_LEVEL_FORMAT)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            if (!DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                && !DeviceType.EXCITER.equals(packetV2.getDeviceType())
                && !DeviceType.REPEATER.equals(packetV2.getDeviceType())) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            if (packetV2.getCommand() == Command.REBOOT) {
                printAnalyzeMessageToLogTopic(packetV2, "重启终端回应: {}={}", packetV2.getDeviceName(),
                             packetV2.getDeviceId());
                sendTransportResponseToTopic(new DeviceRebootResponse(packetV2.getDeviceType(),packetV2.getDeviceId()));
            } else if (packetV2.getCommand() == Command.RESTORE) {
                printAnalyzeMessageToLogTopic(packetV2, "恢复出厂设置回应: {}={}", packetV2.getDeviceName(),
                             packetV2.getDeviceId());
                sendTransportResponseToTopic(new DeviceRestoreResponse(packetV2.getDeviceType(),packetV2.getDeviceId()));
            } else if (packetV2.getCommand() == Command.LOW_LEVEL_FORMAT) {
                printAnalyzeMessageToLogTopic(packetV2, "低级格式化回应: {}={}", packetV2.getDeviceName(),
                             packetV2.getDeviceId());
                sendTransportResponseToTopic(new DeviceFormatResponse(packetV2.getDeviceType(),packetV2.getDeviceId()));
            } else {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
