package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.db.model.TransportWarning;
import com.qhiot.transport.db.service.TransportWarningService;
import com.qhiot.transport.influxdb.model.TransportWarningInflux;
import com.qhiot.transport.influxdb.service.TransportWarningInfluxService;
import com.qhiot.transport.jms.kafka.TransportWarningKafkaSender;
import com.qhiot.transport.message.data.ElectricCabinetData;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 电控箱开合状态上报(0x11)
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/21 19:27
 */
public class ElectricCabinetDecoder extends BaseV2Decoder {

    @Autowired
    private TransportWarningService transportWarningService;

    @Autowired
    private TransportWarningInfluxService transportWarningInfluxService;

//    @Autowired
//    private InsightWarningQueueSender insightWarningQueueSender;

    /**---kafka---*/
    @Autowired
    private TransportWarningKafkaSender transportWarningKafkaSender;
    /**------*/

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.ELECTRIC_CABINET) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            // 0:非法打开 1:合法打开 2:关闭
            int state = commandDataBuf.readByte();
            printAnalyzeMessageToLogTopic(packetV2, "电控箱状态: 状态={}", state);

            final ElectricCabinetData data = new ElectricCabinetData(packetV2.getDeviceId());
            data.setState(state);
//            super.sendTransportDataToQueueAndTopic(data);
            super.sendElectricCabinetData(data);
            if (state == 0) {
                ThreadUtil.submit(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            TransportWarning warning = new TransportWarning();

                            warning.setDeviceId(data.getDeviceId());
                            warning.setWarningCode(TransportWarning.WarningCode.ELECTRIC_CABINET);
                            warning.setWarning("电控箱非法打开");
                            warning.setWarningCount(1);
                            warning.setFirstTime(data.getMessageTime());
                            warning.setLatestTime(data.getMessageTime());

//                            transportWarningService.insertSelective(warning);
                            TransportWarningInflux warningInflux = new TransportWarningInflux(warning);
                            transportWarningInfluxService.insert(warningInflux);

//                            if (insightWarningQueueSender != null) {
//                                insightWarningQueueSender.sendMessage(warning);
//                            }
                            transportWarningKafkaSender.sendMessage(warning);

                        } catch (Exception e) {
                            logger.warn("电控箱非法打开数据出错: ", e.getMessage(), e);
                        }
                    }
                });
            }

            return new byte[0];
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
