package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.codec.util.UpgradeQueue;
import com.qhiot.transport.codec.util.UpgradeQueueManager;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.define.UpgradeCode;
import com.qhiot.transport.message.response.UpgradeResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 固件更新请求包、固件更新数据包回应
 *
 * @author 金建强(ptma@163.com)
 * @version 2016 -07-01 11:11:49
 */
public class FirmwareUpgradeDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || (packetV2.getCommand() != Command.FIRMWARE_UPGRADE)
                    && (packetV2.getCommand() != Command.FIRMWARE_DATA)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }
            if (!DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                    && !DeviceType.EXCITER.equals(packetV2.getDeviceType())
                    && !DeviceType.REPEATER.equals(packetV2.getDeviceType())) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            byte code = commandDataBuf.readByte();
            UpgradeResponse upgradeResponse = new UpgradeResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
            upgradeResponse.setUpgradeCode(code);

            String queueTemplate;
            UpgradeCode upgradeCode = UpgradeCode.valueOf(code);

            if (packetV2.getCommand() == Command.FIRMWARE_UPGRADE) {
                queueTemplate = UpgradeQueue.FIRMWARE_REQUEST_QUEUE;
                printAnalyzeMessageToLogTopic(packetV2, "固件升级请求回应: {}", upgradeCode.toString());
            } else {
                queueTemplate = UpgradeQueue.FIRMWARE_SEGMENT_QUEUE;
                if (packetV2.getCommandDataLength() == 3) {
                    int index = commandDataBuf.readUnsignedShortLE();
                    upgradeResponse.setSegmentIndex(index);
                    printAnalyzeMessageToLogTopic(packetV2, "固件升级分片回应: {}, 分片序号={}", upgradeCode.toString(), index);
                } else {
                    printAnalyzeMessageToLogTopic(packetV2, "固件升级分片回应: {}", upgradeCode.toString());
                }
            }

            UpgradeQueue queue = UpgradeQueueManager.getAndClearQueue(queueTemplate, packetV2.getDeviceId());
            queue.offer(upgradeResponse);

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
