package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.response.GprsBoardParamResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * GPRS-GPS 小板上报参数 (0x02)
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19.
 */
public class GprsBoardParamDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.READ_DEV_PARAM) {
                return chain.doDecode(channel, commandDataBuf, context);
            }
            if (!DeviceType.GPRS_GPS.equals(packetV2.getDeviceType())) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            // IMEI(字符型，15个字节)
            byte[] imeiData = new byte[15];
            commandDataBuf.readBytes(imeiData);
            final String imei = ByteUtil.toAnsi(imeiData);

            // ICCID(字符型，20个字节)
            byte[] iccidData = new byte[20];
            commandDataBuf.readBytes(iccidData);
            final String iccid = ByteUtil.toAnsi(iccidData).toUpperCase();

            printAnalyzeMessageToLogTopic(packetV2, "定位小板: IMEI={}, ICCID={}", imei, iccid);

            GprsBoardParamResponse response = new GprsBoardParamResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
            response.setImei(imei);
            response.setIccid(iccid);
            sendTransportResponseToTopic(response);

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
