package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.data.MonitoringHistoryData;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.response.io.IOWorkmodelReadResponse;
import com.qhiot.transport.message.response.io.IOWorkmodelWriteResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 输入输出运行模式
 *
 * @author 金建强(ptma 163.com)
 * @version 2016 -07-01 09:41:19
 */
public class IOWorkModelDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || !DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                || (packetV2.getCommand() != Command.IO_WORK_MODEL_READ && packetV2.getCommand() != Command.IO_WORK_MODEL_WRITE)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.IO_WORK_MODEL_READ: // 读取运行模式
                    // 0: 自动运行模式
                    // 1: 手动运行模式
                    // 2: 停止模式
                    int workModel = commandDataBuf.readByte();

                    IOWorkmodelReadResponse readResponse = new IOWorkmodelReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setWorkModel(workModel);
                    sendTransportResponseToTopic(readResponse);

                    // 模拟为检测数据， 以便发送到前端监听的 websocket
                    MonitoringHistoryData monitoringData = new MonitoringHistoryData(packetV2.getDeviceType(), packetV2.getDeviceId());
                    monitoringData.setMonitoringId(-1L);
                    monitoringData.setValue(Double.valueOf(workModel));
                    monitoringData.setDataTime(DateUtil.now());
//                    super.sendTransportDataToQueueAndTopic(monitoringData);
                    super.sendMonitoringDataHistory(monitoringData);
                    super.printAnalyzeMessageToLogTopic(packetV2, "读取运行模式回应: 模式={}", workModel);
                    break;
                case Command.IO_WORK_MODEL_WRITE: // 设置运行模式
                    IOWorkmodelWriteResponse writeRespose = new IOWorkmodelWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(writeRespose);

                    printAnalyzeMessageToLogTopic(packetV2, "写入运行模式回应");
                    break;

                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
