package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.data.IccidData;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * ICCID 上报 ICCID 0x18
 *
 * @author 金建强(ptma@163.com)
 * @version 2016 -07-01 02:36:00
 */
public class IccidDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.ICCID) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }
            // ICCID(字符型，20个字节)
            byte[] iccidBytes = new byte[20];
            commandDataBuf.readBytes(iccidBytes);
            final String iccid = ByteUtil.toAnsi(iccidBytes).toUpperCase();

            super.printAnalyzeMessageToLogTopic(packetV2, "ICCID上报：ICCID={}", iccid);
            IccidData iccidData = new IccidData(packetV2.getDeviceId());
            iccidData.setIccid(iccid);
//            super.sendTransportDataToQueueAndTopic(iccidData);
            super.sendIccidData(iccidData);
            return new byte[0];
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
