package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.response.ModbusDataWriteResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * Xodbus 参数写入回应
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19.
 */
public class ModbusDataWriteDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context,
                         DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || (packetV2.getCommand() != Command.MODBUS_DATA_WRITE)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            ModbusDataWriteResponse response =
                    new ModbusDataWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
            response.setSucceed(Boolean.TRUE);
            super.sendTransportResponseToTopic(response);

            printAnalyzeMessageToLogTopic(packetV2, "Modubs 参数写入成功");


            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
