package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.define.ModbusReturnCode;
import com.qhiot.transport.message.response.io.ModbusIOControlResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * modbus指令控制输出IO 回应
 *
 * @author 金建强(ptma 163.com)
 * @version 2016 -07-01 09:41:19
 */
public class ModbusIOControlDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || !DeviceType.GATEWAY.equals(packetV2.getDeviceType())
                || packetV2.getCommand() != Command.MODBUS_IO_CONTROL) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            ModbusIOControlResponse readResponse = new ModbusIOControlResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
            if (packetV2.getCommandDataLength()==8) {//正常原样返回 8 byte
                readResponse.setReturnCode(ModbusReturnCode.NO_EXCEPTION);
            } else if (packetV2.getCommandDataLength()==5) {//异常
                byte deviceAddr = commandDataBuf.readByte();
                byte funCode = commandDataBuf.readByte();
                byte exceptionCode = commandDataBuf.readByte();
                int crc = commandDataBuf.readUnsignedShortLE();

                readResponse.setReturnCode(ModbusReturnCode.valueOf(exceptionCode));
            }

            sendTransportResponseToTopic(readResponse);
            printAnalyzeMessageToLogTopic(packetV2, "Modbus指令控制输出IO回应: 结果={}", readResponse.getReturnCode().getName());

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
