package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.OnboardTag;
import com.qhiot.transport.message.bean.OnboardTagBuilder;
import com.qhiot.transport.message.data.OnboardTagData;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 车载标签上报(0x05)
 * <P>
 * User: 金建强(ptma@163.com)
 * </P>
 * <p>
 * Date: 2016/3/21 16:45
 * </p>
 */
public class OnboardTagDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.ONBOARD_TAG
                || !DeviceType.ONBOARD_TAG.equals(packetV2.getDeviceType())) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }
            final OnboardTag onboardTag = OnboardTagBuilder.read(commandDataBuf);

            printAnalyzeMessageToLogTopic(packetV2, "车载标签：标签={}", onboardTag.getTagId());

            OnboardTagData data = new OnboardTagData(packetV2.getDeviceType(), packetV2.getDeviceId());
            data.setOnboardTag(onboardTag);
//            sendTransportDataToQueueAndTopic(data);
            sendOnboardTagData(data);

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
