package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.OutputPeriodConfig;
import com.qhiot.transport.message.bean.OutputPeriodConfigBuilder;
import com.qhiot.transport.message.response.io.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 输出开关量 周期指令 配置指令解析
 */
public class OutputPeriodConfigDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.SWITCH_OUTPUT_PERIOD_CONFIG_WRITE
                    && packetV2.getCommand() != Command.SWITCH_OUTPUT_PERIOD_CONFIG_READ
                    && packetV2.getCommand() != Command.SWITCH_OUTPUT_PERIOD_CONFIG_DEL
                    && packetV2.getCommand() != Command.SWITCH_OUTPUT_PERIOD_CONFIG_DEL_ALL && packetV2.getCommand() != Command.SWITCH_OUTPUT_PERIOD_CONFIG_READ_ALL)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.SWITCH_OUTPUT_PERIOD_CONFIG_WRITE: // 写输出开关量周期配置
                    OutputPeriodConfigWriteResponse response = new OutputPeriodConfigWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(response);

                    super.printAnalyzeMessageToLogTopic(packetV2, "写输出开关量周期配置回应");
                    break;
                case Command.SWITCH_OUTPUT_PERIOD_CONFIG_READ: // 读输出开关量周期配置
                    OutputPeriodConfig config = OutputPeriodConfigBuilder.read(commandDataBuf);

                    OutputPeriodConfigReadResponse readResponse = new OutputPeriodConfigReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setConfig(config);
                    super.sendTransportResponseToTopic(readResponse);

                    super.printAnalyzeMessageToLogTopic(packetV2, "读输出开关量周期配置回应");
                    break;
                case Command.SWITCH_OUTPUT_PERIOD_CONFIG_READ_ALL: // 读全部输出开关量周期配置
                    List<OutputPeriodConfig> configItems = new ArrayList<OutputPeriodConfig>();
                    OutputPeriodConfig configItem;
                    for (int i = 0; i < OutputPeriodConfig.DATA_COUNT; i++) {
                        configItem = OutputPeriodConfigBuilder.read(commandDataBuf);
                        configItems.add(configItem);
                    }

                    OutputPeriodConfigReadAllResponse readAllResponse = new OutputPeriodConfigReadAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readAllResponse.setConfigs(configItems);
                    super.sendTransportResponseToTopic(readAllResponse);

                    super.printAnalyzeMessageToLogTopic(packetV2, "读输出开关量周期全部配置回应");
                    break;
                case Command.SWITCH_OUTPUT_PERIOD_CONFIG_DEL: // 删除输出开关量周期配置
                    OutputPeriodConfigDeleteResponse delResponse = new OutputPeriodConfigDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(delResponse);

                    super.printAnalyzeMessageToLogTopic(packetV2, "删除输出开关量周期配置回应");
                    break;
                case Command.SWITCH_OUTPUT_PERIOD_CONFIG_DEL_ALL: // 删除输出开关量周期全部配置
                    OutputPeriodConfigDeleteAllResponse delAllResponse = new OutputPeriodConfigDeleteAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(delAllResponse);

                    super.printAnalyzeMessageToLogTopic(packetV2, "删除输出开关量周期全部配置回应");
                    break;

                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
