package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.ArrayUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.data.InputStateData;
import com.qhiot.transport.message.data.OutputStateData;
import com.qhiot.transport.message.response.io.OutputStateResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 开关量输出(0x4D)
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/21 19:08
 */
public class OutputStateDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.SWITCH_OUTPUT_STATE
                || packetV2.getCommandDataLength() != OutputStateData.DATA_LENGTH) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            final OutputStateData outputStateData = new OutputStateData(packetV2.getDeviceType(), packetV2.getDeviceId());
            int[]           states      = new int[16];
            for (int i = 0; i < InputStateData.DATA_LENGTH; i++) {
                byte stateValue = commandDataBuf.readByte();
                states[i] = stateValue == 0x31 ? 1 : 0;
            }
            outputStateData.setStates(states);
//            sendTransportDataToQueueAndTopic(outputStateData);
            sendOutputStateData(outputStateData);

            OutputStateResponse response = new OutputStateResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
            response.setStates(states);
            sendTransportResponseToTopic(response);

            printAnalyzeMessageToLogTopic(packetV2, "输出开关量: [{}]", ArrayUtil.toString(states));
            return new byte[0];
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
