package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.OverloadCurrent;
import com.qhiot.transport.message.bean.OverloadCurrentBuilder;
import com.qhiot.transport.message.response.io.*;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 输入输出关联信息
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class OverloadCurrentDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.OVERLOAD_CURRENT_READ_ALL
                    && packetV2.getCommand() != Command.OVERLOAD_CURRENT_DELETE_ONE
                    && packetV2.getCommand() != Command.OVERLOAD_CURRENT_DELETE_ALL
                    && packetV2.getCommand() != Command.OVERLOAD_CURRENT_READ_ONE
                    && packetV2.getCommand() != Command.OVERLOAD_CURRENT_WRITE_ONE
                    )) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.OVERLOAD_CURRENT_WRITE_ONE: // 写一条
                    OverloadCurrentWriteResponse response = new OverloadCurrentWriteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(response);
                    super.printAnalyzeMessageToLogTopic(packetV2, "写入动力设备监测电流配置回应");
                    break;
                case Command.OVERLOAD_CURRENT_READ_ONE: // 读一条
                    OverloadCurrent config = OverloadCurrentBuilder.read(commandDataBuf);
                    OverloadCurrentReadResponse readResponse = new OverloadCurrentReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setConfig(config);
                    super.sendTransportResponseToTopic(readResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "读动力设备监测电流配置回应");
                    break;
                case Command.OVERLOAD_CURRENT_DELETE_ONE: // 删一条
                    OverloadCurrentDeleteResponse deleteResponse = new OverloadCurrentDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(deleteResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "删动力设备监测电流配置回应");
                    break;
                case Command.OVERLOAD_CURRENT_READ_ALL: // 读全部
                    List<OverloadCurrent> configItems = new ArrayList<OverloadCurrent>();
                    OverloadCurrent configItem;
                    for (int i = 0; i < OverloadCurrent.DATA_COUNT; i++) {
                        configItem = OverloadCurrentBuilder.read(commandDataBuf);
                        configItems.add(configItem);
                    }
                    OverloadCurrentReadAllResponse readAllResponse = new OverloadCurrentReadAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readAllResponse.setConfigs(configItems);
                    super.sendTransportResponseToTopic(readAllResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "读全部动力设备监测电流配置回应");
                    break;
                case Command.OVERLOAD_CURRENT_DELETE_ALL: // 删除全部
                    OverloadCurrentDeleteAllResponse delAllResponse = new OverloadCurrentDeleteAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(delAllResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "删除全部动力设备监测电流配置回应");
                    break;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
