package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.OverloadCurrent;
import com.qhiot.transport.message.bean.OverloadWarning;
import com.qhiot.transport.message.bean.OverloadWarningBuilder;
import com.qhiot.transport.message.response.io.OverloadWarningDeleteAllResponse;
import com.qhiot.transport.message.response.io.OverloadWarningDeleteResponse;
import com.qhiot.transport.message.response.io.OverloadWarningReadAllResponse;
import com.qhiot.transport.message.response.io.OverloadWarningReadResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.ArrayList;
import java.util.List;

/**
 * 输入输出关联信息
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class OverloadWarningDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.OVERLOAD_WARNING_CLEAR
                    && packetV2.getCommand() != Command.OVERLOAD_WARNING_READ_ONE
                    && packetV2.getCommand() != Command.OVERLOAD_WARNING_CLEAR_ALL
                    && packetV2.getCommand() != Command.OVERLOAD_WARNING_READ_ALL
                    )) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.OVERLOAD_WARNING_CLEAR:
                    OverloadWarningDeleteResponse clearResponse = new OverloadWarningDeleteResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(clearResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "清除动力设备监测电流状态回应");
                    break;
                case Command.OVERLOAD_WARNING_READ_ONE:
                    OverloadWarning warningOne = OverloadWarningBuilder.read(commandDataBuf);
                    OverloadWarningReadResponse readWarningResponse = new OverloadWarningReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readWarningResponse.setConfig(warningOne);
                    super.sendTransportResponseToTopic(readWarningResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "读动力设备监测电流状态回应");
                    break;
                case Command.OVERLOAD_WARNING_CLEAR_ALL:
                    OverloadWarningDeleteAllResponse clearAllResponse = new OverloadWarningDeleteAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    super.sendTransportResponseToTopic(clearAllResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "清除全部动力设备监测电流状态回应");
                    break;
                case Command.OVERLOAD_WARNING_READ_ALL:
                    List<OverloadWarning> warningItems = new ArrayList<OverloadWarning>();
                    OverloadWarning warningItem;
                    for (int i = 0; i < OverloadCurrent.DATA_COUNT; i++) {
                        for (int j = 0;j < 3; j++) {
                            warningItem = OverloadWarningBuilder.read(commandDataBuf);
                            warningItem.setIndex(i);
                            warningItem.setType(j);
                            warningItems.add(warningItem);
                        }
                    }
                    OverloadWarningReadAllResponse readAllWarningResponse = new OverloadWarningReadAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readAllWarningResponse.setConfigs(warningItems);
                    super.sendTransportResponseToTopic(readAllWarningResponse);
                    super.printAnalyzeMessageToLogTopic(packetV2, "读全部动力设备监测电流状态回应");
                    break;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
