package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.processor.PowerDataProcessor;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.data.PowerData;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;

/**
 * 终端上电或者掉电 0x07
 *
 * @author 金建强(ptma@163.com)
 * @version 2016 -07-01 02:36:00
 */
public class PowerDecoder extends BaseV2Decoder {

    @Autowired
    private PowerDataProcessor powerDataProcessor;

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.POWER) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }
            // 上电:0X01 ,掉电:0X02
            final byte powerState = commandDataBuf.readByte();
            final int timestamp = commandDataBuf.readIntLE();
            final Date datetime = new Date(timestamp * 1000L);

            PowerData data = new PowerData(packetV2.getDeviceId());
            data.setTimestamp(datetime);
            if (powerState == 0x02) {
                data.setPowerState(0);
                super.printAnalyzeMessageToLogTopic(packetV2, "掉电：时间戳={}", DateUtil.formatDateTime(datetime));
            } else if (powerState == 0x01) {
                data.setPowerState(1);
                super.printAnalyzeMessageToLogTopic(packetV2, "上电：时间戳={}", DateUtil.formatDateTime(datetime));
            }
//            super.sendTransportDataToQueueAndTopic(data);
            super.sendPowerData(data);
            powerDataProcessor.process(data);
            return new byte[0];
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
