package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.decoder.Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.codec.util.Command;
//import com.qhiot.transport.codec.util.GroovyDecoderUtil;
import com.qhiot.transport.db.model.TransportDecoder;
import com.qhiot.transport.message.bean.Rs485Collection;
import com.qhiot.transport.message.bean.Rs485CollectionDataBuilder;
import com.qhiot.transport.message.data.Rs485Data;
import com.qhiot.transport.message.response.Rs485CollectionResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.List;

/**
 * 485 采集数据解析
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19.
 */
public class Rs485CollectionDecoder extends BaseV2Decoder {

    public static final String DECODER_CATEGORY = "packetV2_485";

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || (packetV2.getCommand() != Command.RS485_COLLECTION_OK
                && packetV2.getCommand() != Command.RS485_COLLECTION_FAIL)) {
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.RS485_COLLECTION_OK: // 采集485数据成功
                    final Rs485Collection rs485Collection = Rs485CollectionDataBuilder.read(commandDataBuf);
                    super.printAnalyzeMessageToLogTopic(packetV2, "采集485数据成功");
                    if(rs485Collection != null) {
                        Rs485Data rs485Data = new Rs485Data(packetV2.getDeviceType(), packetV2.getDeviceId());
                        rs485Data.setRs485Collection(rs485Collection);
//                        super.sendTransportDataToQueueAndTopic(rs485Data);
                        super.sendRs485Data(rs485Data);
                        Rs485CollectionResponse response = new Rs485CollectionResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                        response.setRs485Collection(rs485Collection);
                        response.setSucceed(Boolean.TRUE);
                        super.sendTransportResponseToTopic(response);

                        decodePacketV2CollectionData(channel, context, rs485Collection);
                    }
                    return new byte[0];
                case Command.RS485_COLLECTION_FAIL: // 采集失败
                    Rs485CollectionResponse response = new Rs485CollectionResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    response.setSucceed(Boolean.FALSE);
                    super.sendTransportResponseToTopic(response);

                    super.printAnalyzeMessageToLogTopic(packetV2, "485采集数据失败");
                    return null;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    /**
     * 处理 485 采集的数据
     *
     * @param channel
     * @param context
     * @param rs485Collection
     */
    private void decodePacketV2CollectionData(Channel channel, ChainContext context, Rs485Collection rs485Collection) {
        ByteBuf commandDataBuf = ByteBufUtil.directBuffer();
        try {
            commandDataBuf.writeBytes(rs485Collection.getData());
            context.put(ChainContext.COLLECTION_485, rs485Collection);
            executeSubDecoderChain(channel, commandDataBuf, context);
        } finally {
            ByteBufUtil.safeRelease(commandDataBuf);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // 从数据库读取根解码器, 并追加到链路中
//        List<TransportDecoder> decoders = transportDecoderService.selectByCategory(DECODER_CATEGORY);
//        for (TransportDecoder transportDecoder : decoders) {
//            Decoder decoder = GroovyDecoderUtil.parseDecoder(transportDecoder.getContent());
//            if (null != decoder) {
//                addGroovySubDecoder(decoder);
//            }
//        }
    }

}
