package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.Sms;
import com.qhiot.transport.message.bean.SmsBuilder;
import com.qhiot.transport.message.response.SmsReadResponse;
import com.qhiot.transport.message.response.SmsSendContentResponse;
import com.qhiot.transport.message.response.SmsSendMobileResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 短信读写
 * <P>
 * User: 金建强(ptma@163.com)
 * </P>
 * <p>
 * Date: 2016/3/21 19:27
 * </p>
 */
public class SmsDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || (packetV2.getCommand() != Command.SMS_SEND_MOBILE
                    && packetV2.getCommand() != Command.SMS_SEND_CONTENT && packetV2.getCommand() != Command.SMS_READ)) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.SMS_SEND_MOBILE:
                    SmsSendMobileResponse sendMobileResponse = new SmsSendMobileResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(sendMobileResponse);
                    printAnalyzeMessageToLogTopic(packetV2, "指定短信号码回应");
                    break;
                case Command.SMS_SEND_CONTENT:
                    SmsSendContentResponse sendContentResponse = new SmsSendContentResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(sendContentResponse);
                    printAnalyzeMessageToLogTopic(packetV2, "指定短信内容回应");
                    break;
                case Command.SMS_READ:
                    Sms sms = SmsBuilder.read(packetV2.getCommandData());
                    SmsReadResponse readResponse = new SmsReadResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    readResponse.setSms(sms);
                    sendTransportResponseToTopic(readResponse);
                    printAnalyzeMessageToLogTopic(packetV2, "读取短信内容回应");
                    break;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
