package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.Tag24g;
import com.qhiot.transport.message.bean.Tag24gBuilder;
import com.qhiot.transport.message.data.Tag24gData;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 2.4G考勤卡上报(0x06)
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/21 16:45
 */
public class Tag24gDecoder extends BaseV2Decoder {

    //TODO 将解析结果送到消息队列

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.TAG_24G) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }
            final Tag24g tag = Tag24gBuilder.read(commandDataBuf);
            if (tag == null) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            printAnalyzeMessageToLogTopic(packetV2, "2.4G标签：TagId={}, RSSI={}, Voltage={}", tag.getTagId(), tag.getRssi(), tag.getVoltage());

            Tag24gData data = new Tag24gData(packetV2.getDeviceType(), packetV2.getDeviceId());
            data.setTag(tag);
//            super.sendTransportDataToQueueAndTopic(data);
            super.sendTag24gData(data);
            return new byte[0];
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
