package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.Tag780m;
import com.qhiot.transport.message.bean.Tag780mBuilder;
import com.qhiot.transport.message.data.Tag780mData;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 780M无线标签
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/21 16:45
 */
public class Tag780mDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.HEARTBEAT
                || !DeviceType.WIRELESS_TAG780M.equals(packetV2.getDeviceType())) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }
            final Tag780m tag = Tag780mBuilder.read(commandDataBuf);
            if (tag == null) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            printAnalyzeMessageToLogTopic(packetV2, "780M标签：标签Id={}", tag.getTagId());

            Tag780mData data = new Tag780mData(packetV2.getDeviceType(), packetV2.getDeviceId());
            data.setTag(tag);
//            sendTransportDataToQueueAndTopic(data);
            super.sendTag780mData(data);
            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
