package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 倾角检测有效 0x02
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-17 16:19
 */
public class TiltedDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || !DeviceType.TILTED_SENSOR.equals(packetV2.getDeviceType())
                || packetV2.getCommand() != Command.TILTED) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            printAnalyzeMessageToLogTopic(packetV2, "倾倒: {}={}", packetV2.getDeviceName(), packetV2.getDeviceId());

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }

}
