package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.ArrayUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.data.TransformerState;
import com.qhiot.transport.message.response.io.TransformerStateResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 互感器采样(0x35)
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/21 19:08
 */
public class TransformerStateDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.TRANSFORMER_STATE) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            if (packetV2.getCommandDataLength() != (TransformerState.DATA_LENGTH * 2)) {
                printAnalyzeMessageToLogTopic(packetV2, "互感器: 数据长度不匹配");
                return new byte[0];
            }

            final TransformerState state = new TransformerState(packetV2.getDeviceType(), packetV2.getDeviceId());
            int[]           states      = new int[16];
            for (int i = 0; i < TransformerState.DATA_LENGTH; i++) {
                states[i] = commandDataBuf.readUnsignedShortLE();
            }
            state.setStates(states);

//            sendTransportDataToQueueAndTopic(state);
            super.sendTransformerState(state);
            TransformerStateResponse response = new TransformerStateResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
            response.setStates(states);
            sendTransportResponseToTopic(response);

            printAnalyzeMessageToLogTopic(packetV2, "互感器: [{}]", ArrayUtil.toString(state.getStates()));

            return new byte[0];
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
