package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.bean.TransmitTransparently;
import com.qhiot.transport.message.response.TransmitTransparentlyResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 指令透传回应 (0x10)
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/21 19:08
 */
public class TransmitTransparentlyDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.TRANSMIT_TRANSPARENTLY) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            byte[] data = packetV2.getCommandData();

            TransmitTransparentlyResponse response = new TransmitTransparentlyResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
            TransmitTransparently transmitTransparently = new TransmitTransparently();
            transmitTransparently.setData(data);
            response.setTransmitTransparently(transmitTransparently);
            sendTransportResponseToTopic(response);

            printAnalyzeMessageToLogTopic(packetV2, "指令透传回应: HEX=[{}]", ByteUtil.toHexString(data, true));

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
