package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.core.util.DateUtil;
import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.define.DeviceType;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 无线传感器
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/21 16:45
 */
public class WirelessSensorDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null || packetV2.getCommand() != Command.HEARTBEAT
                    || !DeviceType.WIRELESS_SENSOR.equals(packetV2.getDeviceType())) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }
            if (packetV2.getCommandDataLength() > 0) {
                return decodePacketV2WirelessSensorData(channel, packetV2, context);
            }
            return Command.EMPTY_DATA;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    /**
     * 处理无线传感器的数据
     *
     * @param channel Socket 通道
     * @param packetV2 已经解码出来的packetV2
     */
    private byte[] decodePacketV2WirelessSensorData(Channel channel, PacketV2 packetV2, ChainContext context) {
        ByteBuf commandDataBuf = ByteBufUtil.directBuffer();
        try {
            if(hasSubDecoder()) {
                commandDataBuf.writeBytes(packetV2.getCommandData());
                return executeSubDecoderChain(channel, commandDataBuf, context);
            } else {
                return encoderResponsePacklet();
            }
        } finally {
            ByteBufUtil.safeRelease(commandDataBuf);
        }
    }

    private byte[] encoderResponsePacklet() {
        ByteBuf repDataBuf = ByteBufUtil.directBuffer(4);
        try {
            repDataBuf.writeIntLE(DateUtil.getUTCSeconds());
            return ByteBufUtil.getBytesRemaining(repDataBuf);
        } finally {
            ByteBufUtil.safeRelease(repDataBuf);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
    }
}
