package com.qhiot.transport.codec.decoder.v2;

import com.qhiot.transport.codec.ChainContext;
import com.qhiot.transport.codec.DecoderChain;
import com.qhiot.transport.codec.bean.PacketV2;
import com.qhiot.transport.codec.decoder.BaseV2Decoder;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.Command;
import com.qhiot.transport.message.define.DeviceType;
import com.qhiot.transport.message.response.io.WirelessSwitchCloseAllResponse;
import com.qhiot.transport.message.response.io.WirelessSwitchCloseResponse;
import com.qhiot.transport.message.response.io.WirelessSwitchOpenAllResponse;
import com.qhiot.transport.message.response.io.WirelessSwitchOpenResponse;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

/**
 * 无线开关控制器操作回应
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/3/21 16:45
 */
public class WirelessSwitchDecoder extends BaseV2Decoder {

    @Override
    public byte[] decode(Channel channel, ByteBuf commandDataBuf, ChainContext context, DecoderChain chain) {
        commandDataBuf.markReaderIndex();
        try {
            final PacketV2 packetV2 = (PacketV2) context.get(ChainContext.PACKET_V2);
            if (packetV2 == null
                || !DeviceType.WIRELESS_SWITCH.equals(packetV2.getDeviceType())
                || (packetV2.getCommand() != Command.WIRELESS_SWITCH_OPEN
                    && packetV2.getCommand() != Command.WIRELESS_SWITCH_CLOSE
                    && packetV2.getCommand() != Command.WIRELESS_SWITCH_OPEN_ALL && packetV2.getCommand() != Command.WIRELESS_SWITCH_CLOSE_ALL)) {
                commandDataBuf.resetReaderIndex();
                return chain.doDecode(channel, commandDataBuf, context);
            }

            switch (packetV2.getCommand()) {
                case Command.WIRELESS_SWITCH_OPEN:
                    WirelessSwitchOpenResponse openResponse = new WirelessSwitchOpenResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                    sendTransportResponseToTopic(openResponse);
                    printAnalyzeMessageToLogTopic(packetV2, "打开无线开关回应");
                    break;
                case Command.WIRELESS_SWITCH_CLOSE:
                        WirelessSwitchCloseResponse closeResponse = new WirelessSwitchCloseResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                        sendTransportResponseToTopic(closeResponse);
                    printAnalyzeMessageToLogTopic(packetV2, "关闭无线开关回应");
                    break;
                case Command.WIRELESS_SWITCH_OPEN_ALL:
                        WirelessSwitchOpenAllResponse openAllResponse = new WirelessSwitchOpenAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                        sendTransportResponseToTopic(openAllResponse);
                    printAnalyzeMessageToLogTopic(packetV2, "打开全部无线开关回应");
                    break;
                case Command.WIRELESS_SWITCH_CLOSE_ALL:
                        WirelessSwitchCloseAllResponse closeAllResponse = new WirelessSwitchCloseAllResponse(packetV2.getDeviceType(), packetV2.getDeviceId());
                        sendTransportResponseToTopic(closeAllResponse);
                    printAnalyzeMessageToLogTopic(packetV2, "关闭全部无线开关回应");
                    break;
                default:
                    return chain.doDecode(channel, commandDataBuf, context);
            }

            return null;
        } catch (DecodeException e) {
            logger.info(e.getMessage(), e);
            commandDataBuf.resetReaderIndex();
            return chain.doDecode(channel, commandDataBuf, context);
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // Do nothing
    }
}
