package com.qhiot.transport.codec.processor;

import com.qhiot.core.util.DateUtil;
import com.qhiot.core.util.StringUtil;
import com.qhiot.core.util.ThreadUtil;
import com.qhiot.transport.db.model.MonitoringData;
import com.qhiot.transport.db.model.TransportResourceCore;
import com.qhiot.transport.db.service.*;
import com.qhiot.transport.influxdb.model.LinkMonitorDataHistoryInflux;
import com.qhiot.transport.influxdb.service.LinkMonitorDataHistoryInfluxService;
import com.qhiot.transport.message.data.MonitoringHistoryData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * 存储历史数据
 */
@Component
public class HistoryDataSaveProcessor {

    protected final Logger LOGGER = LoggerFactory.getLogger("transport.codec." + getClass().getSimpleName());
    @Autowired
    private MonitoringDataService monitoringDataService;

    @Autowired
    private TransportResourceCore4dbService transportResourceCore4dbService;

    @Autowired
    private LinkMonitorDataHistoryInfluxService linkMonitorDataHistoryInfluxService;


    private ThreadPoolExecutor executor = ThreadUtil.newCachedThreadPool(10, 500, "seem-minitoring-data-processor-%d");

    public void process(final MonitoringHistoryData data) {
        executor.submit(new Runnable() {
            @Override
            public void run() {
                saveToDatabase(data);
                saveToInfluxDB(data);
            }
        });
    }

    private void saveToDatabase(MonitoringHistoryData data) {
        try {
            MonitoringData monitoringData = new MonitoringData();
            monitoringData.setDeviceId(String.valueOf(data.getDeviceId()));
            monitoringData.setMonitoringId(data.getMonitoringId());
            monitoringData.setIndexId(data.getMonitoringIndexId());
            monitoringData.setDataValue(data.getValue());
            monitoringData.setDataTime(data.getDataTime());
            monitoringData.setReceivedTime(DateUtil.now());
            monitoringDataService.saveOrUpdateSelective(monitoringData);
        } catch (Exception e) {
            LOGGER.error("保存监测数据出错: {}, 上下文={}", e.getMessage(), data.getContextId(), e);
        }

    }

    private void saveToInfluxDB(MonitoringHistoryData data) {
        try {
            if (data.getDeviceId() == null || data.getMonitoringId() == null) {
                return;
            }
            TransportResourceCore transportResourceCore = transportResourceCore4dbService.selectByDeviceIdAndMonitoringId(data.getDeviceId(), data.getMonitoringId());
            if (transportResourceCore == null || transportResourceCore.getDeviceId() == null) {
                LOGGER.warn(StringUtil.format("核心配置未查到,网关:{}，测点:{}，指标:{}", data.getDeviceId(), data.getMonitoringId(), data.getMonitoringIndexId()));
                return;
            }
            LinkMonitorDataHistoryInflux linkMonitorDataHistoryInflux = new LinkMonitorDataHistoryInflux();
            linkMonitorDataHistoryInflux.setDeviceId(transportResourceCore.getDeviceId());
            linkMonitorDataHistoryInflux.setTenantId(StringUtil.toStringOrEmpty(transportResourceCore.getTenantId()));
            linkMonitorDataHistoryInflux.setMonitoringCode(StringUtil.toStringOrEmpty(transportResourceCore.getMonitoringCode()));
            linkMonitorDataHistoryInflux.setMonitorCode(StringUtil.toStringOrEmpty(transportResourceCore.getMonitorCode()));

            // 判断数据的产生时间（非上报时间，当时间在n分之内时为0，否则为1，或者按照时间段分级）
            Date now = DateUtil.now();
            Date date = DateUtil.offsetMinute(now, -3);
            int co = date.compareTo(data.getDataTime());
            if (co <= 0) {

            } else {
                //co>0
            }

            linkMonitorDataHistoryInflux.setValue(data.getValue());
            linkMonitorDataHistoryInflux.setDate(data.getDataTime());
//            linkMonitorDataHistoryInflux.setDate(DateUtil.now());
            linkMonitorDataHistoryInfluxService.appointedInsert(linkMonitorDataHistoryInflux);
        } catch (Exception e) {
            LOGGER.error("influxDB保存监测数据出错: {}, 上下文={}", e.getMessage(), data.getContextId(), e);
        }
    }

}
