package com.qhiot.transport.codec.processor;

import com.qhiot.core.util.*;
import com.qhiot.transport.codec.bean.PacketV1;
import com.qhiot.transport.db.dto.TransportResourceMonitoringIndexDTO;
import com.qhiot.transport.db.model.TransportResourceConfig;
import com.qhiot.transport.db.service.*;
import com.qhiot.transport.jms.*;
import com.qhiot.transport.jms.kafka.CodecLogKafkaSender;
import com.qhiot.transport.jms.kafka.MonitoringHistoryDataKafkaSender;
import com.qhiot.transport.jms.kafka.TransportWarningKafkaSender;
import com.qhiot.transport.jms.kafka.WarningDataKafkaSender;
import com.qhiot.transport.message.CodecLogMessage;
import com.qhiot.transport.message.bean.Modbus;
import com.qhiot.transport.message.data.MonitoringHistoryData;
import groovy.lang.Binding;
import groovy.lang.GroovyShell;
import groovy.lang.Script;
import org.codehaus.groovy.control.CompilationFailedException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import tk.mybatis.mapper.entity.Example;

import java.math.BigDecimal;
import java.nio.ByteOrder;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * modbus透传处理器 --罗军
 * 2020 01 10
 *
 * 萧山 中继 透传数据
 */
@Component
public class ModbusDataProcessor {

    protected final Logger                      LOGGER                         = LoggerFactory.getLogger("transport.codec." + getClass().getSimpleName());

    private static final String                 DATA_UPLOAD_TEMPLATE           = "监测数据: {}={}{}, 时间={}, HEX=[{}]";
    private static final String                 DATA_UPLOAD_EXCEPTION_TEMPLATE = "监测数据解析出错: 寄存器类型={}, 寄存器地址={}, 解码方式={}, 数据=[{}], 错误={}";
    private static final String                 TIMESTAMP_WARNING_TEMPLATE     = "监测数据时间戳异常: 时间={}, 系统时间={}";
    private static final String                 NO_CONFIG_WARNING_TEMPLATE     = "未找到测点配置: deviceId={}, registerType={}, storeAddress={}";
    private static final String                 DATA_BOUND_WARNING_TEMPLATE    = "测点\"{}\"监测值超出{}限, 当前值为 {}";

//    @Autowired
//    private CodecLogSender                      codecLogSender;
//    @Autowired
//    private TransportDataQueueSender            transportDataQueueSender;
//    @Autowired
//    private TransportDataTopicSender            transportDataTopicSender;
//    @Autowired
//    private InsightDataQueueSender              insightDataQueueSender;
//    @Autowired
//    private ApiPushDataQueueSender              apiPushDataQueueSender;

    /**---kafka---*/
    @Autowired
    private CodecLogKafkaSender codecLogSender;
    @Autowired
    private MonitoringHistoryDataKafkaSender monitoringHistoryDataKafkaSender;
    /**------*/


    @Autowired
    private TransportResourceConfigService      transportResourceConfigService;

    @Autowired
    private TransportResourceMonitoringService  transportResourceMonitoringService;

    private ExecutorService                     executor                       = ThreadUtil.newCachedThreadPool(10, 1000, "seem-history-data-processor-%d");

    /** 公式脚本缓存 */
    private Map<String, Script>                 formulaScriptMap               = new ConcurrentHashMap<String, Script>();

    public void processorModbusData(final PacketV1 packetV1, final List<Modbus> modbuses) {
        executor.submit(new Runnable() {
            @Override
            public void run() {
                for (Modbus modbus : modbuses) {
                    process(packetV1, modbus);
                }
            }

            private void process(final PacketV1 packetV1, final Modbus modbus) {
                try{
                    LOGGER.debug("modbus数据解析"+modbus);
                    Date samplingTime = DateUtil.convertUTC2Date(modbus.getTimestamp());

                    // 检测时间超过当前系统时间 10 分钟
                    if (DateUtil.diffMinutes(DateUtil.now(), samplingTime) > 10) {
                        String samplingTimeStr = DateUtil.formatDateTime(samplingTime);
                        logWarn(modbus,packetV1, StringUtil.format(TIMESTAMP_WARNING_TEMPLATE, samplingTimeStr, DateUtil.nowString()));
                        return;
                    }

                    //根据寄存器地址（storeAddress） 功能码 解码方式 查找config 根据解码方式，解码
                    Example configExample = new Example(TransportResourceConfig.class);
                    configExample.createCriteria().andEqualTo("deviceId",modbus.getDeviceId())
                        .andEqualTo("registerType",modbus.getFunctionCode()).andEqualTo("valid",1);
//                        .andEqualTo("storeAddress",modbus.getStoreAddress());
                    List<TransportResourceConfig> configs = transportResourceConfigService.selectByExample(configExample);
                    if (CollectionUtil.isNotEmpty(configs)) {
                        for (TransportResourceConfig config : configs) {
                            List<TransportResourceMonitoringIndexDTO> monitorings = transportResourceMonitoringService.selectByDeviceAndConfigIndex(config.getDeviceId(),config.getConfigIndex());
                            //
                            byte method = config.getMethod();
                            switch (modbus.getFunctionCode()){
                                //1bit读写
                                case 1:
                                    break;
                                //2bit只读
                                case 2:
                                    break;
                                //3字节读写
                                case 3:
                                    break;
                                //4字节只读 目前只处理这里
                                case 4:
                                    processWithModbusConfig(monitorings,modbus,method);
                                    break;
                                default:
                                    break;
                            }
                        }
                    }else {
                        LOGGER.debug("modbus数据解析:未能查询到对应的配置");
                    }
                    //根据名称找寻 monitoring ---------操蛋的事情

                    //根据deviceId和
                }catch (Exception e){
                    LOGGER.error(e.getMessage(),e);
                }



            }

            /**
             *
             */
            private void analysData(){
                //从起始地址开始向后
            }


            /**
             * 按照报文的起始地址+解码方式对应长度的倍数，确定每组数据对应的测点，中间部分允许跳过
             *
             * @param
             */
            private void processWithModbusConfig(List<TransportResourceMonitoringIndexDTO> monitorings, Modbus modbus,
                                                    byte method) {
                int storeAddress = modbus.getStoreAddress();
                Map<String,TransportResourceMonitoringIndexDTO> monitoringIndexDTOMap = monitorings.stream()
                    .collect(Collectors.toMap(TransportResourceMonitoringIndexDTO::getMonitoringName, Function.identity(),(key1,key2)->key2));
                byte[] allData = modbus.getData();
                switch (method) {
                    case 0: // Bit
                        break;
                    case 1: // INT8S 8位有符号

                        break;
                    case 2: // INT8U 8位无符号

                        break;
                    case 3: // INT16S_AB

                        break;
                    case 4: // INT16S_BA

                        break;
                    case 5: // INT16U_AB

                        break;
                    case 6: // INT16U_BA
                        if(allData != null&&allData.length > 0 &&allData.length/2>0){
                            for(int i = 0; i < allData.length/2; i++){
                                int actualStoreAddress = storeAddress + i;
                                //解析数据
                                byte[] data = ByteUtil.subBytes(allData, i*2, i*2+2);
                                String parsed = ByteUtil.toHexString(data);
                                int v6 = ByteUtil.toUnsignedShort(data, ByteOrder.LITTLE_ENDIAN);
                                BigDecimal bigDecimal = new BigDecimal(v6);

                                switch (actualStoreAddress){
                                    case 40:
                                        //可见光
                                        //获取对应测点monitoring
                                        TransportResourceMonitoringIndexDTO monitoring40 = monitoringIndexDTOMap.get("可见光");
                                        processFinal(bigDecimal,monitoring40,packetV1,modbus,parsed);
                                        break;
                                    case 41:
                                        //红外光
                                        TransportResourceMonitoringIndexDTO monitoring41 = monitoringIndexDTOMap.get("红外光");
                                        processFinal(bigDecimal,monitoring41,packetV1,modbus,parsed);
                                        break;
                                    case 76:
                                        //累计雨量
                                        TransportResourceMonitoringIndexDTO monitoring76 = monitoringIndexDTOMap.get("时累计雨量");
                                        processFinal(bigDecimal,monitoring76,packetV1,modbus,parsed);
                                        break;
                                    case 77:
                                        //日累计雨量
                                        TransportResourceMonitoringIndexDTO monitoring77 = monitoringIndexDTOMap.get("日累计雨量");
                                        processFinal(bigDecimal,monitoring77,packetV1,modbus,parsed);
                                        break;
                                    default:

                                }
                            }
                        }
                        break;
                    case 7: // Long ABCD
                        break;
                    case 8: // Long CDAB
                        break;
                    case 9: // Long BADC
                        break;
                    case 10: // Long DCBA
                        break;
                    case 11: // float ABCD
                        break;
                    case 12: // float CDAB

                        break;
                    case 13: // float BADC

                        break;
                    case 14: // float DCBA
                        //按4个字节为一组（2B），从读到的起始地址开始，向后推演
                        if(allData != null&&allData.length > 0 &&allData.length/4>0){
                            for(int i = 0; i < allData.length/4; i++){
                                //解析数据
                                byte[] data = ByteUtil.subBytes(allData, i*4, i*4+4);
                                String parsed = ByteUtil.toHexString(data);
                                float v14 = ByteUtil.toFloat(data, ByteOrder.LITTLE_ENDIAN);
                                BigDecimal bigDecimal = new BigDecimal(v14);

                                int actualStoreAddress = storeAddress + i*2;
                                switch (actualStoreAddress){
                                    case 0:
                                        //4-20mA投入式液位计输出信号值
                                        TransportResourceMonitoringIndexDTO monitoring0 = monitoringIndexDTOMap.get("4-20mA投入式液位计输出信号值");
                                        processFinal(bigDecimal,monitoring0,packetV1,modbus,parsed);
                                        break;
                                    case 32:
                                        //电池电压
                                        TransportResourceMonitoringIndexDTO monitoring32 = monitoringIndexDTOMap.get("电池电压");
                                        processFinal(bigDecimal,monitoring32,packetV1,modbus,parsed);
                                        break;
                                    case 34:
                                        //0.5-2.5V投入式液位计输出信号值  原本在16  零时调整 占用了信号值的寄存器地址
                                        TransportResourceMonitoringIndexDTO monitoring34 = monitoringIndexDTOMap.get("液位");
                                        processFinal(bigDecimal,monitoring34,packetV1,modbus,parsed);
                                        break;
                                    case 36:
                                        //温度
                                        TransportResourceMonitoringIndexDTO monitoring36 = monitoringIndexDTOMap.get("温度");
                                        processFinal(bigDecimal,monitoring36,packetV1,modbus,parsed);
                                        break;
                                    case 38:
                                        //温度
                                        TransportResourceMonitoringIndexDTO monitoring38 = monitoringIndexDTOMap.get("湿度");
                                        processFinal(bigDecimal,monitoring38,packetV1,modbus,parsed);
                                        break;
                                    case 42:
                                        //X轴角度
                                        TransportResourceMonitoringIndexDTO monitoring42 = monitoringIndexDTOMap.get("X轴角度");
                                        processFinal(bigDecimal,monitoring42,packetV1,modbus,parsed);
                                        break;
                                    case 44:
                                        //Y轴角度
                                        TransportResourceMonitoringIndexDTO monitoring44 = monitoringIndexDTOMap.get("Y轴角度");
                                        processFinal(bigDecimal,monitoring44,packetV1,modbus,parsed);
                                        break;
                                    case 46:
                                        //Z轴角度
                                        TransportResourceMonitoringIndexDTO monitoring46 = monitoringIndexDTOMap.get("Z轴角度");
                                        processFinal(bigDecimal,monitoring46,packetV1,modbus,parsed);
                                        break;
                                    case 70:
                                        //时累计流量
                                        TransportResourceMonitoringIndexDTO monitoring70 = monitoringIndexDTOMap.get("时累计流量");
                                        processFinal(bigDecimal,monitoring70,packetV1,modbus,parsed);
                                        break;
                                    case 72:
                                        //日累计流量
                                        TransportResourceMonitoringIndexDTO monitoring72 = monitoringIndexDTOMap.get("日累计流量");
                                        processFinal(bigDecimal,monitoring72,packetV1,modbus,parsed);
                                        break;
                                    case 74:
                                        //月累计流量
                                        TransportResourceMonitoringIndexDTO monitoring74 = monitoringIndexDTOMap.get("月累计流量");
                                        processFinal(bigDecimal,monitoring74,packetV1,modbus,parsed);
                                        break;
                                    case 78:
                                        //超声波液位数据
                                        TransportResourceMonitoringIndexDTO monitoring78 = monitoringIndexDTOMap.get("超声波液位数据");
                                        processFinal(bigDecimal,monitoring78,packetV1,modbus,parsed);
                                        break;
                                    default:

                                }
                            }
                        }
                        break;
                    default:

                }

            }

            private void processFinal(BigDecimal bigDecimal,TransportResourceMonitoringIndexDTO monitoring,PacketV1 packetV1,Modbus modbus,String parsed){
                if (bigDecimal != null&&monitoring!=null) {
                    Date samplingTime = DateUtil.convertUTC2Date(modbus.getTimestamp());
                    String samplingTimeStr = DateUtil.formatDateTime(samplingTime);
                    bigDecimal = calculateWithFormula(bigDecimal, monitoring.getFormula());
                    if (monitoring.getValuePrecision() != null && monitoring.getValuePrecision() >= 0) {
                        bigDecimal = bigDecimal.setScale(monitoring.getValuePrecision().intValue(), BigDecimal.ROUND_HALF_UP);
                    } else {
                        bigDecimal = bigDecimal.setScale(2, BigDecimal.ROUND_HALF_UP);
                    }

                    String debugStr = StringUtil.format(DATA_UPLOAD_TEMPLATE, monitoring.getMonitoringName(), bigDecimal.toString(), monitoring.getUnit(), samplingTimeStr, parsed);

                    logDebug(modbus,packetV1, debugStr);

                    MonitoringHistoryData monitoringHistoryData = new MonitoringHistoryData(String.valueOf(modbus.getDeviceId()));
                    monitoringHistoryData.setContextId(packetV1.getContextId());

                    monitoringHistoryData.setMonitoringId(monitoring.getMonitoringId());
                    monitoringHistoryData.setMonitoringName(monitoring.getMonitoringName());
                    monitoringHistoryData.setDataType(monitoring.getDataType());
                    monitoringHistoryData.setDataMap(monitoring.getDataMap());
                    monitoringHistoryData.setValuePrecision(monitoring.getValuePrecision());
                    monitoringHistoryData.setUnit(monitoring.getUnit());
                    monitoringHistoryData.setIcon(monitoring.getIcon());
                    monitoringHistoryData.setValue(bigDecimal.doubleValue());

                    monitoringHistoryData.setDataTime(samplingTime);
                    monitoringHistoryData.setMonitoringIndexId(monitoring.getMonitoringIndexId());

//                    if (transportDataTopicSender != null) {
//                        transportDataTopicSender.sendMessage(monitoringHistoryData);
//                        transportDataQueueSender.sendMessage(monitoringHistoryData);
//                    }
//                    if (insightDataQueueSender != null) {
//                        insightDataQueueSender.sendMessage(monitoringHistoryData);
//                    }
//                    if (apiPushDataQueueSender != null) {
//                        apiPushDataQueueSender.sendMessage(monitoringHistoryData);
//                    }
                    monitoringHistoryDataKafkaSender.sendMessage(monitoringHistoryData);
                }
            }

            /**
             * 使用公式对原始值进行计算
             *
             * @param originalValue
             * @param formula
             * @return
             */
            private BigDecimal calculateWithFormula(BigDecimal originalValue, String formula) {
                if (StringUtil.isBlank(formula)) {
                    return originalValue;
                }
                try {
                    Script script;
                    if (formulaScriptMap.containsKey(formula)) {
                        script = formulaScriptMap.get(formula);
                    } else {
                        script = new GroovyShell().parse(formula);
                        formulaScriptMap.put(formula, script);
                    }
                    Binding binding = new Binding();
                    binding.setVariable("x", originalValue.doubleValue());
                    script.setBinding(binding);

                    Double value = Double.valueOf(script.run().toString());
                    return new BigDecimal(value);
                } catch (CompilationFailedException e) {
                    LOGGER.error("非法的换算公式: formula={}, {}", formula, e.getMessage());
                }
                return originalValue;
            }

            private void logDebug(final Modbus modbus, final PacketV1 packetV1, String debugStr) {
                CodecLogMessage logMessage = CodecLogMessage.analyzeMessage(packetV1.getChannelId(), String.valueOf(modbus.getDeviceId()), packetV1.getContextId(), debugStr);
                LOGGER.debug(logMessage.getMessage());
                if (codecLogSender != null) {
                    codecLogSender.sendMessage(logMessage);
                }
            }

            private void logWarn(final Modbus modbus, final PacketV1 packetV1, String debugStr) {
                CodecLogMessage logMessage = CodecLogMessage.analyzeMessage(packetV1.getChannelId(), String.valueOf(modbus.getDeviceId()), packetV1.getContextId(), debugStr);
                LOGGER.warn(logMessage.getMessage());
                if (codecLogSender != null) {
                    codecLogSender.sendMessage(logMessage);
                }
            }
        });
    }


}
