package com.qhiot.transport.codec.util;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.util.ReferenceCountUtil;

import java.nio.ByteBuffer;

/**
 * @author 金建强(ptma@163.com)
 * @version 2016/4/5 14:36
 */
public class ByteBufUtil {

    public static ByteBuf directBuffer(int initialCapacity) {
        //堆缓冲区 ByteBuf heapBuf = Unpooled.buffer(initialCapacity);
        //直接缓冲区 ByteBuf directBuf = Unpooled.directBuffer(initialCapacity);
        return Unpooled.buffer(initialCapacity);
        //return PooledByteBufAllocator.DEFAULT.directBuffer(initialCapacity);
    }

    public static ByteBuf directBuffer() {
        return Unpooled.buffer();
        //return PooledByteBufAllocator.DEFAULT.directBuffer();
    }

    public static void safeRelease(ByteBuf buffer) {
        if (buffer != null && buffer.refCnt() > 0) {
            ReferenceCountUtil.safeRelease(buffer);
        }
    }

    /**
     * 从 ByteBuf 获取所有可读字节, 不移动读索引
     *
     * @param buffer ByteBuf
     * @return 所有可读字节数组
     */
    public static byte[] getBytesRemaining(final ByteBuf buffer) {
        byte[] bytes = new byte[buffer.readableBytes()];
        if (bytes.length > 0) {
            buffer.getBytes(buffer.readerIndex(), bytes);
        }
        return bytes;
    }

    /**
     * 从 ByteBuffer 获取所有可读字节
     *
     * @param buffer ByteBuffer
     * @return 所有可读字节数组
     */
    public static byte[] getBytesRemaining(final ByteBuffer buffer) {
        ByteBuffer buf = buffer.duplicate();
        byte[] bytes = new byte[buf.remaining()];
        if (bytes.length > 0) {
            buf.get(bytes);
        }
        return bytes;
    }

    /**
     * 从 ByteBuf 读取所有可读字节, 会移动读索引
     *
     * @param buffer ByteBuf
     * @return 所有可读字节数组
     */
    public static byte[] readBytesRemaining(final ByteBuf buffer) {
        byte[] bytes = new byte[buffer.readableBytes()];
        if (bytes.length > 0) {
            buffer.readBytes(bytes);
        }
        return bytes;
    }

    /**
     * 以小端模式写入浮点数
     *
     * @param buf
     * @param value
     */
    public static void writeFloatLE(ByteBuf buf, float value) {
        buf.writeIntLE(Float.floatToRawIntBits(value));
    }

    /**
     * 以小端模式读取浮点
     *
     * @param buf
     * @return
     */
    public static float readFloatLE(ByteBuf buf) {
        return Float.intBitsToFloat(buf.readIntLE());
    }

    public static ByteBuf toByteBuf(byte[] bytes) {
        ByteBuf byteBuf = directBuffer(bytes.length);
        byteBuf.writeBytes(bytes);
        return byteBuf;
    }

}
