package com.qhiot.transport.codec.util;

/**
 * 命令字定义
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-12-23 21:05
 */
public class Command {

    /** 空白命令数据 */
    public static final byte[] EMPTY_DATA                           = {};

    /** 心跳 */
    public static final byte   HEARTBEAT                            = 0x01;

    /** 读终端信息 */
    public static final byte   READ_DEV_PARAM                       = 0x02;

    /** 温湿度数据 */
    public static final byte   HUMITURE_DATA                        = 0x02;

    /** 写终端信息 */
    public static final byte   WRITE_DEV_PARAM                      = 0x03;

    /** 代码更新请求包 */
    public static final byte   FIRMWARE_UPGRADE                     = 0x04;

    /** 代码更新数据包 */
    public static final byte   FIRMWARE_DATA                        = 0x05;

    /** 2.4G 标签卡 */
    public static final byte   TAG_24G                              = 0x06;

    /** 上电或掉电 */
    public static final byte   POWER                                = 0x07;

    /** 透传指令 */
    public static final byte   TRANSMIT_TRANSPARENTLY               = 0x10;

    /** 电控箱状态 */
    public static final byte   ELECTRIC_CABINET                     = 0x11;

    /** 超声波探测结果上告 */
    public static final byte   ULTRASONIC                           = 0x12;

    // ----------------------------------------------------------------------------------------------

    /** 短信目标号码 */
    public static final byte   SMS_SEND_MOBILE                      = 0x12;

    /** 短信目标内容 */
    public static final byte   SMS_SEND_CONTENT                     = 0x13;

    /** 读短信 */
    public static final byte   SMS_READ                             = 0x14;

    /** 电量计数据 */
    public static final byte   COULOMETER_DATA                      = 0x15;
    /** 读取电量计数据 */
    public static final byte   COULOMETER_DATA_READ                 = 0x16;

    /** 终端上报 ICCID */
    public static final byte   ICCID                                = 0x18;

    // ----------------------------------------------------------------------------------------------

    /** 摄像头配置 - 写入一条指令 */
    public static final byte   CAM_CONFIG_WRITE                     = 0x20;
    /** 摄像头配置 - 读取一条指令 */
    public static final byte   CAM_CONFIG_READ                      = 0x21;
    /** 摄像头配置 - 删除一条指令 */
    public static final byte   CAM_CONFIG_DEL                       = 0x22;
    /** 摄像头配置 - 删除全部指令 */
    public static final byte   CAM_CONFIG_DEL_ALL                   = 0x23;
    /** 摄像头配置 - 读取全部指令 */
    public static final byte   CAM_CONFIG_READ_ALL                  = 0x24;
    /** 拍照请求，回应0x25表示成功 */
    public static final byte   CAM_SNAP                             = 0x25;
    /** 拍照失败回应 */
    public static final byte   CAM_SNAP_FAIL                        = 0x26;
    /** 停止照片上传 */
    public static final byte   CAM_SNAP_STOP_UPLOADING              = 0x27;
    /** 照片上传 */
    public static final byte   CAM_SNAP_UPLOADING                   = 0x28;

    // ----------------------------------------------------------------------------------------------

    /** 摄像头配置拍照时间点 - 写入一条指令 */
    public static final byte   CAM_CONFIG_TIME_POINT_WRITE          = 0x29;
    /** 摄像头配置拍照时间点 - 读取一条指令 */
    public static final byte   CAM_CONFIG_TIME_POINT_READ           = 0x2A;
    /** 摄像头配置拍照时间点 - 删除一条指令 */
    public static final byte   CAM_CONFIG_TIME_POINT_DEL            = 0x2B;
    /** 摄像头配置拍照时间点 - 删除全部指令 */
    public static final byte   CAM_CONFIG_TIME_POINT_DEL_ALL        = 0x2C;
    /** 摄像头配置拍照时间点 - 读取全部指令 */
    public static final byte   CAM_CONFIG_TIME_POINT_READ_ALL       = 0x2D;

    // ----------------------------------------------------------------------------------------------

    /** 输入开关量控制配置 - 写入一条指令 */
    public static final byte   SWITCH_INPUT_CONFIG_WRITE            = 0x30;
    /** 输入开关量控制配置 - 读取一条指令 */
    public static final byte   SWITCH_INPUT_CONFIG_READ             = 0x31;
    /** 输入开关量控制配置 - 删除一条指令 */
    public static final byte   SWITCH_INPUT_CONFIG_DEL              = 0x32;
    /** 输入开关量控制配置 - 删除全部指令 */
    public static final byte   SWITCH_INPUT_CONFIG_DEL_ALL          = 0x33;
    /** 输入开关量控制配置 - 读取全部指令 */
    public static final byte   SWITCH_INPUT_CONFIG_READ_ALL         = 0x34;
    /** 输入开关量 当前状态 */
    public static final byte   SWITCH_INPUT_STATE                   = 0x35;
    /** 输入开关量 读取状态 */
    public static final byte   SWITCH_INPUT_STATE_READ              = 0x36;

    // ----------------------------------------------------------------------------------------------

    /** 输出开关量控制配置 - 写入一条指令 */
    public static final byte   SWITCH_OUTPUT_CONFIG_WRITE           = 0x40;
    /** 输出开关量控制配置 - 读取一条指令 */
    public static final byte   SWITCH_OUTPUT_CONFIG_READ            = 0x41;
    /** 输出开关量控制配置 - 删除一条指令 */
    public static final byte   SWITCH_OUTPUT_CONFIG_DEL             = 0x42;
    /** 输出开关量控制配置 - 删除全部指令 */
    public static final byte   SWITCH_OUTPUT_CONFIG_DEL_ALL         = 0x43;
    /** 输出开关量控制配置 - 读取全部指令 */
    public static final byte   SWITCH_OUTPUT_CONFIG_READ_ALL        = 0x44;

    /** 输出开关量 周期指令 配置 - 写入一条指令 */
    public static final byte   SWITCH_OUTPUT_PERIOD_CONFIG_WRITE    = 0x45;
    /** 输出开关量 周期指令 配置 - 读取一条指令 */
    public static final byte   SWITCH_OUTPUT_PERIOD_CONFIG_READ     = 0x46;
    /** 输出开关量 周期指令 配置 - 删除一条指令 */
    public static final byte   SWITCH_OUTPUT_PERIOD_CONFIG_DEL      = 0x47;
    /** 输出开关量 周期指令 配置 - 删除全部指令 */
    public static final byte   SWITCH_OUTPUT_PERIOD_CONFIG_DEL_ALL  = 0x48;
    /** 输出开关量 周期指令 配置 - 读取全部指令 */
    public static final byte   SWITCH_OUTPUT_PERIOD_CONFIG_READ_ALL = 0x49;
    /** 输出开关量 手动控制 - 控制某一路开关量-打开 */
    public static final byte   SWITCH_OUTPUT_OPEN                   = 0x4A;
    /** 输出开关量 手动控制 - 控制某一路开关量-关闭 */
    public static final byte   SWITCH_OUTPUT_CLOSE                  = 0x4B;

    /** 输出开关量 读取状态 */
    public static final byte   SWITCH_OUTPUT_STATE_READ             = 0x4C;
    /** 输出开关量 当前状态 */
    public static final byte   SWITCH_OUTPUT_STATE                  = 0x4D;

    // ----------------------------------------------------------------------------------------------

    /** PLC状态， 包括工作模式,水位信号,16路设备的状态 */
    public static final byte   PLC_STATE                            = 0x50;
    /** 本地模式 */
    public static final byte   PLC_LOCAL                            = 0x51;
    /** 远程模式 */
    public static final byte   PLC_REMOTE                           = 0x52;
    /** 打开某一路PLC设备 */
    public static final byte   PLC_OPEN_FIELD                       = 0x53;
    /** 关闭某一路设备 */
    public static final byte   PLC_CLOSE_FIELD                      = 0x54;
    /** 读取寄存器 */
    public static final byte   PLC_READ_AR                          = 0x55;
    /** 写寄存器 */
    public static final byte   PLC_WRITE_AR                         = 0x56;

    // ----------------------------------------------------------------------------------------------

    /** 互感器状态 */
    public static final byte   TRANSFORMER_STATE_READ               = 0x60;
    public static final byte   TRANSFORMER_STATE                    = 0x61;

    // ----------------------------------------------------------------------------------------------

    /** 485外设采集相关配置 - 写入一条指令 */
    public static final byte   RS485_CONFIG_WRITE                   = 0x70;
    /** 485外设采集相关配置 - 读取一条指令 */
    public static final byte   RS485_CONFIG_READ                    = 0x71;
    /** 485外设采集相关配置 - 删除一条指令 */
    public static final byte   RS485_CONFIG_DEL                     = 0x72;
    /** 485外设采集相关配置 - 删除全部指令 */
    public static final byte   RS485_CONFIG_DEL_ALL                 = 0x73;
    /** 485外设采集相关配置 - 读取全部指令 */
    public static final byte   RS485_CONFIG_READ_ALL                = 0x74;
    /** 485外设采集相关配置 - 采集数据上传 */
    public static final byte   RS485_COLLECTION_OK                  = 0x75;
    /** 485外设采集相关配置 - 采集失败 */
    public static final byte   RS485_COLLECTION_FAIL                = 0x76;
    /** 485外设采集相关配置 - 手动下发一条指令 */
    public static final byte   RS485_COLLECT                        = 0x77;

    // ----------------------------------------------------------------------------------------------

    /** 输入输出运行模式 */
    public static final byte   IO_WORK_MODEL_WRITE                  = 0x0E;
    public static final byte   IO_WORK_MODEL_READ                   = 0x0F;

    // ----------------------------------------------------------------------------------------------

    /** 重启 */
    public static final byte   REBOOT                               = 0x0C;
    /** 恢复出厂设置 */
    public static final byte   RESTORE                              = 0x0D;
    /** 低级格式化 */
    public static final byte   LOW_LEVEL_FORMAT                     = (byte) 0xFE;
    /** 低级写入 */
    public static final byte   LOW_LEVEL_WRITE                      = (byte) 0xFD;

    // ----------------------------------------------------------------------------------------------

    /** 无线开关控制器 打开 */
    public static final byte   WIRELESS_SWITCH_OPEN                 = 0x01;
    /** 无线开关控制器 关闭 */
    public static final byte   WIRELESS_SWITCH_CLOSE                = 0x02;
    /** 无线开关控制器 打开全部 */
    public static final byte   WIRELESS_SWITCH_OPEN_ALL             = 0x03;
    /** 无线开关控制器 关闭全部 */
    public static final byte   WIRELESS_SWITCH_CLOSE_ALL            = 0x04;

    /** 倾倒 */
    public static final byte   TILTED                               = 0x02;
    /** 车载标签上传数据 */
    public static final byte   ONBOARD_TAG                          = 0x05;
    /** 考勤卡 */
    public static final byte   ATTENDANCE_TAG                       = 0x70;
    /** 称重数据 */
    public static final byte   WEIGHT                               = 0x71;
    /** 电表数据 */
    public static final byte   AMMETER                              = 0x72;

    // ----------------------------------------------------------------------------------------------

    /** 动力设备监测电流配置 */
    public static final byte   OVERLOAD_CURRENT_WRITE_ONE           = (byte) 0x19;
    public static final byte   OVERLOAD_CURRENT_READ_ONE            = (byte) 0x78;
    public static final byte   OVERLOAD_CURRENT_DELETE_ONE          = (byte) 0x79;
    public static final byte   OVERLOAD_CURRENT_DELETE_ALL          = (byte) 0x7A;
    public static final byte   OVERLOAD_CURRENT_READ_ALL            = (byte) 0x7B;

    public static final byte   OVERLOAD_WARNING_CLEAR               = (byte) 0x7C;
    public static final byte   OVERLOAD_WARNING_READ_ONE            = (byte) 0x7D;
    public static final byte   OVERLOAD_WARNING_CLEAR_ALL           = (byte) 0x7E;
    public static final byte   OVERLOAD_WARNING_READ_ALL            = (byte) 0x7F;

    // ----------------------------------------------------------------------------------------------

    /** 历史数据 */
    public static final byte   HISTORY_DATA_READ                    = (byte) 0x80;
    public static final byte   HISTORY_DATA_DELETE                  = (byte) 0x81;
    public static final byte   HISTORY_DATA_COUNT                   = (byte) 0x82;
    public static final byte   HISTORY_DATA_UPLOAD                  = (byte) 0x84;

    /** MODBUS 参数写入 */
    public static final byte   MODBUS_DATA_WRITE                    = (byte) 0x85;

    // ----------------------------------------------------------------------------------------------

    /** 传感器配置 */
    public static final byte   RESOURCE_CONFIG_READ_TEN             = (byte) 0x90;
    public static final byte   RESOURCE_CONFIG_DELETE_TEN           = (byte) 0x91;
    public static final byte   RESOURCE_CONFIG_WRITE_TEN            = (byte) 0x92;
    public static final byte   RESOURCE_CONFIG_READ_ONE             = (byte) 0x93;
    public static final byte   RESOURCE_CONFIG_DELETE_ONE           = (byte) 0x94;
    public static final byte   RESOURCE_CONFIG_WRITE_ONE            = (byte) 0x95;
    public static final byte   RESOURCE_CONFIG_DELETE_ALL           = (byte) 0x96;
    public static final byte   RESOURCE_CONFIG_TEST_DATA            = (byte) 0x97;
    public static final byte   RESOURCE_CONFIG_ASK                  = (byte) 0x98;

    // ----------------------------------------------------------------------------------------------

    /** 通过modbus指令控制输出IO */
    public static final byte   MODBUS_IO_CONTROL                    = (byte) 0x99;


    /** ASCII码数据发送和解析 */
    public static final byte   ASCII_PARAM_ASK                      = (byte) 0xA0;
    public static final byte   ASCII_PARAM_REPORT                   = (byte) 0xA1;

    /** AT指令发送和解析 */
    public static final byte   AT_COMMAND_SEND                      = (byte) 0xA2;
    public static final byte   AT_COMMAND_REPORT                    = (byte) 0xA3;

    /** 设备连接参数配置 */
    public static final byte   EQUIPMENT_CONFIG                     = (byte) 0xA4;
    /** 测点配置 */
    public static final byte   EQUIPMENT_MONITORING_CONFIG          = (byte) 0xA5;
    /** 告警配置 */
    public static final byte   WARNING_CONFIG                       = (byte) 0xA6;
    /** 定时逻辑配置 */
    public static final byte   TIMER_CONFIG                         = (byte) 0xA7;

    /** 采集失败 */
    public static final byte   DATA_COLLECT_FAIL                    = (byte) 0xAA;
}
