package com.qhiot.transport.codec.util;

import java.util.Collection;
import java.util.Iterator;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

/**
 * 升级响应队列
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-01-29 15:21
 */
public class UpgradeQueue extends LinkedBlockingQueue<Object> {

    public static final String FIRMWARE_REQUEST_QUEUE = "FIRMWARE_REQUEST_{}";
    public static final String FIRMWARE_SEGMENT_QUEUE = "FIRMWARE_SEGMENT_{}";

    private long            timeMillis;

    public UpgradeQueue() {
        super(Integer.MAX_VALUE);
        timeMillis = System.currentTimeMillis();
    }

    public UpgradeQueue(int capacity) {
        super(capacity);
        timeMillis = System.currentTimeMillis();
    }

    public UpgradeQueue(Collection<Object> c) {
        super(c);
        timeMillis = System.currentTimeMillis();
    }

    public long getTimeMillis() {
        return timeMillis;
    }

    @Override
    public void put(Object e) throws InterruptedException {
        timeMillis = System.currentTimeMillis();
        super.put(e);
    }

    @Override
    public boolean offer(Object e, long timeout, TimeUnit unit) throws InterruptedException {
        timeMillis = System.currentTimeMillis();
        return super.offer(e, timeout, unit);
    }

    @Override
    public Object take() throws InterruptedException {
        timeMillis = System.currentTimeMillis();
        return super.take();
    }

    @Override
    public boolean add(Object o) {
        timeMillis = System.currentTimeMillis();
        return super.add(o);
    }

    @Override
    public boolean offer(Object e) {
        timeMillis = System.currentTimeMillis();
        return super.offer(e);
    }

    public boolean offerTrue() {
        timeMillis = System.currentTimeMillis();
        return super.offer(Boolean.TRUE);
    }

    @Override
    public Object remove() {
        timeMillis = System.currentTimeMillis();
        return super.remove();
    }

    @Override
    public Object poll(long timeout, TimeUnit unit) throws InterruptedException {
        timeMillis = System.currentTimeMillis();
        return super.poll(timeout, unit);
    }

    @Override
    public int remainingCapacity() {
        timeMillis = System.currentTimeMillis();
        return super.remainingCapacity();
    }

    @Override
    public boolean remove(Object o) {
        timeMillis = System.currentTimeMillis();
        return super.remove(o);
    }

    @Override
    public boolean containsAll(Collection<?> c) {
        timeMillis = System.currentTimeMillis();
        return super.containsAll(c);
    }

    @Override
    public boolean addAll(Collection<?> c) {
        timeMillis = System.currentTimeMillis();
        return super.addAll(c);
    }

    @Override
    public boolean removeAll(Collection<?> c) {
        timeMillis = System.currentTimeMillis();
        return super.removeAll(c);
    }

    @Override
    public boolean retainAll(Collection<?> c) {
        timeMillis = System.currentTimeMillis();
        return super.retainAll(c);
    }

    @Override
    public void clear() {
        timeMillis = System.currentTimeMillis();
        super.clear();
    }

    @Override
    public int size() {
        timeMillis = System.currentTimeMillis();
        return super.size();
    }

    @Override
    public boolean isEmpty() {
        timeMillis = System.currentTimeMillis();
        return super.isEmpty();
    }

    @Override
    public boolean contains(Object o) {
        timeMillis = System.currentTimeMillis();
        return super.contains(o);
    }

    @Override
    public Iterator<Object> iterator() {
        timeMillis = System.currentTimeMillis();
        return super.iterator();
    }

    @Override
    public Object[] toArray() {
        timeMillis = System.currentTimeMillis();
        return super.toArray();
    }

    @Override
    public <T> T[] toArray(T[] a) {
        timeMillis = System.currentTimeMillis();
        return super.toArray(a);
    }

    @Override
    public int drainTo(Collection<? super Object> c) {
        timeMillis = System.currentTimeMillis();
        return super.drainTo(c);
    }

    @Override
    public int drainTo(Collection<? super Object> c, int maxElements) {
        timeMillis = System.currentTimeMillis();
        return super.drainTo(c, maxElements);
    }

    @Override
    public Object poll() {
        timeMillis = System.currentTimeMillis();
        return super.poll();
    }

    @Override
    public Object element() {
        timeMillis = System.currentTimeMillis();
        return super.element();
    }

    @Override
    public Object peek() {
        timeMillis = System.currentTimeMillis();
        return super.peek();
    }
}
