package com.qhiot.transport.codec.util;

import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.qhiot.core.util.StringUtil;

import java.util.concurrent.TimeUnit;

/**
 * 终端响应阻塞队列管理器
 *
 * @author 金建强(ptma@163.com)
 */
public class UpgradeQueueManager {

    private final LoadingCache<String, UpgradeQueue> queuesCache;
    private final static int                               EXPIRATION = 600;

    private static class UpgradeQueueManagerHolder {
        final static UpgradeQueueManager INSTANCE = new UpgradeQueueManager();
    }

    private UpgradeQueueManager(){
        //指定一个如果数据不存在获取数据的方法
        CacheLoader<String, UpgradeQueue> cacheLoader = new CacheLoader<String, UpgradeQueue>() {
            @Override
            public UpgradeQueue load(String key) throws Exception {
                return new UpgradeQueue();
            }
        };

        queuesCache = CacheBuilder.newBuilder()
            .expireAfterAccess(EXPIRATION, TimeUnit.SECONDS)
            .build(cacheLoader);
    }

    public static UpgradeQueueManager getInstance() {
        return UpgradeQueueManagerHolder.INSTANCE;
    }

    public static UpgradeQueue getAndClearQueue(String template, Object... values) {
        return UpgradeQueueManager.getInstance().getAndClear(StringUtil.format(template, values));
    }

    public UpgradeQueue getAndClear(String queueName) {
        try {
            return queuesCache.get(queueName);
        } catch (Exception e) {

        }
        UpgradeQueue queue = new UpgradeQueue();
        queuesCache.put(queueName, queue);
        return queue;
    }

}
