package com.qhiot.transport.config;

import com.qhiot.core.util.SpringUtil;
import com.qhiot.transport.netty.server.SocketServer;
import com.qhiot.transport.netty.server.UdpServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.stereotype.Component;

/**
 * Spring 上下文事件监听器
 */
@Component
public class TransportSpringListener implements ApplicationListener<ContextRefreshedEvent> {

    private static final Logger LOGGER = LoggerFactory.getLogger("Init.Transport.Server");

    @Override
    public void onApplicationEvent(ContextRefreshedEvent event) {
        // root application context 没有parent
        if (event.getApplicationContext().getParent() == null) {
            SocketServer socketServer = SpringUtil.getBean(SocketServer.class);
            if (socketServer != null) {
                try {
                    socketServer.start();
                    LOGGER.info("监听: TCP 端口 {}", socketServer.getPort());
                } catch (Exception e) {
                    LOGGER.error("监听 TCP 端口 {} 出错: {}", socketServer.getPort(), e.getMessage(), e);
                }
            }

            UdpServer udpServer = SpringUtil.getBean(UdpServer.class);
            if (udpServer != null) {
                try {
                    udpServer.start();
                    LOGGER.info("监听: UDP 端口 {}", udpServer.getPort());
                } catch (Exception e) {
                    LOGGER.error("监听 UDP 端口 {} 出错: {}", udpServer.getPort(), e.getMessage(), e);
                }
            }
        }
    }

}
