package com.qhiot.transport.context;

import com.qhiot.transport.netty.handler.OnlinePersistence;
import com.qhiot.transport.netty.server.SocketServer;
import com.qhiot.transport.netty.server.UdpServer;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.util.concurrent.Future;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationListener;
import org.springframework.context.event.ContextClosedEvent;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author 金建强(ptma/@163.com)
 * @version 2017/6/15 18:41
 */
@Component("transportServerContextClosedListener")
public class TransportServerContextClosedListener implements ApplicationListener<ContextClosedEvent> {

    private static final Logger LOGGER = LoggerFactory.getLogger(TransportServerContextClosedListener.class);

    @Resource(name = "bossGroup")
    private NioEventLoopGroup bossGroup;

    @Resource(name = "workerGroup")
    private NioEventLoopGroup workerGroup;

    @Resource
    private SocketServer socketServer;


    @Resource(name = "udpBossGroup")
    private NioEventLoopGroup udpBossGroup;

    @Resource(name = "udpWorkerGroup")
    private EventLoopGroup udpWorkerGroup;

    @Resource
    private UdpServer udpServer;

    @Resource
    private OnlinePersistence onlinePersistence;

    @Override
    public void onApplicationEvent(ContextClosedEvent event) {
        if (event.getApplicationContext().getParent() == null) {
            if (onlinePersistence != null){
                onlinePersistence.close();
            }

            if (bossGroup != null) {
                try {
                    Future<?> future = bossGroup.shutdownGracefully(); //3
                    future.sync();
                    LOGGER.debug("bossGroup shutdown.");
                } catch (Exception e) {

                }
            }

            if (workerGroup != null) {
                try {
                    Future<?> future = workerGroup.shutdownGracefully(); //3
                    future.sync();
                    LOGGER.debug("workerGroup shutdown.");
                } catch (Exception e) {

                }
            }
            if (socketServer != null) {
                try {
                    socketServer.stop();
                } catch (Exception e) {

                }
            }

            if (udpBossGroup != null) {
                try {
                    Future<?> future = udpBossGroup.shutdownGracefully(); //3
                    future.sync();
                    LOGGER.debug("udpBossGroup shutdown.");
                } catch (Exception e) {

                }
            }

            if (udpWorkerGroup != null) {
                try {
                    Future<?> future = udpWorkerGroup.shutdownGracefully(); //3
                    future.sync();
                    LOGGER.debug("udpWorkerGroup shutdown.");
                } catch (Exception e) {

                }
            }
            if (udpServer != null) {
                try {
                    udpServer.stop();
                } catch (Exception e) {

                }
            }

        }
    }
}
