package com.qhiot.transport.message.bean;

import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;

/**
 * @author 金建强(ptma@163.com)
 * @version 2016/6/22 11:52
 */
public class DeviceParamBuilder {

    public static DeviceParam read(ByteBuf buf) throws DecodeException {
        try {
            DeviceParam info = new DeviceParam();

            info.initMark = buf.readUnsignedByte();
            buf.readBytes(info.deviceName);
            buf.readBytes(info.serialNumber);
            buf.readBytes(info.softwareVersion);
            buf.readBytes(info.hardwareVersion);
            info.deviceAddr = buf.readIntLE();
            info.antenna1Addr = buf.readUnsignedShortLE();
            info.antenna2Addr = buf.readUnsignedShortLE();
            info.heartbeatPeriod = buf.readUnsignedShortLE();
            info.excitePeriod = buf.readUnsignedShortLE();
            info.ultrasonicDetectPeriod = buf.readUnsignedShortLE();
            info.ultrasonicDetectDistance = buf.readUnsignedShortLE();
            info.frequency125K = buf.readUnsignedShortLE();
            info.routeValidMark = buf.readUnsignedByte();
            info.routeGatewayAddr = buf.readIntLE();
            info.timestamp = buf.readIntLE();
            info.accumulatedRunningTime = buf.readUnsignedShortLE();
            info.reserved1 = buf.readShortLE();
            info.reserved2 = buf.readUnsignedShortLE();
            info.reserved3 = buf.readUnsignedShortLE();
            info.reserved4 = buf.readUnsignedShortLE();
            info.reserved5 = buf.readUnsignedShortLE();
            info.reserved6 = buf.readUnsignedShortLE();
            buf.readBytes(info.chipId);
            return info;
        } catch (RuntimeException e) {
            throw new DecodeException(e);
        }
    }

    public static byte[] toBytes(DeviceParam deviceParam) {
        ByteBuf buf = ByteBufUtil.directBuffer(DeviceParam.DATA_LENGTH);
        try {
            buf.writeByte(deviceParam.initMark);
            buf.writeBytes(deviceParam.deviceName);
            buf.writeBytes(deviceParam.serialNumber);
            buf.writeBytes(deviceParam.softwareVersion);
            buf.writeBytes(deviceParam.hardwareVersion);
            buf.writeIntLE(deviceParam.deviceAddr);
            buf.writeShortLE(deviceParam.antenna1Addr);
            buf.writeShortLE(deviceParam.antenna2Addr);
            buf.writeShortLE(deviceParam.heartbeatPeriod);
            buf.writeShortLE(deviceParam.excitePeriod);
            buf.writeShortLE(deviceParam.ultrasonicDetectPeriod);
            buf.writeShortLE(deviceParam.ultrasonicDetectDistance);
            buf.writeShortLE(deviceParam.frequency125K);
            buf.writeByte(deviceParam.routeValidMark);
            buf.writeIntLE(deviceParam.routeGatewayAddr);

            buf.writeIntLE(deviceParam.timestamp);
            buf.writeShortLE(deviceParam.accumulatedRunningTime);
            buf.writeShortLE(deviceParam.reserved1);
            buf.writeShortLE(deviceParam.reserved2);
            buf.writeShortLE(deviceParam.reserved3);
            buf.writeShortLE(deviceParam.reserved4);
            buf.writeShortLE(deviceParam.reserved5);
            buf.writeShortLE(deviceParam.reserved6);
            buf.writeBytes(deviceParam.chipId);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }
}
