package com.qhiot.transport.message.bean;

import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;

/**
 * 新数据采集配置 - 设备配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/6/23 16:26
 */
public class EquipmentConfigBuilder {

    /**
     * 解码
     * @param buf
     * @return
     */
    public static EquipmentConfig read(ByteBuf buf) {
        try {
            EquipmentConfig config = new EquipmentConfig();
            config.valid = buf.readByte();
            config.commandIndex = buf.readShortLE();
            config.equipmentType = buf.readByte();
            config.manufacturers = buf.readByte();
            config.equipmentModel = buf.readByte();
            config.communicationInterface = buf.readByte();
            config.ip1OrBaudRate = buf.readByte();
            config.ip2OrDataBit = buf.readByte();
            config.ip3OrParityBit = buf.readByte();
            config.ip4OrStopBit = buf.readByte();
            config.portOrReserved1 = buf.readUnsignedShortLE();
            buf.readBytes(config.reserved2);
            config.byteOrder16 = buf.readByte();
            config.byteOrder32 = buf.readByte();
            config.unionIndex = buf.readShortLE();
            buf.readBytes(config.reserved3);
            return config;
        } catch (RuntimeException e) {
            throw new DecodeException(e);
        }
    }

    /**
     * 编码
     * @param config
     * @return
     */
    public static byte[] toBytes(EquipmentConfig config) {
        ByteBuf buf = ByteBufUtil.directBuffer(ResourceConfig.DATA_LENGTH);
        try {
            buf.writeByte(config.valid);
            buf.writeShortLE(config.commandIndex);
            buf.writeByte(config.equipmentType);
            buf.writeByte(config.manufacturers);
            buf.writeByte(config.equipmentModel);
            buf.writeByte(config.communicationInterface);
            buf.writeByte(config.ip1OrBaudRate);
            buf.writeByte(config.ip2OrDataBit);
            buf.writeByte(config.ip3OrParityBit);
            buf.writeByte(config.ip4OrStopBit);
            buf.writeShortLE(config.portOrReserved1);
            buf.writeBytes(config.reserved2);
            buf.writeByte(config.byteOrder16);
            buf.writeByte(config.byteOrder32);
            buf.writeShortLE(config.unionIndex);
            buf.writeBytes(config.reserved3);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }

}
