package com.qhiot.transport.message.bean;

import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;

/**
 * 新数据采集配置 - 设备配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/6/23 16:26
 */
public class EquipmentMonitoringBuilder {

    /**
     * 解码
     * @param buf
     * @return
     */
    public static EquipmentMonitoring read(ByteBuf buf) {
        try {
            EquipmentMonitoring config = new EquipmentMonitoring();
            config.valid = buf.readByte();
            config.configIndex = buf.readShortLE();
            config.equipmentIndex = buf.readShortLE();
            config.protocol = buf.readByte();
            config.address = buf.readUnsignedByte();
            config.registerType = buf.readByte();
            config.registerAddress = buf.readShortLE();
            config.dataLength = buf.readShortLE();
            buf.readBytes(config.reserved2);
            config.method = buf.readByte();
            config.dataQuantity = buf.readByte();

            config.deadZone = ByteBufUtil.readFloatLE(buf);

            config.period = buf.readUnsignedShortLE();
            config.storeRegisterType = buf.readByte();
            config.storeRegisterAddress = buf.readShortLE();
            config.rw = buf.readByte();
            config.unionIndex = buf.readShortLE();
            buf.readBytes(config.reserved5);
            return config;
        } catch (RuntimeException e) {
            throw new DecodeException(e);
        }
    }

    /**
     * 编码
     * @param config
     * @return
     */
    public static byte[] toBytes(EquipmentMonitoring config) {
        ByteBuf buf = ByteBufUtil.directBuffer(EquipmentMonitoring.DATA_LENGTH);
        try {
            buf.writeByte(config.valid);
            buf.writeShortLE(config.configIndex);
            buf.writeShortLE( config.equipmentIndex);
            buf.writeByte(config.protocol);
            buf.writeByte(config.address);
            buf.writeByte(config.registerType);
            buf.writeShortLE(config.registerAddress);
            buf.writeShortLE(config.dataLength);
            buf.writeBytes(config.reserved2);
            buf.writeByte(config.method);
            buf.writeByte(config.dataQuantity);

            ByteBufUtil.writeFloatLE(buf, config.deadZone);

            buf.writeShortLE(config.period);
            buf.writeByte(config.storeRegisterType);
            buf.writeShortLE(config.storeRegisterAddress);
            buf.writeByte(config.rw);
            buf.writeShortLE(config.unionIndex);
            buf.writeBytes(config.reserved5);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }

}
