package com.qhiot.transport.message.bean;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;

/**
 * @author 金建强(ptma@163.com)
 * @version 2016/6/22 11:52
 */
public class GatewayParamBuilder {

    public static GatewayParam read(ByteBuf buf) {
        try {
            GatewayParam info = new GatewayParam();

            info.initMark = buf.readUnsignedByte();
            buf.readBytes(info.deviceName);
            buf.readBytes(info.serialNumber);
            buf.readBytes(info.softwareVersion);
            buf.readBytes(info.hardwareVersion);
            info.gatewayAddr = buf.readIntLE();
            info.serverIP = buf.readUnsignedInt();
            info.serverPort = buf.readUnsignedShortLE();
            info.server2IP = buf.readUnsignedInt();
            info.server2Port = buf.readUnsignedShortLE();
            info.gatewayIP = buf.readUnsignedInt();
            info.subnetMask = buf.readUnsignedInt();
            info.deviceIP = buf.readUnsignedInt();
            info.devicePort = buf.readUnsignedShortLE();
            info.heartbeatPeriod = buf.readUnsignedShortLE();
            info.gprsRssi = buf.readByte();
            info.wirelessPower = buf.readByte();
            info.speakerClosed = buf.readByte();
            info.manualIO = buf.readByte();

            info.reserved1 = buf.readUnsignedShortLE();
            info.reserved2 = buf.readUnsignedShortLE();
            info.reserved3 = buf.readUnsignedShortLE();
            info.reserved4 = buf.readUnsignedShortLE();
            info.reserved5 = buf.readUnsignedShortLE();

            info.reserved6 = buf.readByte();
            info.reserved7 = buf.readByte();
            info.reserved8 = buf.readByte();
            info.reserved9 = buf.readByte();
            info.reserved10 = buf.readByte();

            info.timestamp = buf.readIntLE();
            info.cumulativeOperatingTime = buf.readUnsignedShortLE();
            info.manufactureDate = buf.readIntLE();
            buf.readBytes(info.chipId);
            return info;
        } catch (RuntimeException e) {
            throw new DecodeException(e);
        }
    }

    public static byte[] toBytes(GatewayParam gatewayParam) {
        ByteBuf buf = ByteBufUtil.directBuffer(GatewayParam.DATA_LENGTH);
        try {
            buf.writeByte(gatewayParam.initMark);
            buf.writeBytes(gatewayParam.deviceName);
            buf.writeBytes(gatewayParam.serialNumber);
            buf.writeBytes(gatewayParam.softwareVersion);
            buf.writeBytes(gatewayParam.hardwareVersion);
            buf.writeIntLE(gatewayParam.gatewayAddr);
            buf.writeBytes(ByteUtil.ipToBytes(gatewayParam.serverIP));
            buf.writeShortLE(gatewayParam.serverPort);
            buf.writeBytes(ByteUtil.ipToBytes(gatewayParam.server2IP));
            buf.writeShortLE(gatewayParam.server2Port);
            buf.writeBytes(ByteUtil.ipToBytes(gatewayParam.gatewayIP));
            buf.writeBytes(ByteUtil.ipToBytes(gatewayParam.subnetMask));
            buf.writeBytes(ByteUtil.ipToBytes(gatewayParam.deviceIP));
            buf.writeShortLE(gatewayParam.devicePort);
            buf.writeShortLE(gatewayParam.heartbeatPeriod);
            buf.writeByte(gatewayParam.gprsRssi);
            buf.writeByte(gatewayParam.wirelessPower);
            buf.writeByte(gatewayParam.speakerClosed);
            buf.writeByte(gatewayParam.manualIO);

            buf.writeShortLE(gatewayParam.reserved1);
            buf.writeShortLE(gatewayParam.reserved2);
            buf.writeShortLE(gatewayParam.reserved3);
            buf.writeShortLE(gatewayParam.reserved4);
            buf.writeShortLE(gatewayParam.reserved5);

            buf.writeByte(gatewayParam.reserved6);
            buf.writeByte(gatewayParam.reserved7);
            buf.writeByte(gatewayParam.reserved8);
            buf.writeByte(gatewayParam.reserved9);
            buf.writeByte(gatewayParam.reserved10);

            buf.writeIntLE(gatewayParam.timestamp);
            buf.writeShortLE(gatewayParam.cumulativeOperatingTime);
            buf.writeIntLE(gatewayParam.manufactureDate);
            buf.writeBytes(gatewayParam.chipId);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }
}
