package com.qhiot.transport.message.bean;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;

import java.nio.ByteOrder;

/**
 * @author 金建强(ptma@163.com)
 * @version 2017-03-31 16:04
 */
public class OverloadCurrentBuilder {

    public static OverloadCurrent read(ByteBuf buf) {
        try {
            OverloadCurrent config = new OverloadCurrent();
            config.overloadValid = buf.readByte();
            byte[] floatBytes = new byte[4];
            buf.readBytes(floatBytes);
            config.overloadCurrent = ByteUtil.toFloat(floatBytes, ByteOrder.LITTLE_ENDIAN);
            config.overloadExpire = buf.readByte();

            config.lowloadValid = buf.readByte();
            floatBytes = new byte[4];
            buf.readBytes(floatBytes);
            config.lowloadCurrent = ByteUtil.toFloat(floatBytes, ByteOrder.LITTLE_ENDIAN);
            config.lowloadExpire = buf.readByte();

            config.standbyValid = buf.readByte();
            floatBytes = new byte[4];
            buf.readBytes(floatBytes);
            config.standbyCurrent = ByteUtil.toFloat(floatBytes, ByteOrder.LITTLE_ENDIAN);
            config.standbyExpire = buf.readByte();

            return config;
        } catch (RuntimeException e) {
            throw new DecodeException(e);
        }
    }

    public static byte[] toBytes(OverloadCurrent config) {
        ByteBuf buf = ByteBufUtil.directBuffer(OverloadCurrent.DATA_LENGTH);
        try {
            buf.writeByte(config.overloadValid);
            byte[] floatBytes = ByteUtil.toBytes(config.overloadCurrent, ByteOrder.LITTLE_ENDIAN);
            buf.writeBytes(floatBytes);
            buf.writeByte(config.overloadExpire);

            buf.writeByte(config.lowloadValid);
            floatBytes = ByteUtil.toBytes(config.lowloadCurrent, ByteOrder.LITTLE_ENDIAN);
            buf.writeBytes(floatBytes);
            buf.writeByte(config.lowloadExpire);

            buf.writeByte(config.standbyValid);
            floatBytes = ByteUtil.toBytes(config.standbyCurrent, ByteOrder.LITTLE_ENDIAN);
            buf.writeBytes(floatBytes);
            buf.writeByte(config.standbyExpire);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }
}
