package com.qhiot.transport.message.bean;

import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;

/**
 * 测点配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/6/23 16:26
 */
public class ResourceConfigBuilder {

    /**
     * 解码
     * @param buf
     * @return
     */
    public static ResourceConfig read(ByteBuf buf) {
        try {
            ResourceConfig config = new ResourceConfig();
            config.valid = buf.readByte();
            config.protocol = buf.readByte();
            config.resource = buf.readByte();
            config.commPort = buf.readByte();
            config.baudRate = buf.readByte();
            config.dataBit = buf.readByte();
            config.stopBit = buf.readByte();
            config.parityBit = buf.readByte();
            config.period = buf.readUnsignedShortLE();
            if (config.period == 0) {
                config.period = 300;
            }
            config.address = buf.readUnsignedByte();
            config.functionCode = buf.readByte();
            config.startingAddr = buf.readUnsignedShortLE();
            config.method = buf.readByte();
            config.dataQuantity = buf.readUnsignedShortLE();
            config.commandLength = buf.readByte();
            buf.readBytes(config.command);
            config.registerType = buf.readByte();
            config.storeAddress = buf.readUnsignedShortLE();
            config.upperBound = ByteBufUtil.readFloatLE(buf);
            config.lowerBound = ByteBufUtil.readFloatLE(buf);
            config.upperBoundEnabled = buf.readByte();
            config.lowerBoundEnabled = buf.readByte();
            config.reserved = buf.readByte();
            return config;
        } catch (RuntimeException e) {
            throw new DecodeException(e);
        }
    }

    /**
     * 编码
     * @param config
     * @return
     */
    public static byte[] toBytes(ResourceConfig config) {
        ByteBuf buf = ByteBufUtil.directBuffer(ResourceConfig.DATA_LENGTH);
        try {
            buf.writeByte(config.valid);
            buf.writeByte(config.protocol);
            buf.writeByte(config.resource);
            buf.writeByte(config.commPort);
            buf.writeByte(config.baudRate);
            buf.writeByte(config.dataBit);
            buf.writeByte(config.stopBit);
            buf.writeByte(config.parityBit);
            buf.writeShortLE(config.period);
            buf.writeByte(config.address);
            buf.writeByte(config.functionCode);
            buf.writeShortLE(config.startingAddr);
            buf.writeByte(config.method);
            buf.writeShortLE(config.dataQuantity);
            buf.writeByte(config.commandLength);
            buf.writeBytes(config.command);
            buf.writeByte(config.registerType);
            buf.writeShortLE(config.storeAddress);
            ByteBufUtil.writeFloatLE(buf, config.upperBound);
            ByteBufUtil.writeFloatLE(buf, config.lowerBound);
            buf.writeByte(config.upperBoundEnabled);
            buf.writeByte(config.lowerBoundEnabled);
            buf.writeByte(config.reserved);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }

}
