package com.qhiot.transport.message.bean;

import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;

/**
 * @author 金建强(ptma@163.com)
 * @version 2016/6/23 16:26
 */
public class Rs485ConfigBuilder {

    public static Rs485Config read(ByteBuf buf) {
        try {
            Rs485Config config = new Rs485Config();
            config.commandValid = buf.readByte();
            config.commPort = buf.readByte();
            config.baudRate = buf.readByte();
            config.parityBit = buf.readByte();
            config.period = buf.readUnsignedShortLE();
            config.commandLength = buf.readUnsignedByte();
            config.channel = buf.readByte();
            config.dataType = buf.readUnsignedByte();
            buf.readBytes(config.command);
            return config;
        } catch (RuntimeException e) {
            throw new DecodeException(e);
        }
    }

    public static byte[] toBytes(Rs485Config config) {
        ByteBuf buf = ByteBufUtil.directBuffer(Rs485Config.DATA_LENGTH);
        try {
            buf.writeByte(config.commandValid);
            buf.writeByte(config.commPort);
            buf.writeByte(config.baudRate);
            buf.writeByte(config.parityBit);
            buf.writeShortLE(config.period);
            buf.writeByte(config.commandLength);
            buf.writeByte(config.channel);
            buf.writeByte(config.dataType);
            buf.writeBytes(config.command);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }
}
