package com.qhiot.transport.message.bean;

import com.qhiot.core.util.ByteUtil;
import com.qhiot.transport.codec.exception.DecodeException;
import io.netty.buffer.ByteBuf;

/**
 * @author 金建强(ptma@163.com)
 * @version 2016/6/22 11:52
 */
public class Tag24gBuilder {

    public static Tag24g read(ByteBuf buf) {
        try {
            Tag24g tag24g = new Tag24g();

            /** 前导 2B */
            byte[] lead = new byte[2];
            buf.readBytes(lead);
            if (ByteUtil.equals(lead, new byte[] {0x0D, 0x0D})) {
                /** 长度 1B */
                short length = buf.readUnsignedByte();
                /** 数据 4B */
                byte[]  data = new byte[4];
                buf.readBytes(data);
                /** RSSI 1B */
                byte rssi = buf.readByte();
                /** ID  5B */
                byte[] tagId = new byte[5];
                buf.readBytes(tagId);
                buf.readByte();
                /** 异或校验和 XOR 1B */
                byte xor = buf.readByte();

                tag24g.setRssi(rssi);
                tag24g.setTagId(ByteUtil.toHexString(tagId, false));
            } else if (ByteUtil.equals(lead, new byte[] {0x02, 0x03})) {
                /* 上海仁微
                数据结构定义如下 总共13个字节
                INT8U	Head_2[2]; 		帧头两个字节固定为 02 03
                INT32U  ID;				卡号:小端格式,无符号4字节
                INT8U	RSSI;			信号值:平台显示时要加一个负数符合
                INT16U	Voltage;         卡片电池电压:小端格式/100
                INT32U  Rec_TimeStamp;  时间戳 小端格式
                 */
                long tagId = buf.readUnsignedIntLE();
                int rssi = buf.readByte();
                float voltage = buf.readUnsignedShortLE()/100f;
                int timestamp = buf.readIntLE();

                if (rssi>0) {
                    rssi = rssi * -1;
                }
                tag24g.setRssi(rssi);
                tag24g.setTagId(String.valueOf(tagId));
                tag24g.setVoltage(voltage);
                tag24g.setTimestamp(timestamp);
            } else {
                return null;
            }
            return tag24g;
        } catch (RuntimeException e) {
            throw new DecodeException(e);
        }
    }
}
