package com.qhiot.transport.message.bean;

import com.qhiot.transport.codec.exception.DecodeException;
import com.qhiot.transport.codec.util.ByteBufUtil;
import io.netty.buffer.ByteBuf;

/**
 * 新数据采集配置 - 设备配置
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/6/23 16:26
 */
public class WarningConfigBuilder {

    /**
     * 解码
     * @param buf
     * @return
     */
    public static WarningConfig read(ByteBuf buf) {
        try {
            WarningConfig config = new WarningConfig();
            config.valid = buf.readByte();
            config.configIndex = buf.readShortLE();

            config.registerType1 = buf.readByte();
            config.registerAddress1 = buf.readShortLE();
            config.method1 = buf.readByte();

            config.registerType2 = buf.readByte();
            config.registerAddress2 = buf.readShortLE();
            config.method2 = buf.readByte();

            config.paramRelation = buf.readByte();
            config.condition1 = buf.readByte();
            config.condition2 = buf.readByte();
            config.conditionRelation = buf.readByte();

            config.value1 = ByteBufUtil.readFloatLE(buf);
            config.value2 = ByteBufUtil.readFloatLE(buf);

            config.warningRegisterType = buf.readByte();
            config.warningRegisterAddress = buf.readShortLE();
            config.warningMethod = buf.readByte();

            config.period = buf.readUnsignedShortLE();
            buf.readBytes(config.reserved4);
            return config;
        } catch (RuntimeException e) {
            throw new DecodeException(e);
        }
    }

    /**
     * 编码
     * @param config
     * @return
     */
    public static byte[] toBytes(WarningConfig config) {
        ByteBuf buf = ByteBufUtil.directBuffer(WarningConfig.DATA_LENGTH);
        try {
            buf.writeByte(config.valid);
            buf.writeShortLE(config.configIndex);

            buf.writeByte(config.registerType1);
            buf.writeShortLE(config.registerAddress1);
            buf.writeByte(config.method1);

            buf.writeByte(config.registerType2);
            buf.writeShortLE(config.registerAddress2);
            buf.writeByte(config.method2);

            buf.writeByte(config.paramRelation);
            buf.writeByte(config.condition1);
            buf.writeByte(config.condition2);
            buf.writeByte(config.conditionRelation);

            ByteBufUtil.writeFloatLE(buf, config.value1);
            ByteBufUtil.writeFloatLE(buf, config.value2);

            buf.writeByte(config.warningRegisterType);
            buf.writeShortLE(config.warningRegisterAddress);
            buf.writeByte(config.warningMethod);

            buf.writeShortLE(config.period);
            buf.writeBytes(config.reserved4);

            return ByteBufUtil.getBytesRemaining(buf);
        } finally {
            ByteBufUtil.safeRelease(buf);
        }
    }

}
