package com.qhiot.transport.netty.handler;

import com.qhiot.transport.netty.handler.codec.ByteDecoder;
import com.qhiot.transport.netty.handler.codec.ByteEncoder;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.timeout.IdleStateHandler;

import java.util.concurrent.TimeUnit;

/**
 * 接收Socket新通道时的初始化
 *
 * @author 金建强(ptma@163.com)
 * @version 2015-11-27 14:23.
 */
public class DefaultChannelInitializer extends io.netty.channel.ChannelInitializer<SocketChannel> {

    private ChannelHandler channelHandler;

    private ByteDecoder decoder = new ByteDecoder();

    private ByteEncoder encoder = new ByteEncoder();

    @Override
    protected void initChannel(SocketChannel ch) throws Exception {
        ChannelPipeline pipeline = ch.pipeline();

        // ByteBuf delimiter = Unpooled.copiedBuffer("END".getBytes());
        // pipeline.addLast(new DelimiterBasedFrameDecoder(8192, false, delimiter));

        pipeline.addLast("decoder", decoder);
        pipeline.addLast("encoder", encoder);

        /**
        Linux
             sysctl.conf文件里面设置：
             net.ipv4.tcp_keepalive_time = 10
             net.ipv4.tcp_keepalive_probes = 2
             net.ipv4.tcp_keepalive_intvl = 2

        Windows
            http://www.cnblogs.com/fczjuever/archive/2013/04/05/3000697.html
            HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\Tcpip\Parameters
            新建
            值名称：TCPTimedWaitDelay = DWORD 30 秒
            值类型： KeepAliveInterval = DWORD 2000 毫秒
            重启电脑
         */
        pipeline.addLast("idleStateHandler", new IdleStateHandler(10, 0, 0, TimeUnit.MINUTES));
        pipeline.addLast("handler", channelHandler);
    }

    public void setChannelHandler(ChannelHandler channelHandler) {
        this.channelHandler = channelHandler;
    }

}
