package com.qhiot.transport.netty.transfer;

import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.socket.nio.NioSocketChannel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.TimeUnit;

/**
 * 数据转发工具
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-08-24 16:08
 */
public class Transfer {
    private Logger LOGGER = LoggerFactory.getLogger(Transfer.class);
    private int       port;
    private String    host;
    private Bootstrap bootstrap;
    private Channel transferChannel;
    private boolean   connected;
    private Channel deviceChannel;
    private ChannelFutureListener connectFutureListener;
    private ChannelFutureListener closeFutureListener;

    public Transfer(Channel deviceChannel, Bootstrap bootstrap, String host, int port) {
        this.deviceChannel = deviceChannel;
        this.port = port;
        this.host = host;
        this.bootstrap = bootstrap;
        connected = false;
    }

    public void close() {
        connected = false;
        if (transferChannel != null && transferChannel.isOpen()) {
            transferChannel.close();
        }
        transferChannel = null;
    }

    public void connect() {
        if (connected && (transferChannel != null && transferChannel.isOpen())) {
            return;
        }

        if (transferChannel != null) {
            ((NioSocketChannel) transferChannel).shutdown();
        }

        ChannelFuture future = bootstrap.connect(host, port);
        if (connectFutureListener == null) {
            connectFutureListener = new ChannelFutureListener() {
                @Override
                public void operationComplete(ChannelFuture f) throws Exception {
                    connected = f.isSuccess();
                }
            };
        }
        future.addListener(connectFutureListener);
        future.awaitUninterruptibly(1000, TimeUnit.MILLISECONDS);

        transferChannel = future.channel();

        ChannelFuture closeFuture = transferChannel.closeFuture();
        if (closeFutureListener == null) {
            closeFutureListener = new ChannelFutureListener() {
                @Override
                public void operationComplete(ChannelFuture future) throws Exception {
                    connected = false;
                }
            };
        }
        closeFuture.addListener(closeFutureListener);
    }

    public void writeAndFlush(ByteBuf buf) {
        LOGGER.debug("进入转发");
        if (transferChannel == null || !connected || !transferChannel.isOpen()) {
            connected = false;
            connect();
        }
        if (transferChannel != null) {
            if (((NioSocketChannel) transferChannel).isShutdown()) {
                connected = false;
                connect();
            }
            LOGGER.debug("转发开始");
            transferChannel.writeAndFlush(buf);
        } else {
            LOGGER.debug("通道无法建立");
        }
    }

    public int getPort() {
        return port;
    }

    public String getHost() {
        return host;
    }

}
