package com.qhiot.transport.netty.transfer;

import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;

import java.util.UUID;
import java.util.Vector;

/**
 * 数据转发器, 可以同时转发到多个目标
 *
 * @author 金建强(ptma@163.com)
 * @version 2016-08-24 16:09
 */
public class Transferrer {

    private final String   id;
    private Bootstrap bootstrap;
    private Vector<Transfer> transfers;
    private Channel deviceChannel;

    public Transferrer(Channel deviceChannel, Bootstrap bootstrap) {
        this.deviceChannel = deviceChannel;
        this.bootstrap = bootstrap;
        id = UUID.randomUUID().toString();
        transfers = new Vector<Transfer>();
    }

    public void addTransfer(String host, int port) {
        if (!hasDestination(host, port)) {
            transfers.add(new Transfer(deviceChannel, bootstrap, host, port));
        }
    }

    private boolean hasDestination(String host, int port) {
        for (Transfer client : transfers) {
            if (client.getHost().equals(host) && client.getPort() == port) {
                return true;
            }
        }
        return false;
    }

    public void writeAndFlush(ByteBuf buf) {
        for (Transfer client : transfers) {
            client.writeAndFlush(buf.copy());
        }
    }

    public void close() {
        for (Transfer client : transfers) {
            client.close();
        }
    }

    public String getId() {
        return id;
    }

    public String getDestination() {
        StringBuffer sb = new StringBuffer();
        for (Transfer client : transfers) {
            sb.append(client.getHost()).append(":").append(client.getPort()).append(", ");
        }
        if (sb.length()>2) {
            sb.delete(sb.length()-2, sb.length());
        }
        return sb.toString();
    }
}
