package com.qhiot.websocket;

import com.qhiot.core.util.JsonUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.simp.SimpMessageSendingOperations;
import org.springframework.stereotype.Component;

/**
 * WebSocket & Stomp 订阅广播工具
 *
 * @author 金建强(ptma@163.com)
 * @version 2016/4/6 23:03
 */
@Component
public class WebsocketBroadcaster {

    /** 应用日志广播路径 */
    public static final String LOGGER_DEST_TEMPLATE           = "/topic/logger";

    /** 传输日志广播路径模板 {终端编号} */
    public static final String CODECLOG_DEST_TEMPLATE         = "/topic/transport/codeclog/{}";

    /** 监测数据广播路径模板 {终端编号} */
    public static final String MONITORING_DATA_DEST_TEMPLATE  = "/topic/transport/monitoringdata/{}";

    /** 站点监测数据广播路径模板 {站点编号} */
    public static final String STATION_MONITORING_DATA_DEST_TEMPLATE  = "/topic/transport/station/monitoringdata/{}";

    /** 图片上传进度广播路径模板 {终端编号} */
    public static final String SNAP_PROGRESS_DEST_TEMPLATE    = "/topic/transport/snap/progress/{}";

    /** 固件升级进度广播路径模板 {终端编号} */
    public static final String UPGRADE_PROGRESS_DEST_TEMPLATE = "/topic/transport/upgrade/progress/{}";

    /** 固件升级取消消息广播路径模板 {终端编号} */
    public static final String UPGRADE_CANCEL_DEST_TEMPLATE   = "/topic/transport/upgrade/cancel/{}";

    /** 站点配置下发进度广播路径模板 {站点id} */
    public static final String CONFIG_SEND_PROGRESS_DEST_TEMPLATE = "/topic/transport/config/send/progress/{}";

    /** 终端连接状态广播路径模板 {项目ID} {终端编号} */
    public static final String CONNECTION_STATE_DEST_TEMPLATE = "/topic/transport/connection/{}/{}";

    /** 告警信息广播路径模板 {项目ID}*/
    public static final String WARNING_DEST_TEMPLATE = "/topic/warning/{}";

    /** 通知广播路径模板 {项目ID}*/
    public static final String NOTICE_DEST_TEMPLATE = "/topic/notice/{}";

    /** app定位信息广播路径模板 {项目ID}*/
    public static final String WECHAT_LOCATION_TEMPLATE = "/topic/location/wechat/{}";

    /** 任务取消广播路径模板 {项目ID}*/
    public static final String TASK_NOTICE_TEMPLATE = "/topic/task/notice/{}";

    /** 公厕人流消息推送 {项目ID}*/
    public static final String TASK_TRANSPORT_TOILET = "/topic/transport/toilet/{}";

    /** 第三方接入数据推送 */
    public static final String TASK_THIRD_UNIVESAL_DATA  = "/topic/third/univesal/data/{}";


    public static final String TASK_EXPORT_PROGRESS  = "/topic/export/progress/backup";

    @Autowired
    private SimpMessageSendingOperations messagingTemplate;

    /**
     * 广播对象, 对象会转成 JSON 后再广播
     *
     * @param destination 订阅地址
     * @param message 消息对象
     */
    public void broadcast(String destination, Object message) {
        if(!destination.startsWith("/topic/")){
            throw new RuntimeException("广播地址必须以 /topic/ 开头");
        }
        messagingTemplate.convertAndSend(destination, JsonUtil.toJson(message));
    }

    /**
     * <p>给指定用户发送对象, 对象会转成 JSON 后再发送
     * <p>用户订阅地址为 "/user/{username}/{destination}"</p>
     *
     * @param user 指定的用户
     * @param destination 订阅地址
     * @param message 消息对象
     */
    public void sendToUser(String user, String destination, Object message) {
        messagingTemplate.convertAndSendToUser(user, destination, JsonUtil.toJson(message));
    }
}
